/**
 * Dashboard InterfaceDocuments component.
 * 
 * @namespace Alfresco.dashlet
 * @class Alfresco.dashlet.InterfaceDocuments
 */

(function() {
  /**
   * YUI Library aliases
   */
  var Dom = YAHOO.util.Dom, Event = YAHOO.util.Event;

  /**
   * Alfresco Slingshot aliases
   */
  var $html = Alfresco.util.encodeHTML, $siteURL = Alfresco.util.siteURL;

  
  /**
   * Parser for html elements
   */
  function ElementParser(){
      this.elements=[];

      this.parsers={
          "select-multiple":this.getSelectMultipleValue,
          "text":this.getText
      };
  }

  ElementParser.prototype.registerObject=function(o){
      this.elements.push({o:o});
  };

  ElementParser.prototype.getText=function(o){
        var a=[];
            a.push(o.value);
        return a;
  };

  ElementParser.prototype.getSelectMultipleValue=function(o){
      var a=[];
      for (var i=0;i<o.options.length;i++){
          if (o.options[i].selected) a.push(o.options[i].value);
      }
      return a;
  };

  ElementParser.prototype.getUrlElements=function(){
      var r=[];
      for (var i=0;i<this.elements.length;i++){
          if (Array.isArray(this.elements[i].value)){
              for (var j=0;j<this.elements[i].value.length;j++){
                  r.push(encodeURIComponent(this.elements[i].name)+'='+encodeURIComponent(this.elements[i].value[j]));
              }
          }
      }
      return r;
  };

  ElementParser.prototype.updateAllElements=function(elements){
      for (var i=0;i<this.elements.length;i++){
          this.elements[i].name=this.elements[i].o.name;
          this.elements[i].type=this.elements[i].o.type;
          if (typeof this.parsers[this.elements[i].type]) {
              this.elements[i].value=this.parsers[this.elements[i].type](this.elements[i].o);
          } // else error
      }
  };


  // Dom elements
  var reviewStatusListbox=null;
  var documentTypeListbox=null;
  var areaListbox=null;
  var interfaceReviewersAsText=null;
  var contractSite=null;
  var searchButton=null;
  var resetButton=null;
  var myDataSource=null;
  var myDataTable=null;
  
  /**
   * Request all documents from repo
   * @returns document data structure
   */
  function getDocumentsDataSource(){
    myDataSource = new YAHOO.util.DataSource(Alfresco.constants.PROXY_URI + "lyse/project/" + Alfresco.constants.SITE + "/interface-documents");
    myDataSource.responseType = YAHOO.util.DataSource.TYPE_JSON;
    myDataSource.responseSchema.resultsList = "data";
    return myDataSource;
  }

  

  /**
   * Dashboard InterfaceDocuments constructor.
   * 
   * @param {String}
   *            htmlId The HTML id of the parent element
   * @return {Alfresco.dashlet.InterfaceDocuments} The new component instance
   * @constructor
   */
  Alfresco.dashlet.InterfaceDocuments = function InterfaceDocuments_constructor(htmlId) {
    Alfresco.dashlet.InterfaceDocuments.superclass.constructor.call(this, "Alfresco.dashlet.InterfaceDocuments", htmlId, [ "button", "container", "datasource", "datatable",  "animation" ]);

    // Services
    this.services.preferences = new Alfresco.service.Preferences();

    return this;
  };

  /**
   * Extend from Alfresco.component.Base
   */
  YAHOO.extend(Alfresco.dashlet.InterfaceDocuments, Alfresco.component.Base);

  /**
   * Augment prototype with main class implementation, ensuring overwrite is enabled
   */
  YAHOO.lang.augmentObject(Alfresco.dashlet.InterfaceDocuments.prototype, {
   
    /**
     * Object container for initialization options
     * 
     * @property options
     * @type object
     */
    options : {
      
      /**
       * Maximum number of tasks to display in the dashlet.
       * 
       * @property maxItems
       * @type int
       * @default 50
       */
      maxItems : 50
    },
    
    
    populateFilter : function InterfaceDocuments_populateFilter(){
    	this.populateConstraintListbox(reviewStatusListbox, "lyse:cdlReviewStatusConstraint","listconstraint.lyse_cdlReviewStatusConstraint");
    	this.populateCategoryListbox(documentTypeListbox, "Title-2");
        this.populateCategoryListbox(areaListbox, "Area-system");
    },
    
    populateCategoryListbox : function InterfaceDocuments_populateCategoryListbox(listbox, categoryParentName){
        var setOptionsCallback = 
        {
          success: function(o) 
          {
            var values, i;
            
            try 
            {
              values = YAHOO.lang.JSON.parse(o.responseText);
              listbox.remove(listbox.length - 1); // Remove "Loading..." text
            } 
            catch (x) 
            {
              return;
            }
           
            for(i = 0; i < values.length; i++) 
            {
              var v = values[i];
              var option = new Option('option');
              
              option.value = (v.value === null || v.value === "") ? v.label : v.value;
              option.text = v.label;
              listbox.add(option);
            }
          },
            failure: function(o) 
            {
              Alfresco.util.PopupManager.displayMessage({text: "Could not load values!"});
            },
            scope: this
        };
        
        var url = Alfresco.constants.PROXY_URI + "api/lyse/forms/subcategories/interface/" + Alfresco.constants.SITE +"/" + categoryParentName +"?sort=" + this.options.sort;
        var transaction = YAHOO.util.Connect.asyncRequest('GET', url, setOptionsCallback, null);
      
    },
    
    populateConstraintListbox : function InterfaceDocuments_populateConstraintListbox(listbox, constraintShortName,msg){
    	var setOptionsCallback = 
		{
			success: function(o) 
			{
				var values, i;
				
				try 
				{
					values = YAHOO.lang.JSON.parse(o.responseText);
					listbox.remove(listbox.length - 1); // Remove "Loading..." text
				} 
				catch (x) 
				{
					return;
				}

				for(i = 0; i < values.length; i++) 
				{
					var v = values[i];
					var option = new Option('option');

					option.value = v.value;
					option.text = v.value;
					if (typeof msg != "undefined") {
						option.text=this.msg(msg+"."+v.value);
					}
					listbox.add(option);
				}
			},
	  		failure: function(o) 
	  		{
	  			Alfresco.util.PopupManager.displayMessage({text: "Could not load values!"});
	  		},
	  		scope: this
		};
		var constraintUrl = Alfresco.constants.PROXY_URI + "api/lyse/constraint-allowed-values?constraintQName="+constraintShortName;
		var transaction = YAHOO.util.Connect.asyncRequest('GET', constraintUrl, setOptionsCallback, null);
    	
    },
    
    /**
     * Fired by YUI when parent element is available for scripting
     * 
     * @method onReady
     */
    onReady : function InterfaceDocuments_onReady() {
      var me = this;
      reviewStatusListbox = YAHOO.util.Dom.get(this.id + "-review-status");
      documentTypeListbox = YAHOO.util.Dom.get(this.id + "-document-type");
      areaListbox = YAHOO.util.Dom.get(this.id + "-area");
      interfaceReviewersAsText=YAHOO.util.Dom.get(this.id + "-interface-reviewers-astext");
      contractSite=YAHOO.util.Dom.get(this.id + "-contract_site");
      searchButton = YAHOO.util.Dom.get(this.id + "-search-button");
      resetButton = YAHOO.util.Dom.get(this.id + "-reset-button");
      this.populateFilter();

      var e=new ElementParser();
      e.registerObject(reviewStatusListbox);
      e.registerObject(documentTypeListbox);
      e.registerObject(areaListbox);
      e.registerObject(interfaceReviewersAsText);
      e.registerObject(contractSite);

      Y.util.Event.addListener(searchButton,"click",function(){
        e.updateAllElements();
        var ue=e.getUrlElements();
        var query=ue.join('&');
        myDataTable.load({request:'?'+query});
      });
      Y.util.Event.addListener(resetButton,"click",function(){
        reviewStatusListbox.selectedIndex=-1
        documentTypeListbox.selectedIndex=-1;
        areaListbox.selectedIndex=-1;
        interfaceReviewersAsText.value="";
        contractSite.value="";
        e.updateAllElements();
        myDataTable.load();
      });

      YAHOO.example.Basic = function(){

          /**
           * Render cell as link
           */
          var renderCellLink = function InterfaceDocuments_renderCellAsLink(elCell, oRecord, oColumn, oData)
          {
              elCell.innerHTML = YAHOO.lang.substitute('<a class="interface-dashlet-link" href="{context}page/lyse-datalist-details?nodeRef={nodeRef}" target="_blank" >{title}</a>',
                  {
                      context: Alfresco.constants.URL_CONTEXT,
                      nodeRef: oRecord.getData("noderef"),
                      title: oRecord.getData(oColumn.field)
                  });
          };

          var renderCellStatus = function InterfaceDocuments_renderCellStatus(elCell, oRecord, oColumn, oData)
          {
              elCell.innerHTML = me.msg("listconstraint.lyse_cdlReviewStatusConstraint." + oRecord.getData(oColumn.field));
          };

          /**
           * Render cell reviewers for task
           */
          var renderCellReviewersForTask = function InterfaceDocuments_renderCellReviewersForTask(elCell, oRecord, oColumn, oData)
          {
          	elCell.innerHTML = $html(getInterfaceReviewersFromRecord(oRecord));
          
          };
          
          /**
           * Render cell workflow tasks
           */
          var renderCellWorkflowTasks = function InterfaceDocuments_renderCellWorkflowTasks(elCell, oRecord, oColumn, oData)
          {
            var tasksHtml = '';
            if (oRecord.getData("workflowTasks") != null) {
               for (var i = 0; i < oRecord.getData("workflowTasks").length; i++) {
                  var taskData = oRecord.getData("workflowTasks")[i];
                  tasksHtml += YAHOO.lang.substitute('<a href="{taskUrl}" target="_blank"><img src="{imgUrl}" title="{imgTitle}"/></a>', {
                     taskUrl: Alfresco.constants.URL_PAGECONTEXT + 'lyse-task-edit?taskId=' + $html(taskData.id) + '&workflowId='+ $html(taskData.workflowId),
                     imgUrl: Alfresco.constants.URL_RESCONTEXT + 'components/images/' + (taskData.isPooled == 'true' ? 'people.png' : 'users.png'),
                     imgTitle: $html(taskData.title)
                  });
               }
            }
            elCell.innerHTML = '<div id="' + this.id + '-actions-' + oRecord.getId() + '">' + tasksHtml + '</div>';

          };
          /**
           * Render cell view info
           */
          var renderCellViewInfo = function InterfaceDocuments_renderCellViewInfo(elCell, oRecord, oColumn, oData)
          {
          	var href;
          	
            var viewInfoHtml = '';
            if (oRecord.getData("workflowTasks") != null) {
               for (i = 0; i < oRecord.getData("workflowTasks").length; i++) {
            	  var taskData = oRecord.getData("workflowTasks")[i];
            	  
            	  viewInfoHtml += YAHOO.lang.substitute('<a href="{infoUrl}" target="_blank"><img src="{imgUrl}" title="{imgTitle}"/></a>', {
                     infoUrl: $siteURL('lyse-datalist-details?workflowId=' + $html(taskData.workflowId)),
                     imgUrl: Alfresco.constants.URL_RESCONTEXT + '../components/images/task-16.png',
                     imgTitle: $html("view all information")
                  });
            	  break;
                  
               }
               
               if (oRecord.getData("workflowTasks").length == 0){
               	  // No existing workflow use nodeRef of list item for link 
               	  viewInfoHtml += YAHOO.lang.substitute('<a href="{infoUrl}" target="_blank"><img src="{imgUrl}" title="{imgTitle}"/></a>', {
                       infoUrl: $siteURL('lyse-datalist-details?nodeRef=' + $html(oRecord.getData("noderef"))),
                       imgUrl: Alfresco.constants.URL_RESCONTEXT + '../components/images/task-16.png',
                       imgTitle: $html("view all information")
                    });
            	   
               }
            }
            elCell.innerHTML = '<div id="' + this.id + '-actions-' + oRecord.getId() + '">' + viewInfoHtml + '</div>';

          };

        var myColumnDefs = [
          {key:"contract", label: me.msg("label.contract"), sortable:true, resizeable:true},
          {key:"workflowTasks", label: me.msg("label.task"), sortable:true, resizeable:true, formatter: renderCellWorkflowTasks},
          {key:"", sortable:true, resizeable:true, formatter: renderCellViewInfo},
          {key:"id", label: me.msg("label.id"), sortable:true, resizeable:true, formatter: renderCellLink},
          {key:"title1", label: me.msg("label.title1"), sortable:true, resizeable:true, formatter: renderCellLink},
          {key:"type", label: me.msg("label.type"), sortable:true, resizeable:true},
          {key:"area", label: me.msg("label.area"), sortable:true, resizeable:true},
          {key:"title4", label: me.msg("label.title4"), sortable:true, resizeable:true},
          {key:"dueDate", label: me.msg("label.dueDate"), sortable:true, resizeable:true},
          {key:"reviewDueDate", label: me.msg("label.reviewDueDate"), sortable:true, resizeable:true},
          {key:"status", label: me.msg("label.reviewStatus"), sortable:true, resizeable:true, formatter: renderCellStatus},
          {key:"docno", label: me.msg("label.docno"), sortable:true, resizeable:true},
          {key:"revno", label: me.msg("label.revno"), sortable:true, resizeable:true},
          {key:"revdate", label: me.msg("label.revdate"), sortable:true, resizeable:true},
          {key:"interfaceReviewers", label: me.msg("label.interfaceReviewers"), sortable:true, formatter: renderCellReviewersForTask}
        ];

        myDataSource = getDocumentsDataSource();
        myDataTable = new YAHOO.widget.DataTable(me.id + "-documents", myColumnDefs, myDataSource );
        myDataTable.on('rowDblclickEvent',function(aArgs) {
          var theTarget = aArgs.target;
          var theRecord = this.getRecord(theTarget);
          var data=theRecord.getData();
          var templateUrl = Alfresco.constants.URL_CONTEXT + 'page/lyse-datalist-details?nodeRef=' + data.noderef;
          return window.open(templateUrl, "_blank");
        });

        return {
          oDS: myDataSource,
          oDT: myDataTable
        };
      }();
    }
    
  });
  function getInterfaceReviewersFromRecord(oRecord){
      var reviewersHtml = '';
      if (oRecord.getData("interfaceReviewers") != null) {

          for (i = 0; i < oRecord.getData("interfaceReviewers").length; i++) {
              var reviewer = oRecord.getData("interfaceReviewers")[i];
              reviewersHtml += reviewer.fullName;
              if (i < oRecord.getData("interfaceReviewers").length-1){
            	  reviewersHtml += ", ";
              }
          }
      }
      return reviewersHtml;
 }
})();
