/**
 * Dashboard LyseTasks component.
 *
 * @namespace Alfresco.dashlet
 * @class Alfresco.dashlet.LyseTasks
 */
(function() {
  /**
   * YUI Library aliases
   */
  var Dom = YAHOO.util.Dom, Event = YAHOO.util.Event, Selector = YAHOO.util.Selector;

  /**
   * Alfresco Slingshot aliases
   */
  var $html = Alfresco.util.encodeHTML, $siteURL = Alfresco.util.siteURL;

  /**
   * Dashboard LyseTasks constructor.
   *
   * @param {String}
   *            htmlId The HTML id of the parent element
   * @return {Alfresco.dashlet.LyseTasks} The new component instance
   * @constructor
   */
  Alfresco.dashlet.LyseTasks = function LyseTasks_constructor(htmlId) {
    Alfresco.dashlet.LyseTasks.superclass.constructor.call(this, "Alfresco.dashlet.LyseTasks", htmlId, [ "button", "container", "datasource", "datatable", "paginator", "history", "animation" ]);

    // Services
    this.services.preferences = new Alfresco.service.Preferences();

    return this;
  };

  /**
   * Extend from Alfresco.component.Base
   */
  YAHOO.extend(Alfresco.dashlet.LyseTasks, Alfresco.component.Base);

  /**
   * Augment prototype with Common Workflow actions to reuse createFilterURLParameters
   */
  YAHOO.lang.augmentProto(Alfresco.dashlet.LyseTasks, Alfresco.action.WorkflowActions);

  /**
   * Augment prototype with main class implementation, ensuring overwrite is enabled
   */
  YAHOO.lang.augmentObject(Alfresco.dashlet.LyseTasks.prototype, {
    /**
     * Preferences
     */
    PREFERENCES_TASKS_DASHLET : "",
    PREFERENCES_TASKS_DASHLET_FILTER : "",

    PREF_SORT_ASCENDING : ".sortAscending",
    PREF_SORT_FIELD :".sortField",

    /**
     * Object container for initialization options
     *
     * @property options
     * @type object
     */
    options : {

     /**
      * Flag indicating whether sort direction is ascending (true) or descending (false)
      *
      * @property sortAscending
      * @type boolean
      */
     sortAscending: true,

     /**
       * Task types not to display
       *
       * @property hiddenTaskTypes
       * @type object
       * @default []
       */
      hiddenTaskTypes : [],

      /**
       * Maximum number of tasks to display in the dashlet.
       *
       * @property maxItems
       * @type int
       * @default 50
       */
      maxItems : 50,

      /**
       * Filter look-up: type to display value and query value
       *
       * @property filters
       * @type Object
       */
      filters : {},

      siteFilter : "undefined",

      /**
       * OrderByOptions: ordering of the result
       *
       * @property orderByOptions
       * @type Object
       */
      orderByOptions : {},


      /**
       * Date filter look-up: type to display value and query value
       *
       * @property filters
       * @type Object
       */
      dateFilters : {},

      /**
       * Exclude filter look-up: type to display value and query value
       *
       * @property filters
       * @type Object
       */
      excludeFilters : {}
    },

    /**
     * Fired by YUI when parent element is available for scripting
     *
     * @method onReady
     */
    onReady : function LyseTasks_onReady() {
      // Create filter menu
      this.widgets.filterMenuButton = Alfresco.util.createYUIButton(this, "filters", this.onFilterSelected, {
        type : "menu",
        menu : "filters-menu",
        lazyloadmenu : false
      });

      // Load preferences (after which the appropriate tasks will be displayed)
      this.PREFERENCES_TASKS_DASHLET = this.services.preferences.getDashletId(this, "tasks");
      this.PREFERENCES_TASKS_DASHLET_FILTER = this.PREFERENCES_TASKS_DASHLET + ".filter";
      var prefs = this.services.preferences.get();


      this.widgets.siteFilter = Alfresco.util.createYUIButton(this, "sitefilters", this.onSiteFilter, {
        type: "menu",
        menu: "sitefilters-menu",
        lazyloadmenu: false
      });
      this.widgets.siteFilter.set("label","Filter by All " + Alfresco.constants.MENU_ARROW_SYMBOL);
      this.widgets.siteFilter.value = this.options.siteFilter;

      var filterwidget = this.widgets.siteFilter;
      if(Alfresco.constants.SITE) {
      	 Alfresco.util.Ajax.request({
            url: Alfresco.constants.PROXY_URI + "api/lyse/get/project/sites/" + Alfresco.constants.SITE,
            successCallback:
            {
               fn: function(response){
         		  var items = response.json;
         		  if(items){
	         		if(items.length < 1){
      					filterwidget.destroy();
	         		}
         			for (var i = 0; i < items.length; i++) {
						var newMenuItem = this.widgets.siteFilter._menu.addItem("Filter by " + items[i].title);
						newMenuItem.value = items[i].shortName;
               		}
      				this.widgets.siteFilter.removeClass('hidden');
               	  }else{
      				filterwidget.destroy();
               	  }
               },
               scope: this
            }
         });
      }else{
      	filterwidget.destroy();
      }

      // Select the preferred filter in the ui
      var filter = Alfresco.util.findValueByDotNotation(prefs, this.PREFERENCES_TASKS_DASHLET_FILTER, "activeTasks");
      filter = this.options.filters.hasOwnProperty(filter) ? filter : "activeTasks";
      this.widgets.filterMenuButton.set("label", this.msg("filter." + filter) + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
      this.widgets.filterMenuButton.value = filter;

      // Create orderBy menu
      this.widgets.orderByMenuButton = Alfresco.util.createYUIButton(this, "orderBy", this.onOrderBySelected, {
        type : "menu",
        menu : "orderBy-menu",
        lazyloadmenu : false
      });
      this.PREFERENCES_TASKS_DASHLET_ORDER_BY = this.PREFERENCES_TASKS_DASHLET + ".orderBy";

      // Select the preferred orderBy in the ui
      var orderBy = Alfresco.util.findValueByDotNotation(prefs, this.PREFERENCES_TASKS_DASHLET_ORDER_BY, "dueDate");
      orderBy = this.options.orderByOptions.hasOwnProperty(orderBy) ? orderBy : "dueDate";
      this.widgets.orderByMenuButton.set("label", this.msg("orderBy." + orderBy) + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
      this.widgets.orderByMenuButton.value = orderBy;

      // Create date filter menu
      this.widgets.dateFilterMenuButton = Alfresco.util.createYUIButton(this, "dateFilters", this.onDateFilterSelected, {
        type : "menu",
        menu : "dateFilters-menu",
        lazyloadmenu : false
      });
      this.PREFERENCES_TASKS_DASHLET_DATE_FILTER = this.PREFERENCES_TASKS_DASHLET + ".dateFilter";

      // Select the preferred date filter in the ui
      var dateFilter = Alfresco.util.findValueByDotNotation(prefs, this.PREFERENCES_TASKS_DASHLET_DATE_FILTER, "all");
      dateFilter = this.options.dateFilters.hasOwnProperty(dateFilter) ? dateFilter : "all";
      this.widgets.dateFilterMenuButton.set("label", this.msg("dateFilter." + dateFilter) + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
      this.widgets.dateFilterMenuButton.value = dateFilter;

      // Create exclude filter menu
      this.widgets.excludeFilterMenuButton = Alfresco.util.createYUIButton(this, "excludeFilters", this.onExcludeFilterSelected, {
        type : "menu",
        menu : "excludeFilters-menu",
        lazyloadmenu : false
      });
      this.PREFERENCES_TASKS_DASHLET_EXCLUDE_FILTER = this.PREFERENCES_TASKS_DASHLET + ".excludeFilter";

      // Select the preferred exclude filter in the ui
      var excludeFilter = Alfresco.util.findValueByDotNotation(prefs, this.PREFERENCES_TASKS_DASHLET_EXCLUDE_FILTER, "none");
      excludeFilter = this.options.excludeFilters.hasOwnProperty(excludeFilter) ? excludeFilter : "none";
      this.widgets.excludeFilterMenuButton.set("label", this.msg("excludeFilter." + excludeFilter) + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
      this.widgets.excludeFilterMenuButton.value = excludeFilter;

      // Sort Direction button
      this.widgets.sortAscending = Alfresco.util.createYUIButton(this, "sortAscending-button", this.onSortAscending);
      if (this.widgets.sortAscending !== null)
      {
         this.widgets.sortAscending.set("title", this.msg(this.options.sortAscending ? "button.sort.descending" : "button.sort.ascending"));
         if (!this.options.sortAscending)
         {
            Dom.addClass(this.widgets.sortAscending.get("element"), "sort-descending");
         }
      }


      // Display the toolbar now that we have selected the filter
      Dom.removeClass(Selector.query(".toolbar div", this.id, true), "hidden");

      // Prepare webscript url to task instances
      var taskProperties = ["lysewf_site", "bpm_priority", "bpm_status", "bpm_dueDate", "bpm_description", "lysewf_relatedDatalistItem", "lysewf_taskStartTime" ];
      var webscript = YAHOO.lang.substitute("api/site-task-instances?site={site}&authority={authority}&properties={properties}", {
        site : (Alfresco.constants.SITE.length > 0) ? Alfresco.constants.SITE : "allsites",
        authority : encodeURIComponent(Alfresco.constants.USERNAME),
        properties : taskProperties.join(",")
      });

      /**
       * Create datatable with a simple pagination that only displays number of results. The pagination is handled in the "base" data source url and can't be changed in the dashlet
       */
      this.widgets.alfrescoDataTable = new Alfresco.util.DataTable({
        dataSource : {
          url : Alfresco.constants.PROXY_URI + webscript,
          filterResolver : this.bind(function() {
            // Reuse method form WorkflowActions for dateFilter
            var dateFilter = this.widgets.dateFilterMenuButton.value;
            var dateFilterParameters = this.options.dateFilters[dateFilter];
            var params = this.substituteParameters(dateFilterParameters) || "";

            // ...and then add the other filter
            var filter = this.widgets.filterMenuButton.value;
            var filterParameters = this.options.filters[filter];
            var filteredOtherFilter = this.substituteParameters(filterParameters) || "";

            // ...and then add the orderBy
            var orderBy = this.widgets.orderByMenuButton.value;
            var orderByParameters = this.options.orderByOptions[orderBy];

            // ...and the sortAscending/Descending
            var sortOrder = this.options.sortAscending ? "asc" : "desc";

            // ...and then the exclude filter
            var excludeFilter = this.widgets.excludeFilterMenuButton.value;
            var excludeFilterParameters = this.options.excludeFilters[excludeFilter];
            var excludeFilterArray = Object.assign([], this.options.hiddenTaskTypes);
            if (excludeFilterParameters.length > 0) {
              excludeFilterArray.push(excludeFilterParameters);
            }

            // Join the filters
            params = params + "&exclude=" + excludeFilterArray.join(",");
            params = params + "&" + filteredOtherFilter;
            params = params + "&" + orderByParameters;
            params = params + "&sortOrder=" + sortOrder;
            params = params + "&filtersite=" + this.options.siteFilter;
            return params;
          })
        },
        dataTable : {
          container : this.id + "-tasks",
          columnDefinitions : [ {
            key : "isPooled",
            sortable : false,
            formatter : this.bind(this.renderCellIcons),
            width : 24
          }, {
            key : "title",
            sortable : false,
            formatter : this.bind(this.renderCellTaskInfo)
          }, {
            key : "startDate",
            sortable : false,
            formatter : this.bind(this.renderCellDate)
          } , {
            key : "name",
            sortable : false,
            formatter : this.bind(this.renderCellActions),
            width : 45
          } ],
          config : {
            MSG_EMPTY : this.msg("message.noTasks")
          }
        },
        paginator : {
          history : false,
          hide : false,
          config : {
            containers : [ this.id + "-paginator" ],
            template : "{FirstPageLink} {PreviousPageLink} {CurrentPageReport} {NextPageLink} {LastPageLink}",
            firstPageLinkLabel : "&lt;&lt;",
            previousPageLinkLabel : "&lt;",
            nextPageLinkLabel : "&gt;",
            lastPageLinkLabel : "&gt;&gt;",
            pageReportTemplate : this.msg("pagination.template.page-report"),
            rowsPerPage : this.options.maxItems
          }
        }
      });

      // Override DataTable function to set custom empty message
      var me = this, dataTable = this.widgets.alfrescoDataTable.getDataTable(), original_doBeforeLoadData = dataTable.doBeforeLoadData;

      dataTable.doBeforeLoadData = function LyseTasks_doBeforeLoadData(sRequest, oResponse, oPayload) {
        // Hide the paginator if there are fewer rows than would cause pagination
        if (oResponse.results.length === 0) {
          Dom.addClass(this.configs.paginator.getContainerNodes(), "hidden");
        } else {
          Dom.removeClass(this.configs.paginator.getContainerNodes(), "hidden");
        }

        if (oResponse.results.length === 0) {
          oResponse.results.unshift({
            isInfo : true,
            title : me.msg("empty.title"),
            description : me.msg("empty.description")
          });
        }

        return original_doBeforeLoadData.apply(this, arguments);
      };
    },

    /**
     * Reloads the list with the new filter and updates the filter menu button's label
     *
     * @param p_sType
     *            {string} The event
     * @param p_aArgs
     *            {array} Event arguments
     */
    onFilterSelected : function LyseTasks_onFilterSelected(p_sType, p_aArgs) {
      var menuItem = p_aArgs[1];

      if (menuItem) {
        // Set the currentSkipCount to 0 so after changing the filter the paging starts from the first page
        // Otherwise an error can occur if the previous taks page has more pages than the new page
        this.widgets.alfrescoDataTable.currentSkipCount = 0;

        this.widgets.filterMenuButton.set("label", menuItem.cfg.getProperty("text") + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
        this.widgets.filterMenuButton.value = menuItem.value;

        // Will call filterResolver to get the correct parameters
        this.widgets.alfrescoDataTable.reloadDataTable();

        // Save preferences
        this.services.preferences.set(this.PREFERENCES_TASKS_DASHLET_FILTER, menuItem.value);
      }
    },

    /**
     * Reloads the list with the new filter and updates the filter menu button's label
     *
     * @param p_sType
     *            {string} The event
     * @param p_aArgs
     *            {array} Event arguments
     */
    onExcludeFilterSelected : function LyseTasks_onExcludeFilterSelected(p_sType, p_aArgs) {
      var menuItem = p_aArgs[1];

      if (menuItem) {
        // Set the currentSkipCount to 0 so after changing the filter the paging starts from the first page
        // Otherwise an error can occur if the previous taks page has more pages than the new page
        this.widgets.alfrescoDataTable.currentSkipCount = 0;

        this.widgets.excludeFilterMenuButton.set("label", menuItem.cfg.getProperty("text") + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
        this.widgets.excludeFilterMenuButton.value = menuItem.value;

        // Will call filterResolver to get the correct parameters
        this.widgets.alfrescoDataTable.reloadDataTable();

        // Save preferences
        this.services.preferences.set(this.PREFERENCES_TASKS_DASHLET_EXCLUDE_FILTER, menuItem.value);
      }
    },

    /**
     * Reloads the list with the new date filter and updates the date filter menu button's label
     *
     * @param p_sType
     *            {string} The event
     * @param p_aArgs
     *            {array} Event arguments
     */
    onDateFilterSelected : function LyseTasks_onFilterSelected(p_sType, p_aArgs) {
      var menuItem = p_aArgs[1];

      if (menuItem) {
        // Set the currentSkipCount to 0 so after changing the filter the paging starts from the first page
        // Otherwise an error can occur if the previous taks page has more pages than the new page
        this.widgets.alfrescoDataTable.currentSkipCount = 0;

        this.widgets.dateFilterMenuButton.set("label", menuItem.cfg.getProperty("text") + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
        this.widgets.dateFilterMenuButton.value = menuItem.value;

        // Will call filterResolver to get the correct parameters
        this.widgets.alfrescoDataTable.reloadDataTable();

        // Save preferences
        this.services.preferences.set(this.PREFERENCES_TASKS_DASHLET_DATE_FILTER, menuItem.value);
      }
    },

    /**
     * Reloads the list with a new ordering and updates the orderBy menu button's label
     *
     * @param p_sType
     *            {string} The event
     * @param p_aArgs
     *            {array} Event arguments
     */
    onOrderBySelected : function LyseTasks_onOrderBySelected(p_sType, p_aArgs) {
      var menuItem = p_aArgs[1];

      if (menuItem) {
        // Set the currentSkipCount to 0 so after changing the filter the paging starts from the first page
        // Otherwise an error can occur if the previous taks page has more pages than the new page
        this.widgets.alfrescoDataTable.currentSkipCount = 0;

        this.widgets.orderByMenuButton.set("label", menuItem.cfg.getProperty("text") + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
        this.widgets.orderByMenuButton.value = menuItem.value;

        // Will call filterResolver to get the correct parameters
        this.widgets.alfrescoDataTable.reloadDataTable();

        // Save preferences
        this.services.preferences.set(this.PREFERENCES_TASKS_DASHLET_ORDER_BY, menuItem.value);
      }
    },

    onSiteFilter : function LyseTasks_onSiteFilter(p_sType, p_aArgs) {
      var menuItem = p_aArgs[1];

      if (menuItem) {
      	this.options.siteFilter = menuItem.value;

        this.widgets.alfrescoDataTable.currentSkipCount = 0;

		var text = menuItem.value;
		if(menuItem.element && menuItem.element.children.length > 0 && menuItem.element.children[0].innerHTML != ""){
			text = menuItem.element.children[0].innerHTML;
		}

        this.widgets.siteFilter.set("label", text + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
        this.widgets.siteFilter.value = menuItem.value;

        // Will call filterResolver to get the correct parameters
        this.widgets.alfrescoDataTable.reloadDataTable();
      }
    },

    /**
     * Sort direction button click handler
     *
     * @method onSortAscending
     * @param e {object} DomEvent
     * @param p_obj {object} Object passed back from addListener method
     */
    onSortAscending: function DL_onSortAscending(e, p_obj)
    {
       // Clear sort information when the sort button is clicked (this shouldn't
       // be called when the table view sorts via column header click)...
       this.widgets.alfrescoDataTable.widgets.dataTable.set("sortedBy", null);

       this.options.sortAscending = !this.options.sortAscending;
       this.widgets.sortAscending.set("title", this.msg(this.options.sortAscending ? "button.sort.descending" : "button.sort.ascending"));
       if (this.options.sortAscending)
       {
          Dom.removeClass(this.widgets.sortAscending.get("element"), "sort-descending");
       }
       else
       {
          Dom.addClass(this.widgets.sortAscending.get("element"), "sort-descending");
       }
       this.services.preferences.set(this.PREF_SORT_ASCENDING, this.options.sortAscending);

       // Set the currentSkipCount to 0 so after changing the filter the paging starts from the first page
       // Otherwise an error can occur if the previous taks page has more pages than the new page
       this.widgets.alfrescoDataTable.currentSkipCount = 0;
       // Will call filterResolver to get the correct parameters
       this.widgets.alfrescoDataTable.reloadDataTable();
    },


    /**
     * Priority & pooled icons custom datacell formatter
     */
    renderCellIcons : function LyseTasks_onReady_renderCellIcons(elCell, oRecord, oColumn, oData) {
      var data = oRecord.getData(), desc = "";

      if (data.isInfo) {
        oColumn.width = 52;
        Dom.setStyle(elCell, "width", oColumn.width + "px");
        Dom.setStyle(elCell.parentNode, "width", oColumn.width + "px");

        desc = '<img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/help-task-bw-32.png" />';
      } else {
        var priority = data.properties["bpm_priority"], priorityMap = {
          "1" : "high",
          "2" : "medium",
          "3" : "low"
        }, priorityKey = priorityMap[priority + ""], pooledTask = data.isPooled;

        desc = '<img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/priority-' + priorityKey + '-16.png" title="' + this.msg("label.priority", this.msg("priority." + priorityKey)) + '"/>';
        if (pooledTask) {
          desc += '<br/><img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/pooled-task-16.png" title="' + this.msg("label.pooledTask") + '"/>';
        }
      }

      // Lyse, Remove references to priority and unassigned/pooled tasks (both icon in list and menu filter)
      desc = "";
      elCell.innerHTML = desc;
    },

    /**
     * Task info custom datacell formatter
     */
    renderCellTaskInfo: function LyseTasks_onReady_renderCellTaskInfo(elCell, oRecord, oColumn, oData)
    {
      var data = oRecord.getData(),
          desc = "";

      if (data.isInfo)
      {
        desc += '<div class="empty"><h3>' + data.title + '</h3>';
        desc += '<span>' + data.description + '</span></div>';
      }
      else
      {
        var taskId = data.id, workflowId = data.workflowInstance.id;

        message = data.properties["bpm_description"],
            dueDateStr = data.properties["bpm_dueDate"],
            dueDate = dueDateStr ? Alfresco.util.fromISO8601(dueDateStr) : null,
            today = new Date(),
            type = data.title,
            status = data.properties["bpm_status"],
            assignee = data.owner;

        // if there is a property label available for the status use that instead
        if (data.propertyLabels && Alfresco.util.isValueSet(data.propertyLabels["bpm_status"], false))
        {
          status = data.propertyLabels["bpm_status"];
        }
        // if message is the same as the task type show the <no message> label
        if (message == type)
        {
          message = this.msg("workflow.no_message");
        }

        var href;
        if (data.isEditable)
        {
          href = $siteURL('lyse-task-edit?taskId=' + taskId + '&referrer=tasks&workflowId=' + workflowId);
        }
        else
        {
          href = $siteURL('lyse-datalist-details?workflowId=' + workflowId);
        }

        var viewLink = document.createElement("a");
        viewLink.setAttribute("class", "theme-color-1");
        viewLink.setAttribute("title", this.msg("title.viewTask"));
        viewLink.setAttribute("href",  href);
        viewLink.setAttribute("target", "_blank");
        if (message != null) {
        	viewLink.innerHTML = $html(message);
        }
        elCell.appendChild(viewLink);

        desc += YAHOO.lang.substitute('<span class="{cssClass}" title="{title}">{type} {dueDate}</span>', {
          cssClass: (dueDate && (today > dueDate) ? "task-delayed" : ""),
          title: dueDate ? this.msg("title.dueOn", Alfresco.util.formatDate(dueDate, "longDate")) : "",
          type: type,
          dueDate: dueDate ? " | " + Alfresco.util.formatDate(dueDate, "longDate") : ""
        });

        var el = document.createElement("div");
        el.innerHTML = desc;
        elCell.appendChild(el);

      }
    },

    /**
     * Render cell date
     */
    renderCellDate: function LyseTasks_onReady_renderCellDate(elCell, oRecord, oColumn, oData)
    {
      var data = oRecord.getData();
      if (data.isInfo) {
        oColumn.width = 0;
        Dom.setStyle(elCell, "width", oColumn.width + "px");
        Dom.setStyle(elCell.parentNode, "width", oColumn.width + "px");
      }
      else {
        startDateStr = data.properties.lysewf_taskStartTime;
        startDate = startDateStr ? Alfresco.util.formatDate(Alfresco.util.fromISO8601(startDateStr), "yyyy-mm-dd") : "";

        dueDateStr = data.properties.bpm_dueDate;
        dueDate = dueDateStr ? Alfresco.util.formatDate(Alfresco.util.fromISO8601(dueDateStr), "yyyy-mm-dd") : "";

        var el = document.createElement("div");
        el.innerHTML = "Created: " + startDate + ", Due: " + dueDate;
        elCell.appendChild(el);
      }
    },

    /**
     * Actions custom datacell formatter
     */
    renderCellActions : function LyseTasks_onReady_renderCellActions(elCell, oRecord, oColumn, oData) {
      var data = oRecord.getData(), desc = "";

      if (data.isInfo) {
        oColumn.width = 0;
        Dom.setStyle(elCell, "width", oColumn.width + "px");
        Dom.setStyle(elCell.parentNode, "width", oColumn.width + "px");
      } else {
        var workflowId = data.workflowInstance.id;
        var viewLink = document.createElement("a");
        viewLink.setAttribute("class", "view-task");
        viewLink.setAttribute("title", this.msg("title.viewAction"));
        viewLink.setAttribute("href", Alfresco.constants.URL_PAGECONTEXT + "lyse-datalist-details?workflowId=" + workflowId);
        viewLink.setAttribute("target", "_blank");
        viewLink.innerHTML = '&nbsp;';
        elCell.appendChild(viewLink);
      }
    }

  });
})();
