/**
 * Dashboard ProjectSites component.
 *
 * @namespace Alfresco.dashlet
 * @class Alfresco.dashlet.ProjectSites
 */
(function()
{

   var Dom = YAHOO.util.Dom,
       Event = YAHOO.util.Event,
       Selector = YAHOO.util.Selector;

   var $html = Alfresco.util.encodeHTML,
       $links = Alfresco.util.activateLinks;

   var FAV_EVENTCLASS = Alfresco.util.generateDomId(null, "fav-site"),
       IMAP_EVENTCLASS = Alfresco.util.generateDomId(null, "imap-site"),
       DELETE_EVENTCLASS = Alfresco.util.generateDomId(null, "del-site");


   Alfresco.dashlet.ProjectSites = function ProjectSites_constructor(htmlId)
   {
      Alfresco.dashlet.ProjectSites.superclass.constructor.call(this, "Alfresco.dashlet.ProjectSites", htmlId, ["datasource", "datatable", "animation"]);

      // Initialise prototype properties
      this.sites = [];

	  this.sortOptions = [];
	
      // Services
      this.services.preferences = new Alfresco.service.Preferences();

      return this;
   };

   YAHOO.extend(Alfresco.dashlet.ProjectSites, Alfresco.component.Base,
   {
   
      PREFERENCES_PROJECT_SITES_DASHLET: "",
      PREFERENCES_PROJECT_SITES_DASHLET_FILTER: "",
      PREFERENCES_PROJECT_SITES_DASHLET_SORT: "",
      sites: null,
      favSites: null,
      imapfavSites: null,
      filter: null,

      options:
      {
         validFilters:
         {
            "all": true,
            "active": true,
            "inactive": true
         },
         imapEnabled: false,
         listSize: 100
      },

      onReady: function ProjectSites_onReady()
      {
         var me = this;

         // Fetch preferences
         this.PREFERENCES_PROJECT_SITES_DASHLET = this.services.preferences.getDashletId(this, "sites");
         this.PREFERENCES_PROJECT_SITES_DASHLET_FILTER = this.PREFERENCES_PROJECT_SITES_DASHLET + ".filter";
         this.PREFERENCES_PROJECT_SITES_DASHLET_SORT = this.PREFERENCES_PROJECT_SITES_DASHLET + ".sort";

         // Create Dropdown filter
         this.widgets.filter = Alfresco.util.createYUIButton(this, "filter", this.onFilterChanged,
         {
            type: "menu",
            menu: "filter-menu",
            lazyloadmenu: false
         });
         
         // Create Dropdown sort
         this.widgets.sort = Alfresco.util.createYUIButton(this, "sort", this.onSortChanged,
         {
            type: "menu",
            menu: "sort-menu",
            lazyloadmenu: false
         });

         // DataSource definition
         this.widgets.dataSource = new YAHOO.util.DataSource(this.sites,
         {
            responseType: YAHOO.util.DataSource.TYPE_JSARRAY
         });

         // DataTable column defintions
         var columnDefinitions =
         [
            { key: "icon", label: "Icon", sortable: false, formatter: this.bind(this.renderCellIcon), width: 52 },
            { key: "detail", label: "Description", sortable: false, formatter: this.bind(this.renderCellDetail) },
            { key: "phase", label: "Phase", sortable: false, formatter: this.bind(this.renderCellPhase), width: 150 }
         ];

         // DataTable definition
         this.widgets.dataTable = new YAHOO.widget.DataTable(this.id + "-project-sites", columnDefinitions, this.widgets.dataSource,
         {
            MSG_EMPTY: this.msg("message.datatable.loading")
         });

         // Override abstract function within DataTable to set custom empty message
         this.widgets.dataTable.doBeforeLoadData = function ProjectSites_doBeforeLoadData(sRequest, oResponse, oPayload)
         {
            if ((oResponse.results.length === 0) || (oResponse.results.length === 1 && oResponse.results[0].shortName === "swsdp"))
            {
               oResponse.results.unshift(
               {
                  isInfo: true,
                  title: me.msg("empty.title"),
                  description: me.msg("empty.description") + (oResponse.results.length === 1 ? "<p>" + me.msg("empty.description.sample-site") + "</p>" : "")
               });
            }
            return true;
         };

         // Add animation to row delete
         this.widgets.dataTable._deleteTrEl = function ProjectSites__deleteTrEl(row)
         {
            var scope = this,
               trEl = this.getTrEl(row);

            var changeColor = new YAHOO.util.ColorAnim(trEl,
            {
               opacity:
               {
                  to: 0
               }
            }, 0.25);
            changeColor.onComplete.subscribe(function()
            {
               YAHOO.widget.DataTable.prototype._deleteTrEl.call(scope, row);
            });
            changeColor.animate();
         };

         /**
          * Hook favourite site events
          */
         var registerEventHandler = function ProjectSites_onReady_registerEventHandler(cssClass, fnHandler)
         {
            var fnEventHandler = function ProjectSites_onReady_fnEventHandler(layer, args)
            {
               var owner = YAHOO.Bubbling.getOwnerByTagName(args[1].anchor, "div");
               if (owner !== null)
               {
                  fnHandler.call(me, args[1].target.offsetParent, owner);
               }

               return true;
            };
            YAHOO.Bubbling.addDefaultAction(cssClass, fnEventHandler);
         };

         registerEventHandler(FAV_EVENTCLASS, this.onFavouriteSite);
         registerEventHandler(IMAP_EVENTCLASS, this.onImapFavouriteSite);
         registerEventHandler(DELETE_EVENTCLASS, this.onDeleteSite);

         // Enable row highlighting
         this.widgets.dataTable.subscribe("rowMouseoverEvent", this.widgets.dataTable.onEventHighlightRow);
         this.widgets.dataTable.subscribe("rowMouseoutEvent", this.widgets.dataTable.onEventUnhighlightRow);

         // Load sites & preferences
         this.initPreferences();
         this.loadSites();
      },

      /**
       * Date drop-down changed event handler
       *
       * @method onFilterChanged
       * @param p_sType {string} The event
       * @param p_aArgs {array}
       */
      onSortChanged: function ProjectSites_onSortChanged(p_sType, p_aArgs)
      {
         var menuItem = p_aArgs[1];
         if (menuItem)
         {
            this.widgets.sort.set("label", menuItem.cfg.getProperty("text") + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
            this.widgets.sort.value = menuItem.value;

            // Save preferences
            this.services.preferences.set(this.PREFERENCES_PROJECT_SITES_DASHLET_SORT, menuItem.value,
            {
               successCallback:
               {
                  fn: function() 
                  {
                     // Update local cached copy of current filter
                     this.sort = menuItem.value;

                     // Reload the sites list
                     this.loadSites();
                  },
                  scope: this
               }
            });
         }
      },
      
      /**
       * Date drop-down changed event handler
       *
       * @method onFilterChanged
       * @param p_sType {string} The event
       * @param p_aArgs {array}
       */
      onFilterChanged: function ProjectSites_onFilterChanged(p_sType, p_aArgs)
      {
         var menuItem = p_aArgs[1];
         if (menuItem)
         {
            this.widgets.filter.set("label", menuItem.cfg.getProperty("text") + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
            this.widgets.filter.value = menuItem.value;

            // Save preferences
            this.services.preferences.set(this.PREFERENCES_PROJECT_SITES_DASHLET_FILTER, menuItem.value,
            {
               successCallback:
               {
                  fn: function() 
                  {
                     // Update local cached copy of current filter
                     this.filter = menuItem.value;

                     // Reload the sites list
                     this.loadSites();
                  },
                  scope: this
               }
            });
         }
      },

      /**
       * Init cached state from User Preferences
       *
       * @method initPreferences
       */
      initPreferences: function ProjectSites_initPreferences()
      {
         var prefs = this.services.preferences.get();

         // Retrieve fav sites preferences
         var favSites = Alfresco.util.findValueByDotNotation(prefs, "org.alfresco.share.sites.favourites");
         if (favSites === null)
         {
            favSites = {};
         }
         this.favSites = favSites;
         var imapfavSites = Alfresco.util.findValueByDotNotation(prefs, "org.alfresco.share.sites.imapFavourites");
         if (imapfavSites === null)
         {
            imapfavSites = {};
         }
         this.imapfavSites = imapfavSites;

         var recentSites = Alfresco.util.findValueByDotNotation(prefs, "org.alfresco.share.sites.recent");
         if (recentSites === null)
         {
            recentSites = {};
         }

         var recentSitesArray = [];

         for (key in recentSites)
         {
            recentSitesArray.push(recentSites[key]);
         }

         this.recentSites = recentSites;
         this.recentSitesArray = recentSitesArray;

         // Retrieve the preferred filter for the UI
         var filter = Alfresco.util.findValueByDotNotation(prefs, this.PREFERENCES_PROJECT_SITES_DASHLET_FILTER, "all");
         this.filter = this.options.validFilters.hasOwnProperty(filter) ? filter : "all";
         this.sort = "Recent";
         
      },

      /**
       * Load sites list
       *
       * @method loadSites
       */
      loadSites: function ProjectSites_loadSites()
      {
         // Load sites
         Alfresco.util.Ajax.request(
         {
            url: Alfresco.constants.PROXY_URI + "api/lyse/get/projects?filter=" + this.filter,
            successCallback:
            {
               fn: this.onSitesLoaded,
               scope: this
            }
         });
      },

      /**
       * Process user preferences after sites data has loaded and display the sites list
       *
       * @method onSitesLoaded
       * @param p_response {object} Response from "api/people/{userId}/sites" query
       */
      onSitesLoaded: function ProjectSites_onSitesLoaded(p_response)
      {
         var p_items = p_response.json,
             siteManagers, i, j, k, l, ii = 0;

         // Select the preferred filter in the ui
         this.widgets.filter.set("label", this.msg("filter.project." + this.filter) + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
         this.widgets.filter.value = this.filter;

         // Select the preferred sort in the ui
         this.widgets.sort.set("label", this.msg("sort.by.label") + this.sort + " " + Alfresco.constants.MENU_ARROW_SYMBOL);
         this.widgets.sort.value = this.sort;
         
         // Display the toolbar buttons now that we have selected the filter
         this.widgets.filter.removeClass('hidden');
	     this.widgets.sort.removeClass('hidden');

         for (i = 0, j = p_items.length; i < j; i++)
         {
            p_items[i].isSiteManager = p_items[i].siteRole === "SiteManager";
            p_items[i].isFavourite = !this.favSites[p_items[i].shortName] ? false : this.favSites[p_items[i].shortName];
            p_items[i].isRecent = Alfresco.util.arrayContains(this.recentSitesArray, p_items[i].shortName);
            if (this.imapfavSites)
            {
               p_items[i].isIMAPFavourite = !this.imapfavSites[p_items[i].shortName] ? false : this.imapfavSites[p_items[i].shortName];
            }
         }

         this.sites = [];
         // this.widgets.filter.value is the filtered value.
         for (i = 0, j = p_items.length; i < j; i++)
         {
            var site = YAHOO.lang.merge({}, p_items[i]);
            this.sites[ii] = site;
            ii++;
            
         	if(site.phase && this.sortOptions.indexOf(site.phase) === -1) {
            	this.sortOptions.push(site.phase);
				var newMenuItem = this.widgets.sort._menu.addItem(this.msg("sort.by.label") + site.phase);
				newMenuItem.value = site.phase;
            }
         }
         
         if(this.sort === "Recent") {
	         this.sites.sort(function(a, b)
	         {
        		return (a.isRecent === b.isRecent)? 0 : a.isRecent? -1 : 1;
	         });
         } else if(this.sort === "Favorite") {
	         this.sites.sort(function(a, b)
	         {
        		return (a.isFavourite === b.isFavourite)? 0 : a.isFavourite? -1 : 1;
	         });
         } else if(this.sortOptions.indexOf(this.sort) !== -1) {
         
         	 // First sort them by name.
	         this.sites.sort(function(a, b)
	         {
	            var name1 = a.phase ? a.phase.toLowerCase() : a.phase.toLowerCase(),
	                name2 = b.phase ? b.phase.toLowerCase() : b.phase.toLowerCase();
	            return (name1 > name2) ? 1 : (name1 < name2) ? -1 : 0;
	         });
	         
	         // Put all matched conditions first in temporary array.
	         var sortedList = [];
	         for (var i = 0; i < this.sites.length; i++) {
			    if (this.sites[i].phase === this.sort) {
    				sortedList.unshift(this.sites[i]);
			    }else{
    				sortedList.push(this.sites[i]);
			    }
			 }
			 
			 // Replace array with sorted array.
	         this.sites = sortedList;
         }
         
         
         var successHandler = function ProjectSites_onSitesUpdate_success(sRequest, oResponse, oPayload)
         {
            oResponse.results=this.sites;
            this.widgets.dataTable.set("MSG_EMPTY", "");
            this.widgets.dataTable.onDataReturnInitializeTable.call(this.widgets.dataTable, sRequest, oResponse, oPayload);
         };

         this.widgets.dataSource.sendRequest(this.sites,
         {
            success: successHandler,
            scope: this
         });
      },

      /**
       * Generate "Favourite" UI
       *
       * @method generateFavourite
       * @param record {object} DataTable record
       * @return {string} HTML mark-up for Favourite UI
       */
      generateFavourite: function ProjectSites_generateFavourite(record)
      {
         var html = "";

         if (record.getData("isFavourite"))
         {
            html = '<a class="favourite-action ' + FAV_EVENTCLASS + ' enabled" title="' + this.msg("favourite.site.remove.tip") + '" tabindex="0"></a>';
         }
         else
         {
            html = '<a class="favourite-action ' + FAV_EVENTCLASS + '" title="' + this.msg("favourite.site.add.tip") + '" tabindex="0">' + this.msg("favourite.site.add.label") + '</a>';
         }

         return html;
      },

      /**
       * Generate "IMAP Favourite" UI
       *
       * @method generateIMAPFavourite
       * @param record {object} DataTable record
       * @return {string} HTML mark-up for Favourite UI
       */
      generateIMAPFavourite: function ProjectSites_generateIMAPFavourite(record)
      {
         var html = "";

         if (record.getData("isIMAPFavourite"))
         {
            html = '<a class="favourite-action favourite-imap ' + IMAP_EVENTCLASS + ' enabled" title="' + this.msg("favourite.imap-site.remove.tip") + '" tabindex="0"></a>';
         }
         else
         {
            html = '<a class="favourite-imap ' + IMAP_EVENTCLASS + '" title="' + this.msg("favourite.imap-site.add.tip") + '" tabindex="0">' + this.msg("favourite.imap-site.add.label") + '</a>';
         }

         return html;
      },

      /**
       * Icon custom datacell formatter
       *
       * @method renderCellIcon
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellIcon: function ProjectSites_renderCellIcon(elCell, oRecord, oColumn, oData)
      {
         Dom.setStyle(elCell, "width", oColumn.width + "px");
         Dom.setStyle(elCell.parentNode, "width", oColumn.width + "px");
         
         var site = oRecord.getData(),
            img = site.isInfo ? "help-site-bw-32.png" : "filetypes/generic-site-32.png";

         elCell.innerHTML = '<img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/' + img + '" />';
      },

      /**
       * Name & description custom datacell formatter
       *
       * @method renderCellDetail
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellDetail: function ProjectSites_renderCellDetail(elCell, oRecord, oColumn, oData)
      {
         var site = oRecord.getData(),
            description = '<span style="display:none;" class="faded">' + this.msg("details.description.none") + '</span>',
            desc = "";

         if (site.isInfo)
         {
            desc += '<div class="empty"><h3>' + site.title + '</h3>';
            desc += '<span>' + site.description + '</span></div>';
         }
         else
         {
         
			var classes = "theme-color-1";
			if(site.disabled == true){
				classes = classes + " disabled-project";
			}
			
            // Description non-blank?
            if (site.description && site.description !== "")
            {
               description = $links($html(site.description));
	            desc += '<h3 class="site-title"><a href="' + Alfresco.constants.URL_PAGECONTEXT + 'site/' + site.shortName + '/dashboard" class="theme-color-1 '+ classes +' ">' + $html(site.title) + '</a></h3>';
	            desc += '<div class="detail"><span>' + description + '</span></div>';
            }else{
	            desc += '<h3 class="site-title"><a href="' + Alfresco.constants.URL_PAGECONTEXT + 'site/' + site.shortName + '/dashboard" class="theme-color-1 '+ classes +'">' + $html(site.title) + '</a></h3>';
	            desc += '<div style="display:none;" class="detail"><span>' + description + '</span></div>';
            }
            
			 
            /* Favourite / IMAP */
            desc += '<div class="detail detail-social">';
            desc +=    '<span class="item item-social">' + this.generateFavourite(oRecord) + '</span>';
            if (this.options.imapEnabled)
            {
               desc +=    '<span class="item item-social item-separator">' + this.generateIMAPFavourite(oRecord) + '</span>';
            }
            desc += '</div>';
         }

         elCell.innerHTML = desc;
      },

      /**
       * Project Phase custom datacell formatter
       *
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellPhase: function ProjectSites_renderCellPhase(elCell, oRecord, oColumn, oData)
      {
         var site = oRecord.getData();
            
         Dom.setStyle(elCell, "width", oColumn.width + "px");
         Dom.setStyle(elCell.parentNode, "width", oColumn.width + "px");
         var desc = "<label>" + site.phase + "</label>";
         // No actions for now...
         elCell.innerHTML = desc;
      },

      /**
       * Adds an event handler that adds or removes the site as favourite site
       *
       * @method onFavouriteSite
       * @param row {object} DataTable row representing site to be actioned
       */
      onFavouriteSite: function ProjectSites_onFavouriteSite(row)
      {
         var record = this.widgets.dataTable.getRecord(row),
            site = record.getData(),
            siteId = site.shortName;

         site.isFavourite = !site.isFavourite;

         this.widgets.dataTable.updateRow(record, site);
         var fnPref = site.isFavourite ? "favouriteSite" : "unFavouriteSite";
         
         // Assume the call will succeed, but register a failure handler to replace the UI state on failure
         var responseConfig =
         {
            failureCallback:
            {
               fn: function ProjectSites_onFavouriteSite_failure(event, obj)
               {
                  // Reset the flag to it's previous state
                  var record = obj.record,
                     site = record.getData();

                  site.isFavourite = !site.isFavourite;
                  this.widgets.dataTable.updateRow(record, site);
                  Alfresco.util.PopupManager.displayPrompt(
                  {
                     text: this.msg("message.save.failure")
                  });
               },
               scope: this,
               obj:
               {
                  record: record
               }
            },
            successCallback:
            {
               fn: function ProjectSites_onFavouriteSite_success(event, obj)
               {
                  var record = obj.record,
                     site = record.getData();

                  YAHOO.Bubbling.fire(site.isFavourite ? "favouriteSiteAdded" : "favouriteSiteRemoved", site);
               },
               scope: this,
               obj:
               {
                  record: record
               }
            }
         };

         this.services.preferences[fnPref].call(this.services.preferences, siteId, responseConfig);

         // Update cached local copy
         this.favSites[siteId] = site.isFavourite;
      },

      /**
       * Adds an event handler that adds or removes the site as favourite site
       *
       * @method onImapFavouriteSite
       * @param row {object} DataTable row representing site to be actioned
       */
      onImapFavouriteSite: function ProjectSites_onImapFavouriteSite(row)
      {
         var record = this.widgets.dataTable.getRecord(row),
            site = record.getData(),
            siteId = site.shortName;

         site.isIMAPFavourite = !site.isIMAPFavourite;

         this.widgets.dataTable.updateRow(record, site);

         // Assume the call will succeed, but register a failure handler to replace the UI state on failure
         var responseConfig =
         {
            failureCallback:
            {
               fn: function ProjectSites_onImapFavouriteSite_failure(event, obj)
               {
                  // Reset the flag to it's previous state
                  var record = obj.record,
                     site = record.getData();

                  site.isIMAPFavourite = !site.isIMAPFavourite;
                  this.widgets.dataTable.updateRow(record, site);
                  Alfresco.util.PopupManager.displayPrompt(
                  {
                     text: this.msg("message.save.failure")
                  });
               },
               scope: this,
               obj:
               {
                  record: record
               }
            }
         };

         // Update user preferences
         this.services.preferences.set(Alfresco.service.Preferences.IMAP_FAVOURITE_SITES + "." + siteId, site.isIMAPFavourite, responseConfig);

         // Update cached local copy
         this.imapfavSites[siteId] = site.isIMAPFavourite;
      },

      /**
       * Searches the current recordSet for a record with the given parameter value
       *
       * @method _findRecordByParameter
       * @param p_value {string} Value to find
       * @param p_parameter {string} Parameter to look for the value in
       */
      _findRecordByParameter: function ProjectSites__findRecordByParameter(p_value, p_parameter)
      {
        var recordSet = this.widgets.dataTable.getRecordSet();
        for (var i = 0, j = recordSet.getLength(); i < j; i++)
        {
           if (recordSet.getRecord(i).getData(p_parameter) === p_value)
           {
              return recordSet.getRecord(i);
           }
        }
        return null;
      }
   });
})();