(function () {
  /**
   * Constructor.
   *
   * @param {String} htmlId The HTML id of the parent element
   * @return {Lyse.component.CivilPunchItemsOverview} The new component instance
   * @constructor
   */
  Lyse.component.CivilPunchItemsOverview= function CivilPunchItemsOverview_constructor(htmlId) {
    Lyse.component.CivilPunchItemsOverview.superclass.constructor.call(this, "Lyse.component.CivilPunchItemsOverview", htmlId, ["container", "datasource", "datatable"]);
  };

  /**
   * Extend Alfresco.component.Base
   */
  YAHOO.extend(Lyse.component.CivilPunchItemsOverview, Alfresco.component.Base,
      {
    /**
     * Fired by YUI when parent element is available for scripting.
     * Component initialisation, including instantiation of YUI widgets and event listener binding.
     *
     * @method onReady
     */
    onReady: function() {
      this.createDataTable();
      var toggleCivilPunchItemsOverview = (function () {
        var container = Dom.get(this.id + "-civil-punch-items-overview");
        var button = Dom.get("toggleCivilPunchItemsOverview-button");

        var display = container.style.display;
        if(display == "none") {
          container.style.display = 'block';
          button.innerHTML = "Hide Civil punch items overview";
        } else {
          container.style.display = 'none';
          button.innerHTML = "Show Civil punch items overview";
        }
      }).bind(this);

      var oToggleCivilPunchItemsOverview = new YAHOO.widget.Button({
        id: "toggleCivilPunchItemsOverview",
        type: "button",
        label: "Hide Civil punch items overview",
        container: "hide-civil-punch-items-overview"
      });

      oToggleCivilPunchItemsOverview.on("click", toggleCivilPunchItemsOverview);
    },

    /**
     * Builds the url for the data table
     *
     * @method buildUrl
     * @retrun {string} The url for the data table
     */
    buildUrl: function CivilPunchItemsOverview_buildUrl() {
      var url = Alfresco.constants.PROXY_URI + "api/lyse/punch-items-overview?nodeRef={nodeRef}&";

      return YAHOO.lang.substitute(url,
          {
        nodeRef: encodeURIComponent(this.options.nodeRef)
          });
    },

    /**
     * Creates the data table
     *
     * @method createDataTable
     */
    createDataTable: function MQVersionHistory_createDataTable() {
      this.widgets.alfrescoDataTable = new Alfresco.util.DataTable(
          {
            dataSource: {
              url: this.buildUrl(),
              config: {
                responseSchema: {
                  resultsList: 'civilPunchItems',
                  fields: ["punchItem", "status", "noderef", "punchCategory", "punchType", "duedate", "workflowid"]
                }
              }
            },
            dataTable: {
              container: this.id + "-civil-punch-items-overview",
              columnDefinitions: [
                                  {key: "punchItem", label: this.msg("column.punchItem"), sortable: true, formatter: this.renderCivilPunchItem.bind(this)},
                                  {key: "status", label: this.msg("column.status"), sortable : true},
                                  {key: "punchCategory", label: this.msg("column.punchCategory"), sortable : true},
                                  {key: "punchType", label: this.msg("column.punchType"), sortable : true},
                                  {key: "duedate", label: this.msg("column.duedate"), sortable : true},
                                  {key: "actions", label: this.msg("column.actions"), formatter: this.renderActions.bind(this)}
                                  ],
                                  config: {
                                    MSG_EMPTY: this.msg("no.result"),
                                    className: ""
                                  }
            }
          });
      // Attach event to go to datalist item
      YAHOO.util.Event.delegate(
          this.id,
          "click",
          this.goToItem.bind(this),
          "a.goto-item"
      );
    },
    renderActions: function (elCell) {
      elCell.innerHTML = "<div class='actions'>" +
      "<a class='goto-item' title='View item' href='#'>" + "<img src="+Alfresco.constants.URL_CONTEXT +"res/components/images/list-16.png></a>" +
      "</div>";
    },

    renderCivilPunchItem: function (elCell, record, column, value) {
      var maxLength = 55;
      if (value) {
        if (value.length >= maxLength) {
            elCell.innerHTML = value.substr(0, maxLength) + "...";
        }
        else {
            elCell.innerHTML = value;
        }
      }
    },

    goToItem : function(event,element) {
      var tr = element.parentNode.parentNode.parentNode;
      var record = this.widgets.alfrescoDataTable.getDataTable().getRecord(tr);
      var data = record.getData();
      var nodeRef = data.noderef;
      var workflowId = data.workflowid;
      var templateUrl;
      var siteShortNameUrl = "";
      if (Alfresco.constants.SITE) {
        siteShortNameUrl = "/site/" + Alfresco.constants.SITE;
      }

      Alfresco.util.Ajax.jsonGet({
        url: Alfresco.constants.PROXY_URI + "lyse/api/permissions/" + Alfresco.util.NodeRef(nodeRef).uri,
        responseContentType : "application/json",
        successCallback: {
            fn: function (response) {
                var permissions = response.json;
                if(permissions.permission !== false) {
                  Alfresco.util.PopupManager.displayMessage({
                    text: this.msg('message.opened_in_new_window')
                  });

                  if(workflowId != null && workflowId != "") {
                    templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?workflowId='+workflowId;
                    return window.open(templateUrl, "_blank");
                  } else {
                    templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?nodeRef=' + nodeRef;
                    return window.open(templateUrl, "_blank");
                  }
                } else {
                  Alfresco.util.PopupManager.displayMessage({
                    text: "You don't have permission to view this"
                  });
                }
            },
            scope: this
        }
    });
    }
  });
})();


