(function()
{
   /**
    * YUI Library aliases
    */
   var Dom = YAHOO.util.Dom,
       Event = YAHOO.util.Event,
       Selector = YAHOO.util.Selector,
       KeyListener = YAHOO.util.KeyListener,
       Bubbling = YAHOO.Bubbling;

   /**
    * Alfresco Slingshot aliases
    */
   var $html = Alfresco.util.encodeHTML,
      $links = Alfresco.util.activateLinks,
      $combine = Alfresco.util.combinePaths,
      $userProfile = Alfresco.util.userProfileLink;


   Alfresco.component.ExtDataGrid = function(htmlId)
   {
      Alfresco.component.ExtDataGrid.superclass.constructor.call(this, htmlId);
      YAHOO.Bubbling.on("afterFormRuntimeInit", this.onAfterFormRuntimeInit, this);
      //YAHOO.Bubbling.on("beforeFormRuntimeInit", this.onBeforeFormRuntimeInit, this);
      this.totalRecordsUpper = null;
      return this;
   };

   /**
    * Extend Alfresco.component.DataGrid
    */
   YAHOO.extend(Alfresco.component.ExtDataGrid, Alfresco.component.DataGrid);

   /**
    * Augment prototype with Common Actions module
    */
   YAHOO.lang.augmentProto(Alfresco.component.ExtDataGrid, Alfresco.service.DataListActions);

   /**
    * Augment prototype with main class implementation, ensuring overwrite is enabled
    */
   YAHOO.lang.augmentObject(Alfresco.component.ExtDataGrid.prototype,
   {
       /**
        * Overrides the one in datagrid.js since it did not seem to work with paging enabled!
        *
        * Searches the current recordSet for a record with the given parameter value
        *
        * @method _findRecordByParameter
        * @private
        * @param p_value {string} Value to find
        * @param p_parameter {string} Parameter to look for the value in
        */
       _findRecordByParameter: function DataGrid__findRecordByParameter(p_value, p_parameter)
       {
           // Handle pagination
           var offset = 0;
           if (this.options.usePagination) {
               offset = this.widgets.paginator.getStartIndex();
           }
           var recordSet = this.widgets.dataTable.getRecordSet();
           for (var i = 0, j = recordSet.getLength(); i < j; i++)
           {
               if (recordSet.getRecord(i + offset).getData(p_parameter) == p_value)
               {
                   return recordSet.getRecord(i + offset);
               }
           }
           return null;
       },

         /**
       * Returns workflow tasks custom datacell formatter
       *
       * @method fnRenderCellWorkflowTasks
       */
      fnRenderCellWorkflowTasks: function DataGrid_fnRenderCellWorkflowTasks()
      {
         var scope = this;

         /**
          * Workflow tasks custom datacell formatter
          *
          * @method renderCellWorkflowTasks
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return function DataGrid_renderCellWorkflowTasks(elCell, oRecord, oColumn, oData)
         {
            var tasksHtml = '';
            if(oRecord.getData('workflowTasks') === undefined)
            {
              tasksHtml = loadingGif();
              elCell.innerHTML = '<div class="loading-wrapper" id="' + scope.id + '-actions-' + oRecord.getId() + '">' + tasksHtml + '</div>';
            }
            else if (oRecord.getData("workflowTasks") != null) {
               for (i = 0; i < oRecord.getData("workflowTasks").length; i++) {
                  var taskData = oRecord.getData("workflowTasks")[i];

                  var img = Alfresco.constants.URL_RESCONTEXT + 'components/images/' + (taskData.isPooled === 'true' ? '/people.png' : 'users.png');
                  if(taskData.name == "lysewf:shareForInformation" || taskData.name == "lysewf:shareForCommenting" || taskData.name == "lysewf:shareInformOriginator"){
                  	img = Alfresco.constants.URL_RESCONTEXT + 'components/images/header/' + (taskData.isPooled === 'true' ? 'people.png' : 'users.png');
                  }
                  var urlContext = Alfresco.constants.URL_PAGECONTEXT + 'site/' + Alfresco.constants.SITE;
                  tasksHtml += YAHOO.lang.substitute('<a href="{taskUrl}" target="_blank"><img src="{imgUrl}" title="{imgTitle}"/></a>', {
                     taskUrl: urlContext + '/lyse-task-edit?taskId=' + $html(taskData.id) + '&workflowId='+ $html(taskData.workflowId),
                     imgUrl: img,
                     imgTitle: $html(taskData.title)
                  });
               }
               elCell.innerHTML = '<div id="' + scope.id + '-actions-' + oRecord.getId() + '">' + tasksHtml + '</div>';
            }
         };
      },

      /**
       * Returns responsible (workflow tasks) custom datacell formatter
       *
       * @method fnRenderCellResponsiblesForTasks
       */
      fnRenderCellResponsiblesForTasks: function DataGrid_fnRenderCellResponsiblesForTasks()
      {
         var scope = this;

         /**
          * Responsible for tasks custom datacell formatter
          *
          * @method renderCellResponsiblesForTasks
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return (function DataGrid_renderCellResponsiblesForTasks(elCell, oRecord, oColumn, oData) {
            var responsiblesHtml = getTaskResponsibleFromRecord(oRecord);
            elCell.innerHTML = '<div id="' + scope.id + '-actions-' + oRecord.getId() + '">' + responsiblesHtml + '</div>';
            if (!window.ec){
                window.ec=elCell;
                window.or=oRecord;
            }
         }).bind(this);
      },

      /**
       * Returns share responsible (workflow tasks) custom datacell formatter
       *
       * @method fnRenderCellShareResponsiblesForTasks
       */
      fnRenderCellShareResponsiblesForTasks: function DataGrid_fnRenderShareCellResponsiblesForTasks()
      {
         var scope = this;

         /**
          * Share Responsible for tasks custom datacell formatter
          *
          * @method renderCellResponsiblesForTasks
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return (function DataGrid_renderShareCellResponsiblesForTasks(elCell, oRecord, oColumn, oData) {
            var responsiblesHtml = getShareTaskResponsibleFromRecord(oRecord);
            elCell.innerHTML = '<div id="' + scope.id + '-actions-' + oRecord.getId() + '">' + responsiblesHtml + '</div>';
            if (!window.ec){
                window.ec=elCell;
                window.or=oRecord;
            }
         }).bind(this);
      },

      /**
       * Returns actions custom datacell formatter
       *
       * @method fnRenderCellActions
       */
      fnRenderCellActions: function DataGrid_fnRenderCellActions()
      {
         var scope = this;

         /**
          * Actions custom datacell formatter
          *
          * @method renderCellActions
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return function DataGrid_renderCellActions(elCell, oRecord, oColumn, oData)
         {
            Dom.setStyle(elCell, "width", "110px");
            Dom.setStyle(elCell.parentNode, "width", "110px");

            elCell.innerHTML = '<div id="' + scope.id + '-actions-' + oRecord.getId() + '" class="hidden"></div>';
         };
      },

      /**
       * Return data type-specific formatter
       *
       * @method getCellFormatter
       * @return {function} Function to render read-only value
       */
      getCellFormatter: function DataGrid_getCellFormatter()
      {
         var scope = this;

         /**
          * Data Type custom formatter
          *
          * @method renderCellDataType
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return function DataGrid_renderCellDataType(elCell, oRecord, oColumn, oData)
         {
            var html = "";

            // Populate potentially missing parameters
            if (!oRecord)
            {
               oRecord = this.getRecord(elCell);
            }
            if (!oColumn)
            {
               oColumn = this.getColumn(elCell.parentNode.cellIndex);
            }

            if (oRecord && oColumn)
            {
               if (!oData)
               {
                  oData = oRecord.getData("itemData")[oColumn.field];
               }

               if (oData)
               {
                  var datalistColumn = scope.datalistColumns[oColumn.key];
                  if (datalistColumn)
                  {
                     oData = YAHOO.lang.isArray(oData) ? oData : [oData];
                     for (var i = 0, ii = oData.length, data; i < ii; i++)
                     {
                        data = oData[i];
                        switch (datalistColumn.dataType.toLowerCase())
                        {
                           case "cm:person":
                        	   if (ii > 2 && i === 2){
                         		  html +="<span>...</span>"
                         	  }else if (i < 2){
                         		 html += '<span class="person">' + $userProfile(data.metadata, data.displayValue) + '</span>';
                         	  }
                              break;

                           case "datetime":
                              html += Alfresco.util.formatDate(Alfresco.util.fromISO8601(data.value), scope.msg("date-format.shortDate"));
                              break;

                           case "date":
                              html += Alfresco.util.formatDate(Alfresco.util.fromISO8601(data.value), scope.msg("date-format.shortDate"));
                              break;

                           case "cm:content":
                        	   if (data.displayValue.indexOf("fileholder.txt")!== -1){
                        		   html += "";
                        	   }else{
                        		   html += $html(data.displayValue);
                        	   }
                        	   break;
                           case "cm:cmobject":
                           case "cm:folder":
                        	  // Need to show all replies
                        	  if (datalistColumn.name === "lysedl:replies"){
                        		  if (data.linkValue){
                        			  html += '<a href="' + Alfresco.util.siteURL((data.metadata === "container" ? 'folder' : 'document') + '-details?nodeRef=' + data.linkValue) + '">';
                        		  }else{
                        			  html += '<a href="' + Alfresco.util.siteURL((data.metadata === "container" ? 'folder' : 'document') + '-details?nodeRef=' + data.value) + '">';
                        		  }
                                  html += '<img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/filetypes/' + Alfresco.util.getFileIcon(data.displayValue, (data.metadata == "container" ? 'cm:folder' : null), 16) + '" width="16" alt="' + $html(data.displayValue) + '" title="' + $html(data.displayValue) + '" />';
                                  html += ' ' + $html(data.displayValue) + '</a>'
                        	  }else {
	                        	  if (ii > 2 && i === 2){
	                        		  html +="<span>...</span>"
	                        	  }else if (i < 2){
	                        		  if (data.linkValue){
	                        			  html += '<a href="' + Alfresco.util.siteURL((data.metadata === "container" ? 'folder' : 'document') + '-details?nodeRef=' + data.linkValue) + '">';
	                        		  }else{
                                          if (datalistColumn.name === "lyse:rrmRiskAssessmentReport"){
                                              html += '<a target="_blank" href="' + Alfresco.constants.URL_CONTEXT + 'page/lyse-datalist-details?nodeRef=' + data.value + '">';
                                          }else {
                                              html += '<a href="' + Alfresco.util.siteURL((data.metadata === "container" ? 'folder' : 'document') + '-details?nodeRef=' + data.value) + '">';
                                          }
	                        		  }
	                                  html += '<img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/filetypes/' + Alfresco.util.getFileIcon(data.displayValue, (data.metadata === "container" ? 'cm:folder' : null), 16) + '" width="16" alt="' + $html(data.displayValue) + '" title="' + $html(data.displayValue) + '" />';
	                                  html += ' ' + $html(data.displayValue) + '</a>'
	                        	  }
                        	  }
                              break;

                           case "discussion":
                        	   html+= scope.msg("datalist.comments.count", data.displayValue);
                        	   break;
                           case "boolean":
                              html+= scope.msg(data.value ? "label.yes" : "label.no");
                              break;
                           case "long":
                              html+= YAHOO.util.Number.format(data.displayValue, {thousandsSeparator: '&nbsp;'});
                              break;
                           case "double":
                               if(YAHOO.util.History.getQueryStringParameter('list') == 'Milestones and invoicing') {
                                   html += YAHOO.util.Number.format(data.displayValue, {
                                       thousandsSeparator: '&nbsp;',
                                       decimalPlaces: 2
                                   });
                               }else{
                                   html+= YAHOO.util.Number.format(data.displayValue, {thousandsSeparator: '&nbsp;'});
                               }
                               break;

                           default:
                           	  if(data.metadata === "personAsText"){
                                html += '<span class="person">';
                           	  }
                              if (datalistColumn.name === "cm:creator") {
                                  html += '<span class="person">' + $userProfile(data.metadata, data.displayValue) + '</span>';
                              } else if (datalistColumn.name === "lyse:mccCommissioningPackagesMCPackagesAsText") {
                                var displayValues = data.displayValue.split(",");
                                if(displayValues.length > 0) {
                                  for(var j = 0; j< displayValues.length; j++) {
                                    html += '<span>' + displayValues[j] + '</span><br\>';
                                  }
                                }
                              } else if ($html(data.displayValue.length) > 40){
                                // If richtext editor was used, this property contains html, strip that out
                                html += $html(strip(data.displayValue.substring(0,40) + '...'));
                                var domid = Alfresco.util.generateDomId(elCell);
                                new YAHOO.widget.Tooltip(domid +"tooltip",
                                    { context:elCell,
                                      text:  $html(strip(data.displayValue))
                                    }

                                  );
                              } else {
                                // If richtext editor was used, this property contains html, strip that out
                                html += $html(strip(data.displayValue));
                              }

                           	  if(data.metadata === "personAsText"){
                                html += '<span>';
                           	  }
                              break;
                        }

                        if (i < ii - 1)
                        {
                           html += "<br />";
                        }
                     }

                     // Extra fields in datagrid that should be clickable to open dataitem
                     var makeClickable = [
                         "dl:itemId",
                         "lyse:melNameEng",
                         "lyse:qaHeading",
                         "lyse:punchDescription",
                         "lyse:iasHeading",
                         "lyse:hseHeading",
                         "lyse:hseNecForm",
                         "lyse:hseDescription",
                         "lyse:epbDescription",
                         "lysedl:userFeedbackTitle",
                         "lysedl:vorHeading",
                         "lysedl:issueTitle",
                         "lysedl:interfaceActionTitle",
                         "lysedl:ncrHeading",
                         "lysedl:correspondenceTitle",
                         "cm:title",
                         "lysedl:milestoneHeading",
                         "lyse:title4",
                         "lysedl:civilCorrespondenceTitle",
                         "lysedl:civilVorHeading"
                     ];
                     if (makeClickable.indexOf(datalistColumn.name) !== -1) {
	                   	 html = "<span style='cursor: pointer; text-decoration: underline;'>" + html+ "</span>";
	                   	 elCell.onclick = function() {
	                   		 scope.onActionView(oRecord.getData());
	                   	 };
                     }

                   }
               }
            }

            elCell.innerHTML =   html ;
         };
      },




      /**
       * Fired by YUI when parent element is available for scripting
       *
       * @method onReady
       */
      onReady: function ExtDataGrid_onReady()
      {

        // Following function is for use by windows opened by this window to request a
        // refresh of the datagrid: window.opener.refreshDataGrid()
        // There is possibly a cleaner way than using a global function for this.
        window.refreshDataGrid = (function () {
          this._updateDataGrid.call(this,
            {
              page: this.currentPage
            });
        }).bind(this);

    	 Alfresco.component.ExtDataGrid.superclass.onReady.call(this);
    	 //this.options.usePagination = false;

    	 // when calling with a nodeRef in #view, this will open the form.
         if (window.location.hash && window.location.hash.indexOf("#view=") === 0) {
             var hash = window.location.hash;
             var nodeRef = hash.replace("#view=", "");

             this.onActionView({
                nodeRef: nodeRef
             });
          } else if (window.location.hash && window.location.hash.indexOf("#edit=") === 0) {
             var hash = window.location.hash;
             var nodeRef = hash.replace("#edit=", "");

             this.onActionEdit({
                nodeRef: nodeRef
             });
          }

         // FilterForm Submit button
         this.widgets.filterFormSubmit = Alfresco.util.createYUIButton(this, "filterform-submit", this.onFilterFormSubmit);
         this.widgets.filterFormClear = Alfresco.util.createYUIButton(this, "filterform-clear", this.onFilterFormClear);

         Alfresco.util.createTwister(this.id+"-wf-image", "wf-image", false);
         Alfresco.util.createTwister(this.id+"-filterHeader", "datalistformfilter1", false);
         /*var filterTwister1 = Dom.get(this.id+"-filterHeader");
         var twisterPanel1 = Dom.getNextSibling(filterTwister1);
         */

         Alfresco.util.createTwister(this.id+"-title", "datalistformfilter2", false);
       /*  var filterTwister2 = Dom.get(this.id+"-title");
         var twisterPanel2 = Dom.getNextSibling(filterTwister2);
         */

      },
       onShowWorkflowHistoryList:function(item)
       {
           Lyse.DatalistActions.onShowWorkflowHistoryList.call(this,item.nodeRef);
       },
       onShowShareHistoryList:function(item)
       {
           Lyse.DatalistActions.onShowShareHistoryList.call(this,item.nodeRef);
       },

       onShowEditors:function(item)
       {
           Lyse.DatalistActions.onShowEditors.call(this,item.nodeRef);
       },

      /**
       * Submit a contractors document for review.
       *
       * @method onActionSubmitForReview
       * @param item {object} Object literal representing one data item
       */
      onActionSubmitForReview:function(item)
      {
         Lyse.DatalistActions.onActionSubmitForReview.call(this,item,this.datalistMeta.itemType,function()
         {
             this.refreshItem(item.nodeRef, "actions.submit-for-review-success");
         });
      },

      /**
       * Start the workflow.
       *
       * @method onActionStartWorkflow
       * @param item {object} Object literal representing one data item
       */
      onActionStartWorkflow: function ExtDataGrid_onActionStartWorkflow(item)
      {
          Lyse.DatalistActions.onActionStartWorkflow.call(this,item.nodeRef,function()
          {
              this.refreshItem(item.nodeRef, "actions.start-workflow-success");
          });

      },

      onActionStartConfirmIFCWorkflow: function ExtDataGrid_onActionStartConfirmIFCWorkflow(item)
      {
          Lyse.DatalistActions.onActionStartConfirmIFCWorkflow.call(this,item,function()
          {
              this.refreshItem(item, "actions.start-workflow-success");
          });

      },

     onActionStartMQWorkflow: function ExtDataGrid_onActionStartWorkflow(item)
     {
       Lyse.DatalistActions.onActionStartWorkflow.call(this,item.nodeRef,function()
       {
         this.refreshItem(item.nodeRef, "actions.start-workflow-success");
       });

     },
       onActionStartShareForCommentingWorkflow:function(item)
       {
           Lyse.DatalistActions.onActionStartShareForCommentingWorkflow.call(this,item.nodeRef,function()
           {
               this.refreshItem(item.nodeRef, "actions.start-share-workflow.success");
               this._updateDataGrid.call(this,
                 {
                   page: this.currentPage
                 });
           });
       },
       onActionUpdateOrderAccommodationTask: function DataGrid_onActionUpdateOrderAccommodationTask(item) {

           // Redirect to the assigned task if there is only one
           var taskId = item.workflowTasks[0].id;
           var workflowId = item.workflowTasks[0].workflowId;
           window.location.href = YAHOO.lang.substitute("{context}lyse-task-edit?taskId="+ taskId +"&workflowId="+ workflowId,
               {
                   context: Alfresco.constants.URL_PAGECONTEXT,
                   taskId: taskId
               });

       },
      /**
       * Generate meetings PDF with the selected datalist items
       *
       * @method onActionGenerateMeetingPdf
       * @param item {object} Object literal representing one data item
       */
      onActionGenerateMeetingPdf: function ExtDataGrid_onActionGenerateMeetingPdf(item) {
   	    var nodeRef = Alfresco.util.NodeRef(item.nodeRef);

        Alfresco.util.Ajax.jsonGet(
        {
          url: Alfresco.constants.PROXY_URI + "lyse/api/generate-meeting-pdf/"+nodeRef.uri,
          responseContentType : "application/json",
          successCallback: {
            fn: function () {
	          Alfresco.util.PopupManager.displayMessage(
	          {
	            text: this.msg("actions.generate-meeting-pdf.success")
	          });
	          //Reload page so the newly generated PDF shows up in the datagrid
	          location.reload();
            },
            scope: this
          },

          failureCallback:
          {
            fn: function (response)
            {
              Alfresco.util.PopupManager.displayPrompt(
     	      {
     	        title: this.msg("actions.generate-meeting-pdf.failure"),
     	        text: response.json.message
     	      });
         	},
         	scope: this
          }
        });
      },
       onActionStartShareForInformationWorkflow:function(item)
       {
           Lyse.DatalistActions.onActionStartShareForInformationWorkflow.call(this,item.nodeRef,function()
           {
               this.refreshItem(item.nodeRef, "actions.start-share-workflow.success");
               this._updateDataGrid.call(this,
                 {
                   page: this.currentPage
                 });
           });
       },
      /**
       * Upload new revision of contractors document.
       *
       * @method onActionUploadNewRevision
       * @param item {object} Object literal representing one data item
       */
      onActionUploadNewRevision: function(item)
      {
        Lyse.DatalistActions.onActionUploadNewRevision.call(this,item);
      },

      /**
       * Upload new revision of an IFC document.
       *
       * @method onActionUploadIFC
       * @param item {object} Object literal representing one data item
       */
      onActionUploadIFC: function(item)
      {
        Lyse.DatalistActions.onActionUploadIFC.call(this, item);
      },

      /**
       * Upload new revision of an AS BUILT document.
       *
       * @method onActionUploadAsBuilt
       * @param item {object} Object literal representing one data item
       */
      onActionUploadAsBuilt: function(item)
      {
        Lyse.DatalistActions.onActionUploadAsBuilt.call(this,item,this.datalistMeta.itemType);
      },

       /**
        * Upload as built.
        *
        * @method onActionUploadAsBuilt
        * @param item {object} Object literal representing one data item
        */
       onActionUploadAsBuiltOld: function ExtDataGrid_onActionUploadAsBuiltOld(item)
       {
           // Intercept before dialog show
           var doBeforeDialogShow = function DataGrid_onActionEdit_doBeforeDialogShow(p_form, p_dialog)
           {
               Alfresco.util.populateHTML(
                   [ p_dialog.id + "-dialogTitle", this.msg("label.upload-as-built.title") ]
               );

           };

           var templateUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form/site/" + Alfresco.constants.SITE + "?itemKind={itemKind}&itemId={itemId}&mode={mode}&submitType={submitType}&formId={formId}&showCancelButton=true",
               {
                   itemKind: "node",
                   itemId: item.nodeRef,
                   mode: "edit",
                   submitType: "json",
                   formId: "as-built"
               });

           // Using Forms Service, so always create new instance
           var editDetails = new Alfresco.module.SimpleDialog(this.id + "-editDetails");
           editDetails.setOptions(
               {
                   width: "1000px",
                   templateUrl: templateUrl,
                   actionUrl: null,
                   destroyOnHide: true,
                   doBeforeDialogShow:
                   {
                       fn: doBeforeDialogShow,
                       scope: this
                   },
                   onSuccess:
                   {
                       fn: function DataGrid_onActionEdit_success(response)
                       {
                           if(fn)
                           {
                               fn.call(this,response);
                           }

                       },
                       scope: this
                   },
                   onFailure:
                   {
                       fn: function DataGrid_onActionEdit_failure(response)
                       {
                           if (response.serverResponse.responseText.contains("error.responsiblesMissing")){
                               Alfresco.util.PopupManager.displayPrompt(
                                   {
                                       text: this.msg("message.details.responsiblesMissing")
                                   });
                           }else {
                               Alfresco.util.PopupManager.displayPrompt(
                                   {
                                       text: this.msg("message.details.failure")
                                   });
                           }
                       },
                       scope: this
                   }
               }).show();
       },

       /**
       * Called when files are selected from the input element.
       *
       * @method onBrowseButtonClick
       * @param event {object} a file selection "change" event
       */
      onFileSelection: function ExtDataGrid_onFileSelection(evt)
      {
    	 // FileList object
         var files = evt.target.files;
         if (files != null)
         {
             var statusSpan = Dom.get(this.fileUploadDialog + "-status-span");
             statusSpan.innerHTML = files[0].name;
             Dom.removeClass(statusSpan, "hidden");
         }
         else
         {
            // No files selected, do nothing.
         }
      },

       onActionAdminEdit: function DataGrid_onActionAdminEdit(item) {
           this.onActionEdit(item, "admin-edit");
       },

     onActionAdminDeleteWorkflow:function(item)
     {
       Alfresco.util.PopupManager.displayPrompt(
         {
           title: Alfresco.util.message("datalist.prompt.admin-delete-workflow"),
           text: Alfresco.util.message("datalist.prompt.admin-delete-workflow.confirm"),
           buttons: [
             {
               text: Alfresco.util.message("button.yes"),
               handler: function()
               {
                 this.destroy();
                 Alfresco.util.Ajax.jsonDelete(
                   {
                     url: Alfresco.constants.PROXY_URI + "api/lyse/delete-workflow/" + item.itemData.prop_lysedl_workflowId.value + "?nodeRef=" + item.nodeRef,
                     successCallback:
                     {
                       fn: function (response){
                         Alfresco.util.PopupManager.displayMessage(
                           {
                             title:Alfresco.util.message("datalist.prompt.admin-delete-workflow"),
                             text:Alfresco.util.message("message.success")
                           });
                         location.reload();
                       },
                       scope: this
                     }
                   });
               },
               isDefault: true
             },
             {
               text: Alfresco.util.message("button.no"),
               handler: function()
               {
                 this.destroy();
               }
             }]
         });
     },

	  /**
       * Edit Data Item pop-up
       *
       * @method onActionEdit
       * @param item {object} Object literal representing one data item
       */
      onActionEdit:function(item,formId)
      {
          Lyse.DatalistActions.onActionEdit.call(this,item.nodeRef,formId,function()
          {
              this.refreshItem(item.nodeRef, "message.details.success");
          });
      },
      /**
       * Edit Data Item pop-up
       *
       * @method onActionEdit
       * @param item {object} Object literal representing one data item
       */
      onActionView: function DataGrid_onActionEdit(item)
      {
        var templateUrl;
        //First check if this view form shall have an attachment document library
        Alfresco.util.Ajax.jsonGet(
            {
              url: Alfresco.constants.PROXY_URI + "api/lyse/attachment-folder?nodeRef=" +item.nodeRef,
              successCallback:
              {
                fn: function (response){
                  var siteShortNameUrl = "";
                  if (Alfresco.constants.SITE) {
                    siteShortNameUrl = "/site/" + Alfresco.constants.SITE;
                  }
                  // To make this work, make sure the lysedl:workflowId is added to the data grid form!
                  var workflowId = item.itemData.prop_lysedl_workflowId;

                  if (response.json !== undefined && response.json.nodeRef !== "") {
                    //Success, this datalist item has an attachment folder
                    var attachmentFolder = response.json.nodeRef;

                    if(workflowId != null) {
                      templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?workflowId=' + workflowId.value.toString() + "&docLib=" + attachmentFolder;
                    } else {
                      templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?nodeRef=' + item.nodeRef + "&docLib=" + attachmentFolder;
                    }

                    // Some browsers has a minimal warning about new tab. Alert user.
                    Alfresco.util.PopupManager.displayMessage({
                      text: this.msg('message.opened_in_new_window')
                    });

                    return window.open(templateUrl, "_blank");
                  } else {
                    //Datalist item does not have an attachment folder, proceed as usual.
                    // Some browsers has a minimal warning about new tab. Alert user.
                    Alfresco.util.PopupManager.displayMessage({
                      text: this.msg('message.opened_in_new_window')
                    });

                    if(workflowId != null) {
                      templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?workflowId='+workflowId.value.toString();
                      return window.open(templateUrl, "_blank");
                    } else {
                      templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?nodeRef=' + item.nodeRef;
                      return window.open(templateUrl, "_blank");
                    }
                  }
                },
                scope: this
              }
            });
      },

      /**
       * Link to doclib
       *
       * @method onActionHandleAttachments
       * @param item {object} Object literal representing one data item
       */
      onActionHandleAttachments: function DataGrid_onActionHandleAttachments(item)
      {
            // Get the nodeRef of the attachment folder
            Alfresco.util.Ajax.jsonGet(
            {
               url: Alfresco.constants.PROXY_URI + "api/lyse/attachment-folder?nodeRef=" +item.nodeRef,
               successCallback:
               {
                fn: function (response){
   	               if (response.json !== undefined)
   	               {
   	            	    var attachmentFolder = response.json.nodeRef;
   	            	    var doclibUrl = Alfresco.constants.URL_CONTEXT + 'page/attachment-doclib?docLib=' + attachmentFolder;
   	            	    return window.open(doclibUrl, "_blank");
   	               }
   	            },
                scope: this
               }
            });
      },

      /**
       * Start new iteration of the workflow for the MQ datalist item
       *
       * @method onActionStartNewMQVersion
       * @param item {object} Object literal representing one data item
       */
      onActionStartNewMQVersion: function(item)
      {
            // Signal the receiver task
            Alfresco.util.Ajax.jsonGet(
            {
              url: Alfresco.constants.PROXY_URI + "lyse/api/civil-mq-start-new-version-task?nodeRef=" +item.nodeRef,
              successCallback: {
                fn: function () {
                  Alfresco.util.PopupManager.displayMessage(
                    {
                      text: this.msg("actions.start-new-mq-version.success")
                    });
                  var nodeRefs =  [item.nodeRef];
                  Alfresco.util.Ajax.jsonPost({
                    url : Alfresco.constants.PROXY_URI + "/api/lyse/datatable-metadata",
                    dataObj: nodeRefs,
                    responseContentType : "application/json",
                    successCallback : {
                      fn : function(response) {
                        //Reload
                        location.reload();
                        var taskId = response.json[0].workflowTasks[0].id;
                        var workflowId = response.json[0].workflowTasks[0].workflowId;
                        window.open(Alfresco.constants.URL_PAGECONTEXT +'lyse-task-edit?taskId=' + taskId + '&workflowId=' + workflowId, "_blank");
                      },
                      scope : this,
                    }
                  })
                },
                scope: this
              },

               failureCallback:
               {
                 fn: function (response)
                 {
                   Alfresco.util.PopupManager.displayPrompt(
                 {
                   title: this.msg("actions.start-new-mq-version.failure"),
                   text: response.json.message
                 });
               },
               scope: this
               }
            });
      },

      /**
       * Data Item created event handler
       *
       * @method onDataItemCreated
       * @param layer {object} Event fired
       * @param args {array} Event parameters (depends on event type)
       */
      onDataItemCreated: function DataGrid_onDataItemCreated(layer, args)
      {
         var displayWorkflowPrompt = function (me) {
             Alfresco.util.PopupManager.displayPrompt(
             {
                title: Alfresco.util.message("datalist.prompt.startworkflow"),
                text: '<div class="bd" style="text-align:left;font-weight:bold">' + $html(Alfresco.util.message("datalist.prompt.startworkflow.confirm.firstPart")) + '</div>'
                   + '<div class="bd" style="text-align:left">' + $html(Alfresco.util.message("datalist.prompt.startworkflow.confirm.secondPart"))
                   + '<br/>' + $html(Alfresco.util.message("datalist.prompt.startworkflow.confirm.thirdPart")) + '</div>',
                noEscape: true,
                buttons: [
                {
                   text: Alfresco.util.message("button.yes"),
                   handler: function()
                   {
                      this.destroy();
                      me.onActionStartWorkflow(obj);
                   },
                isDefault: true
                },
                {
                   text: Alfresco.util.message("button.no"),
                   handler: function()
                   {
                      this.destroy();
                   }
                }]
             });
         };

         var displayGeneratePdfPrompt = function (me) {
        	 Alfresco.util.PopupManager.displayPrompt(
             {
                title: Alfresco.util.message("datalist.prompt.generatepdf"),
                text: Alfresco.util.message("datalist.prompt.generatepdf.confirm"),
                buttons: [
                {
                   text: Alfresco.util.message("button.yes"),
                   handler: function()
                   {
                      this.destroy();
                      me.onActionGenerateMeetingPdf(obj);
                   },
                isDefault: true
                },
                {
                   text: Alfresco.util.message("button.no"),
                   handler: function()
                   {
                      this.destroy();
                   }
                }]
             });
         };
         var obj = args[1];
         if (obj && (obj.nodeRef !== null))
         {
            var nodeRef = new Alfresco.util.NodeRef(obj.nodeRef);
            var workflowRelated = false;
            var meetingRelated = false;

            // Lookup if this is a workflow enabled node
            Alfresco.util.Ajax.jsonGet(
            {
               url: Alfresco.constants.PROXY_URI + "api/lyse/" + nodeRef.uri + "/workflow-related",
               successCallback:
               {
                fn: function (response){
   	               if (response.json !== undefined)
   	               {
   	            	    workflowRelated = response.json.workflowRelated;
   	            	    var datalistType = response.json.datalistType;
   	            	    var me = this;

   	            	    // Ask user if she want to start a workflow.
	   	            	if (workflowRelated && datalistType !== "civilMeasuredQuantity"){
	   	            	  displayWorkflowPrompt(me);
	   	   	            }
   	                }
   	           },
               scope: this
             }
           });

           // Lookup if this is a meeting related node + if there's associations meetingItems and meetingGeneratedPDF is empty
           Alfresco.util.Ajax.jsonGet(
		   {
		     url: Alfresco.constants.PROXY_URI + "api/lyse/" + nodeRef.uri + "/meeting-related",
             successCallback:
             {
               fn: function (response){
   	              if (response.json !== undefined)
   	              {
   	                meetingRelated = response.json.meetingRelated;
   	                var me = this;

   	                // Ask user if she want to generate a pdf
   	          	    if (meetingRelated) {
   	                displayGeneratePdfPrompt(me);
	   	            }
   	              }
   	            },
                scope: this
             }
		   });
         }

         if (obj && (obj.nodeRef !== null))
         {
            var nodeRef = new Alfresco.util.NodeRef(obj.nodeRef);
            // Reload the node's metadata
            Alfresco.util.Ajax.jsonPost(
            {
               url: Alfresco.constants.PROXY_URI + "slingshot/datalists/item/node/" + nodeRef.uri,
               dataObj: this._buildDataGridParams(),
               successCallback:
               {
                  fn: function DataGrid_onDataItemCreated_refreshSuccess(response)
                  {
                     var item = response.json.item;
                     var fnAfterUpdate = function DataGrid_onDataItemCreated_refreshSuccess_fnAfterUpdate()
                     {
                        var recordFound = this._findRecordByParameter(item.nodeRef, "nodeRef");
                        if (recordFound !== null)
                        {
                           var el = this.widgets.dataTable.getTrEl(recordFound);
                           Alfresco.util.Anim.pulse(el);
                        }
                        if(nodeRef.nodeRef){
             				this.refreshItem(nodeRef.nodeRef, "message.details.success");
             			}
                     };
                     this.afterDataGridUpdate.push(fnAfterUpdate);

                     // Lyse change, add new row to top of datatable.
                     this.widgets.dataTable.addRow(item, 0);
                  },
                  scope: this
               },
               failureCallback:
               {
                  fn: function DataGrid_onDataItemCreated_refreshFailure(response)
                  {
                     Alfresco.util.PopupManager.displayPrompt(
                     {
                        text: this.msg("message.create.refresh.failure")
                     });
                  },
                  scope: this
               }
            });
         }
      },

      /**
       *
       * Called when the Ajax call to the Start workflow API succeeds.
       *
       * @method onStartWorkflowSuccess
       * @param {Object} response
       */
      onStartWorkflowSuccess: function consoleChannels_onStartWorkflowSuccess(response)
      {
         Alfresco.util.PopupManager.displayMessage(
         {
            text: this.msg("startworkflow.success")
         });

      },
      /**
       * DataTable set-up and event registration
       *
       * @method _setupDataTable
       * @protected
       */
      _setupDataTable: function DataGrid__setupDataTable(columns)
      {
         // YUI DataTable column definitions
         var columnDefinitions =
         [
            { key: "nodeRef", label: "", sortable: false, formatter: this.fnRenderCellSelected(), width: 16, className: "selectCell"  }
         ];

         if(this.datalistMeta.itemType=="lysedl:hseNec" ||
             	this.datalistMeta.itemType=="lysedl:civilQualityControl" ||
              this.datalistMeta.itemType=="lysedl:civilMeasuredQuantity" ||
              this.datalistMeta.itemType=="lysedl:genericActionList" ||
             	this.datalistMeta.itemType=="lysedl:genericDocumentList" ||
             	this.datalistMeta.itemType=="lysedl:meetingMinutes" ||
             	this.datalistMeta.itemType=="lysedl:hseMeetingList" ||
             	this.datalistMeta.itemType=="lysedl:civilReporting")
              {
                columnDefinitions.push(
                  {

                    key:"locked",
                    label:"",
                    sortable:false,
                    formatter:(function(elCell,oRecord)
                    {
                      if(oRecord.getData("isLocked"))
                      {
                        elCell.innerHTML = '<img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/padlockred.gif" width="16" title="' + Alfresco.util.message("lyse.locked") + '" />';
                      }
                      else
                      {
                        elCell.innerHTML = '';
                      }

                    }).bind(this)
                  }
                );
              }

         // Add actions as second column
         columnDefinitions.push(
             { key: "actions", label:  this.msg("label.column.actions") , sortable: false, formatter: this.fnRenderCellActions(), width: 80 }
         );

    	// Kept exclude lists in case needed in future.
         var excludelist=[ 'lysedl:logisticScheduleDatalist','lysedl:melList','lysedl:monthlyReport','lysedl:civilReporting','lysedl:contact','lysedl:meetingMinutes','lysedl:document','lysedl:contract','lysedl:userFeedback','lysedl:variousDocumentList','lysedl:mccVariousDocumentList','lysedl:hseNec','lysedl:grunnEierMatriseDataList','lysedl:civilVariousDocumentList','lysedl:civilQualityControl'];
         excludelist=[];

         if (excludelist.indexOf(this.datalistMeta.itemType)<0) { // If itemType not in excludelist
            columnDefinitions.push(
                 { key: "workflowTasks", label: this.msg("label.column.workflowTasks") , sortable: true, formatter: this.fnRenderCellWorkflowTasks()/*, width: 32 */}
            );

            columnDefinitions.push( {
                key: "responsiblesForTasks",
                label:  this.msg("label.column.responsibles") ,
                sortable: true,
                formatter: this.fnRenderCellResponsiblesForTasks(),
                sortOptions:
                {
                    field: "hva er opp med det?",
                    sortFunction: function(item1,item2,desc,field){
                        if(!YAHOO.lang.isValue(item1)) {
                            return (!YAHOO.lang.isValue(item2)) ? 0 : 1;
                        } else if(!YAHOO.lang.isValue(item2)) {
                            return -1;
                        }

                        function getVal(d){
                            return getTaskResponsibleFromRecord(d);
                        }

                        val1=getVal(item1);
                        val2=getVal(item2);
                        ret=0;
                        if (val1<val2) ret =-1;
                        if (val2<val1) ret =1;
                        if (desc) ret=-ret;
                        return ret;
                    }
                }
                /*, width: 32 */}
            );
         }



         var column;
         for (var i = 0, ii = this.datalistColumns.length; i < ii; i++)
         {
            column = this.datalistColumns[i];
            if (column.name === "lysedl:workflowId") {
               // Don't show! The field must still be present in the "datagrid" form definition or else the field value is not available
               continue;
            }

            // well lets hack the categories label for this type...:(
            var label = column.label;

            // General label override.
            // If there is a message with the expected form, this is used instead.
            // This means the 'hacks' further below can be replaced.
            var label_override_key="column.label.override."+column.formsName;
            var label_override=this.msg(label_override_key);
            if (label_override !== label_override_key) label = label_override;

            if (this.datalistMeta.itemType === "lysedl:irlDataList" && column.name === "cm:categories"){
            	label = this.msg("form.label.irl.area-system");
            }
            if (this.datalistMeta.itemType === "lysedl:iasDataList" && column.name === "cm:categories"){
            	label = this.msg("form.label.irl.area-system");
            }
            if (this.datalistMeta.itemType === "lysedl:epbDataList" && column.name === "cm:categories"){
            	label = this.msg("form.label.irl.area-system");
            }
            if (this.datalistMeta.itemType === "lysedl:tenderList" && column.name === "cm:title"){
            	label = this.msg("form.label.appendix");
            }
            // And the same for these, since we cannot override the label/label-id in the "datagrid" form
            else if (this.datalistMeta.itemType === "lysedl:variationOrderRequestItem" && column.name === "lysedl:vorCostConsequence"){
               label =   this.msg("column.label.vorCostConsequence")  ;
            }
            else if (this.datalistMeta.itemType === "lysedl:variationOrderRequestItem" && column.name === "lysedl:vorConsequenceNOK"){
               label = this.msg("column.label.vorConsequenceNOK") ;
            }
            else if (this.datalistMeta.itemType === "lysedl:variationOrderRequestItem" && column.name === "lysedl:vorConsequenceEUR"){
               label =  this.msg("column.label.vorConsequenceEUR") ;
            }
            else if (this.datalistMeta.itemType === "lysedl:variationOrderRequestItem" && column.name === "lysedl:vorScheduleConsequence"){
               label =  this.msg("column.label.vorScheduleConsequence") ;
            }
            else if (this.datalistMeta.itemType === "lysedl:civilVariationOrderRequestItem" && column.name === "lysedl:civilVorCostConsequence"){
                label =   this.msg("column.label.civilVorCostConsequence")  ;
            }
            else if (this.datalistMeta.itemType === "lysedl:civilVariationOrderRequestItem" && column.name === "lysedl:civilVorConsequenceNOK"){
                label = this.msg("column.label.civilVorConsequenceNOK") ;
            }
            else if (this.datalistMeta.itemType === "lysedl:civilVariationOrderRequestItem" && column.name === "lysedl:civilVorConsequenceEUR"){
                label =  this.msg("column.label.civilVorConsequenceEUR") ;
            }
            else if (this.datalistMeta.itemType === "lysedl:civilVariationOrderRequestItem" && column.name === "lysedl:civilVorScheduleConsequence"){
                label =  this.msg("column.label.civilVorScheduleConsequence") ;
            }

            else if (this.datalistMeta.itemType === "lysedl:correspondenceItem" && column.name === "lysedl:attachments"){
                label =  this.msg("form.label.letter") ;
            }
            else if (this.datalistMeta.itemType === "lysedl:riskReducingMeasures" && column.name === "lyse:hseActionMeetingNotes"){
                label =  this.msg("label.hse.meetingNotes.rrm") ;
            }

            columnDefinitions.push(
            {
               key: this.dataResponseFields[i],
               label: label,
               sortable: true,
               sortOptions:
               {
                  field: column.formsName,
                  sortFunction: this.getSortFunction()
               },
               formatter: this.getCellFormatter(column.dataType)
            });
         }

         var formatGreenRows = (function(elTr, oRecord) {
           YAHOO.Bubbling.on("permissionsLoaded",(function()
           {
             // #39843 Only format green rows for MCC Commissioning packages
             if (decodeURI(elTr.baseURI).indexOf("Commissioning packages") != -1 || oRecord._oData.itemData.prop_lyse_mccCommissioningPackagesArea != null) {
               if(oRecord._oData.permissions.userAccess.isMCCCommissioningPackageReady) {
                 elTr.style.backgroundColor="#98fb98";
               } else {
                 elTr.removeAttribute("style");
               }
             }

           }));
           return true;
         }).bind(this);

         // DataTable definition
         var me = this;
         this.widgets.dataTable = new YAHOO.widget.DataTable(this.id + "-grid", columnDefinitions, this.widgets.dataSource,
         {
            renderLoopSize: this.options.usePagination ? 16 : 32,
            initialLoad: false,
            dynamicData: true,
            "MSG_EMPTY": this.msg("message.empty"),
            "MSG_ERROR": this.msg("message.error"),
            paginator: this.widgets.paginator,
            formatRow: formatGreenRows
         });

         this.widgets.paginator.unsubscribe('changeRequest',this.widgets.dataTable.onPaginatorChangeRequest);
         this.widgets.paginator.subscribe('changeRequest', this.onDataGridPaging,this,true);

         this.widgets.dataTable.unsubscribe("theadCellClickEvent", this.widgets.dataTable.onEventSortColumn);
         this.widgets.dataTable.subscribe("theadCellClickEvent",this.onDataGridSort,this,true);

         // Update totalRecords with value from server
         this.widgets.dataTable.handleDataReturnPayload = function DataGrid_handleDataReturnPayload(oRequest, oResponse, oPayload)
         {
            me.totalRecords = oResponse.meta.totalRecords;
            me.totalRecordsUpper = oResponse.meta.totalRecordsUpper;
            oResponse.meta.pagination =
            {
               rowsPerPage: me.options.pageSize,
               recordOffset: (me.currentPage - 1) * me.options.pageSize
            };
            return oResponse.meta;
         };

         // It could be simplified to get paginator, but i want to make sure im using correct refrence, YUI might do copies somewhere?
         var dataTablePaginator = this.widgets.dataTable.get("paginator");

         // Override abstract function within DataTable to set custom error message
         this.widgets.dataTable.doBeforeLoadData = function DataGrid_doBeforeLoadData(sRequest, oResponse, oPayload)
         {
            if (oResponse.error)
            {
               try
               {
                  var response = YAHOO.lang.JSON.parse(oResponse.responseText);
                  me.widgets.dataTable.set("MSG_ERROR", response.message);
               }
               catch(e)
               {
                  me._setDefaultDataTableErrors(me.widgets.dataTable);
               }
            }

            /* 		Tried to use paginators built in functions to set the value, it didn't work.
             * 		dataTablePaginator.setStartIndex(oResponse.meta.paginationRecordOffset,true);
             * 		Below may be a hacky solution but it works.
             **/
            dataTablePaginator._configs.recordOffset.value = oResponse.meta.paginationRecordOffset;

            // We don't get an renderEvent for an empty recordSet, but we'd like one anyway
            if (oResponse.results.length === 0)
            {
               this.fireEvent("renderEvent",
               {
                  type: "renderEvent"
               });
            }

            // Must return true to have the "Loading..." message replaced by the error message
            return true;
         };

         // File checked handler
         this.widgets.dataTable.subscribe("checkboxClickEvent", function(e)
         {
            var id = e.target.value;
            this.selectedItems[id] = e.target.checked;
            Bubbling.fire("selectedItemsChanged");
         }, this, true);

         // Rendering complete event handler
         this.widgets.dataTable.subscribe("renderEvent", function()
         {
            Alfresco.logger.debug("DataTable renderEvent");

            // IE6 fix for long filename rendering issue
            if (YAHOO.env.ua.ie < 7)
            {
               var ie6fix = this.widgets.dataTable.getTableEl().parentNode;
               ie6fix.className = ie6fix.className;
            }

            // Deferred functions specified?
            for (var i = 0, j = this.afterDataGridUpdate.length; i < j; i++)
            {
               this.afterDataGridUpdate[i].call(this);
            }
            this.afterDataGridUpdate = [];

            // Update the paginator if it's been created
            if (this.widgets.paginator)
            {
               Alfresco.logger.debug("Setting paginator state: page=" + this.currentPage + ", totalRecords=" + this.totalRecords);

               this.widgets.paginator.setState(
               {
                  page: this.currentPage,
                  totalRecords: this.totalRecords
               });

               if (this.totalRecordsUpper)
               {
                  this.widgets.paginator.set("pageReportTemplate", this.msg("pagination.template.page-report.more"));
               }
               else
               {
                  this.widgets.paginator.set("pageReportTemplate", this.msg("pagination.template.page-report"));
               }

               this.widgets.paginator.render();
            }
         }, this, true);

         // Enable row highlighting
         this.widgets.dataTable.subscribe("rowMouseoverEvent", this.onEventHighlightRow, this, true);
         this.widgets.dataTable.subscribe("rowMouseoutEvent", this.onEventUnhighlightRow, this, true);
      },

      /**
       * Retrieves the Data List from the Repository
       *
       * @method populateDataGrid
       */
      populateDataGrid: function DataGrid_populateDataGrid()
      {
    	 this.populateFilterForm();
         if (!YAHOO.lang.isObject(this.datalistMeta))
         {
            return;
         }

         this.renderDataListMeta();

         // Query the visible columns for this list's item type
         Alfresco.util.Ajax.jsonGet(
         {
            url: $combine(Alfresco.constants.URL_SERVICECONTEXT, "components/data-lists/config/columns?itemType=" + encodeURIComponent(this.datalistMeta.itemType)
                + "&destination=" + this.datalistMeta.nodeRef + "&site=" + Alfresco.constants.SITE),
            successCallback:
            {
               fn: this.onDatalistColumns,
               scope: this
            },
            failureCallback:
            {
               fn: this._onDataListFailure,
               obj:
               {
                  title: this.msg("message.error.columns.title"),
                  text: this.msg("message.error.columns.description")
               },
               scope: this
            }
         });
      },

      populateFilterForm: function ExtDataGrid_populateFilterForm()
      {
          var filterFormUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&mode={mode}&formId={formId}&destination={destination}&submitType=json&showCancelButton=false&showSubmitButton=false&site={site}",
    	   {
              itemKind: "type",
              itemId: this.datalistMeta.itemType,
              destination: this.datalistMeta.nodeRef,
              formId:"filter",
              mode: "create",
              submitType: "json",
              site: Alfresco.constants.SITE
           });
    	  var data =
          {
             htmlid: this.id + "-extDgFilterForm"
          };

    	  Alfresco.util.Ajax.request(
		   {
	          url: filterFormUrl,
	          dataObj:data,
	          successCallback:
	          {
	             fn: this.onFilterFormTemplateLoaded,
	             scope: this
	          },
	          failureMessage: "Could not load dialog template from '" + this.options.templateUrl + "'.",
	          scope: this,
	          execScripts: true
	      });
      },

      /**
       * Event callback when filtr template has been loaded
       *
       * @method onFilterFormTemplateLoaded
       * @param response {object} Server response from load template XHR request
       */
      onFilterFormTemplateLoaded: function ExtDataGrid_onFilterFormTemplateLoaded(response)
      {
    	   Dom.get(this.id + "-filterform").innerHTML = response.serverResponse.responseText;
    	   //show filter form
    	   Dom.setStyle(this.id + "-filter", "visibility", "inherit");

      },

      onFilterFormSubmit: function ExtDataGrid_onFilterFormSubmit(){
    	  //alert(this.formsRuntime.getFormData());
    	  YAHOO.Bubbling.fire("changeFilter",
          {
             filterOwner: this.id,
             filterId: "filterform",
             filterData: Alfresco.util.toQueryString(this.formsRuntime.getFormData())
          });
      },

      onFilterFormClear: function ExtDataGrid_onFilterFormClear(){
          //alert(this.formsRuntime.getFormData());
    	  this.currentFilter="";
    	  this.formsRuntime.reset();
    	  // Some specialized controls - e.g. auto-complete  or object-finder don't implement reset
    	  YAHOO.Bubbling.fire("resetForm");
          // Seems nobody is is listening
    	  // YAHOO.Bubbling.fire("resetFinder"); // Seems nobody is is listening
          this.onFilterFormSubmit();
         // So this is the best way to get a "most of the time right" reset
         location.hash = '';
      },

      onChangeFilter: function DataGrid_onChangeFilter(layer, args)
      {
         var obj = args[1];
         if (obj !== null)
         {
        	this._addSortingToNewFilter(obj, this.sortAsc, this.sortColumn);
            var filter = Alfresco.util.cleanBubblingObject(obj);
            if(obj.filterId === "filterform"){
              this.currentPage = 1;
              this.widgets.paginator.setPage(1,true);
            }
            this._updateDataGrid.call(this,
            {
               filter: filter,
               page: this.currentPage
            });
       	 	//Alfresco.component.ExtDataGrid.superclass.onChangeFilter.call(this, layer, args);
         }
      },

      onFilterFormSave: function ExtDataGrid_onFilterFormSave(){
    	  Alfresco.util.PopupManager.getUserInput({
    		  okButtonText : "Save"
    	  });

      },

      /**
       * DataGrid View Filter changed event handler
       *
       * @method onFilterChanged
       * @param layer {object} Event fired (unused)
       * @param args {array} Event parameters (new filterId)
       */
      onFilterChanged: function DataGrid_onFilterChanged(layer, args)
      {
         var obj = args[1];
         if ((obj !== null) && (obj.filterId !== null))
         {
            obj.filterOwner = obj.filterOwner || Alfresco.util.FilterManager.getOwner(obj.filterId);

            // Should be a filterId in the arguments
            this.currentFilter = Alfresco.util.cleanBubblingObject(obj);
            Alfresco.logger.debug("DL_onFilterChanged: ", this.currentFilter);

            if (this.currentFilter.filterId === "filterform" && this.formsRuntime){
            	// refill filter form
            	var filterData =  Alfresco.util.getQueryStringParameters(this.currentFilter.filterData);

            	// get the form element
                var form = Dom.get(this.formsRuntime.formId);
                if (form !== null)
                {
                   var length = form.elements.length;
                   for (var i = 0; i < length; i++)
                   {
                      var element = form.elements[i];
                      var name = element.name;
                      if (name === "-" || element.disabled || element.type === "button")
                      {
                         continue;
                      }
                      if (name === undefined || name === "")
                      {
                         name = element.id;
                      }
                      var value = YAHOO.lang.trim(element.value);
                      if (name && filterData[name])
                      {
                    	//try to find form control component
                    	var control = Alfresco.util.ComponentManager.get(element.attributes.id.value+"-cntrl");
                    	if (control && control.name === "Alfresco.FilterDateRange"){
                    		var values = filterData[name].split("TO");
                    		if (values[0] !== ""){
                    			control.currentFromDate = values[0];
                    			var dateEntry = Alfresco.util.fromISO8601(control.currentFromDate).toString(control.msg("form.control.date-picker.entry.date.format"));
                            	Dom.get(control.id + "-date-from").value = dateEntry;
                    		}
                            if (values[1] !== ""){
                            	 control.currentToDate = values[1];
                             	var dateEntry = Alfresco.util.fromISO8601(control.currentToDate).toString(control.msg("form.control.date-picker.entry.date.format"));
                             	Dom.get(control.id + "-date-to").value = dateEntry;
                            }
                    	}else if (element.name === "prop_cm_categories"||filterData[name].indexOf("'")!==-1){
                    		element.value=filterData[name];

                    	}else if (element.type === "hidden" && filterData[name].split(",").length > 0 && !element.name.endsWith("_added"))
                        {
                    		element.value=filterData[name];
                    		var selectElement = Dom.get(element.id+"-entry");
                    		if (selectElement && selectElement.type !== "checkbox"){
                    		   values = filterData[name].split(",");

                               for (var j = 0, jj = selectElement.options.length; j < jj; j++)
                               {
                                  if (Alfresco.util.arrayContains(values,selectElement.options[j].value))
                                  {
                                	  selectElement.options[j].selected= true;
                                  }else{
                                	  selectElement.options[j].selected= false;
                                  }
                               }
                    		}else {
                    			element.value=filterData[name];
                    		}

                        }
                    	else{
                    		element.value=filterData[name];
                    	}

                      }
                   }
                }
            }else if (this.currentFilter.filterId !== "filterform"){
            	//clear form
            	this.populateFilterForm();
            }
         }
      },

     /**
      * Event handler called when the "onBeforeFormRuntimeInit" event is received.
      *
      * @method onBeforeFormRuntimeInit
      * @param layer {String} Event type
      * @param args {Object} Event arguments
      * <pre>
      *    args.[1].component: Alfresco.FormUI component instance,
      *    args.[1].runtime: Alfresco.forms.Form instance
      * </pre>
      */
     /* onBeforeFormRuntimeInit: function ExtDataGrid_onBeforeFormRuntimeInit(layer, args)
     {
        this.formUI = args[1].component;
        this.formsRuntime = args[1].runtime;
     },*/


     /**
      * Event handler called when the "onAfterFormRuntimeInit" event is received.
      *
      * @method onAfterFormRuntimeInit
      * @param layer {String} Event type
      * @param args {Object} Event arguments
      * <pre>
      *    args.[1].component: Alfresco.FormUI component instance,
      *    args.[1].runtime: Alfresco.forms.Form instance
      * </pre>
      */
      onAfterFormRuntimeInit: function ExtDataGrid_onAfterFormRuntimeInit(layer, args)
      {
         var rt = args[1].runtime;
         if (rt.formId === this.id + "-extDgFilterForm-form") {
            this.formsRuntime = rt;
            var me = this,
                form = Dom.get(this.formsRuntime.formId);

            YAHOO.util.Event.removeListener(form, "keypress");
            YAHOO.util.Event.removeListener(form, KeyListener.KEYDOWN);
            /**
             * Prevent the Enter key from causing a double form submission
             */
            var fnStopEvent = function(id, keyEvent)
            {
               var event = keyEvent[1],
                   target = event.target ? event.target : event.srcElement;

               var targetId = target.id;
               if (!(targetId.contains(me.id) && targetId.contains("-extDgFilterForm")))
               {
                  //this is not from filterForm
                  return false;
               }

               if (target.tagName === "TEXTAREA")
               {
                  // Allow linefeeds in textareas
                  return false;
               }
               else if (target.tagName === "BUTTON" || Dom.hasClass(target, "yuimenuitemlabel"))
               {
                  // Eventlisteners for buttons and menus must be notified that the enter key was entered
               }
               else
               {
                  var targetName = target.name;
                  if (targetName && (targetName !== "-"))
                  {
                     me.onFilterFormSubmit(event);
                  }
                  Event.stopEvent(event);
                  return false;
               }
            };
            var enterListener = new KeyListener(form,
                {
                   keys: KeyListener.KEY.ENTER
                }, fnStopEvent, YAHOO.env.ua.ie > 0 ? KeyListener.KEYDOWN : "keypress");
            enterListener.enable();
            if (this.currentFilter.filterId === "filterform"){

               YAHOO.Bubbling.fire("filterChanged",
                   {
                      filterOwner: this.id,
                      filterId: this.currentFilter.filterId,
                      filterData: Alfresco.util.toQueryString(this.currentFilter.filterData)
                   });
            }
         }
      },
     /**
      * Build URI parameter string for doclist JSON data webscript
      *
      * @method _buildDataGridParams
      * @param p_obj.filter {string} [Optional] Current filter
      * @return {Object} Request parameters. Can be given directly to Alfresco.util.Ajax, but must be JSON.stringified elsewhere.
      */
     _buildDataGridParams: function DataGrid__buildDataGridParams(p_obj)
     {
        var request =
        {
           fields: this.dataRequestFields,
           page : this.currentPage,
           size : this.options.pageSize,
           total : this.totalRecords,
           noCache : new Date().getTime()
        };

        if (p_obj && p_obj.filter)
        {
        	if (p_obj.filter.sortField !=null && this.datalistColumns[p_obj.filter.sortField].dataType == "double"){
        		request.filter =
		        {
		        	// #36352 Alfresco canned queries does not support sorting on doubles, "short circuit" to use solr search.
        			filterId: "short circuit to solrSearch",
		            filterData:  Alfresco.util.getQueryStringParameters(p_obj.filter.filterData),
		            sortField: p_obj.filter.sortField,
		            sortAsc: p_obj.filter.sortAsc,
		            page : this.currentPage,
		            size : this.options.pageSize
		        };
		    } else if (!p_obj.filter.sortField){
       		    request.filter =
		        {
        	 	    filterId: p_obj.filter.filterId,
		            filterData:  Alfresco.util.getQueryStringParameters(p_obj.filter.filterData),
		            page : this.currentPage,
		            size : this.options.pageSize
		        };
        	} else {
        		request.filter =
		        {
        			filterId: p_obj.filter.filterId,
		            filterData:  Alfresco.util.getQueryStringParameters(p_obj.filter.filterData),
		            sortField: p_obj.filter.sortField,
		            sortAsc: p_obj.filter.sortAsc,
		            page : this.currentPage,
		            size : this.options.pageSize
		        };
        	}
        }
        this.widgets.dataSource.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);
        return request;

     },

       refreshItem: function ExtendedDataGrid_refreshItem(nodeRef, successMessageKey) {
            Alfresco.util.Ajax.jsonPost(
               {
                   url: Alfresco.constants.PROXY_URI + "slingshot/datalists/item/node/" + new Alfresco.util.NodeRef(nodeRef).uri,
                   dataObj: this._buildDataGridParams(),
                   successCallback:
                   {
                       fn: function refreshItem_refreshSuccess(response)
                       {
                           // See #44485
                           setTimeout(function() {
                             // Fire "itemUpdated" event
                             Bubbling.fire("dataItemUpdated",
                                 {
                                     item: response.json.item
                                 });

                             // https://redmine.redpill-linpro.com/issues/41027
                             this.updateMetadataNode(nodeRef);
                             this.updatePermission(nodeRef);
                             // Display success message
                             Alfresco.util.PopupManager.displayMessage(
                                 {
                                     text: this.msg(successMessageKey)
                                 });
                           }.bind(this), 200);
                       },
                       scope: this
                   },
                   failureCallback:
                   {
                       fn: function refreshItem_refreshFailure(response)
                       {
                           Alfresco.util.PopupManager.displayPrompt(
                               {
                                   text: this.msg("message.details.failure")
                               });
                       },
                       scope: this
                   }
               });
       },

     updateMetadataNode:function(nodeRef)
     {
       var record = this._findRecordByParameter(nodeRef, "nodeRef");
       return this.updateMetadata([record]);
     },
     /**
      * Update workflowTask and responsible information for one nodeRef
      *
      * @method updateMetadata
      */
     updateMetadata: function DataGrid_updateMetadata(records)
     {
       var nodeRefs = records.map(function(record)
       {
         return record._oData.nodeRef;
       });
       Alfresco.util.Ajax.jsonPost({
         url : Alfresco.constants.PROXY_URI + "/api/lyse/datatable-metadata",
         dataObj: nodeRefs,
         responseContentType : "application/json",
         successCallback : {
           fn : function(response)
           {
             var metadataList = response.json;
             // update rows with this data
             records.forEach((function(record,index)
             {
               //Yea this is confusing. It has to be like this because the updateCell call overwrites the values with the passed in value but only
               //if it finds the field. Responsibles is not a field so it can't be set like workflowTasks..
               var workflowTasks = metadataList[index].workflowTasks;
               record._oData.responsibles = metadataList[index].responsibles;
               this.widgets.dataTable.updateCell(record,this.widgets.dataTable.getColumn('workflowTasks'),workflowTasks);
               this.widgets.dataTable.updateCell(record,this.widgets.dataTable.getColumn('responsiblesForTasks'));
             }).bind(this));
           },
           scope : this,
         }
       });

     },


     updatePermission:function(nodeRef)
     {
       var record = this._findRecordByParameter(nodeRef, "nodeRef");
       return this.updatePermissions([record]);
     },

     /**
      * Update permission information for one nodeRef
      *
      * @method updateMetadata
      */
     updatePermissions: function DataGrid_updatePermissions(records)
     {
       var nodeRefs = records.map(function(record)
       {
         return record._oData.nodeRef;
       });
       Alfresco.util.Ajax.jsonPost({
         url : Alfresco.constants.PROXY_URI + "/lyse/api/permissions",
         dataObj:{refs:nodeRefs},
         responseContentType : "application/json",
         successCallback : {
           fn : function(response)
           {
             var metadataList = response.json;
             // update rows with this data
             records.forEach((function(record,index)
             {
               record._oData.permissions.userAccess = metadataList[index];

             }).bind(this));

             if(records.length === 1)
             {
               // See #44485
               setTimeout(function() {
                 this.widgets.dataTable.updateRow(records[0],records[0]._oData);
               }.bind(this), 500);
             }
             else
             {
               this.widgets.dataTable.updateRows(this.currentPage * this.options.pageSize,records.map(function(record)
               {
                 return record._oData;
               }));
             }

             YAHOO.Bubbling.fire("permissionsLoaded");
           },
           scope : this,
         }
       });

     },
     /**
      * DataSource set-up and event registration
      *
      * @method _setupDataSource
      * @protected
      */
     _setupDataSource: function DataGrid__setupDataSource()
     {
        var listNodeRef = new Alfresco.util.NodeRef(this.datalistMeta.nodeRef);

        for (var i = 0, ii = this.datalistColumns.length; i < ii; i++)
        {
           var column = this.datalistColumns[i],
              columnName = column.name.replace(":", "_"),
              fieldLookup = (column.type === "property" ? "prop" : "assoc") + "_" + columnName;

           this.dataRequestFields.push(columnName);
           this.dataResponseFields.push(fieldLookup);
           this.datalistColumns[fieldLookup] = column;
        }

        // DataSource definition
        this.widgets.dataSource = new YAHOO.util.DataSource(Alfresco.constants.PROXY_URI + "slingshot/datalists/data/node/" + listNodeRef.uri + "/" + "datagrid",
        {
           connMethodPost: true,
           responseType: YAHOO.util.DataSource.TYPE_JSON,
           maxCacheEntries : 0,
           connXhrMode:"cancelStaleRequests",
           responseSchema:
           {
              resultsList: "items",
              metaFields:
              {
                 paginationRecordOffset: "startIndex",
                 totalRecords: "totalRecords",
                 totalRecordsUpper: "totalRecordsUpper"
              }
           }
        });
        this.widgets.dataSource.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);

        // Intercept data returned from data webscript to extract custom metadata
        this.widgets.dataSource.doBeforeCallback = function DataGrid_doBeforeCallback(oRequest, oFullResponse, oParsedResponse)
        {
        	this.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);
            this.connMgr.setDefaultXhrHeader(Alfresco.util.Ajax.JSON);

           // Container userAccess event
           var permissions = oFullResponse.metadata.parent.permissions;
           if (permissions && permissions.userAccess)
           {
              Bubbling.fire("userAccess",
              {
                 userAccess: permissions.userAccess
              });
           }

           return oParsedResponse;
        };
     },

       /**
        * Custom event handler to highlight row.
        *
        * @method onEventHighlightRow
        * @param oArgs.event {HTMLEvent} Event object.
        * @param oArgs.target {HTMLElement} Target element.
        */
       onEventHighlightRow: function DataGrid_onEventHighlightRow(oArgs)
       {
           // elActions is the element id of the active table cell where we'll inject the actions
           var elActions = Dom.get(this.id + "-actions-" + oArgs.target.id);
           // Inject the correct action elements into the actionsId element
           if (elActions)
           {
               elActions.innerHTML = "";
               // Call through to get the row highlighted by YUI
               this.widgets.dataTable.onEventHighlightRow.call(this.widgets.dataTable, oArgs);

               // Clone the actionSet template node from the DOM
               var record = this.widgets.dataTable.getRecord(oArgs.target.id),
                   clone = Dom.get(this.id + "-actionSet").cloneNode(true);
               if(record==null)
               {
                   console.debug("Could not get record for " + oArgs.target.id);
                   return;
               }

               // Token replacement
               clone.innerHTML = YAHOO.lang.substitute(window.unescape(clone.innerHTML), this.getActionUrls(record));

               // Generate an id
               clone.id = elActions.id + "_a";

               // Simple view by default
               Dom.addClass(clone, "simple");

               // Trim the items in the clone depending on the user's access
               var userAccess = record.getData("permissions").userAccess,
                   actionLabels = record.getData("actionLabels") || {};

               // Inject the current filterId to allow filter-scoped actions
               userAccess["filter-" + this.currentFilter.filterId] = true;

               // Remove any actions the user doesn't have permission for
               var actions = YAHOO.util.Selector.query("div", clone),
                   action, aTag, spanTag, actionPermissions, aP, i, ii, j, jj;

               for (i = 0, ii = actions.length; i < ii; i++)
               {
                   action = actions[i];
                   aTag = action.firstChild;
                   spanTag = aTag.firstChild;
                   if (spanTag && actionLabels[action.className])
                   {
                       spanTag.innerHTML = $html(actionLabels[action.className]);
                   }

                   if (aTag.rel !== "")
                   {
                       actionPermissions = aTag.rel.split(",");
                       for (j = 0, jj = actionPermissions.length; j < jj; j++)
                       {
                           aP = actionPermissions[j];
                           // Support "negative" permissions
                           if ((aP.charAt(0) == "~") ? !!userAccess[aP.substring(1)] : !userAccess[aP])
                           {
                               clone.removeChild(action);
                               break;
                           }
                       }
                   }
               }

               // Need the "More >" container?
               var splitAt = this.options.splitActionsAt;
               actions = YAHOO.util.Selector.query("div", clone);
               if (actions.length > splitAt)
               {
                   var moreContainer = Dom.get(this.id + "-moreActions").cloneNode(true);
                   var containerDivs = YAHOO.util.Selector.query("div", moreContainer);
                   // Insert the two necessary DIVs before the splitAt action item
                   Dom.insertBefore(containerDivs[0], actions[splitAt]);
                   Dom.insertBefore(containerDivs[1], actions[splitAt]);
                   // Now make action items after the split, children of the 2nd DIV
                   var index, moreActions = actions.slice(splitAt);
                   for (index in moreActions)
                   {
                       if (moreActions.hasOwnProperty(index))
                       {
                           containerDivs[1].appendChild(moreActions[index]);
                       }
                   }
               }

               elActions.appendChild(clone);
           }

           if (this.showingMoreActions)
           {
               this.deferredActionsMenu = elActions;
           }
           else if (!Dom.hasClass(document.body, "masked"))
           {
               this.currentActionsMenu = elActions;
               // Show the actions
               Dom.removeClass(elActions, "hidden");
               this.deferredActionsMenu = null;
           }
       },
     /**
      * Updates all Data Grid data by calling repository webscript with current list details
      *
      * @method _updateDataGrid
      * @private
      * @param p_obj.filter {object} Optional filter to navigate with
      */
     _updateDataGrid: function DataGrid__updateDataGrid(p_obj)
     {
        p_obj = p_obj || {};
        Alfresco.logger.debug("DataGrid__updateDataGrid: ", p_obj.filter);
        var successFilter = YAHOO.lang.merge({}, p_obj.filter !== undefined ? p_obj.filter : this.currentFilter),

           me = this,
           params =
           {
              filter: successFilter
           };
        this._addSortingToNewFilter(params.filter, this.sortAsc, this.sortColumn);

        // Clear the current document list if the data webscript is taking too long
        var fnShowLoadingMessage = function DataGrid_fnShowLoadingMessage()
        {
           Alfresco.logger.debug("DataGrid__uDG_fnShowLoadingMessage: slow data webscript detected.");
           // Check the timer still exists. This is to prevent IE firing the event after we cancelled it. Which is "useful".
           if (this.timerShowLoadingMessage)
           {
              this.loadingMessageShowing = Alfresco.util.PopupManager.displayMessage(
              {
                 displayTime: 0,
                 text: '<span class="wait">' + $html(this.msg("message.loading")) + '</span>',
                 noEscape: true
              });
           }
        };

        // Reset the custom error messages
        this._setDefaultDataTableErrors(this.widgets.dataTable);

        // More Actions menu no longer relevant
        this.showingMoreActions = false;

       var destroyLoaderMessage = (function DataGrid__uDG_destroyLoaderMessage()
       {
         if (this.timerShowLoadingMessage)
         {
           // Stop the "slow loading" timed function
           this.timerShowLoadingMessage.cancel();
           this.timerShowLoadingMessage = null;
         }

         if (this.loadingMessageShowing)
         {
           // Safe to destroy
           this.loadingMessageShowing.destroyWithAnimationsStop();
           this.loadingMessageShowing = null;
         }
       }).bind(this);
        // Slow data webscript message
       destroyLoaderMessage();
        this.loadingMessageShowing = false;
        this.timerShowLoadingMessage = YAHOO.lang.later(this.options.loadingMessageDelay, this, fnShowLoadingMessage);




        var successHandler = function DataGrid__uDG_successHandler(sRequest, oResponse, oPayload)
        {
           destroyLoaderMessage();
           // Updating the DotaGrid may change the item selection
           var fnAfterUpdate = function DataGrid__uDG_sH_fnAfterUpdate()
           {
              Bubbling.fire("selectedFilesChanged");
           };
           this.afterDataGridUpdate.push(fnAfterUpdate);

           Alfresco.logger.debug("currentFilter was:", this.currentFilter, "now:", successFilter);
           this.currentFilter = successFilter;
           this.currentPage = p_obj.page || 1;
           Bubbling.fire("filterChanged", successFilter);
           this.widgets.dataTable.onDataReturnReplaceRows.call(this.widgets.dataTable, sRequest, oResponse, oPayload);

           this.widgets.dataSource.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);
           this.widgets.dataSource.connMgr.setDefaultXhrHeader(Alfresco.util.Ajax.JSON);
           // Get permissions and workflowtasks and responsibles.
            var records = this.widgets.dataTable.getRecordSet()._records.filter(function(record)
            {
                return record._oData.nodeRef;
            });



          this.updateMetadata(records);
          this.updatePermissions(records);


        };

        var failureHandler = function DataGrid__uDG_failureHandler(sRequest, oResponse)
        {
           destroyLoaderMessage();
           // Clear out deferred functions
           this.afterDataGridUpdate = [];

           if (oResponse.status == 401)
           {
              // Our session has likely timed-out, so refresh to offer the login page
              window.location.reload(true);
           }
           else
           {
              try
              {
                 var response = YAHOO.lang.JSON.parse(oResponse.responseText);
                 this.widgets.dataTable.set("MSG_ERROR", response.message);
                 this.widgets.dataTable.showTableMessage(response.message + ". " + Alfresco.util.message("message.info.refresh"), YAHOO.widget.DataTable.CLASS_ERROR);
                 if (oResponse.status == 404)
                 {
                    // Site or container not found - deactivate controls
                    Bubbling.fire("deactivateAllControls");
                 }
              }
              catch(e)
              {
                 this._setDefaultDataTableErrors(this.widgets.dataTable);
              }
           }
        };

        // Update the DataSource
        var requestParams = this._buildDataGridParams(params);
        Alfresco.logger.debug("DataSource requestParams: ", requestParams);

        // TODO: No-cache? - add to URL retrieved from DataSource
        // "&noCache=" + new Date().getTime();

        if (Alfresco.util.CSRFPolicy.isFilterEnabled())
        {
           this.widgets.dataSource.connMgr.initHeader(Alfresco.util.CSRFPolicy.getHeader(), Alfresco.util.CSRFPolicy.getToken(), false);
        }
        this.widgets.dataSource.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);
        this.widgets.dataSource.connMgr.setDefaultXhrHeader(Alfresco.util.Ajax.JSON);

        this.widgets.dataSource.sendRequest(YAHOO.lang.JSON.stringify(requestParams),
        {
           success: successHandler,
           failure: failureHandler,
           scope: this
        });
     },

     onDataGridPaging: function DataGrid_onDataGridPaging(paginator, args)
     {
    	this.currentPage = paginator.page;
        this._updateDataGrid.call(this,
        {
           page: this.currentPage
        });
     },

     onDataGridSort: function DataGrid_onDataGridSort(e, dataGrid)
     {
    	YAHOO.util.Event.stopEvent(e.event);
    	var column = dataGrid.widgets.dataTable.getColumn(e.target.cellIndex);
    	if (!column.sortable) {
    		this._removeSortingClasses();
    		return; // not sortable
    	}

    	var sortField = column.key;
    	if (dataGrid.sortColumn && dataGrid.sortColumn === sortField) {
    		dataGrid.sortAsc = (dataGrid.sortAsc) ? false : true;
    	} else {
    		dataGrid.sortColumn = sortField;
    		dataGrid.sortAsc = true;
    	}
   		dataGrid.currentFilter.sortField = sortField;
   		dataGrid.currentFilter.sortAsc = dataGrid.sortAsc;

   		dataGrid._updateDataGrid.call(dataGrid,
	    {
   			page: dataGrid.currentPage
	    });

   		this._addSortingClasses(e.target);

     },

     _removeSortingClasses : function ExtDataGrid_removeSortingClasses() {
		var addClass = YAHOO.widget.DataTable.CLASS_DESC;
		var removeClass = YAHOO.widget.DataTable.CLASS_ASC;

		var headers = this.widgets.dataTable.getColumnSet().flat;
		for (var i = 0; headers.length > i; i++) {
			var header = headers[i].getThEl();
			if (Dom.hasClass(header, addClass))
				Dom.removeClass(header, addClass);
			if (Dom.hasClass(header, removeClass))
				Dom.removeClass(header, removeClass);
		}
	},
     _addSortingClasses : function ExtDataGrid_addSortingClasses(elm) {
    	this._removeSortingClasses();
		if (this.sortAsc) {
			var addClass = YAHOO.widget.DataTable.CLASS_ASC;
			var removeClass = YAHOO.widget.DataTable.CLASS_DESC;
		} else {
			var addClass = YAHOO.widget.DataTable.CLASS_DESC;
			var removeClass = YAHOO.widget.DataTable.CLASS_ASC;
		}
		Dom.addClass(elm, addClass);
		if (Dom.hasClass(elm, removeClass))
			Dom.removeClass(elm, removeClass);
	},


     _addSortingToNewFilter : function ExtDataGrid_addSortingToNewFilter(newFilter, sortAsc, sortField)
     {
    	 if (null == newFilter.sortField) {
    		 newFilter.sortField = sortField;
    	 }
    	 if (null == newFilter.sortAsc) {
    		 newFilter.sortAsc = sortAsc;
    	 }

     },

     /**
      * Renders Data List metadata, i.e. title and description
      *
      * @method renderDataListMeta
      */
     renderDataListMeta: function DataGrid_renderDataListMeta()
     {
    	 Alfresco.component.ExtDataGrid.superclass.renderDataListMeta.call(this);

         if (!YAHOO.lang.isObject(this.datalistMeta))
         {
            return;
         }


         // #38118, uncomment when Lyse have distributed all new diagrams.
         /*Alfresco.util.populateHTML(
            [ this.id + "-wf-image", this.msg("form.label.workflowDiagram")  ],
            [ this.id + "-wf-image-description", '<img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/workflow-diagrams/' + this.datalistMeta.itemType.replace(':', '_') + '.png"/>']);
         */


    	 var url = YAHOO.lang.substitute(Alfresco.constants.PROXY_URI + "/api/metadata?nodeRef={nodeRef}", {
    		 nodeRef : this.datalistMeta.nodeRef
    	 });
    	 Alfresco.util.Ajax.request({
             method : Alfresco.util.Ajax.GET,
             url : url,
             responseContentType : Alfresco.util.Ajax.JSON,
             successCallback : {
                fn : function DataGrid_populateLastUpdatedField(response)
                {
                   if (response && response.json)
                   {
                      var lastUpdate = response.json.properties["{http://www.alfresco.org/model/content/1.0}modified"];
                      var date = new Date(lastUpdate);
                      var info = this.msg("form.label.lastUpdate") + ": ";
                      info += YAHOO.lang.trim(date.toUTCString().replace("GMT", ""));
                      Alfresco.util.populateHTML([ this.id + "-last-updated", $html(info) ]);
                   }
                },
                scope : this,
             }
         });
       },

       /**
        * Return data type-specific sorter
        *
        * @method getSortFunction
        * @return {function} Function to sort column by
        */
       getSortFunction: function DataGrid_getSortFunction()
       {
           /**
            * Data Type custom sorter
            *
            * @method sortFunction
            * @param a {object} Sort record a
            * @param b {object} Sort record b
            * @param desc {boolean} Ascending/descending flag
            * @param field {String} Field to sort by
            */
           return function DataGrid_sortFunction(a, b, desc, field)
           {
               var fieldA = a.getData().itemData[field],
                   fieldB = b.getData().itemData[field];

               if (YAHOO.lang.isArray(fieldA))
               {
                   fieldA = fieldA[0];
               }
               if (YAHOO.lang.isArray(fieldB))
               {
                   fieldB = fieldB[0];
               }

               // Deal with empty values
               if (!YAHOO.lang.isValue(fieldA))
               {
                   return (!YAHOO.lang.isValue(fieldB)) ? 0 : 1;
               }
               else if (!YAHOO.lang.isValue(fieldB))
               {
                   return -1;
               }

               var valA = fieldA.value,
                   valB = fieldB.value;

               // Lyse change: use the display value if it exists and is a string, the original code would only use displayValue if fieldA starts with "workspace://SpacesStore"
               if (typeof valA === "string" && YAHOO.lang.isValue(fieldA.displayValue))
               {
                   valA = fieldA.displayValue;
                   valB = fieldB.displayValue;
               }

               return YAHOO.util.Sort.compare(valA, valB, desc);
           };
       }

   }, true);

   function loadingGif()
  {
    return '<span class="loading"></span>';
  }
   function getTaskResponsibleFromRecord(oRecord){
        var responsiblesHtml = '';
        if(oRecord.getData('responsibles') === undefined)
        {
          return "";//loadingGif();
        }
        else if (oRecord.getData("responsibles") != null) {

            for (i = 0; i < oRecord.getData("responsibles").length; i++) {
                var responsible = oRecord.getData("responsibles")[i];
                responsiblesHtml += responsible.name;
                if (i < oRecord.getData("responsibles").length-1){
                        responsiblesHtml += ", ";
                }
            }
        }
        return responsiblesHtml;
   }

   function getShareTaskResponsibleFromRecord(oRecord){
       var responsiblesHtml = '';
       if(oRecord.getData('shareResponsibles') === undefined)
       {
         return "";//loadingGif();
       }
       else if (oRecord.getData("shareResponsibles") != null) {

           for (i = 0; i < oRecord.getData("shareResponsibles").length; i++) {
               var responsible = oRecord.getData("shareResponsibles")[i];
               responsiblesHtml += responsible.name;
               if (i < oRecord.getData("shareResponsibles").length-1){
                       responsiblesHtml += ", ";
               }
           }
       }
       return responsiblesHtml;
  }

  function strip(html) {
     //https://stackoverflow.com/questions/822452/strip-html-from-text-javascript/47140708#47140708
    if(!html){
      return "";
    }
    var doc = new DOMParser().parseFromString(html, 'text/html');

    return doc.body.textContent || "";

  }
})();
