(function () {
  /**
   * YUI Library aliases
   */
  var Event = YAHOO.util.Event;

  /**
   * Constructor.
   *
   * @param {String} htmlId The HTML id of the parent element
   * @return {Lyse.component.MCPackagesOverview} The new component instance
   * @constructor
   */
  Lyse.component.MCPackagesOverview= function MCPackagesOverview_constructor(htmlId) {
    Lyse.component.MCPackagesOverview.superclass.constructor.call(this, "Lyse.component.MCPackagesOverview", htmlId, ["container", "datasource", "datatable"]);
  };

  /**
   * Extend Alfresco.component.Base
   */
  YAHOO.extend(Lyse.component.MCPackagesOverview, Alfresco.component.Base,
      {
    /**
     * Fired by YUI when parent element is available for scripting.
     * Component initialisation, including instantiation of YUI widgets and event listener binding.
     *
     * @method onReady
     */
    onReady: function() {
      this.createDataTable();
      var toggleMCPackagesOvreview = (function () {
        var container = Dom.get(this.id + "-mc-packages-overview");
        var button = Dom.get("toggleMCPackagesOverview-button");

        var display = container.style.display;
        if(display == "none") {
          container.style.display = 'block';
          button.innerHTML = "Hide MC packages overview";
        } else {
          container.style.display = 'none';
          button.innerHTML = "Show MC packages overview";
        }
      }).bind(this);

      var oToggleMCPackagesOverview = new YAHOO.widget.Button({
        id: "toggleMCPackagesOverview",
        type: "button",
        label: "Hide MC packages overview",
        container: "hide-mc-packages-overview"
      });

      oToggleMCPackagesOverview.on("click", toggleMCPackagesOvreview);
    },

    /**
     * Builds the url for the data table
     *
     * @method buildUrl
     * @retrun {string} The url for the data table
     */
    buildUrl: function MCPackagesOverview_buildUrl() {
      var url = Alfresco.constants.PROXY_URI + "api/lyse/mc-packages-overview?nodeRef={nodeRef}&";

      return YAHOO.lang.substitute(url,
          {
        nodeRef: encodeURIComponent(this.options.nodeRef)
          });
    },

    /**
     * Creates the data table
     *
     * @method createDataTable
     */
    createDataTable: function MQVersionHistory_createDataTable() {
      this.widgets.alfrescoDataTable = new Alfresco.util.DataTable(
          {
            dataSource: {
              url: this.buildUrl(),
              config: {
                responseSchema: {
                  resultsList: 'mcPackages',
                  fields: ["mcpackage", "status", "contract", "area", "fat", "mcplanned", "mcforecast", "noderef", "workflowid"]
                }
              }
            },
            dataTable: {
              container: this.id + "-mc-packages-overview",
              columnDefinitions: [
                                  {key: "mcpackage", label: this.msg("column.mcpackage"), sortable: true},
                                  {key: "status", label: this.msg("column.status"), sortable : true},
                                  {key: "contract", label: this.msg("column.contract"), sortable : true},
                                  {key: "area", label: this.msg("column.area"), sortable : true},
                                  {key: "fat", label: this.msg("column.fat"), sortable : true},
                                  {key: "mcplanned", label: this.msg("column.mcplanned"), sortable : true},
                                  {key: "mcforecast", label: this.msg("column.mcforecast"), sortable : true},
                                  {key: "actions", label: this.msg("column.actions"), formatter: this.renderActions.bind(this)}
                                  ],
                                  config: {
                                    MSG_EMPTY: this.msg("no.result"),
                                    className: ""
                                  }
            }
          });

      // Attach event to view datalist item
      YAHOO.util.Event.delegate(
          this.id,
          "click",
          this.viewItem.bind(this),
          "a.view-item"
      );

      // Attach event to go to datalist item
      YAHOO.util.Event.delegate(
          this.id,
          "click",
          this.goToItem.bind(this),
          "a.goto-item"
      );
    },
    renderActions: function (elCell) {
      elCell.innerHTML = "<div class='actions'>" +
      /* #39221 disabled for now
       * "<a class='view-item' title='View item' href='#'>" + "<img src="+Alfresco.constants.URL_CONTEXT +"res/components/images/search.gif>" + "</a>" +
       * */
      "<a class='goto-item' title='View item' href='#'>" + "<img src="+Alfresco.constants.URL_CONTEXT +"res/components/images/list-16.png></a>" +
      "</div>";
    },

    viewItem : function(event,element) {
      var tr = element.parentNode.parentNode.parentNode;
      var record = this.widgets.alfrescoDataTable.getDataTable().getRecord(tr);
      var data = record.getData();

      // Stop browser from using href attribute
      Event.preventDefault(event);

      var templateUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&mode={mode}",
          {
        itemKind: "node",
        itemId: data.noderef,
        mode: "view"
          });

      // Using Forms Service, so always create new instance
      var htmlId = this.id +  "-showVersion-";
      var showVersion = new Alfresco.module.SimpleViewDialog(htmlId);
      showVersion.setOptions(
          {
            width: "60em",
            templateUrl: templateUrl,
            actionUrl: null,
            destroyOnHide: true
          });
      showVersion.show();
      YAHOO.util.Event.onAvailable(htmlId + "-dialog", function() {
        Alfresco.util.populateHTML([ htmlId+ "-dialogTitle", "View item" ]);
      });
    },
    goToItem : function(event,element) {
      var tr = element.parentNode.parentNode.parentNode;
      var record = this.widgets.alfrescoDataTable.getDataTable().getRecord(tr);
      var data = record.getData();
      var nodeRef = data.noderef;
      var workflowId = data.workflowid;
      var templateUrl;
      //First check if this view form shall have an attachment document library
      Alfresco.util.Ajax.jsonGet(
          {
            url: Alfresco.constants.PROXY_URI + "api/lyse/attachment-folder?nodeRef=" +nodeRef,
            successCallback:
            {
              fn: function (response){
                var siteShortNameUrl = "";
                if (Alfresco.constants.SITE) {
                  siteShortNameUrl = "/site/" + Alfresco.constants.SITE;
                }
                if (response.json !== undefined && response.json.nodeRef !== "") {
                  //Success, this datalist item has an attachment folder
                  var attachmentFolder = response.json.nodeRef;
                  if(workflowId != "" ) {
                    templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?workflowId=' + workflowId + "&docLib=" + attachmentFolder;
                  } else {
                    templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?nodeRef=' + nodeRef + "&docLib=" + attachmentFolder;
                  }

                  // Some browsers has a minimal warning about new tab. Alert user.
                  Alfresco.util.PopupManager.displayMessage({
                    text: this.msg('message.opened_in_new_window')
                  });

                  return window.open(templateUrl, "_blank");
                } else {
                  //Datalist item does not have an attachment folder, proceed as usual.
                  // Some browsers has a minimal warning about new tab. Alert user.
                  Alfresco.util.PopupManager.displayMessage({
                    text: this.msg('message.opened_in_new_window')
                  });

                  if(workflowId != null) {
                    templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?workflowId='+workflowId.value.toString();
                    return window.open(templateUrl, "_blank");
                  } else {
                    templateUrl = Alfresco.constants.URL_CONTEXT + 'page'+siteShortNameUrl+'/lyse-datalist-details?nodeRef=' + nodeRef;
                    return window.open(templateUrl, "_blank");
                  }
                }
              },
              scope: this
            }
          });
    }
  });
})();


