(function () {
  /**
   * YUI Library aliases
   */
  Event = YAHOO.util.Event;

  /**
   * Constructor.
   *
   * @param {String} htmlId The HTML id of the parent element
   * @return {Lyse.component.MQVersionHistory} The new component instance
   * @constructor
   */
  Lyse.component.MQVersionHistory = function MQVersionHistory_constructor(htmlId) {
    Lyse.component.MQVersionHistory.superclass.constructor.call(this, "Lyse.component.MQVersionHistory", htmlId, ["container", "datasource", "datatable"]);
  };

  /**
   * Extend Alfresco.component.Base
   */
  YAHOO.extend(Lyse.component.MQVersionHistory, Alfresco.component.Base,
      {
    /**
     * Fired by YUI when parent element is available for scripting.
     * Component initialisation, including instantiation of YUI widgets and event listener binding.
     *
     * @method onReady
     */
    onReady: function() {
      this.createDataTable();
    },

    /**
     * Builds the url for the data table
     *
     * @method buildUrl
     * @retrun {string} The url for the data table
     */
    buildUrl: function MQVersionHistory_buildUrl() {
      var url = Alfresco.constants.PROXY_URI + "api/lyse/measured-quantity-info?nodeRef={nodeRef}";

      return YAHOO.lang.substitute(url,
          {
        nodeRef: encodeURIComponent(this.options.nodeRef),
          });
    },

    /**
     * Creates the data table
     *
     * @method createDataTable
     */
    createDataTable: function MQVersionHistory_createDataTable() {
      this.widgets.alfrescoDataTable = new Alfresco.util.DataTable(
      {
        dataSource: {
          url: this.buildUrl(),
          config: {
            responseSchema: {
              resultsList: 'versions',
              fields: ["revNo", "version", "quantity", "comments", "nodeRef", "startDate", "endDate", "versionRef", "status"]
            }
          }
        },
        dataTable: {
          container: this.id + "-mq-version-history",
          columnDefinitions: [
            {key: "revNo", label: this.msg("column.revision")},
            {key: "quantity", label: this.msg("column.quantity")},
            {key: "status", label: this.msg("column.status")},
            {key: "actions", label: this.msg("column.actions"), formatter: this.renderActions.bind(this)}
          ],
          config: {
            MSG_EMPTY: this.msg("no.result"),
            className: "" // default = alfresco-datatable, which hides the column headings
          }
        }
      });

      this.widgets.alfrescoTotalQuantityDataTable = new Alfresco.util.DataTable(
          {
            dataSource: {
              url: this.buildUrl(),
              config: {
                responseSchema: {
                  resultsList: 'totalQuantity',
                  fields: ["totalQuantity"]
                }
              }
            },
            dataTable: {
              container: this.id + "-mq-version-accumulated",
              columnDefinitions: [
                {key: "totalQuantity", label: this.msg("column.totalQuantity")}
              ],
              config: {
                MSG_EMPTY: this.msg("no.result"),
                className: ""
              }
            }
          });

      // Attach event to view comments
      YAHOO.util.Event.delegate(
          this.id,
          "click",
          this.displayOldComments.bind(this),
          "a.view-comments"
      );

      // Attach event to view datalist item
      YAHOO.util.Event.delegate(
          this.id,
          "click",
          this.displayOld.bind(this),
          "a.view-item"
      );
    },

    renderActions: function (elCell) {
      elCell.innerHTML = "<div class='actions'>" +
          //"<a class='view-comments' href='#'>" + this.msg("button.comment") + "</a>" +
          "<a class='view-item' href='#'>" + this.msg("button.item") + "</a>" +
          "</div>";
    },

    displayOld : function(event,element) {
      var tr = element.parentNode.parentNode.parentNode;
      var record = this.widgets.alfrescoDataTable.getDataTable().getRecord(tr)
      var data = record.getData();

      // Stop browser from using href attribute
      Event.preventDefault(event);

      var templateUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&mode={mode}",
      {
        itemKind: "node",
        itemId: data.versionRef,
        mode: "view"
      });

      // Using Forms Service, so always create new instance
      var htmlId = this.id +  "-showVersion-" + data.version;
      var showVersion = new Alfresco.module.SimpleViewDialog(htmlId);
      showVersion.setOptions(
          {
            width: "60em",
            templateUrl: templateUrl,
            actionUrl: null,
            destroyOnHide: true
          });
      showVersion.show();
      YAHOO.util.Event.onAvailable(htmlId + "-dialog", function() {
        Alfresco.util.populateHTML([ htmlId+ "-dialogTitle", "Version "+data.version ]);
      });
    },

    displayOldComments : function(event,element) {
      var tr = element.parentNode.parentNode.parentNode;
      var record = this.widgets.alfrescoDataTable.getDataTable().getRecord(tr)
      var data = record.getData();

      // Stop browser from using href attribute
      Event.preventDefault(event);

      // construct the url to call
      var url = YAHOO.lang.substitute(Alfresco.constants.PROXY_URI + "api/lyse/{nodeRef}/comments?reverse=true&startIndex=0&pageSize=5&startDate={startDate}&endDate={endDate}",
        {
          nodeRef: data.nodeRef.replace(":/", ""),
          startDate: data.startDate,
          endDate: data.endDate
        });

      Alfresco.util.Ajax.jsonGet(
          {
            url: url,
            successCallback:
            {
              fn: function(response) {
                if(response.json !== undefined){
                  var div = document.createElement("div");
                  var message = "";
                  if(response.json.items.length > 0) {
                    for(var i=0; i<response.json.items.length;i++){
                      // Building the comments
                      var comments =
                      "<div><b>"
                      +response.json.items[i].author.firstName + response.json.items[i].author.lastName
                      +" ("+ new Date(response.json.items[i].createdOn).toLocaleString() + "),"
                      +"</b>"
                      +"<br/>" + response.json.items[i].content
                      +"</div><br/>"
                      message += comments;
                    }
                  } else {
                    message = "No comments"
                  }
                  div.innerHTML = message;
                  div.style.textAlign = "left";
                  Alfresco.util.PopupManager.displayPrompt(
                      {
                          text: div,
                          title: "Comments",
                          close: true,
                          noEscape: true
                      });
                }
              },
              scope: this
            },
          });
    }
  });
})();



