(function () {
    /**
     * YUI Library aliases
     */
    Event = YAHOO.util.Event;

    /**
     * Constructor.
     *
     * @param {String} htmlId The HTML id of the parent element
     * @return {Lyse.component.RevisionHistoryControl} The new component instance
     * @constructor
     */
    Lyse.component.RevisionHistoryControl = function RevisionHistoryControl_constructor(htmlId) {
        Lyse.component.RevisionHistoryControl.superclass.constructor.call(this, "Lyse.component.RevisionHistoryControl", htmlId, ["container", "datasource", "datatable"]);
    };

    /**
     * Extend Alfresco.component.Base
     */
    YAHOO.extend(Lyse.component.RevisionHistoryControl, Alfresco.component.Base,
        {
            onToggleRevisionHistory: function RevisionHistoryControl_onToggleRevisionHistory() {
                var container = Dom.get(this.id + "-revision-history");
                var button = Dom.get(this.id + "-show-hide-revision-history-button");

                var display = container.style.display;
                if (display === "none") {
                    container.style.display = 'block';
                    button.innerHTML = this.msg("label.hideRevisionHistory");
                } else {
                    container.style.display = 'none';
                    button.innerHTML = this.msg("label.showRevisionHistory");
                }
            },

            /**
             * Fired by YUI when parent element is available for scripting.
             * Component initialisation, including instantiation of YUI widgets and event listener binding.
             *
             * @method onReady
             */
            onReady: function RevisionHistoryControl_onReady() {
                Alfresco.util.createYUIButton(this, "show-hide-revision-history", this.onToggleRevisionHistory);
                this.createDataTable();
            },

            /**
             * Builds the url for the data table
             *
             * @method buildUrl
             * @retrun {string} The url for the data table
             */
            buildUrl: function RevisionHistoryControl_buildUrl() {
                var url = Alfresco.constants.PROXY_URI + "api/lyse/revision-history?nodeRef={nodeRef}&revisionProperty={revisionProperty}";

                return YAHOO.lang.substitute(url,
                    {
                        nodeRef: encodeURIComponent(this.options.nodeRef),
                        revisionProperty: encodeURIComponent(this.options.revisionProperty)
                    });
            },

            /**
             * Creates the data table
             *
             * @method createDataTable
             */
            createDataTable: function RevisionHistory_createDataTable() {
                this.widgets.alfrescoDataTable = new Alfresco.util.DataTable(
                    {
                        dataSource: {
                            url: this.buildUrl(),
                            config: {
                                responseSchema: {
                                    resultsList: 'revisions',
                                    fields: ["revision", "version", "versionRef", "nodeRef", "comments", "currentVersion", "startDate", "endDate", "status"]
                                }
                            }
                        },
                        dataTable: {
                            container: this.id + "-revision-history",
                            columnDefinitions: [
                                {key: "revision", label: this.msg("column.revision")},
                                //{key: "version", label: this.msg("column.version")},
                                {key: "endDate", label: this.msg("column.date"), formatter: this.renderCellDate.bind(this)},
                                {key: "status", label: this.msg("column.status"), formatter: this.renderStatus.bind(this)},
                                {key: "actions", label: this.msg("column.actions"), formatter: this.renderActions.bind(this)}
                            ],
                            config: {
                                MSG_EMPTY: this.msg("no.result"),
                                className: "" // default = alfresco-datatable, which hides the column headings
                            }
                        }
                    });

                // Attach event to view datalist item
                YAHOO.util.Event.delegate(
                    this.id,
                    "click",
                    this.displayVersion.bind(this),
                    "a.view-item"
                );
            },

            renderActions: function (elCell) {
                elCell.innerHTML = "<div class='actions'><a class='view-item' href='#'>" + this.msg("button.item") + "</a></div>";
            },

            renderStatus: function (elCell, oRecord, oColumn, oData) {
                // #43102 - Dirty hack "Approved - Current" should just be displayed as "Approved"
                elCell.innerHTML = oData !== null ? oData.replace("- Current", "") : "";
            },

            renderCellDate: function WorkflowHistory_renderCellDate(elCell, oRecord, oColumn, oData) {
                if (oData) {
                    var date = new Date(oData);
                    elCell.innerHTML = Alfresco.util.formatDate(date, this.msg("date-format.shortDate")) + " " + Alfresco.util.formatDate(date, this.msg("date-format.shortTime"));

                }
            },

            displayVersion: function (event, element) {
                var tr = element.parentNode.parentNode.parentNode;
                var record = this.widgets.alfrescoDataTable.getDataTable().getRecord(tr);
                var data = record.getData();

                // Stop browser from using href attribute
                Event.preventDefault(event);

                var templateUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&mode={mode}",
                    {
                        itemKind: "node",
                        itemId: data.versionRef,
                        mode: "view"
                    });

                // Using Forms Service, so always create new instance
                var htmlId = this.id + "-showVersion-" + data.version;
                var showVersion = new Alfresco.module.SimpleViewDialog(htmlId);
                showVersion.setOptions(
                    {
                        width: "60em",
                        templateUrl: templateUrl,
                        actionUrl: null,
                        destroyOnHide: true
                    });
                showVersion.show();
                YAHOO.util.Event.onAvailable(htmlId + "-dialog", function () {
                    Alfresco.util.populateHTML([htmlId + "-dialogTitle", "Version " + data.version]);
                });
            }

        });
})();



