// @overridden projects/slingshot/source/web/components/document-details/document-actions.js

/*
 * This file overrides multiple functions found in document-actions.js
 *
 * Redmine issues: #29044
 * Override function:  _onActionDeleteConfirm
 * If the deleted node is an attachment associated to a datalistitem this function will redirect to the correct
 * datalist after the delete is done, instead of the standard behaviou that is navigation to the parent document
 * library node (where this attachment actually resides).
 * @autor erik.billerby@redpill-linpro.com
 *
 * Redmine issue: #36538
 * Override function: onNewVersionUploadCompleteCustom
 * if there is no site available this function will now find one, activitypost need a sitename.
 * @autor herman@redpill-linpro.com
 *
 */
(function (__onActionDeleteConfirm, onNewVersionUploadCompleteCustom) {

    /**
     * This is a copy of the original one with one modification: instead of redirecting to the document library of the
     * repository, it now closes the window OR redirects to the user home.
     */
    Alfresco.DocumentActions.prototype._onActionDeleteConfirm = function Lyse_DocumentActions_onActionDeleteConfirm(asset) {
      var path = asset.location.path,
        fileName = asset.fileName,
        displayName = asset.displayName,
        nodeRef = new Alfresco.util.NodeRef(asset.nodeRef),
        parentLocation = DocumentActions_getParentLocation(path, this.options.siteId);

        var doDelete = function(callbackUrl) {
            this.modules.actions.genericAction(
                {
                    success: {
                        activity: {
                            siteId: this.options.siteId,
                            activityType: "file-deleted",
                            page: "documentlibrary",
                            activityData: {
                                fileName: fileName,
                                path: path,
                                nodeRef: nodeRef.toString()
                            }
                        },
                        callback: {
                            fn: function DocumentActions_oADC_success(data) {
                              if(window.history.length>1)
                              {
                                window.history.back();
                              }
                              else if (window.opener || window.history.length === 1) {
                                Lyse.util.closeWindow();
                              }
                              else if (callbackUrl != null) {
                                window.location = callbackUrl;
                              }
                              else {
                                Lyse.util.navigateForward();
                              }
                            }
                        }
                    },
                    failure: {
                        message: this.msg("message.delete.failure", displayName)
                    },
                    webscript: {
                        method: Alfresco.util.Ajax.DELETE,
                        name: "file/node/{nodeRef}",
                        params: {
                            nodeRef: nodeRef.uri
                        }
                    }
                });

        };

        Alfresco.util.Ajax.jsonGet({
            url : Alfresco.constants.PROXY_URI + "/api/lyse/source-associated-datalistitem?nodeRef=" + asset.nodeRef,
            successCallback : {
                fn : function(response) {
                    var dlUrl = response.json.data.datalistUrl;
                    doDelete.call(this, dlUrl);
                },
                scope : this
            },
            failureCallback:
            {
                fn: function(response)
                {
                    doDelete.call(this, null);
                },
                scope: this
            }

        });

    };

    /**
     * This replaces alfrescos default function and if no site name is found, it calls a webscript to find it.
     */
    Alfresco.DocumentActions.prototype.onNewVersionUploadCompleteCustom = function Lyse_DocumentActions_onNewVersionUploadCompleteCustom(complete) {
         this.recordData.jsNode.setNodeRef(complete.successful[0].nodeRef);
         var mydocumentDetailsUrl = this.getActionUrls(this.recordData).documentDetailsUrl + "#newVersionUpload";
         var oldNodeRef = this.recordData.jsNode.nodeRef.nodeRef, newNodeRef = this.recordData.nodeRef;
         var siteName = this.options.siteId;

         // Declare function that make activity post
         var activityPost = function () {
             Alfresco.Share.postActivity(siteName, "org.alfresco.documentlibrary.file-updated", complete.successful[0].fileName, "document-details?nodeRef="+complete.successful[0].nodeRef,
            {
                 fileName: complete.successful[0].fileName,
                 nodeRef: complete.successful[0].nodeRef
             },
             function(){
                 window.location = mydocumentDetailsUrl;
	             if (oldNodeRef == newNodeRef)
	             {
                     window.location.reload();
	             }
             });
         }

         // Check if sitename exists, if not: call webscript, then call "activityPost"
		 if (!siteName) {
		      Alfresco.util.Ajax.jsonGet({
		    	  url: Alfresco.constants.PROXY_URI + "/api/lyse/" + Alfresco.util.NodeRef(complete.successful[0].nodeRef).uri + "/site-shortname",
		    	  responseContentType : "application/json",
		    	  successCallback:
		    	  {
		    		  fn: function (response){
		    			  siteName = response.json.shortName;
		    			  activityPost();
		    		  },
		    		  scope: this
		    	  }
		      });
		} else {
			// If name already exists, skip webscript and call "activityPost" direct.
			activityPost();
		}
    };
}(Alfresco.DocumentActions.prototype._onActionDeleteConfirm, Alfresco.DocumentActions.prototype.onNewVersionUploadCompleteCustom));
