var console = console || {log:function(e){},error:function(){}};

(function()
{
    window.SVK = window.SVK || {};

    var rplpDataTable = {};

    var clone = function(obj) {
        if (null == obj || "object" != typeof obj) return obj;
        var copy = obj.constructor();
        for (var attr in obj) {
            if (obj.hasOwnProperty(attr)) copy[attr] = obj[attr];
        }
        return copy;
    };

    rplpDataTable.formatters = {

        /*
        Send in a selectFieldId that we bind to.
        */
        selectBoxFormatterFactory : function(selectFieldId){
            var field = document.getElementById(selectFieldId);
            return function(el, oRecord, oColumn, oData){
                var recordValue = oRecord.getData(oColumn.key);

                // Reset
                el.innerHTML = '';

                // if the recordValue is undefined we pretend it is recordValue
                if(recordValue === undefined){
                    recordValue = "";
                }
                
                if(field.options === undefined || field.options.length === 0){
                	el.innerHTML = Alfresco.util.message("svk.label.dt.couldNotRenderReviewGroup");
                	return undefined;
                }

                // Loop through all options
                for(var i = 0; i < field.options.length; i += 1){
                    var option = field.options[i];
                    var optionValue = option.value;

                    // If this option
                    if(optionValue == recordValue){
                        el.innerHTML = option.text;
                        return undefined;
                    }
                }

                return undefined;
            }
        },
        trueFalseIndicatorFactory : function(options){
        	if(options === undefined){
        		options = {};
        	}
            var indicatorType = options.indicatorType;

            if(indicatorType === undefined){
                indicatorType = "checkbox";
            }

            return function(el, oRecord, oColumn, oData){
                //console.log(el, oRecord, oColumn.key, oData);

                var checkboxIndicator = "";
                if(oRecord.getData(oColumn.key) == true){
                    checkboxIndicator = "<input type='checkbox' checked='checked' disabled='disabled' />";
                }else{
                    checkboxIndicator = "<input type='checkbox' disabled='disabled' />";
                }

                if(indicatorType == "checkbox"){
                    el.innerHTML = checkboxIndicator;
                }
            }
        },
        linkFormatterFactory : function(options){
            return function(el, oRecord, oColumn, oData){
                var href=options.linkHrefFunction(oRecord);
                var text=oRecord.getData(options.linkTextProperty);
                if(href === undefined){
                    el.innerHTML = text;
                    return null;
                }
                var linkElement = '<a href="' + href + '">' + text + '</a>';
                el.innerHTML = linkElement;
                return true;
            }
        },
        imageFormatterFactory : function(options,optionalOptions){
            return function(el, oRecord, oColumn, oData){
            	options.showImageFunction(oRecord.getData("inheritParentPermission"),oRecord);
                var cssClass = "";
                var titleText = "";
                if(options.showPointer === true){
                    cssClass = 'show-pointer';
                }
/*
                if(options.getTitleTextFunction != undefined){
                	var authorizedMembers = options.getTitleTextFunction();
                	titleText = "Permission to read:\n\n";
                	for(var group in authorizedMembers){
                		titleText += authorizedMembers[group].displayName+"\n";
                	}
                	
                }
                var imgSrc = options.getImageSource(oRecord.getData("inheritParentPermission"));
                var img = '<div class ="comment-img-box"><a href="#" id="commentpermission"><img src="' + imgSrc + '" class="' + cssClass + '" title="' + titleText + '" /></a>';
*/
                var img = '<div class ="comment-img-box">';
                // This section is to get the "Reply" icon and functionallity in the same column/row as the padlock.
                if(optionalOptions && optionalOptions != undefined){
                	 if(optionalOptions.showImageFunction != undefined && optionalOptions.showImageFunction(oRecord.getData("allowAnswer"), oRecord) === false){
                	 }else{
		                var secondTitleText = "";
		                if(optionalOptions.getTitleTextFunction != undefined){
		                	secondTitleText = optionalOptions.getTitleTextFunction();
		                }
		                var secondImgSrc = optionalOptions.getImageSource();
		                var img = img + '<a href="#" id="commentreply"><img src="' + secondImgSrc + '" class="' + cssClass + '" title="' + secondTitleText + '" /></a>';
                	 }
	            }
                
                el.innerHTML = img+"</div>";
            	return true;
            }
        },
        dateFormatterFactory : function(options){
            return function(el, oRecord, oColumn, oData){
                var d = oRecord.getData(oColumn.key);

                if(d === undefined){
                    return undefined;
                }
                
                el.innerHTML = d.substr(0,10);
                
                return true;

                try{
                    d = new Date(d);
                }catch(err){
                    return undefined;
                }

                var str = "";
                str += d.getFullYear();
                str += '-';
                var month = d.getMonth() + 1;
                if(month <= 9){
                    month = '0' + month;
                }
                str += month
                str += '-';

                var day = d.getDate();
                if(day <= 9){
                    day = '0' + day;
                }
                str += day;

                el.innerHTML = str;
            }
        }
    };


    rplpDataTable.getDataTable = function(options){
        /**
         * Initiates a DataTable from the configuration passed
         * @param {object} options.beforeCallbackFunctions A object with keys describing rowData and a function that returns a value
         * @param {string} dataSourceURL to datasource API
         * @param {string} currentStateIdentifierKeyName name of the idenfifier in data that maps to currentState
        */
    	
    	//console.log(options);

        /*
        == beforeCallbackFunctions ==

        {
            someValue : function(parsedResponseRow, currentRowState){
                if(currentRowState === undefined){
                    return '';
                }
                // Add logic to check in the current row state for different values and return a value
                return 'welcome back.';
            }
        }
        This function NEEDS to be able to handle a "undefined" currentRowState, if the row has NO state, t
        */

        if(options.currentState === undefined){
            options.currentState = [];
        }


        if(options.dataSourceURL === undefined){
            console.error('dateSoruceURL parameter Needs to be passed as an option');
        }

        if(options.callbacks === undefined){
            options.callbacks = {};
        }

        // Default callbacks.
        if(options.callbacks['reset_rows_editor'] === undefined){
            options.callbacks['reset_rows_editor'] = function(dataTableConf){
                //console.log('reset', dataTableConf);
                for(key in dataTableConf.rowsEditor.fields){
                    document.getElementById(dataTableConf.rowsEditor.fields[key].fieldId).value = "";
                }
            }
        }

        var columnDefs = options.columnDefs;
        
        //console.log(dataTableConf);

        var dataTableConf = {
        	tableId : options.tableId,
            dataSourceURL : options.dataSourceURL,
            doBeforeCallback : undefined,
            currentState : options.currentState,
            initialState : clone(options.currentState),
            currentStateIdentifierKeyName : options.currentStateIdentifierKeyName,
            beforeCallbackFunctions: options.beforeCallbackFunctions,
            responseSchema: options.responseSchema,
            columnDefs: columnDefs,
            checkboxClickEvents : options.checkboxClickEvents,
            actionButtons: options.actionButtons,
            serializerConf: options.serializerConf,
            callbacks : options.callbacks,
            rowsEditor: options.rowsEditor,
            defaultValues : options.defaultValues || {},
            layout: options.layout || {},
            messages : options.messages || {}
        };

        //console.log('Messages', dataTableConf.messages);


        dataTableConf.callEvent = function(event_name){
            dataTableConf.callbacks[event_name](dataTableConf);
        };


        dataTableConf.getCurrentStateRowObject = function(id){
            for(var i = 0; i < dataTableConf.currentState.length; i += 1){
                var r = dataTableConf.currentState[i];
                if(r[dataTableConf.currentStateIdentifierKeyName] == id){
                    return r;
                }
            }

            return undefined;
        };

        dataTableConf.getInitialStateRowObject = function(id){
            for(var i = 0; i < dataTableConf.initialState.length; i += 1){
                var r = dataTableConf.initialState[i];
                if(r[dataTableConf.currentStateIdentifierKeyName] == id){
                    return r;
                }
            }

            return undefined;
        };
        /*
        Wraps all the beforeCallbackFunctions
        */
        dataTableConf.doBeforeCallback = function(oRequest, oFullResponse, oParsedResponse, oCallback){
            var parsedResponse = oParsedResponse;
            
            for(var i = 0; i < parsedResponse.results.length; i += 1){
                var responseRow = parsedResponse.results[i];
                // For each row in the response
                // The current row state, holds data like if it is selected and whatnot.
                var currentStateId = responseRow[dataTableConf.currentStateIdentifierKeyName];
                var currentRowState = dataTableConf.getCurrentStateRowObject(currentStateId);

                // If the row has a currentRowState, set that sate
                if(currentRowState !== undefined){
                    // There is a currentRowState. We will just set the responsrow to the current state object. I reccon.
                    for(var currentRowStateKey in currentRowState){
                        var currentRowStateValue = currentRowState[currentRowStateKey];
                        if(currentRowStateValue != undefined){
                            // If the value is something, set it.
                            responseRow[currentRowStateKey] = currentRowStateValue;
                        }
                    }
                }

                for(var columnName in dataTableConf.defaultValues){
                    var conf = dataTableConf.defaultValues[columnName];
                    responseRow[columnName] = conf.value;
                }
                /*
                We will now go through all functions defined in beforeCallbackFunctions, execute them and set the return value
                */
                if(dataTableConf.beforeCallbackFunctions != undefined){
                    for(var keyValue in dataTableConf.beforeCallbackFunctions){
                        // keyValue is the name of the parameter we should set on responseRow
                        responseRow[keyValue] = dataTableConf.beforeCallbackFunctions[keyValue](
                                responseRow, currentRowState
                        );
                    }
                }
            }

            return parsedResponse;
        };

        dataTableConf.dataSource = new YAHOO.util.XHRDataSource(
                dataTableConf.dataSourceURL,
                {
                    responseType : YAHOO.util.XHRDataSource.TYPE_JSON,
                    responseSchema : dataTableConf.responseSchema,
                    //doBeforeCallback : doBeforeCallback
                    doBeforeCallback : dataTableConf.doBeforeCallback
                }
            );

        // Initiate the data table!
        dataTableConf.dataTable = new YAHOO.widget.ScrollingDataTable(dataTableConf.tableId, dataTableConf.columnDefs, dataTableConf.dataSource,
                {
                    height: dataTableConf.layout.height || "30em",
                    width: dataTableConf.layout.width || "600px",
                    MSG_EMPTY: dataTableConf.messages.empty || Alfresco.util.message('label.svkDocumentDataTableEmptyMessage'),
                    MSG_SORTDESC: Alfresco.util.message('label.svkDocumentDataTableSortAscendingMessage'),
                    MSG_SORTASC: Alfresco.util.message('label.svkDocumentDataTableSortDecendingMessage')
                }
        );

        dataTableConf.toJSON = function(rowFormatterFunction){
            return JSON.stringify(dataTableConf.serialize(rowFormatterFunction));
        };

        dataTableConf.serialize = function(rowFormatterFunction, extraFields){
            var data = [];

            var records = dataTableConf.dataTable.getRecordSet().getRecords();
            
            var serializerConfFields = dataTableConf.serializerConf.fields;
            
            extraFields = extraFields || {};
            
            for(dataKey in extraFields){
            	serializerConfFields[dataKey] = extraFields[dataKey];
            }

            for(var i = 0; i < records.length; i += 1){
                var record = records[i];
                var recordData = {};

                var currentStateRowObject = undefined;
                var initialStateRowObject = undefined;
                if(dataTableConf.currentStateIdentifierKeyName != undefined){
                    var id = record.getData(dataTableConf.currentStateIdentifierKeyName);
                    currentStateRowObject = dataTableConf.getCurrentStateRowObject(id);
                    initialStateRowObject = dataTableConf.getInitialStateRowObject(id);

                    recordData['hasChanged'] = record.getData('hasChanged') === true;

                    if(dataTableConf.serializerConf.identifierName != undefined){
                        recordData[dataTableConf.serializerConf.identifierName] = id;
                    }
                }

                for(dataKey in serializerConfFields){
                    var dataFormatter = serializerConfFields[dataKey];
                    var value = record.getData(dataKey);
                    
                    if(typeof(dataFormatter) === "function"){
                        value = dataFormatter(value, currentStateRowObject);
                    }
                    recordData[dataKey] = value;
                }

                if(typeof(rowFormatterFunction) === 'function'){
                    var result = rowFormatterFunction(recordData, initialStateRowObject);

                    if(result != undefined){
                        data.push(result);
                    }
                }else{
                    data.push(recordData);
                }
            }

            return data;
        };

        if(options.checkboxClickEvents != undefined){
            dataTableConf.dataTable.subscribe('checkboxClickEvent', function(oArgs){
                var target = oArgs.target;
                var column = this.getColumn(target);
                var record = this.getRecord(target);

                var func = options.checkboxClickEvents[column.key];

                if(func == undefined){
                    return false;
                }

                var id = record.getData(dataTableConf.currentStateIdentifierKeyName);

                var currentRowState = dataTableConf.getCurrentStateRowObject(id);

                if(currentRowState === undefined){
                    currentRowState = {};
                }

                func(record, currentRowState, target.checked);

                return true;
            });
        }

        if(options.linkClickEvents != undefined){
        	dataTableConf.dataTable.subscribe('linkClickEvent', function(e){
                var target = e.target;
                var record = this.getRecord(target);

                var func = options.linkClickEvents[target.id];

                if(func == undefined){
                    return false;
                }

                var id = record.getData(dataTableConf.currentStateIdentifierKeyName);

                var currentRowState = dataTableConf.getCurrentStateRowObject(id);

                if(currentRowState === undefined){
                    currentRowState = {};
                }

                func(record, currentRowState);

                return true;
            });
        }

        dataTableConf.actionButtonFunctions = {
            selectAll : function(){
                var records = dataTableConf.dataTable.getRecordSet().getRecords();
                var conf = dataTableConf.actionButtons['selectAll'];

                for(var i = 0; i < records.length; i += 1){
                    var record = records[i];
                    record.setData('selected', true);
                }

                dataTableConf.dataTable.render();

                if(conf.postClickEvent != undefined){
                    dataTableConf.callEvent(conf.postClickEvent);
                }
            },
            selectNone: function(){
                var records = dataTableConf.dataTable.getRecordSet().getRecords();
                var conf = dataTableConf.actionButtons['selectNone'];

                for(var i = 0; i < records.length; i += 1){
                    var record = records[i];
                    record.setData('selected', false);
                }

                dataTableConf.dataTable.render();
                if(conf.postClickEvent != undefined){
                    dataTableConf.callEvent(conf.postClickEvent);
                }
            }
        };

        dataTableConf.getRecordsWithProperty = function(name, value){
            var filteredRecords = [];

            var records = dataTableConf.dataTable.getRecordSet().getRecords();
            for(var i = 0; i < records.length; i += 1){
                var record = records[i];
                if(record.getData(name) === value){
                    filteredRecords.push(record);
                }
            }

            return filteredRecords;
        };


        if(options.actionButtons != undefined){
        	//console.log('ActionButtons', options.actionButtons);
        	
            for(key in dataTableConf.actionButtons){
                //console.log(key);
                var actionButtonConf = options.actionButtons[key];
                //console.log('id', actionButtonConf);
                YAHOO.util.Event.addListener(
                    actionButtonConf.id, 'click', dataTableConf.actionButtonFunctions[key]
                );
                //console.log('Bound..');
            }
        }

        if(dataTableConf.rowsEditor != undefined){
            for(key in dataTableConf.rowsEditor.fields){
                var conf = dataTableConf.rowsEditor.fields[key];

                YAHOO.util.Event.addListener(conf.fieldId, 'change', function(conf){
                    return function(e){
                        var targetElement = YAHOO.util.Event.getTarget(e);
                        var selectedValue = targetElement.value;

                        // If the
                        if(targetElement === ""){
                            return undefined;
                        }

                        if(conf.resetAfterChange === true || conf.resetAfterChange === undefined){
                            dataTableConf.callEvent('reset_rows_editor');
                        }

                        // Get all selected records and set data on them
                        var records = dataTableConf.getRecordsWithProperty('selected', true);

                        for(var i=0;i<records.length;i++){
                            var record = records[i];

                            // Set data on record
                            record.setData(conf.valueKey, selectedValue);

                            // We set hasChanged on ALL items that have been changed.
                            record.setData('hasChanged', true);
                        }

                        // Re-render and change data
                        dataTableConf.dataTable.render();
                        dataTableConf.callEvent('data_changed');
                        return true;

                    }
                }(conf));
            }
        }
        
        // Refresh the table
        dataTableConf.refreshData = function(){
        	dataTableConf.dataTable.getDataSource().sendRequest('', { success: dataTableConf.dataTable.onDataReturnInitializeTable, scope: dataTableConf.dataTable});
        }

        // Return it so we can bind more stuff on it later. maby.
        return dataTableConf;
    };
    
    SVK.DataTable = rplpDataTable;

})();
