// @overridden projects/slingshot/source/web/components/documentlibrary/documentlist-view-table.js

(function(_renderView) {

  /*
   * Overridden in order to insert lastComment column, that renders the timestamp for the last comment on a node.
   */
  Alfresco.DocumentListTableViewRenderer.prototype.renderView = function Lyse_DL_SVR_renderView (scope, sRequest, oResponse, oPayload) {
    
    // Add a class to help with CSS selectors...
    var container = Dom.get(scope.id + this.parentElementIdSuffix);
    Dom.addClass(container, 'alf-table');
    
    var viewRendererInstance = this;
    
    scope.widgets.dataTable.set('draggableColumns', false);
    
    var container = Dom.get(scope.id + this.parentElementIdSuffix);
    var oRecordSet = scope.widgets.dataTable.getRecordSet();
    Dom.addClass(container, 'alf-table');
    Dom.removeClass(Dom.get(scope.id + "-table-config-button-container"), 'hidden');
    
    // Always hide the fileName column...
    // This has been done due to time-constraints and the issues surrounding the other views (most notably the detailed
    // and simple views) changing the widths of the fileName field. The simplest solution was to just always hide this 
    // field and rely on the JSON blob to add in columns that identify the item.
    scope.widgets.dataTable.hideColumn("fileName");
    
    // Not all of the columns are properties, the selector, indicators and actions are not properties and
    // need to be handled by specific configuration defined in the JSON BLOB...
    if (this.additionalJsonConfig != null)
    {
       // Add/remove actions column as appropriate
       if (this.additionalJsonConfig.actions != null && this.additionalJsonConfig.actions.show == "false")
       {
          scope.widgets.dataTable.hideColumn("actions");
       }
       else
       {
          scope.widgets.dataTable.showColumn("actions");
       }
       
       // Add/remove the indicators column...
       if (this.additionalJsonConfig.indicators != null && this.additionalJsonConfig.indicators.show == "false")
       {
          scope.widgets.dataTable.hideColumn("status");
       }
       else
       {
          scope.widgets.dataTable.showColumn("status");
       }
       
       // Add/remove the selection box column...
       if (this.additionalJsonConfig.selector != null && this.additionalJsonConfig.selector.show == "false")
       {
          scope.widgets.dataTable.hideColumn("nodeRef");
       }
       else
       {
          scope.widgets.dataTable.showColumn("nodeRef");
       }
       
       // Add/remove the thumbnail column...
       if (this.additionalJsonConfig.thumbnail != null && this.additionalJsonConfig.thumbnail.show == "false")
       {
          scope.widgets.dataTable.hideColumn("thumbnail");
       }
       else
       {
          scope.widgets.dataTable.showColumn("thumbnail");
       }
       
       // If a JSON blob has been provided then is should define the columns that need to be
       // rendered in the detailed view. Iterate over this configuration and add in the columns
       // defined making use of the data provided...
       if (this.additionalJsonConfig.propertyColumns != null && 
           this.additionalJsonConfig.propertyColumns.length != null)
       {
          for (var i=0; i<this.additionalJsonConfig.propertyColumns.length; i++)
          {
             var propColumn = this.additionalJsonConfig.propertyColumns[i];
             if (propColumn.property != null)
             {
                var width = null;
                var rawWidth = propColumn.width;
                if (rawWidth != null)
                {
                   width = parseInt(rawWidth);
                   if (width === NaN)
                   {
                      width = null;
                   }
                }
                
                var columnDef = {
                   key: propColumn.property,
                   label: propColumn.property,
                   formatter: (propColumn.link == "true") ? this.fnRenderCellLinkProperty() : this.fnRenderCellProperty(),
                   sortable: false,
                   width: this.getConfigWidth(propColumn)
                };
                if (propColumn.label != null)
                {
                   columnDef.label = scope.msg(propColumn.label);
                }
                scope.widgets.dataTable.insertColumn(columnDef);
             }
          }
       }
       
       // Lyse change: Add a "Last comment" column which is populated by an XHR request
       var lastCommentColumn = {
         key: 'lastComment',
         label: scope.msg('label.lastComment'),
         formatter: function(elCell, oRecord, oColumn, oData){
         	var nodeRef = oRecord.getData().nodeRef;
 	        // Find date and time for last comment
            Alfresco.util.Ajax.jsonGet(
            {
               url: Alfresco.constants.PROXY_URI + "lyse/api/last-comment?nodeRef=" +nodeRef,
               successCallback:
               {
                fn: function (response){
   	               if (response.json && response.json.lastComment)
   	               {
   	            	    var lastComment = response.json.lastComment;
   	            	    elCell.innerHTML = Alfresco.util.formatDate(lastComment);
   	               }
   	            },
                scope: this
               }
            });
            
         },
         sortable: false
       };
 
 	   scope.widgets.dataTable.insertColumn(lastCommentColumn);
 	   
 	   // Lyse change: Add a "Number of files" column which is populated by an XHR request
     var noOfFiles = {
       key: 'noOfFiles',
       label: scope.msg('label.noOfFiles'),
       formatter: function(elCell, oRecord, oColumn, oData){
       	var nodeRef = oRecord.getData().nodeRef;
        // Find number of files below folder
          Alfresco.util.Ajax.jsonGet(
          {
             url: Alfresco.constants.PROXY_URI + "lyse/api/number-of-files?nodeRef=" +nodeRef,
             successCallback:
             {
              fn: function (response){
 	               if (response.json && response.json.fileCount)
 	               {
 	            	    var fileCount = response.json.fileCount;
 	            	    elCell.innerHTML = fileCount;
 	               }
 	               
 	            },
              scope: this
             }
          });
          
       },
       sortable: false
     };

       // Get modified column, insert after modified-index or if not found at 9.
       var modifiedCell = scope.widgets.dataTable.getColumn("modified");
       if(modifiedCell){
         scope.widgets.dataTable.insertColumn(noOfFiles, modifiedCell._nTreeIndex + 1);
       }else{
         scope.widgets.dataTable.insertColumn(noOfFiles, 9);
       }

       // Lyse change: Recursively get comment count (if applicable) and set the comments count
       var commentsCount = {
         key: 'commentsCount',
         label: scope.msg('label.commentsCount'),
         formatter: function(elCell, oRecord, oColumn, oData){
          var nodeRef = oRecord.getData().nodeRef;
          // Find number of files below folder
          Alfresco.util.Ajax.jsonGet(
          {
             url: Alfresco.constants.PROXY_URI + "lyse/api/number-of-comments?nodeRef=" +nodeRef,
             successCallback:
             {
              fn: function (response){
                 if (response.json && response.json.commentCount)
                 {
                    var commentCount = response.json.commentCount;
                    elCell.innerHTML = commentCount;
                 }
                 
              },
              scope: this
             }
          });
         },
         sortable: false
       };

       var commentsCountCell = scope.widgets.dataTable.getColumn("fm:commentCount");
       if(commentsCountCell){
         var commentsCountCellIndex = commentsCountCell._nTreeIndex;
         scope.widgets.dataTable.removeColumn(commentsCountCell);
         scope.widgets.dataTable.insertColumn(commentsCount, commentsCountCellIndex);
       }
       else{
         scope.widgets.dataTable.insertColumn(commentsCount);
       }

       // Lyse change: Add a permission column which is populated by an XHR request
       var readPermissions = {
          key: 'permission',
          label: '',
          formatter: function(elCell, oRecord, oColumn, oData){
        	 
             // Setup variables for webscript & dialog
            var nodeRef = oRecord.getData().nodeRef;
            var nodeRefRegEx = new RegExp(/^(\w+):\/\/(\w+)\/([\w\d\-]+)$/);
            var nodeRefParts = nodeRef.match(nodeRefRegEx);
            var url = Alfresco.constants.PROXY_URI +  'api/lyse/getPermissions/' + nodeRefParts[1] + '/' + nodeRefParts[2] + '/' + nodeRefParts[3];
            var groups = [];
            Alfresco.util.Ajax.jsonGet(
              {
                 url: url,
                 successCallback:
                 {
                  fn: function (response){
                     groups = YAHOO.lang.JSON.parse(response.serverResponse.responseText).data;
                     if (groups) {

                       // Successfully received a json, populate column with correct data
                       var padlock = Alfresco.constants.URL_CONTEXT + 'res/components/images/padlock.gif';
                       var padLockDefault = Alfresco.constants.URL_CONTEXT + 'res/components/images/padlockred.gif';
                       var titleText = "Permission to read:\n\n";
                       for(var group in groups){
                         if(groups[group].hasOwnProperty('inherit')){
                             // If we have inherit, do not add any groups.
                             titleText += "<<All Members>>";
                             padLockDefault = padlock;
                             groups = null;
                             break;
                         }else{
                             // Add group to title-list
                           if(groups[group].hasOwnProperty('displayName')){
                                 titleText += groups[group].displayName+"\n";
                           }
                         }
                       }
               }
               var img = '<img src="' + padLockDefault + '" title="' + titleText + '" />';
                 elCell.innerHTML = img;

                 // If we may change permissions, we make the picture to a link
                 if(oRecord.getData().node.permissions.user.ChangePermissions){
                   elCell.addEventListener("click", function(){
                   var changePermissionDialog = new Alfresco.ChangePermissionDialog(this.id + "-changePermissionDialog");
                 var templateUrl = Alfresco.constants.URL_SERVICECONTEXT + "components/comments/change-permission-dialog";
                     var shortName = (Alfresco.constants.SITE != "") ? Alfresco.constants.SITE : (this.site != null) ? this.site : "";
                 changePermissionDialog.setOptions(
                 {
                 nodeRef: this.nodeRef,
                 site: shortName,
                 url: 'api/lyse/getPermissions/',
                 width: "40em",
                 templateUrl: templateUrl,
                 destroyOnHide: true,
                   preDefinedChoices: groups,
                   onSuccess:
                     {
                       fn: function ()
                       {
                       YAHOO.Bubbling.fire("metadataRefresh", {});
                       },
                       scope: this
                     }
                 });
                 changePermissionDialog.show();
                   });
                 elCell.nodeRef = nodeRef;
                 }
                  },
                  scope: this
                 }
              });
         },
         sortable: false
       };

      // Get the current sort data...
      var oSortedBy = scope.widgets.dataTable.get("sortedBy") || {};
      var oState = scope.widgets.dataTable.getState();

      YAHOO.util.Dom.setStyle(scope.id + this.parentElementIdSuffix, 'display', '');
      scope.widgets.dataTable.onDataReturnInitializeTable.call(scope.widgets.dataTable, sRequest, oResponse, oPayload);

      // Reset the sort state...
      scope.widgets.dataTable.set('sortedBy', oState.sortedBy);
      var titleCell = scope.widgets.dataTable.getColumn("cm:title");
      if(titleCell){
        scope.widgets.dataTable.removeColumn(titleCell);
      }
      var descriptionCell = scope.widgets.dataTable.getColumn("cm:description");
      if(descriptionCell){
        scope.widgets.dataTable.removeColumn(descriptionCell);
      }
    }
  };
}
(Alfresco.DocumentListTableViewRenderer.prototype.renderView));
