
/**
 * AttachmentDocLib component.
 *
 * @namespace Alfresco
 * @class Alfresco.AttachmentDocLib
 */
(function()
    {
  /**
   * YUI Library aliases
   */
  var Dom = YAHOO.util.Dom,
  Event = YAHOO.util.Event;

  /**
   * AttachmentDocLib constructor.
   *
   * @param {String} htmlId The HTML id of the control element
   * @return {Alfresco.AttachmentDocLib} The new AttachmentDocLib instance
   * @constructor
   */
  Alfresco.AttachmentDocLib = function(htmlId)
  {
    Alfresco.AttachmentDocLib.superclass.constructor.call(this, "Alfresco.AttachmentDocLib", htmlId);

    return this;
  };

  YAHOO.extend(Alfresco.AttachmentDocLib, Alfresco.component.Base,
      {
    /**
     * Object container for initialization options
     *
     * @property options
     * @type object
     */
    options:
    {

      /**
       * current nodeRef
       *
       * @property editorParameters
       * @type object
       */
      nodeRef: null
    },


    /**
     * Fired by YUI when parent element is available for scripting.
     * Component initialisation, including instantiation of YUI widgets and event listener binding.
     *
     * @method onReady
     */
    onReady: function AttachmentDocLib_onReady() {
      var datalistNodeRef;
      var attachmentFolder;
      var workflowId = Alfresco.util.parseURL().queryParams.workflowId;
      var createAttachmentButton = (function () {
        Alfresco.util.Ajax.jsonGet({
          url: Alfresco.constants.PROXY_URI + "api/lyse/attachment-folder?nodeRef=" +datalistNodeRef,
          successCallback: {
            fn: (function (response) {
              if (response.json !== undefined && response.json.nodeRef !== "") {
                //Success, this datalist item has an attachment folder
                attachmentFolder = response.json.nodeRef;

                var showAttachmentDocLib = (function (event) {
                  // Stop browser from using href attribute
                  Event.preventDefault(event);
                  var templateUrl = Alfresco.constants.URL_CONTEXT + 'page';

                  // #45603
                  // Sometimes (like in task forms) Alfresco.constants.SITE is not set, then it should be omitted in the URL.
                  if(Alfresco.constants.SITE != "") {
                    templateUrl += '/site/'+ Alfresco.constants.SITE;
                  }

                  templateUrl += '/lyse-datalist-details?';
                  if(workflowId) {
                    templateUrl += 'workflowId='+ workflowId;
                  } else {
                    templateUrl += 'nodeRef='+ datalistNodeRef;
                  }
                  templateUrl += '&docLib=' + attachmentFolder;

                  // Add anchor to the attachment doclib
                  templateUrl += '#attachment-doclib-header';
                  // Some browsers has a minimal warning about new tab. Alert user.
                  Alfresco.util.PopupManager.displayMessage({
                    text: "Target is opened in new window or tab"
                  });

                  return window.open(templateUrl, "_blank");
                }).bind(this);

                // Only add button once
                if(Dom.get("showAttachmentDocLib") == null) {
                  var oShowAttachmentDocLib = new YAHOO.widget.Button("attachmentButton", {
                    id: "showAttachmentDocLib",
                    type: "button"
                  });
                  oShowAttachmentDocLib.on("click", showAttachmentDocLib);
                }
              }
            }).bind(this)
          }
        });
      }).bind(this);

      // Let's try and find the noderef from the form
      YAHOO.Bubbling.on("CommentsControl", function() {
        var commentsControl = Alfresco.util.ComponentManager.findFirst("Alfresco.CommentsControls");
        if (commentsControl && commentsControl.options.itemNodeRef) {
          datalistNodeRef = commentsControl.options.itemNodeRef;
          createAttachmentButton();
        } else {
          Alfresco.util.Ajax.jsonGet({
            url: Alfresco.constants.PROXY_URI + "/api/lyse/noderef-from-workflowid/" + workflowId,
            successCallback: {
              fn: function (response){
                if (response.json !== undefined && response.json.nodeRef !== "") {
                  //Success, this datalist item has an attachment folder
                  datalistNodeRef = response.json.nodeRef;
                  createAttachmentButton();
                }
              },
              scope: this
            }
          });
        }
        // Unsubscribe from the event to prevent invocation of this method duplicate of times
        YAHOO.Bubbling.unsubscribe("CommentsControl");
      });
    }
  });
})();

