if (!Lyse) {
    var Lyse = {};
}

if (!Lyse.component) {
    Lyse.component = {};
}

(function () {
    /**
     * YUI Library aliases
     */
    var Dom = YAHOO.util.Dom,
        Event = YAHOO.util.Event;

    /**
     * Alfresco Slingshot aliases
     */
    var $html = Alfresco.util.encodeHTML;

    /**
     * NodeDetailsPopup constructor.
     *
     * @param {String} htmlId The HTML id of the control element
     * @return {Lyse.component.NodeDetailsPopup} The new NodeDetailsPopup instance
     * @constructor
     */
    Lyse.component.NodeDetailsPopup = function (htmlId) {
        Lyse.component.NodeDetailsPopup.superclass.constructor.call(this, "Lyse.component.NodeDetailsPopup", htmlId);

        return this;
    };

    YAHOO.extend(Lyse.component.NodeDetailsPopup, Alfresco.component.Base,
        {
            /**
             * Object container for initialization options
             *
             * @property options
             * @type object
             */
            options: {

                /**
                 * current nodeRef
                 */
                nodeRef: null,
                mode: "view",
                width: "1000px",
                attachmentFolder: null
            },

            setOptions: function NodeDetailsPopup_setOptions(obj)
            {
                this.options = YAHOO.lang.merge(this.options, obj);
                return this;
            },

            /**
             * Fired by YUI when parent element is available for scripting.
             * Component initialisation, including instantiation of YUI widgets and event listener binding.
             *
             * @method onReady
             */
            onReady: function NodeDetailsPopup_onReady() {
                Alfresco.util.createYUIButton(this, "viewDetailsButton", this.onButtonClick);
		        // Fetch & check doclib
			    Alfresco.util.Ajax.jsonGet( {
			      url: Alfresco.constants.PROXY_URI + "api/lyse/attachment-folder?nodeRef=" + this.options.nodeRef,
			      successCallback: {
			        fn: function (response) {
			          if (response.json !== undefined && response.json.nodeRef !== "") {
			            //Success, this datalist item has an attachment folder
			            this.options.attachmentFolder = response.json.nodeRef;
			          }
			        }, scope: this }
			    });
            },

            onButtonClick: function NodeDetailsPopup_onButtonClick(arg) {
            
                    // Depending on the type, popup the right page
		            Alfresco.util.Ajax.jsonGet(
		            {
		               url: Alfresco.constants.PROXY_URI + "api/metadata?nodeRef=" +this.options.nodeRef,
		               successCallback:
		               {
                     fn: function (response) {
                       if (response.json !== undefined) {
                         var nodeType = response.json.type;
                         var pageName;
                         if (nodeType == '{http://www.alfresco.org/model/content/1.0}content') {
                           pageName = '/document-details';
                         } else if (nodeType == '{http://www.alfresco.org/model/content/1.0}folder') {
                           pageName = '/folder-details';
                         } else {
                           pageName = '/lyse-datalist-details';
                         }
                         var urlContext = Alfresco.constants.URL_PAGECONTEXT;
                         if(Alfresco.constants.SITE) {
                           urlContext += 'site/' + Alfresco.constants.SITE + '/';
                         }

                         var workflowId = response.json.properties["{http://ns.lyse.no/model/datalist/1.0}workflowId"];
                         
                         var doclib = "";
                         if(this.options.attachmentFolder){
                         	doclib = "&docLib=" + this.options.attachmentFolder;
                         }
                         
                         
                         if(workflowId)
                         {
                           window.open(YAHOO.lang.substitute("{context}" + pageName + "?workflowId={workflowId}" + doclib,
                             {
                               context: urlContext,
                               workflowId: workflowId
                             }));
                         }
                         else
                         {
                           window.open(YAHOO.lang.substitute("{context}" + pageName + "?nodeRef={nodeRef}" + doclib,
                             {
                               context: urlContext,
                               nodeRef: this.options.nodeRef
                             }));
                         }

                       }

                     },
		                scope: this
		               }
		            });
            
                
            }
        });
})();
