
/**
 * VersionControl component.
 *
 * @namespace Alfresco
 * @class Alfresco.VersionControl
 */
(function()
{
   /**
    * YUI Library aliases
    */
   var Dom = YAHOO.util.Dom;

   /**
    * VersionControl constructor.
    *
    * @param {String} htmlId The HTML id of the control element
    * @return {Alfresco.VersionControl} The new VersionControl instance
    * @constructor
    */
   Alfresco.VersionControl = function(htmlId)
   {
      Alfresco.VersionControl.superclass.constructor.call(this, "Alfresco.VersionControl", htmlId);

      return this;
   };

   YAHOO.extend(Alfresco.VersionControl, Alfresco.component.Base,
   {
	   /**
	       * Object container for initialization options
	       *
	       * @property options
	       * @type object
	       */
	      options:
	      {

	         /**
	          * current nodeRef
	          *
	          * @property editorParameters
	          * @type object
	          */
	         nodeRef: null
	      },


      /**
       * Fired by YUI when parent element is available for scripting.
       * Component initialisation, including instantiation of YUI widgets and event listener binding.
       *
       * @method onReady
       */
      onReady: function VersionControl_onReady()
      {
    	   Alfresco.util.Ajax.jsonGet(
	         {
	            url: Alfresco.constants.URL_SERVICECONTEXT + "components/datalist/datalist-versions",
	            dataObj:
	            {
	        	   htmlid: this.id,
	               nodeRef: this.options.nodeRef
	            },
	            successCallback:
	            {
	               fn: this.onTemplateLoaded,
	               scope: this
	            },
	            execScripts: true
	         });

    	   var toggleVersionHistory = function () {
    	     var childNode = document.querySelectorAll('div[id$="prop_cm_versionLabel-cntrl"]')[0];
    	     var hideVersionButton = Dom.get("HideVersionHistory-button");
    	     var display = childNode.style.display;

    	     if(display == "none") {
    	       childNode.style.display = 'block';
    	       hideVersionButton.innerHTML = "Hide version history";
    	     } else {
    	       childNode.style.display = 'none';
    	       hideVersionButton.innerHTML = "Show version history";
    	     }
    	   };

    	   var exists = document.getElementById('HideVersionHistory');
    	   if(exists == null) {
    	     // Let's find our version label div
    	     var childNode = document.querySelectorAll('div[id$="prop_cm_versionLabel-cntrl"]')[0];
    	     if(childNode != null) {
    	       //Get the parent div
    	       var parentNode = childNode.parentNode;
    	       var element = document.createElement("SPAN");
    	       element.id="hide-version-history-container";
    	       parentNode.insertBefore(element, parentNode.children[1]);
    	       parentNode.children[2].style.display = 'none';
    	       var oToggleVersionHistoryButton = new YAHOO.widget.Button({
    	         id: "HideVersionHistory",
    	         type: "button",
    	         label: "Show version history",
    	         container: "hide-version-history-container"
    	       });
    	       oToggleVersionHistoryButton.on("click", toggleVersionHistory);
    	     }
    	   }
      },

      /**
       * Event callback when this component has been reloaded via AJAX call
       *
       * @method onTemplateLoaded
       * @param response {object} Server response from load template XHR request
       */
      onTemplateLoaded: function AmSD_onTemplateLoaded(response)
      {

    	 // Inject the template from the XHR request into a new DIV element
         var containerDiv = Dom.get(this.id);
         containerDiv.innerHTML = response.serverResponse.responseText;
      }
   });
})();