// @overridden projects/slingshot/source/web/components/invite/invitationlist.js
/**
 * No changes here yet, but #39008 will probably need this (been elaborating) so lets keep it for a while.
 * renderCellActions override
 * Change from original: Changed webscript to custom webscript.
 */

(function(__setupDataTable) {
   Alfresco.InvitationList.prototype._setupDataTable = function Lyse_InvitationList_setupDataTable() {
       /**
        * DataTable Cell Renderers
        *
        * Each cell has a custom renderer defined as a custom function. See YUI documentation for details.
        * These MUST be inline in order to have access to the Alfresco.InvitationList class (via the "me" variable).
        */
       var me = this;

       var $html = Alfresco.util.encodeHTML;



       /**
        * Description/detail custom datacell formatter
        *
        * @method renderCellDescription
        * @param elCell {object}
        * @param oRecord {object}
        * @param oColumn {object}
        * @param oData {object|string}
        */
       var renderCellDescription = function Lyse_InvitationList_renderCellDescription(elCell, oRecord, oColumn, oData)
       {
          // we currently render all results the same way
          var record = oRecord.getData(),
             name = YAHOO.lang.trim(record.firstName + " " + record.lastName),
             userName = "",
             email = record.email;

          if (record.userName !== undefined && record.userName.length > 0)
          {
              userName = "(" + record.userName + ")";
          }
          elCell.innerHTML = '<h3 class="itemname">' + $html(name) + ' <span class="lighter theme-color-1">' + $html(userName) + '</span></h3><div class="detail">' + $html(email) + '</div>';
       };

       /**
        * Role selector datacell formatter
        *
        * @method renderCellRole
        * @param elCell {object}
        * @param oRecord {object}
        * @param oColumn {object}
        * @param oData {object|string}
        */
       var renderCellRole = function Lyse_InvitationList_renderCellActions(elCell, oRecord, oColumn, oData)
       {
          Dom.setStyle(elCell.parentNode, "width", oColumn.width + "px");
          Dom.setStyle(elCell, "overflow", "visible");

          // cell where to add the element

          var cell = new YAHOO.util.Element(elCell),
             id = oRecord.getData('id'),
             buttonId = me.id + '-roleselector-' + id;

          // create a clone of the template
          var actionsColumnTemplate = Dom.get(me.id + '-role-column-template'),
             templateInstance = actionsColumnTemplate.cloneNode(true);
          templateInstance.setAttribute("id", "actionsDiv" + id);
          Dom.removeClass(templateInstance, "hidden");

          // define the role dropdown menu and the event listeners
          var rolesMenu = [], role;
          for (var i = 0, j = me.options.roles.length; i < j; i++)
          {
             role = me.options.roles[i];
             rolesMenu.push(
             {
                text: me.msg("role." + role),
                value: role,
                onclick:
                {
                   fn: me.onRoleSelect,
                   obj:
                   {
                      record: oRecord,
                      role: role
                   },
                   scope: me
                }
             });
          }

          // Insert the templateInstance to the column.
          cell.appendChild (templateInstance);

          // Create a yui button for the role selector.
          var fButton = Dom.getElementsByClassName("role-selector-button", "button", templateInstance);
          var button = new YAHOO.widget.Button(fButton[0],
          {
             type: "menu",
             name: buttonId,
             label: me.getRoleLabel(oRecord) + " " + Alfresco.constants.MENU_ARROW_SYMBOL,
             menu: rolesMenu
          });
          me.listWidgets[id] =
          {
             button: button
          };
       };

       /**
        * Remove user datacell formatter
        *
        * @method renderCellRemoveButton
        * @param elCell {object}
        * @param oRecord {object}
        * @param oColumn {object}
        * @param oData {object|string}
        */
       var renderCellRemoveButton = function Lyse_InvitationList_renderCellRemoveButton(elCell, oRecord, oColumn, oData)
       {
          var desc =
             '<span id="' + me.id + '-removeInvitee">' +
             '  <a href="#" class="remove-item-button"><span class="removeIcon">&nbsp;</span></a>' +
             '</span>';
          elCell.innerHTML = desc;
       };

       // DataTable column defintions
       var columnDefinitions =
       [
          { key: "user", label: "User", sortable: false, formatter: renderCellDescription },
          { key: "role", label: "Role", sortable: false, formatter: renderCellRole },
          { key: "remove", label: "Remove", sortable: false, formatter: renderCellRemoveButton }
       ];

       var msgEmpty = "";
       // Sets up the appropriate message for the right page.
       if (me.id.indexOf("_invite_") > 0)
       {
         msgEmpty = this.msg("invitationlist.empty-list");
       }
       else if (me.id.indexOf("_add-users_") > 0)
       {
         msgEmpty = this.msg("added-users-list.direct-add-instructions");
       }

       // DataTable definition
       this.widgets.dataTable = new YAHOO.widget.DataTable(this.id + "-inviteelist", columnDefinitions, this.widgets.dataSource,
       {
         MSG_EMPTY: msgEmpty
       });
   };

   Alfresco.InvitationList.prototype._finalizeInvites = function Lyse_InvitationList_finalizeInvites(inviteData) {
     /**
      * Called when all invites have been processed
      *
      * @method _finalizeInvites
      * @param resultData {Object}
      * @private
      */
        // remove the entries that were successful
        for (var i = inviteData.successes.length - 1; i >= 0; i--)
        {
           this.widgets.dataTable.deleteRow(inviteData.successes[i]);
        }

        // remove wait message
        this.widgets.feedbackMessage.destroy();

        // inform the user
        Alfresco.util.PopupManager.displayMessage(
        {
           text: this.msg("message.inviteresult", inviteData.successes.length, inviteData.failures.length)
        });
        if(inviteData.failures.length > 0 ){
          Alfresco.util.PopupManager.displayPrompt({
            title: "Oops, something went wrong!",
            text : inviteData.message,
            modal: true,
            close: true
          });
        }

        // update the invite button
        this._enableDisableInviteButton();
   }

   Alfresco.InvitationList.prototype._doInviteUser = function Lyse_InvitationList_doInviteUser(inviteData)
   {
      // fetch the record to process
      var record = inviteData.recs[inviteData.index];

      // success handler
      var success = function InvitationList__doInviteUser_success(response)
      {
         inviteData.successes.push(inviteData.index);
         inviteData.index++;
         this._processInviteData(inviteData);
      };

      var failure = function InvitationList__doInviteUser_failure(response)
      {
         inviteData.failures.push(inviteData.index);
         inviteData.index++;
         inviteData.message = response.json.message;
         this._processInviteData(inviteData);
      };

      // We have to do a backend call for each invited person
      var serverPath = window.location.protocol + "//" + window.location.host + Alfresco.constants.URL_CONTEXT;

      Alfresco.util.Ajax.request(
      {
         url: Alfresco.constants.PROXY_URI + "api/sites/" + this.options.siteId + "/invitations",
         method: "POST",
         requestContentType: "application/json",
         responseContentType: "application/json",
         dataObj:
         {
            invitationType: "NOMINATED",
            inviteeUserName: record.getData('userName') || "",
            inviteeRoleName: record.getData('role'),
            inviteeFirstName: record.getData('firstName'),
            inviteeLastName: record.getData('lastName'),
            inviteeEmail: record.getData('email'),
            serverPath: serverPath,
            acceptURL: 'page/accept-invite',
            rejectURL: 'page/reject-invite'
         },
         successCallback:
         {
            fn: success,
            scope: this
         },
         failureCallback:
         {
            fn: failure,
            scope: this
         }
      });
   }
}(Alfresco.InvitationList.prototype._setupDataTable));
