
/**
 * DatalistManager tool component.
 *
 * @namespace Alfresco
 * @class Alfresco.DatalistManager
 */
(function()
{
   /**
    * YUI Library aliases
    */
   var Dom = YAHOO.util.Dom;

    /**
     * Alfresco Slingshot aliases
     */
    var $html = Alfresco.util.encodeHTML;

   /**
    * DatalistManager constructor.
    *
    * @param {String} htmlId The HTML id of the parent element
    * @return {Alfresco.DatalistManager} The new DatalistManager instance
    * @constructor
    */
   Alfresco.DatalistManager = function(htmlId)
   {
	  this.id = htmlId;
	  this.buttons = [];
	  this.container = null;
	  this.admin = null;
      this.name = "Alfresco.DatalistManager";
      Alfresco.DatalistManager.superclass.constructor.call(this, "Alfresco.DatalistManager", htmlId, ["button", "container", "datasource", "datatable", "json"]);

      return this;
   };


   YAHOO.extend(Alfresco.DatalistManager, Alfresco.component.Base,
   {
      onReady: function DatalistManager_onReady() {
         this.widgets.newList = Alfresco.util.createYUIButton(this, "buttonNew", this.onNewDatalist,
         {
            disabled: false
         });
         // Reference to self - used in inline functions
         var me = this;
         var uriSearchResults = Alfresco.constants.PROXY_URI + "lyse/datalists/lists/site/"+Alfresco.constants.SITE;
         this.widgets.dataSource = new YAHOO.util.DataSource(uriSearchResults,
         {
            responseType: YAHOO.util.DataSource.TYPE_JSON,
            connXhrMode: "queueRequests",
            responseSchema:
            {
                resultsList: "items"
            }
         });
         this.widgets.dataSource.doBeforeParseData = function SiteFinder_doBeforeParseData(oRequest , oFullResponse)
         {
	  		me.admin = oFullResponse.admin;
	  		me.container = oFullResponse.container;
        	this.container = oFullResponse.container;
            var updatedResponse = oFullResponse;
            if (oFullResponse)
            {
               var items = [];
               for (var x = 0, y = oFullResponse.datalists.length; x < y; x++){
                 var type= oFullResponse.datalists[x].type;
                 if(type == "lysedl:interfaceAction" ||
                   type == "lysedl:issueList" ||
                   type == "lysedl:actionWithProject" ||
                   type == "lysedl:hseAction" ||
                   type == "lysedl:mccAction" ||
                   type == "lysedl:civilAction"){
                   continue;
                 } // Ignore}
                 items.push(oFullResponse.datalists[x]);
               }
               // we need to wrap the array inside a JSON object so the DataTable is happy
               updatedResponse =
               {
                  "items": items
               };
            }
            return updatedResponse;
         };

         this._setupDataTable();
         var successHandler = function SiteFinder__pS_successHandler(sRequest, oResponse, oPayload)
         {
            this.widgets.dataTable.onDataReturnInitializeTable.call(this.widgets.dataTable, sRequest, oResponse, oPayload);
         };

         var failureHandler = function SiteFinder__pS_failureHandler(sRequest, oResponse)
         {
            if (oResponse.status == 401){
               window.location.reload();
            }
            else{
               try{
                  var response = YAHOO.lang.JSON.parse(oResponse.responseText);
                  this.widgets.dataTable.set("MSG_ERROR", response.message);
                  this.widgets.dataTable.showTableMessage(Alfresco.util.encodeHTML(response.message), YAHOO.widget.DataTable.CLASS_ERROR);
               }
               catch(e){
                  this._setDefaultDataTableErrors(this.widgets.dataTable);
               }
            }
         };

         this.widgets.dataSource.sendRequest('', {
            success: successHandler,
            failure: failureHandler,
            scope: this
         },this.widgets.dataTable);
         this.isReady = true;
      },


      _hideDatalist: function DatalistManager_hideDatalist(event, item)
	  {
		 this.buttons[item.name+'-buttonShow'].button.set('disabled', false);
		 this.buttons[item.name+'-buttonHide'].button.set('disabled', true);

         Alfresco.util.Ajax.jsonPost({
             url: Alfresco.constants.PROXY_URI + "lyse/api/sitedatalistmanager/" +  new Alfresco.util.NodeRef(item.nodeRef).uri + "/hide",
             successCallback:{
                 fn: function refreshItem_refreshSuccess(response){
                 // Do nothing for now.
                 },
                 scope: this
             }
         });
	  },


	  _refreshDatatable: function DatalistManager_refreshDatatable(){
	  	// TODO: Refresh specific item istead?
      	location.reload();
	  },

    /**
     * YUI WIDGET EVENT HANDLERS
     * Handlers for standard events fired from YUI widgets, e.g. "click"
     */

    onNewDatalist: function DataLists_onNewDatalist(e, p_obj)
    {
         Alfresco.module.getCreateDatalistInstance(this.container).show();

    },

	  _editDatalist: function DatalistManager_editDatalist(event, item){
          var templateUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&mode={mode}&submitType={submitType}&showCancelButton=true",
          {
              itemKind: "node",
              itemId: item.nodeRef,
              mode: "edit",
              submitType: "json"
          });

          var doBeforeDialogShow = function DataGrid_onActionEdit_doBeforeDialogShow(p_form, p_dialog){
              Alfresco.util.populateHTML([ p_dialog.id + "-dialogTitle", this.msg("label.edit-row.title") ]);
              Alfresco.util.populateHTML([ p_dialog.id + "-dialogHeader", " " ]);
              Alfresco.util.populateHTML([ p_dialog.id + "-dialogHeader", " " ]);

              // #43930 temporarly disable edit title in dialog.
              Dom.setAttribute(p_dialog.id + "_prop_cm_title", "readonly", true);
              Dom.setAttribute(p_dialog.id + "_prop_cm_title", "style", "background-color:#eee; border:1px solid #ABADB3;");
          };

          var editDetails = new Alfresco.module.SimpleDialog(this.id + "-editDetails-" + Alfresco.util.generateDomId());
          editDetails.setOptions({
              width: "1000px",
              templateUrl: templateUrl,
              actionUrl: null,
              destroyOnHide: true,
              doBeforeDialogShow:
              {
                  fn: doBeforeDialogShow,
                  scope: this
              },
              onSuccess:
              {
                  fn: function DataGrid_onActionEdit_success(response){
	          		Alfresco.util.PopupManager.displayMessage({
	                	text: this.msg("message.datalist.update.success")
	                });
					this._refreshDatatable();
                  },
                  scope: this
              },
              onFailure:
              {
                  fn: function DataGrid_onActionEdit_failure(response){
                      Alfresco.util.PopupManager.displayPrompt({
                          text: this.msg(" message.datalist.update.failure")
                      });
                  },
                  scope: this
              }
          }).show();
	  },


      _showDatalist: function DatalistManager_showDatalist(event, item)
	  {
		 var URL = "";
		 if(item.nodeRef){
		 	URL = Alfresco.constants.PROXY_URI + "lyse/api/sitedatalistmanager/" +  new Alfresco.util.NodeRef(item.nodeRef).uri + "/show";
		 }else{
		 	URL = Alfresco.constants.PROXY_URI + "lyse/api/sitedatalistmanager/create/"+Alfresco.constants.SITE;
		 }
         Alfresco.util.Ajax.jsonPost({
             url: URL,
         	 dataObj: item,
             successCallback:{
                 fn: function refreshItem_refreshSuccess(response){
					 this.buttons[item.name+'-buttonShow'].button.set('disabled', true);
					 this.buttons[item.name+'-buttonHide'].button.set('disabled', false);
					 this.buttons[item.name+'-buttonShow'].button._setLabel("Show");
                	 if(!item.nodeRef){
						this._refreshDatatable();
		 			}
                 },
                 scope: this
             }
         });
	    },
      _setupDataTable: function DatalistManager_setupDataTable()
      {
         var me = this;
         renderCellDatalist = function DatalistManager_renderCellDatalist(elCell, oRecord, oColumn, oData){
           var record = oRecord.getData();
           elCell.innerHTML = '<span>' + record.title + '</span>';
         };
         renderCellDescription = function DatalistManager_renderCellDescription(elCell, oRecord, oColumn, oData){
           var record = oRecord.getData();
           elCell.innerHTML = '<span style="white-space: pre-wrap;"><p>'+record.description+'</p></span>';
         };
         renderCellAction = function DatalistManager_renderCellAction(elCell, oRecord, oColumn, oData){
              var record = oRecord.getData();
                elCell.innerHTML = '<span id="' + record.name + '-buttonHide"></span><span id="' + record.name + '-buttonShow"></span></span><span id="' + record.name + '-buttonEdit"></span>';

                // create the hide/show button
                var buttonHide = new YAHOO.widget.Button({
                  container: record.name + '-buttonHide',
                  label: "Hide",
                  onclick:
                    {
                      fn: me._hideDatalist,
                      obj: oRecord.getData(),
                      scope: me
                    }
                });
                var buttonShow = new YAHOO.widget.Button({
                  container: record.name + '-buttonShow',
                  label: "Show",
                  onclick:
                    {
                      fn: me._showDatalist,
                      obj: oRecord.getData(),
                      scope: me
                    }
                });
                var buttonEdit = new YAHOO.widget.Button({
                  container: record.name + '-buttonEdit',
                  label: "Edit",
                  onclick:
                    {
                      fn: me._editDatalist,
                      obj: oRecord.getData(),
                      scope: me
                    }
                });
                // button-disable-logic
                if (me.admin == "true") {
                  if (!record.nodeRef) {
                    buttonHide.set('disabled', true);
                    buttonEdit.set('disabled', true);
                    buttonShow._setLabel("Add");
                  } else if (record.hidden === "true") {
                    buttonHide.set('disabled', true);
                  } else {
                    buttonShow.set('disabled', true);
                  }
                } else {
                  buttonShow.set('disabled', true);
                  buttonHide.set('disabled', true);
                  buttonEdit.set('disabled', true);
                }

                if (record.querylist === "true") {
                  buttonEdit.addClass('hidden');
                  buttonEdit.set('disabled', true);
                }

                // store the buttons
                me.buttons[record.name + "-buttonShow"] = {
                  button: buttonShow
                };
                me.buttons[record.name + "-buttonHide"] = {
                  button: buttonHide
                };
            };

             var columnDefinitions =
             [
                {
                   key: "shortName", label: "Datalist", sortable: false, formatter: renderCellDatalist, width: 200
                },
                {
                   key: "description", label: "Description", sortable: false, formatter: renderCellDescription
                },
                {
                   key: "button", label: "Actions", sortable: false, formatter: renderCellAction, width: 175
                }
             ];

             // DataTable definition
             this.widgets.dataTable = new YAHOO.widget.DataTable(this.id + "-datalist-manager", columnDefinitions, this.widgets.dataSource,
             {
                renderLoopSize: 32,
                initialLoad: false,
                MSG_EMPTY: this.msg("message.instructions")
             });


             // Override abstract function within DataTable to set custom error message
             this.widgets.dataTable.doBeforeLoadData = function SiteFinder_doBeforeLoadData(sRequest, oResponse, oPayload)
             {
                if (oResponse.error)
                {
                   try
                   {
                      var response = YAHOO.lang.JSON.parse(oResponse.responseText);
                      this.widgets.dataTable.set("MSG_ERROR", response.message);
                   }
                   catch(e)
                   {
                      me._setDefaultDataTableErrors(me.widgets.dataTable);
                   }
                }
                else if (oResponse.results)
                {
                   if (oResponse.results.length === 0)
                   {
                      me.widgets.dataTable.set("MSG_EMPTY", '<span style="white-space: nowrap;">' + me.msg("message.empty") + '</span>');
                   }
                   me.renderLoopSize = Alfresco.util.RENDERLOOPSIZE;
                }
                // Must return true to have the "Searching..." message replaced by the error message
                return true;
             };
      }
   });
})();
