// @overridden projects/web-framework-commons/source/web/components/object-finder/object-finder.js
/**
 * RPLP root namespace.
 *
 * @namespace RPLP
 */
if (typeof RPLP == "undefined" || !RPLP) {
	var RPLP = {};
}

(function(onReady) {
    Alfresco.ObjectFinder.prototype.onReady = function lyse_onReady() {
			         this._createSelectedItemsControls();
               if (!this.options.disabled)
               {
                  // Control is NOT in view mode
                  if (this.options.compactMode)
                  {
                     Dom.addClass(this.pickerId, "compact");
                  }

                  this._createNavigationControls();
                  var itemGroupActionsContainerEl = Dom.get(this.id + "-itemGroupActions");
                  if (itemGroupActionsContainerEl)
                  {
                     // Create an "Add/Select" button that will display a picker to add items
                     if (this.options.allowSelectAction)
                     {
                        var addButtonEl = document.createElement("button");
                        itemGroupActionsContainerEl.appendChild(addButtonEl);

                        var addButtonLabel = this.options.selectActionLabel;
                        if(this.pickerId.indexOf("site-groups")!==-1) {
                          // Custom label on select button - https://redmine.redpill-linpro.com/issues/43654
                          addButtonLabel = "Edit";
                        }
                        if (this.options.selectActionLabelId && this.options.selectActionLabelId.length !== "")
                        {
                           addButtonLabel = this.msg(this.options.selectActionLabelId);
                        }
                        this.widgets.addButton = Alfresco.util.createYUIButton(this, null, this.onAddButtonClick,
                        {
                           label: addButtonLabel,
                           disabled: true
                        }, addButtonEl);
                     }
                     // Create a "Remove all" button to remove all items (if component is in "list" mode)
                     if (this.options.allowRemoveAllAction && this.options.displayMode == "list")
                     {
                        var removeAllButtonEl = document.createElement("button");
                        itemGroupActionsContainerEl.appendChild(removeAllButtonEl);
                        this.widgets.removeAllButton = Alfresco.util.createYUIButton(this, null, this.onRemoveAllButtonClick,
                        {
                           label: this.msg("button.removeAll"),
                           disabled: true
                        }, removeAllButtonEl);
                     }
                  }
                  if (this.options.allowRemoveAction && this.options.displayMode == "list")
                  {
                     this.options.listItemActions.push(
                     {
                        name: "remove-list-item",
                        event: "removeListItem",
                        label: "form.control.object-picker.remove-item"
                     });
                  }
                  this.widgets.ok = Alfresco.util.createYUIButton(this, "ok", this.onOK, {additionalClass:"alf-primary-button"});
                  this.widgets.cancel = Alfresco.util.createYUIButton(this, "cancel", this.onCancel);

                  // force the generated buttons to have a name of "-" so it gets ignored in
                  // JSON submit. TODO: remove this when JSON submit behaviour is configurable
                  Dom.get(this.id + "-ok-button").name = "-";
                  Dom.get(this.id + "-cancel-button").name = "-";

                  this.widgets.dialog = Alfresco.util.createYUIPanel(this.pickerId,
                  {
                     width: "60em"
                  });
                  this.widgets.dialog.hideEvent.subscribe(this.onCancel, null, this);
                  Dom.addClass(this.pickerId, "object-finder");
               }
               if(window.RPLP != undefined && window.RPLP.global_alf_members != undefined) {
                 this.options.currentValue = window.RPLP.global_alf_members;
                 this.options.selectCurrentValues = true;
               }

               this._loadSelectedItems();

    };
}(Alfresco.ObjectFinder.prototype.onReady));

(function(getAddedItems) {
    Alfresco.ObjectFinder.prototype.getAddedItems = function lyse_getAddedItems() {
         var addedItems = [],
            currentItems = Alfresco.util.arrayToObject(this.options.currentValue.split(","));

         for (var item in this.selectedItems)
         {
            if (this.selectedItems.hasOwnProperty(item))
            {
               if (!(item in currentItems) || this.options.selectCurrentValues)
               {
                  addedItems.push(item);
               }
            }
         }
         return addedItems;


    };
}(Alfresco.ObjectFinder.prototype.getAddedItems));


(function(onRenderCurrentValue) {

   var $html = Alfresco.util.encodeHTML,
      $hasEventInterest = Alfresco.util.hasEventInterest;

    Alfresco.ObjectFinder.prototype.onRenderCurrentValue = function lyse_onRenderCurrentValue(layer, args) {
         // Check the event is directed towards this instance
         if ($hasEventInterest(this, args))
         {
            this._adjustCurrentValues();

            var items = this.selectedItems,
               displayValue = "";

            if (items === null)
            {
               displayValue = "<span class=\"error\">" + this.msg("form.control.object-picker.current.failure") + "</span>";
            }
            else
            {
               var item, link;
               if (this.options.displayMode == "list")
               {
                  var l = this.widgets.currentValuesDataTable.getRecordSet().getLength();
                  if (l > 0)
                  {
                     this.widgets.currentValuesDataTable.deleteRows(0, l);
                  }
               }

               for (var key in items)
               {
                  if (items.hasOwnProperty(key))
                  {
                     item = items[key];

                     // Special case for tags, which we want to render differently to categories
                     if (item.type == "cm:category" && item.displayPath.indexOf("/categories/Tags") !== -1)
                     {
                        item.type = "tag";
                     }

                     if (this.options.showLinkToTarget && this.options.targetLinkTemplate !== null)
                     {
                        if (this.options.displayMode == "items")
                        {
                           link = null;
                           if (YAHOO.lang.isFunction(this.options.targetLinkTemplate))
                           {
                              link = this.options.targetLinkTemplate.call(this, item);
                           }
                           else
                           {
                          	 //Discard template, build link from scratch
                        	 if(item.type.startsWith("lysedl:")) {
                        	   var linkTemplate = (item.site) ? Alfresco.constants.URL_PAGECONTEXT + "site/{site}/lyse-datalist-details?nodeRef={nodeRef}" : Alfresco.constants.URL_PAGECONTEXT + "document-details?nodeRef={nodeRef}";
                        	 }else{
                        	   var linkTemplate = (item.site) ? Alfresco.constants.URL_PAGECONTEXT + "site/{site}/document-details?nodeRef={nodeRef}" : Alfresco.constants.URL_PAGECONTEXT + "document-details?nodeRef={nodeRef}";
                        	 }
                             link = YAHOO.lang.substitute(linkTemplate,
                             {
                               nodeRef : item.nodeRef,
                               site : item.site
                             });
                           }
                           if (document.location.href.indexOf("/data-lists") > 0) {
                              // Open the file in a new window if we are in a data list view, we are probably in a lightbox then which would be lost when navigating back here
                              displayValue += this.options.objectRenderer.renderItem(item, 16,
                                  "<div>{icon} <a target='_blank' href='" + link + "'>{name}</a></div>");
                           }
                           else {
                              displayValue += this.options.objectRenderer.renderItem(item, 16,
                                  "<div>{icon} <a href='" + link + "' id='" + $html(item.nodeRef) + "'>{name}</a></div>");
                           }
                        }
                        else if (this.options.displayMode == "list")
                        {
                           this.widgets.currentValuesDataTable.addRow(item);
                        }
                     }
                     else
                     {
                        if (this.options.displayMode == "items")
                        {
                           if (item.type === "tag")
                           {
                              displayValue += this.options.objectRenderer.renderItem(item, null, "<div class='itemtype-tag'>{name}</div>");
                           }
                           else
                           {
                              displayValue += this.options.objectRenderer.renderItem(item, 16, "<div class='itemtype-" + $html(item.type) + "' style='word-wrap: break-word;'>{icon} {name}</div>");
                           }
                        }
                        else if (this.options.displayMode == "list")
                        {
                           this.widgets.currentValuesDataTable.addRow(item);
                        }
                     }
                  }
               }
               if (this.options.displayMode == "items")
               {
                  Dom.get(this.id + "-currentValueDisplay").innerHTML = displayValue;
               }
            }
            this._enableActions();
         }
       };
}(Alfresco.ObjectFinder.prototype.onRenderCurrentValue));



(function(onParentDetails) {
   var $html = Alfresco.util.encodeHTML,
      $hasEventInterest = Alfresco.util.hasEventInterest;

    Alfresco.ObjectFinder.prototype.onParentDetails = function lyse_onParentDetails(layer, args) {

         // Check the event is directed towards this instance
         if ($hasEventInterest(this, args))
         {
            var obj = args[1];
            if (obj && obj.parent && this.widgets.navigationMenu)
            {
               var arrItems = [],
                  item = obj.parent,
                  navButton = this.widgets.navigationMenu,
                  navMenu = navButton.getMenu(),
                  navGroup = navMenu.getItemGroups()[0],
                  indent = "";

               // Create array, deepest node first in final array
               while (item)
               {
                  arrItems = [item].concat(arrItems);
                  item = item.parent;
               }

               var i, ii;
               for (i = 0, ii = navGroup.length; i < ii; i++)
               {
                  navMenu.removeItem(0, 0, true);
               }

               item = arrItems[arrItems.length - 1];
               var value = item.type == "st:site" ? "{title}" : "{name}";
               navButton.set("label", this.options.objectRenderer.renderItem(item, 16, '<div><span class="item-icon">{icon}</span><span class="item-name">' + value + '</span></div>'));

               // Navigation Up button
               if (arrItems.length > 1)
               {
                  this.widgets.folderUp.set("value", arrItems[arrItems.length - 2]);
                  this.widgets.folderUp.set("disabled", false);
               }
               else
               {
                  this.widgets.folderUp.set("disabled", true);
               }

               var menuItem;
               for (i = 0, ii = arrItems.length; i < ii; i++)
               {
                  item = arrItems[i];

                  var value = item.type == "st:site" ? "{title}" : "{name}";
                  menuItem = new YAHOO.widget.MenuItem(this.options.objectRenderer.renderItem(item, 16, indent + '<span class="item-icon">{icon}</span><span class="item-name">' + value + '</span>'),
                  {
                     value: item.nodeRef
                  });
                  menuItem.cfg.addProperty("label",
                  {
                     value: item.name
                  });
                  navMenu.addItem(menuItem, 0);
                  indent += "&nbsp;&nbsp;&nbsp;";
               }

               navMenu.render();
            }
         }


    };
}(Alfresco.ObjectFinder.prototype.onParentDetails));


(function(fnRenderCellListItemName) {
   var $html = Alfresco.util.encodeHTML;

    Alfresco.ObjectFinder.prototype.fnRenderCellListItemName = function lyse_fnRenderCellListItemName() {

         var scope = this;

         /**
          * Action item custom datacell formatter
          *
          * @method fnRenderCellListItemName
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return function ObjectFinder_fnRenderCellListItemName(elCell, oRecord, oColumn, oData)
         {
            var item = oRecord.getData(),
               modifiedBy =  item.modifier ? $html(item.modifier) : scope.msg("label.none"),
               modifiedOn = item.modified ? Alfresco.util.formatDate(Alfresco.util.fromISO8601(item.modified)) : null,
               itemName = item.displayName?item.displayName:item.name,
               title = $html(itemName.indexOf("fileholder.txt") >= 0 ? "Upload a document" : itemName);
            if (title.indexOf("Upload a document") === -1 && scope.options.showLinkToTarget && scope.options.targetLinkTemplate !== null)
            {
               var link;
               if (YAHOO.lang.isFunction(scope.options.targetLinkTemplate)){
                  link = scope.options.targetLinkTemplate.call(scope, oRecord.getData());
               }else{
                  //Discard template, build link from scratch
                  var linkTemplate = (item.site) ? Alfresco.constants.URL_PAGECONTEXT + "site/{site}/document-details?nodeRef={nodeRef}" : Alfresco.constants.URL_PAGECONTEXT + "document-details?nodeRef={nodeRef}";
                  link = YAHOO.lang.substitute(linkTemplate,{
                     nodeRef : item.nodeRef,
                     site : item.site
                  });
               }

               if (document.location.href.indexOf("/data-lists") > 0) {
                  // Open the file in a new window if we are in a data list view, we are probably in a lightbox then which would be lost when navigating back here
                  title = '<a target="_blank" href="' + link + '">' + title + '</a>';
               }
               else {
                  title = '<a href="' + link + '">' + title + '</a>';
               }
            }

            // Fetch nodeRef properties, replace html with new info about modifier & modified date.
            if(item.nodeRef != null){
              var lastUpdatedUser = Alfresco.util.generateDomId(null, "last-updated-user");
              var template = '<h3 class="name">' + title + '</h3>';
              template += '<div class="description" id='+lastUpdatedUser+'>' + scope.msg("form.label.modifiedby") + ': ' + modifiedBy + '</div>';
              template += '<div class="viewmode-label" id="last-updated-date">' + scope.msg("form.control.object-picker.modified-on") + ': ' + (modifiedOn ? modifiedOn : scope.msg("label.none")) + '</div>';
              if(this.configs.element.id.contains('assoc_lysewf_civilVorContrasignedDocument')){
                template += '<input type="hidden" id="civilVorContrasignedDocumentModifiedOn" value='+ (item.modified ? item.modified : scope.msg("label.none")) + '>';
              }
              elCell.innerHTML = template;
              var url = Alfresco.constants.PROXY_URI + "slingshot/doclib/node/" + item.nodeRef.replace("://", "/");
	          Alfresco.util.Ajax.request({
	            method : Alfresco.util.Ajax.GET,
	            url : url,
	            responseContentType : Alfresco.util.Ajax.JSON,
	            successCallback : {
	              fn : function(response) {
	                if (response && response.json && response.json.item) {
                    var lastUpdater = response.json.item["modifiedBy"];
                    if(lastUpdater != null) {
                      var userInfo = scope.msg("form.label.modifiedby") + ": " + lastUpdater;
                      Alfresco.util.populateHTML([lastUpdatedUser, $html(userInfo) ]);
                    }
	                }
	              },
	              scope : this,
                updateUser : lastUpdatedUser
	            }
	          });
            }
         }
    };
}(Alfresco.ObjectFinder.prototype.fnRenderCellListItemName));

(function(fnRenderCellListItemActions) {
    Alfresco.ObjectFinder.prototype.fnRenderCellListItemActions = function lyse_fnRenderCellListItemActions() {


         var scope = this;

         /**
          * Action item custom datacell formatter
          *
          * @method fnRenderCellListItemActions
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return function ObjectFinder_fnRenderCellListItemActions(elCell, oRecord, oColumn, oData)
         {
            if (oColumn.width)
            {
               Dom.setStyle(elCell, "width", oColumn.width + (YAHOO.lang.isNumber(oColumn.width) ? "px" : ""));
               Dom.setStyle(elCell.parentNode, "width", oColumn.width + (YAHOO.lang.isNumber(oColumn.width) ? "px" : ""));
            }

            // While waiting for the package item actions, only render the actions (remove) in non editable mode
            if (scope.options.disabled === false)
            {
               var links = "", link, listAction;
               for (var i = 0, il = scope.options.listItemActions.length; i < il; i++)
               {
                  listAction = scope.options.listItemActions[i];
                  if (listAction.event)
                  {
                    if(listAction.event == "uploadNewListItemRevision") {
                      Alfresco.util.Ajax.jsonGet({
                        url: Alfresco.constants.PROXY_URI + "lyse/api/permissions/" + Alfresco.util.NodeRef(oRecord.getData("nodeRef")).uri,
                        responseContentType : "application/json",
                        successCallback: {
                            fn: function (response) {
                              var permissions = response.json;
                              if(permissions.edit || permissions.unlockItem) {
                                links += '<div class="list-action"><a href="#" class="' + listAction.name + ' ' + ' list-action-event-' + scope.eventGroup + ' ' + listAction.event+ '" title="' + scope.msg(listAction.label) + '" tabindex="0">' + scope.msg(listAction.label) + '</a></div>';
                                elCell.innerHTML = links;
                              }
                            },
                            scope: this
                        }
                      });
                    } else {
                      links += '<div class="list-action"><a href="#" class="' + listAction.name + ' ' + ' list-action-event-' + scope.eventGroup + ' ' + listAction.event+ '" title="' + scope.msg(listAction.label) + '" tabindex="0">' + scope.msg(listAction.label) + '</a></div>';
                    }
                  }
                  else
                  {
                     link = null;
                     if (YAHOO.lang.isFunction(listAction.link))
                     {
                        link = listAction.link.call(this, oRecord.getData());
                     }
                     else if (YAHOO.lang.isString(listAction.link))
                     {
                        link = YAHOO.lang.substitute(listAction.link, oRecord.getData());
                     }
                     links += '<div class="list-action"><a href="' + link + '" class="' + listAction.name + '" title="' + scope.msg(listAction.label) + '" tabindex="0">' + scope.msg(listAction.label) + '</a></div>';
                  }
               }
               elCell.innerHTML = links;
            }
         }


    };
}(Alfresco.ObjectFinder.prototype.fnRenderCellListItemActions));

(function(_createNavigationControls) {
    Alfresco.ObjectFinder.prototype._createNavigationControls = function lyse__createNavigationControls() {

          var me = this;

          if (this._inAuthorityMode())
          {
             // only show the search box for authority mode
             Dom.setStyle(this.pickerId + "-folderUpContainer", "display", "none");
             Dom.setStyle(this.pickerId + "-navigatorContainer", "display", "none");
             Dom.setStyle(this.pickerId + "-searchContainer", "display", "block");

             // setup search widgets
             this.widgets.searchButton = new YAHOO.widget.Button(this.pickerId + "-searchButton");
             this.widgets.searchButton.on("click", this.onSearch, this.widgets.searchButton, this);

             // force the generated buttons to have a name of "-" so it gets ignored in
             // JSON submit. TODO: remove this when JSON submit behaviour is configurable
             Dom.get(this.pickerId + "-searchButton").name = "-";

             // register the "enter" event on the search text field
             var zinput = Dom.get(this.pickerId + "-searchText");
             new YAHOO.util.KeyListener(zinput,
             {
                keys: 13
             },
             {
                fn: me.onSearch,
                scope: this,
                correctScope: true
             }, "keydown").enable();
          }
          else
          {
         	if(this._inDatalistMode())
         	{
         	  // setup search widgets
               Dom.setStyle(this.pickerId + "-searchContainer", "display", "block");
               this.widgets.searchButton = new YAHOO.widget.Button(this.pickerId + "-searchButton");
               this.widgets.searchButton.on("click", this.onSearch, this.widgets.searchButton, this);
               Dom.get(this.pickerId + "-searchButton").name = "-";
               // register the "enter" event on the search text field
               var zxinput = Dom.get(this.pickerId + "-searchText");
               new YAHOO.util.KeyListener(zxinput,
               {
                  keys: 13
               },
               {
                  fn: me.onSearch,
                  scope: this,
                  correctScope: true
               }, "keydown").enable();
         	}
             // Up Navigation button
             this.widgets.folderUp = new YAHOO.widget.Button(this.pickerId + "-folderUp",
             {
                disabled: true
             });
             this.widgets.folderUp.on("click", this.onFolderUp, this.widgets.folderUp, this);

             // Navigation drop-down menu
             this.widgets.navigationMenu = new YAHOO.widget.Button(this.pickerId + "-navigator",
             {
                type: "menu",
                menu: this.pickerId + "-navigatorMenu",
                lazyloadmenu: false
             });

             // force the generated buttons to have a name of "-" so it gets ignored in
             // JSON submit. TODO: remove this when JSON submit behaviour is configurable
             Dom.get(this.pickerId + "-folderUp-button").name = "-";
             Dom.get(this.pickerId + "-navigator-button").name = "-";

             this.widgets.navigationMenu.getMenu().subscribe("click", function (p_sType, p_aArgs)
             {
                var menuItem = p_aArgs[1];
                if (menuItem)
                {
                   YAHOO.Bubbling.fire("parentChanged",
                   {
                      eventGroup: me,
                      label: menuItem.cfg.getProperty("label"),
                      nodeRef: menuItem.value
                   });
                }
             });

             // Optional "Create New" UI controls
             if (Dom.get(this.pickerId + "-createNew"))
             {
                // Create New - OK button
                this.widgets.createNewOK = new YAHOO.widget.Button(this.pickerId + "-createNewOK",
                {
                   disabled: true
                });
                this.widgets.createNewOK.on("click", this.onCreateNewOK, this.widgets.createNewOK, this);

                // Create New - Cancel button
                this.widgets.createNewCancel = new YAHOO.widget.Button(this.pickerId + "-createNewCancel",
                {
                   disabled: true
                });
                this.widgets.createNewCancel.on("click", this.onCreateNewCancel, this.widgets.createNewCancel, this);
             }
          }
    };
}(Alfresco.ObjectFinder.prototype._createNavigationControls));


(function() {
    Alfresco.ObjectFinder.prototype._inDatalistMode = function lyse__inDatalistMode() {
         return (this.options.itemFamily == "datalist");
    };
}());


(function(_generatePickerChildrenUrlParams) {
    Alfresco.ObjectRenderer.prototype._generatePickerChildrenUrlParams = function lyse__generatePickerChildrenUrlParams(searchTerm) {
         var params = "?selectableType=" + this.options.itemType + "&searchTerm=" + encodeURIComponent(searchTerm) +
                      "&excludeConsumers=" + this.options.excludeConsumers + "&includeBaseGroups=" + this.options.includeBaseGroups + "&excludeContractorGroups=" + this.options.excludeContractorGroups +
                      "&excludeContainerGroups=" + this.options.excludeContainerGroups +"&relatedSiteTypeToFetchFrom=" + this.options.relatedSiteTypeToFetchFrom +
                      "&addNoneOption=" + this.options.addNoneOption  + "&includeAnonymous=" + this.options.includeAnonymous + "&includeOnlyCommGroups=" + this.options.includeOnlyCommGroups;
         if(this.options.siteShortName && this.options.siteShortName!=="")
         {
             params += "&siteShortName=" + this.options.siteShortName;
         }
         if (this.options.createNewItemUri !== "")
         {
            params += "&size=" + this.options.maxSearchResults;
         }
         else
         {
            params += "&size=" + this.options.maxChildResults;
         }


         // if an XPath start location has been provided and it has not been resolved
         // yet, pass it to the pickerchildren script as a parameter
         if (!this.startLocationResolved && this.objectFinder.options.startLocation &&
              this.objectFinder.options.startLocation.charAt(0) == "/")
         {
            params += "&xpath=" + encodeURIComponent(this.objectFinder.options.startLocation);
         }

         // has a rootNode been specified?
         if (this.objectFinder.options.rootNode)
         {
            var rootNode = null;

            if (this.objectFinder.options.rootNode.charAt(0) == "{")
            {
               if (this.objectFinder.options.rootNode == "{companyhome}")
               {
                  rootNode = "alfresco://company/home";
               }
               else if (this.objectFinder.options.rootNode == "{userhome}")
               {
                  rootNode = "alfresco://user/home";
               }
               else if (this.objectFinder.options.rootNode == "{siteshome}")
               {
                  rootNode = "alfresco://sites/home";
               }
               else if (this.options.rootNode == "{shared}")
               {
                  rootNode = "alfresco://company/shared";
               }
            }
            else
            {
               // rootNode is either an xPath expression or a nodeRef
               rootNode = this.objectFinder.options.rootNode;
            }
            if (rootNode !== null)
            {
               params += "&rootNode=" + encodeURIComponent(rootNode);
            }
         }

         //set the selectableMimeType parameter if it's configured
         if(this.objectFinder.options.selectableMimeType)
         {
            params += "&selectableMimeType=" + encodeURI(this.objectFinder.options.selectableMimeType);
         }

         if (this.objectFinder.options.params)
         {
            params += "&" + encodeURI(this.objectFinder.options.params);
         }

         return params;

    };
}(Alfresco.ObjectRenderer.prototype._generatePickerChildrenUrlParams));


