var Lyse = Lyse || {};
Lyse.component = Lyse.component || {};

(function () {
    /**
     * YUI Library aliases
     */
    var Dom = YAHOO.util.Dom,
        Event = YAHOO.util.Event;

    /**
     * Alfresco Slingshot aliases
     */
    var $html = Alfresco.util.encodeHTML;

    /**
     * Constructor.
     *
     * @param {String} htmlId The HTML id of the parent element
     * @return {Lyse.component.ShowEditors} The new component instance
     * @constructor
     */
    Lyse.component.ShowEditors = function ShowEditors_constructor(htmlId) {
        Lyse.component.ShowEditors.superclass.constructor.call(this, "Lyse.component.ShowEditors", htmlId, ["container", "datasource", "datatable"]);
    };

    /**
     * Extend Alfresco.component.Base
     */
    YAHOO.extend(Lyse.component.ShowEditors, Alfresco.component.Base,
        {
            /**
             * Fired by YUI when parent element is available for scripting.
             * Component initialisation, including instantiation of YUI widgets and event listener binding.
             *
             * @method onReady
             */
            onReady: function ShowEditors_onReady() {
                this.createDataTable();
                if (Dom.get(this.id + "-close-button") !== null)
                {
                    Alfresco.util.createYUIButton(this, "close-button", null,
                        {
                            type: "submit"
                        });
                    Dom.get(this.id + "-close-button").name = "-";
                }

            },

            /**
             * Builds the url for the data table
             *
             * @method buildUrl
             * @retrun {string} The url for the data table
             */
            buildUrl: function ShowEditors_buildUrl() {
                var url = Alfresco.constants.PROXY_URI + "lyse/api/write-permissions?nodeRef={nodeRef}";

                return YAHOO.lang.substitute(url,
                    {
                        nodeRef: encodeURIComponent(this.options.nodeRef)
                    });
            },

            /**
             * Creates the data table
             *
             * @method createDataTable
             */
            createDataTable: function ShowEditors_createDataTable() {

                this.widgets.alfrescoDataTable = new Alfresco.util.DataTable(
                    {
                        dataSource: {
                            url: this.buildUrl(),
                            config: {
                            	responseType: YAHOO.util.DataSource.TYPE_JSON,
                                responseSchema: {
                                    resultsList: 'users'
                                }
                            }
                        },
                        dataTable: {
                            container: this.id + "-show-editors-table",
                            columnDefinitions: [
                                {key: "user", label: this.msg("column.user"), formatter: this.bind(this.renderCellUser), width: 450}
                            ],
                            config: {
                                MSG_EMPTY: this.msg("no.result"),
                                className: "" // default = alfresco-datatable, which hides the column headings
                            }
                        }
                    });
                
            },


            renderCellUser: function ShowEditors_renderCellUser(elCell, oRecord, oColumn, oData) {
                elCell.innerHTML = $html(oRecord.getData("user"));
            }

        });
})();
