(function()
{
   /**
    * YUI Library aliases
    */
   var Dom = YAHOO.util.Dom,
       Event = YAHOO.util.Event,
       Selector = YAHOO.util.Selector,
       KeyListener = YAHOO.util.KeyListener,
       Bubbling = YAHOO.Bubbling;

   /**
    * Alfresco Slingshot aliases
    */
   var $html = Alfresco.util.encodeHTML,
      $links = Alfresco.util.activateLinks,
      $combine = Alfresco.util.combinePaths,
      $userProfile = Alfresco.util.userProfileLink;


   Alfresco.component.ExtDataGrid = function(htmlId)
   {
      Alfresco.component.ExtDataGrid.superclass.constructor.call(this, htmlId);
      YAHOO.Bubbling.on("afterFormRuntimeInit", this.onAfterFormRuntimeInit, this);
      YAHOO.Bubbling.on("setSiteTitle", this.onSetSiteTitle, this);
      //YAHOO.Bubbling.on("beforeFormRuntimeInit", this.onBeforeFormRuntimeInit, this);
      this.totalRecordsUpper = null;
      return this;
   };

   /**
    * Extend Alfresco.component.DataGrid
    */
   YAHOO.extend(Alfresco.component.ExtDataGrid, Alfresco.component.DataGrid);

   /**
    * Augment prototype with Common Actions module
    */
   YAHOO.lang.augmentProto(Alfresco.component.ExtDataGrid, Alfresco.service.DataListActions);

   /**
    * Augment prototype with main class implementation, ensuring overwrite is enabled
    */
   YAHOO.lang.augmentObject(Alfresco.component.ExtDataGrid.prototype,
   {


         /**
       * Returns workflow tasks custom datacell formatter
       *
       * @method fnRenderCellWorkflowTasks
       */
      fnRenderCellWorkflowTasks: function DataGrid_fnRenderCellWorkflowTasks()
      {
         var scope = this;

         /**
          * Workflow tasks custom datacell formatter
          *
          * @method renderCellWorkflowTasks
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return function DataGrid_renderCellWorkflowTasks(elCell, oRecord, oColumn, oData)
         {
            var tasksHtml = '';
            if (oRecord.getData("workflowTasks") != null) {
               for (i = 0; i < oRecord.getData("workflowTasks").length; i++) {
                  var taskData = oRecord.getData("workflowTasks")[i];

                  var img = Alfresco.constants.URL_RESCONTEXT + 'components/images/' + (taskData.isPooled === 'true' ? '/people.png' : 'users.png');
                  if(taskData.name == "lysewf:shareForInformation" || taskData.name == "lysewf:shareForCommenting" || taskData.name == "lysewf:shareInformOriginator"){
                  	img = Alfresco.constants.URL_RESCONTEXT + 'components/images/header/' + (taskData.isPooled === 'true' ? 'people.png' : 'users.png');
                  }
                  var urlContext = Alfresco.constants.URL_PAGECONTEXT + 'site/' + Alfresco.constants.SITE;
                  tasksHtml += YAHOO.lang.substitute('<a href="{taskUrl}" target="_blank"><img src="{imgUrl}" title="{imgTitle}"/></a>', {
                     taskUrl: urlContext + '/lyse-task-edit?taskId=' + $html(taskData.id) + '&workflowId='+ $html(taskData.workflowId),
                     imgUrl: img,
                     imgTitle: $html(taskData.title)
                  });

               }
            }
            elCell.innerHTML = '<div id="' + scope.id + '-actions-' + oRecord.getId() + '">' + tasksHtml + '</div>';
         };
         /**
          * Formatter for getting site title from shortname
          *
          * @method fnRenderCellWorkflowTasks
          */
         fnRenderCellSiteInfo: function DataGrid_fnRenderCellSiteInfo()
         {
            var scope = this;

            /**
             * Shortname custom datacell formatter
             *
             * @method renderCellSiteInfo
             * @param elCell {object}
             * @param oRecord {object}
             * @param oColumn {object}
             * @param oData {object|string}
             */
            return function DataGrid_renderCellSiteInfo(elCell, oRecord, oColumn, oData)
            {

               elCell.innerHTML = 'Hello goodbye';
            };
         }
      },

      /**
       * Returns responsible (workflow tasks) custom datacell formatter
       *
       * @method fnRenderCellResponsiblesForTasks
       */
      fnRenderCellResponsiblesForTasks: function DataGrid_fnRenderCellResponsiblesForTasks()
      {
         var scope = this;

         /**
          * Responsible for tasks custom datacell formatter
          *
          * @method renderCellResponsiblesForTasks
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return function DataGrid_renderCellResponsiblesForTasks(elCell, oRecord, oColumn, oData) {
            var responsiblesHtml = getTaskResponsibleFromRecord(oRecord);
            elCell.innerHTML = '<div id="' + scope.id + '-actions-' + oRecord.getId() + '">' + responsiblesHtml + '</div>';
            if (!window.ec){
                window.ec=elCell;
                window.or=oRecord;
            }
         };
      },

      /**
       * Returns actions custom datacell formatter
       *
       * @method fnRenderCellActions
       */
      fnRenderCellActions: function DataGrid_fnRenderCellActions()
      {
         var scope = this;

         /**
          * Actions custom datacell formatter
          *
          * @method renderCellActions
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return function DataGrid_renderCellActions(elCell, oRecord, oColumn, oData)
         {
            Dom.setStyle(elCell, "width", "50px");
            Dom.setStyle(elCell.parentNode, "width", "50px");

            elCell.innerHTML = '<div id="' + scope.id + '-actions-' + oRecord.getId() + '" class="hidden"></div>';
         };
      },

      /**
       * Return data type-specific formatter
       *
       * @method getCellFormatter
       * @return {function} Function to render read-only value
       */
      getCellFormatter: function DataGrid_getCellFormatter()
      {
         var scope = this;

         /**
          * Data Type custom formatter
          *
          * @method renderCellDataType
          * @param elCell {object}
          * @param oRecord {object}
          * @param oColumn {object}
          * @param oData {object|string}
          */
         return function DataGrid_renderCellDataType(elCell, oRecord, oColumn, oData)
         {
            var html = "";

            // Populate potentially missing parameters
            if (!oRecord)
            {
               oRecord = this.getRecord(elCell);
            }
            if (!oColumn)
            {
               oColumn = this.getColumn(elCell.parentNode.cellIndex);
            }

            if (oRecord && oColumn)
            {
               if (!oData)
               {
                  oData = oRecord.getData("itemData")[oColumn.field];
               }

               if (oData)
               {
                  var datalistColumn = scope.datalistColumns[oColumn.key];
                  if (datalistColumn)
                  {
                     oData = YAHOO.lang.isArray(oData) ? oData : [oData];
                     for (var i = 0, ii = oData.length, data; i < ii; i++)
                     {
                        data = oData[i];

                        switch (datalistColumn.dataType.toLowerCase())
                        {
                           case "cm:person":
                        	   if (ii > 2 && i === 2){
                         		  html +="<span>...</span>"
                         	  }else if (i < 2){
                         		 html += '<span class="person">' + $userProfile(data.metadata, data.displayValue) + '</span>';
                         	  }
                              break;

                           case "datetime":
                              html += Alfresco.util.formatDate(Alfresco.util.fromISO8601(data.value), scope.msg("date-format.shortDate"));
                              break;

                           case "date":
                              html += Alfresco.util.formatDate(Alfresco.util.fromISO8601(data.value), scope.msg("date-format.shortDate"));
                              break;

                           case "cm:content":
                        	   if (data.displayValue.indexOf("fileholder.txt")!== -1){
                        		   html += "";
                        	   }else{
                        		   html += $html(data.displayValue);
                        	   }
                        	   break;
                           case "cm:cmobject":
                           case "cm:folder":
                        	  // Need to show all replies
                        	  if (datalistColumn.name === "lysedl:replies"){
                        		  if (data.linkValue){
                        			  html += '<a href="' + Alfresco.util.siteURL((data.metadata === "container" ? 'folder' : 'document') + '-details?nodeRef=' + data.linkValue) + '">';
                        		  }else{
                        			  html += '<a href="' + Alfresco.util.siteURL((data.metadata === "container" ? 'folder' : 'document') + '-details?nodeRef=' + data.value) + '">';
                        		  }
                                  html += '<img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/filetypes/' + Alfresco.util.getFileIcon(data.displayValue, (data.metadata == "container" ? 'cm:folder' : null), 16) + '" width="16" alt="' + $html(data.displayValue) + '" title="' + $html(data.displayValue) + '" />';
                                  html += ' ' + $html(data.displayValue) + '</a>'
                        	  }else {
	                        	  if (ii > 2 && i === 2){
	                        		  html +="<span>...</span>"
	                        	  }else if (i < 2){
	                        		  if (data.linkValue){
	                        			  html += '<a href="' + Alfresco.util.siteURL((data.metadata === "container" ? 'folder' : 'document') + '-details?nodeRef=' + data.linkValue) + '">';
	                        		  }else{
	                        			  html += '<a href="' + Alfresco.util.siteURL((data.metadata === "container" ? 'folder' : 'document') + '-details?nodeRef=' + data.value) + '">';
	                        		  }
	                                  html += '<img src="' + Alfresco.constants.URL_RESCONTEXT + 'components/images/filetypes/' + Alfresco.util.getFileIcon(data.displayValue, (data.metadata === "container" ? 'cm:folder' : null), 16) + '" width="16" alt="' + $html(data.displayValue) + '" title="' + $html(data.displayValue) + '" />';
	                                  html += ' ' + $html(data.displayValue) + '</a>'
	                        	  }
                        	  }
                              break;

                           case "discussion":
                        	   html+= scope.msg("datalist.comments.count", data.displayValue);
                        	   break;
                           case "boolean":
                              html+= scope.msg(data.value ? "label.yes" : "label.no");
                              break;
                           case "long":
                              html+= YAHOO.util.Number.format(data.displayValue, {thousandsSeparator: '&nbsp;'});
                              break;
                           case "double":
                               if(YAHOO.util.History.getQueryStringParameter('list') == 'Milestones and invoicing') {
                                   html += YAHOO.util.Number.format(data.displayValue, {
                                       thousandsSeparator: '&nbsp;',
                                       decimalPlaces: 2
                                   });
                               }else{
                                   html+= YAHOO.util.Number.format(data.displayValue, {thousandsSeparator: '&nbsp;'});
                               }
                               break;

                           default:
                              if (datalistColumn.name === "cm:creator") {
                                  html += '<span class="person">' + $userProfile(data.metadata, data.displayValue) + '</span>';
                              }
                        	  else if ($html(data.displayValue.length) > 40){

                        		  // If richtext editor was used, this property contains html, strip that out
                        		  if (isHTML(data.displayValue)){
                        			  data.displayValue = strip(data.displayValue);
                        		  }
                        		  html += $html(data.displayValue.substring(0,40) + '...');
                        		  var domid = Alfresco.util.generateDomId(elCell);
                        		  new YAHOO.widget.Tooltip(domid +"tooltip",
                        				  { context:elCell,
                        				  text:  $html(data.displayValue)});
                        	  }else{
                        		  // If richtext editor was used, this property contains html, strip that out
                        		  if (isHTML(data.displayValue)){
                        			  data.displayValue = strip(data.displayValue);
                        		  }
                        		  html += $html(data.displayValue);
                        	  }

                              break;
                        }

                        if (i < ii - 1 && i <= 1)
                        {
                           html += "<br />";
                        }
                     }

                     // Modify data content shown in table cells
                     switch (datalistColumn.name) {
                          case "dl:itemId":
                          case "lysedl:vorHeading":
                          case "lysedl:issueTitle":
                          case "lysedl:interfaceActionTitle":
                          case "lysedl:ncrHeading":
                          case "lysedl:correspondenceTitle":
                          case "cm:title":
                          case "lysedl:milestoneHeading":
                          case "lyse:title4":
                          case "lyse:civilTitle4":
                          case "lyse:iasHeading":
                          case "lysedl:civilVorHeading":
                              html = "<span style='cursor: pointer; text-decoration: underline;'>" + html+ "</span>";
                              elCell.onclick = function() {
                                scope.onActionView(oRecord.getData());
                              };

                              break;
                          case "lyse:irlRequirementSpecification":
                          case "lyse:irlDocumentsToBeProvided":
                              var elem = document.createElement('textarea');
                              elem.innerHTML = html;
                              html = elem.value;
                              break;
                          case "lyse:shortname":
                              // Lookup site title
                              YAHOO.Bubbling.fire("setSiteTitle",
                              {
                            	  cell: elCell,
                            	  data: data.value
                              });
                              break;

                     }
                   }
               }
            }

            elCell.innerHTML =   html ;
         };
      },

      /**
       * Set site title handler.
       *
       * @method onSetSiteTitle
       * @param e {object} DomEvent
       * @param args {array} Event parameters (depends on event type)
       */
      onSetSiteTitle: function ExtDataGrid_onSetSiteTitle(e, args)
      {
          Alfresco.util.Ajax.jsonGet(
                  {
                     url: Alfresco.constants.PROXY_URI + "api/lyse/site-shortname?shortname=" + args[1].data,
                     successCallback:
                     {
                      fn: function (response){
         	               if (response.json !== undefined)
         	               {
         	            	  args[1].cell.innerHTML= response.json.title;
         	                }
         	           },
                       scope: this
                      },
                      failureCallback:
                      {
                         fn: function (response){
                        	 html = data.value
                         },
                         scope: this
                      }
                   });

      },

      /**
       * Fired by YUI when parent element is available for scripting
       *
       * @method onReady
       */
      onReady: function ExtDataGrid_onReady()
      {

        var _this=this;
        // Following function is for use by windows opened by this window to request a
        // refresh of the datagrid: window.opener.refreshDataGrid()
        // There is possibly a cleaner way than using a global function for this.
        window.refreshDataGrid=function(s){
          _this.onFilterFormSubmit();
        }

    	 Alfresco.component.ExtDataGrid.superclass.onReady.call(this);
    	 //this.options.usePagination = false;

    	 // when calling with a nodeRef in #view, this will open the form.
         if (window.location.hash && window.location.hash.indexOf("#view=") === 0) {
             var hash = window.location.hash;
             var nodeRef = hash.replace("#view=", "");

             this.onActionView({
                nodeRef: nodeRef
             });
          } else if (window.location.hash && window.location.hash.indexOf("#edit=") === 0) {
             var hash = window.location.hash;
             var nodeRef = hash.replace("#edit=", "");

             this.onActionEdit({
                nodeRef: nodeRef
             });
          }

         // FilterForm Submit button
         this.widgets.filterFormSubmit = Alfresco.util.createYUIButton(this, "filterform-submit", this.onFilterFormSubmit);
         this.widgets.filterFormClear = Alfresco.util.createYUIButton(this, "filterform-clear", this.onFilterFormClear);

         Alfresco.util.createTwister(this.id+"-filterHeader", "datalistformfilter1", false);
         Alfresco.util.createTwister(this.id+"-title", "datalistformfilter2", false);
      },

      onShowWorkflowHistoryList: function ExtDataGrid_onShowWorkflowHistoryList(item) {
         var nodeRef = item.nodeRef;
         var panel = Alfresco.util.PopupManager.displayWebscript(
             {
                title: this.msg("dialog.title.workflow-history"),
                url: YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "/components/workflow/lyse-workflow-history?nodeRef={nodeRef}", { nodeRef: item.nodeRef }),
                properties: {
                }
             }
         );
         // Register the ESC key to close the panel
         var escapeListener = new YAHOO.util.KeyListener(document,
             {
                keys: YAHOO.util.KeyListener.KEY.ESCAPE
             },
             {
                fn: function(id, keyEvent)
                {
                   this.destroy();
                },
                scope: panel,
                correctScope: true
             });
         escapeListener.enable();
      },

       onShowShareHistoryList: function ExtDataGrid_onShowShareHistoryList(item) {
           var nodeRef = item.nodeRef;
           var panel = Alfresco.util.PopupManager.displayWebscript(
               {
                   title: this.msg("dialog.title.share-history"),
                   url: YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "/components/workflow/lyse-share-history?nodeRef={nodeRef}", { nodeRef: item.nodeRef }),
                   properties: {
                   }
               }
           );
           // Register the ESC key to close the panel
           var escapeListener = new YAHOO.util.KeyListener(document,
               {
                   keys: YAHOO.util.KeyListener.KEY.ESCAPE
               },
               {
                   fn: function(id, keyEvent)
                   {
                       this.destroy();
                   },
                   scope: panel,
                   correctScope: true
               });
           escapeListener.enable();
       },

        /**
         * Show Upload redlined form.
         *
         * @method onActionRedlinedForm
         * @param item {object} Object literal representing one data item
         */
        onActionUploadRedlinedForm: function ExtDataGrid_onActionUploadRedlinedForm(datagrid, item, fn)
        {


            // Intercept before dialog show
            var doBeforeDialogShow = function DataGrid_onActionEdit_doBeforeDialogShow(p_form, p_dialog)
            {
                Alfresco.util.populateHTML(
                    [ p_dialog.id + "-dialogTitle", this.msg("actions.upload-redlined") ]
                );

            };

            var templateUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form/site/" + Alfresco.constants.SITE + "?itemKind={itemKind}&itemId={itemId}&mode={mode}&submitType={submitType}&formId={formId}&showCancelButton=true",
                {
                    itemKind: "node",
                    itemId: item.nodeRef,
                    mode: "edit",
                    submitType: "json",
                    formId: "upload-redlined"
                });

            // Using Forms Service, so always create new instance
            var editDetails = new Alfresco.module.SimpleDialog(this.id + "-editDetails");
            editDetails.setOptions(
                {
                    width: "1000px",
                    templateUrl: templateUrl,
                    actionUrl: null,
                    destroyOnHide: true,
                    doBeforeDialogShow:
                    {
                        fn: doBeforeDialogShow,
                        scope: this
                    },
                    onSuccess:
                    {
                        fn: function DataGrid_onActionEdit_success(response)
                        {
                            if(fn)
                            {
                                fn.call(this,response);
                            }

                        },
                        scope: this
                    },
                    onFailure:
                    {
                        fn: function DataGrid_onActionEdit_failure(response)
                        {
                            if (response.serverResponse.responseText.contains("error.responsiblesMissing")){
                                Alfresco.util.PopupManager.displayPrompt(
                                    {
                                        text: this.msg("message.details.responsiblesMissing")
                                    });
                            }else {
                                Alfresco.util.PopupManager.displayPrompt(
                                    {
                                        text: this.msg("message.details.failure")
                                    });
                            }
                        },
                        scope: this
                    }
                }).show();
        },


      onActionUploadRedlined: function ExtDataGrid_onActionUploadRedlined(item)
      {

          if (!this.fileUpload)
          {
              this.fileUpload = Alfresco.getFileUploadInstance();
          }
          var nodeToUpdate = "";

          nodeToUpdate = item.itemData.assoc_lyse_engRedlinedAttachment.value.toString();

          var singleUpdateConfig = {
              updateNodeRef: nodeToUpdate,
              suppressRefreshEvent: true,
              overwrite: true,
              mode: this.fileUpload.MODE_SINGLE_UPDATE,
              onFileUploadComplete: {
                  fn: function(complete) {
                      if (complete.failed === undefined || complete.failed.length === 0) {
                          this.onActionUploadRedlinedForm(this, item, function() {
                              var me = this;

                              Alfresco.util.PopupManager.displayPrompt({
                                  title: Alfresco.util.message("datalist.prompt.uploadRedlined"),
                                  text: Alfresco.util.message("datalist.prompt.confirmRedlined"),
                                  buttons: [{
                                          text: Alfresco.util.message("button.yes"),
                                          handler: function() {
                                              this.destroy();
                                              me.onActionStartConfirmRedlinedDrawingWorkflow(item);
                                          },
                                          isDefault: true
                                      },
                                      {
                                          text: Alfresco.util.message("button.no"),
                                          handler: function() {
                                              this.destroy();
                                              Alfresco.util.Ajax.jsonPut({
                                                  url: Alfresco.constants.PROXY_URI +
                                                      "api/lyse/set-status/" +
                                                      item.nodeRef.replace("://", "/") +
                                                      "?statusValue=redlined",
                                                  successCallback: {
                                                      fn: function(response) {
                                                          me.refreshItem(item.nodeRef, "message.details.success");
                                                      },
                                                      scope: me
                                                  }
                                              });
                                          }
                                      }
                                  ]
                              });

                          });
                      }
                  },
                  scope: this
              }
          };
          if (Alfresco.util.isValueSet(this.options.siteId))
          {
              singleUpdateConfig.siteId = this.options.siteId;
              singleUpdateConfig.containerId = "documentLibrary";
          }

          this.fileUpload.show(singleUpdateConfig);

          // Hide the version information div
          this.fileUploadDialog = this.fileUpload.uploader.id;
          Dom.addClass(Dom.get(this.fileUploadDialog + "-versionSection-div"), "hidden");
          var fileSelectionButton = Dom.get(this.fileUploadDialog + "-file-selection-button-overlay-button");
          var fileSelectionInput = Dom.getNextSibling(fileSelectionButton);
          Event.addListener(fileSelectionInput, "change", this.onFileSelection, this, true);
      },

      /**
       * Called when files are selected from the input element.
       *
       * @method onBrowseButtonClick
       * @param event {object} a file selection "change" event
       */
      onFileSelection: function ExtDataGrid_onFileSelection(evt)
      {
          // FileList object
          var files = evt.target.files;
          if (files != null)
          {
              var statusSpan = Dom.get(this.fileUploadDialog + "-status-span");
              statusSpan.innerHTML = files[0].name;
              Dom.removeClass(statusSpan, "hidden");
          }
          else
          {
              // No files selected, do nothing.
          }
      },


       onActionStartConfirmRedlinedDrawingWorkflow: function ExtDataGrid_onActionStartConfirmRedlinedDrawingWorkflow(item) {

           var payload = {
               firstUserTask: "confirmRedlined",
               nodes: [{
                   nodeRef: item.nodeRef
               }]
           };

           Alfresco.util.Ajax.jsonPost({
               url: Alfresco.constants.PROXY_URI + "/lyse/api/start-workflow",
               responseContentType: "application/json",
               dataObj: payload,
               successCallback: {
                   fn: function ExtDataGrid_onActionStartConfirmRedlinedDrawingWorkflow_success(response) {

                       var refreshItem = true;
                       if (response.json) {
                           if (response.json.data.workflows.length > 0) {
                               if (response.json.data.workflows[0].tasks.length === 1) {
                                   // Redirect to the assigned task if there is only one
                                   var taskId = response.json.data.workflows[0].tasks[0].taskId;
                                   var workflowId = response.json.data.workflows[0].workflowId;

                                   refreshItem = false;
                                   Alfresco.util.PopupManager.displayMessage({
                                       text: this.msg("actions.start-workflow-success")
                                   });
                                   window.location.href = YAHOO.lang.substitute("{context}lyse-task-edit?taskId={taskId}&workflowId=" + workflowId, {
                                       context: Alfresco.constants.URL_PAGECONTEXT,
                                       taskId: taskId
                                   });


                               }
                           }
                       }
                       if (refreshItem) {
                           this.refreshItem(item.nodeRef, "actions.start-workflow-success");
                       }

                   },
                   scope: this
               },
               failureCallback: {
                   fn: function ExtDataGrid_onActionStartConfirmRedlinedDrawingWorkflow_failure(response) {
                       Alfresco.util.PopupManager.displayPrompt({
                           text: response.json.message
                       });
                   },
                   scope: this
               }
           });
       },

      /**
       * Called when files are selected from the input element.
       *
       * @method onBrowseButtonClick
       * @param event {object} a file selection "change" event
       */
      onFileSelection: function ExtDataGrid_onFileSelection(evt)
      {
    	 // FileList object
         var files = evt.target.files;
         if (files != null)
         {
             var statusSpan = Dom.get(this.fileUploadDialog + "-status-span");
             statusSpan.innerHTML = files[0].name;
             Dom.removeClass(statusSpan, "hidden");
         }
         else
         {
            // No files selected, do nothing.
         }
      },



	  /**
       * Edit Data Item pop-up
       *
       * @method onActionEdit
       * @param item {object} Object literal representing one data item
       */
      onActionEdit: function DataGrid_onActionEdit(item, formId)
      {
         var scope = this;

         // Intercept before dialog show
         var doBeforeDialogShow = function DataGrid_onActionEdit_doBeforeDialogShow(p_form, p_dialog)
         {
            Alfresco.util.populateHTML(
               [ p_dialog.id + "-dialogTitle", this.msg("label.edit-row.title") ]
            );

            /**
             * No full-page edit view for v3.3
             *
            // Data Item Edit Page link button
            Alfresco.util.createYUIButton(p_dialog, "editDataItem", null,
            {
               type: "link",
               label: scope.msg("label.edit-row.edit-dataitem"),
               href: scope.getActionUrls(item).editMetadataUrl
            });
             */
         };

         var templateUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&mode={mode}&submitType={submitType}&site={site}&showCancelButton=true",
         {
            itemKind: "node",
            itemId: item.nodeRef,
            mode: "edit",
            submitType: "json",
            site: Alfresco.constants.SITE
         });

          if (formId !== undefined) {
              templateUrl += "&formId=" + formId;
          }

         // Using Forms Service, so always create new instance
         var editDetails = new Alfresco.module.SimpleDialog(this.id + "-editDetails");
         editDetails.setOptions(
         {
            width: "1000px",
            templateUrl: templateUrl,
            actionUrl: null,
            destroyOnHide: true,
            doBeforeDialogShow:
            {
               fn: doBeforeDialogShow,
               scope: this
            },
            onSuccess:
            {
               fn: function DataGrid_onActionEdit_success(response)
               {
                   this.refreshItem(item.nodeRef, "message.details.success");
               },
               scope: this
            },
            onFailure:
            {
               fn: function DataGrid_onActionEdit_failure(response)
               {
                  Alfresco.util.PopupManager.displayPrompt(
                  {
                     text: this.msg("message.details.failure")
                  });
               },
               scope: this
            }
         }).show();
      },
      /**
       * Edit Data Item pop-up
       *
       * @method onActionEdit
       * @param item {object} Object literal representing one data item
       */
      onActionView: function DataGrid_onActionEdit(item)
      {
		 var templateUrl;
         var workflowId = item.itemData.prop_lysedl_workflowId;
         //First check if this view form shall have an attachment document library
         Alfresco.util.Ajax.jsonGet(
         {
            url: Alfresco.constants.PROXY_URI + "api/lyse/attachment-folder?nodeRef=" +item.nodeRef,
            successCallback:
            {
             fn: function (response){
	               if (response.json !== undefined && response.json.nodeRef !== "")
	               {
	            	   //Success, this datalist item has an attachment folder
	            	   var attachmentFolder = response.json.nodeRef;
	                   templateUrl = Alfresco.constants.URL_CONTEXT + 'page/lyse-datalist-details?nodeRef=' + item.nodeRef + "&docLib=" + attachmentFolder;

                      // Some browsers has a minimal warning about new tab. Alert user.
                      Alfresco.util.PopupManager.displayMessage({
                        text: this.msg('message.opened_in_new_window')
                      });

	                   return window.open(templateUrl, "_blank");
	               } else {
	            	   //Datalist item does not have an attachment folder, proceed as usual.

                      // Some browsers has a minimal warning about new tab. Alert user.
                      Alfresco.util.PopupManager.displayMessage({
                        text: this.msg('message.opened_in_new_window')
                      });

	            	   //For some reason item.itemData.prop_lysedl_workflowId is null even if the node clearly had a lysedl:workflowId
	                   //This is a workaround and uses the item.workflowTasks[0].workflowId which contains the same data.
	                   if(workflowId != null || (item.workflowTasks != undefined && item.workflowTasks.length > 0 && item.workflowTasks[0].workflowId != null)) {
	                     if(workflowId == null) {
	                       workflowId = item.workflowTasks[0].workflowId;
	                     } else {
	                       workflowId = item.itemData.prop_lysedl_workflowId.value.toString();
	                     }
	                     templateUrl = Alfresco.constants.URL_CONTEXT + 'page/lyse-datalist-details?workflowId='+workflowId;
	                     return window.open(templateUrl, "_blank");
	                   } else {
	                     templateUrl = Alfresco.constants.URL_CONTEXT + 'page/lyse-datalist-details?nodeRef=' + item.nodeRef;
	                     return window.open(templateUrl, "_blank");
	                   }
	               }
	            },
             scope: this
            }
         });
      },

      /**
       * Link to doclib
       *
       * @method onActionHandleAttachments
       * @param item {object} Object literal representing one data item
       */
      onActionHandleAttachments: function DataGrid_onActionHandleAttachments(item)
      {
            // Get the nodeRef of the attachment folder
            Alfresco.util.Ajax.jsonGet(
            {
               url: Alfresco.constants.PROXY_URI + "api/lyse/attachment-folder?nodeRef=" +item.nodeRef,
               successCallback:
               {
                fn: function (response){
   	               if (response.json !== undefined)
   	               {
   	            	    var attachmentFolder = response.json.nodeRef;
   	            	    var doclibUrl = Alfresco.constants.URL_CONTEXT + 'page/attachment-doclib?docLib=' + attachmentFolder;
   	            	    return window.open(doclibUrl, "_blank");
   	               }
   	            },
                scope: this
               }
            });
      },



      /**
       * DataTable set-up and event registration
       *
       * @method _setupDataTable
       * @protected
       */
      _setupDataTable: function DataGrid__setupDataTable(columns)
      {
         // YUI DataTable column definitions
         var columnDefinitions =
         [
            { key: "nodeRef", label: "", sortable: false, formatter: this.fnRenderCellSelected(), width: 16 }
         ];

         // Add actions as second column
         columnDefinitions.push(
             { key: "actions", label:  this.msg("label.column.actions") , sortable: false, formatter: this.fnRenderCellActions(), width: 80 }
         );

         // According to Margrethe (I cannot remember why cdl was in the exclude list) https://redmine.redpill-linpro.com/issues/46255
         //var excludelist=['lysedl:cdlDataList'];
         var excludelist=[];
         if (excludelist.indexOf(this.datalistMeta.itemType)<0) { // If itemType not in excludelist
	         columnDefinitions.push(
	             { key: "workflowTasks", label: this.msg("label.column.workflowTasks") , sortable: true, formatter: this.fnRenderCellWorkflowTasks()/*, width: 32 */}
	         );
            columnDefinitions.push( {
                key: "responsiblesForTasks",
                label:  this.msg("label.column.responsibles") ,
                sortable: true,
                formatter: this.fnRenderCellResponsiblesForTasks(),
                sortOptions:
                {
                    field: "hva er opp med det?",
                    sortFunction: function(item1,item2,desc,field){
                        if(!YAHOO.lang.isValue(item1)) {
                            return (!YAHOO.lang.isValue(item2)) ? 0 : 1;
                        } else if(!YAHOO.lang.isValue(item2)) {
                            return -1;
                        }

                        function getVal(d){
                            return getTaskResponsibleFromRecord(d);
                        }

                        val1=getVal(item1);
                        val2=getVal(item2);
                        ret=0;
                        if (val1<val2) ret =-1;
                        if (val2<val1) ret =1;
                        if (desc) ret=-ret;
                        return ret;
                    }
                }
                /*, width: 32 */}
            );
         }

         var column;
         for (var i = 0, ii = this.datalistColumns.length; i < ii; i++)
         {
            column = this.datalistColumns[i];
            if (column.name === "lysedl:workflowId") {
               // Don't show! The field must still be present in the "datagrid" form definition or else the field value is not available
               continue;
            }

            var label = column.label;

            // General label override.
            if (this.datalistMeta.itemType === "lysedl:cdlDataList" && column.name === "lyse:shortname"){
            	label = this.msg("form.label.shortname");
            }
            if (this.datalistMeta.itemType === "lysedl:companysDocumentDataList" && column.name === "lyse:shortname"){
            	label = this.msg("form.label.shortname");
            }
            columnDefinitions.push(
            {
               key: this.dataResponseFields[i],
               label: label,
               sortable: true,
               sortOptions:
               {
                  field: column.formsName,
                  sortFunction: this.getSortFunction()
               },
               formatter: this.getCellFormatter(column.dataType)
            });
         }


      // Define a custom row formatter function
      var reviewStatusRowFormatter = function(elTr, oRecord) {
          if (oRecord.getData("itemData").prop_lyse_engDrawingStatus != undefined){
        	  if ((oRecord.getData("itemData").prop_lyse_engDrawingStatus.value == "under-revision")||(oRecord.getData("itemData").prop_lyse_engDrawingStatus.value == "revoked")||(oRecord.getData("itemData").prop_lyse_engDrawingStatus.value == "void")){
        		  Dom.addClass(elTr, 'mark');
        	  }
          }
          return true;
      };

         // DataTable definition
         var me = this;
         this.widgets.dataTable = new YAHOO.widget.DataTable(this.id + "-grid", columnDefinitions, this.widgets.dataSource,
         {
            renderLoopSize: this.options.usePagination ? 16 : 32,
            initialLoad: false,
            dynamicData: true,
            formatRow: reviewStatusRowFormatter,
            "MSG_EMPTY": this.msg("message.empty"),
            "MSG_ERROR": this.msg("message.error"),
            paginator: this.widgets.paginator
         });

         this.widgets.paginator.unsubscribe('changeRequest',this.widgets.dataTable.onPaginatorChangeRequest);
         this.widgets.paginator.subscribe('changeRequest', this.onDataGridPaging,this,true);

         this.widgets.dataTable.unsubscribe("theadCellClickEvent", this.widgets.dataTable.onEventSortColumn);
         this.widgets.dataTable.subscribe("theadCellClickEvent",this.onDataGridSort,this,true);

         // Update totalRecords with value from server
         this.widgets.dataTable.handleDataReturnPayload = function DataGrid_handleDataReturnPayload(oRequest, oResponse, oPayload)
         {
            me.totalRecords = oResponse.meta.totalRecords;
            me.totalRecordsUpper = oResponse.meta.totalRecordsUpper;
            oResponse.meta.pagination =
            {
               rowsPerPage: me.options.pageSize,
               recordOffset: (me.currentPage - 1) * me.options.pageSize
            };
            return oResponse.meta;
         };

         // It could be simplified to get paginator, but i want to make sure im using correct refrence, YUI might do copies somewhere?
         var dataTablePaginator = this.widgets.dataTable.get("paginator");

         // Override abstract function within DataTable to set custom error message
         this.widgets.dataTable.doBeforeLoadData = function DataGrid_doBeforeLoadData(sRequest, oResponse, oPayload)
         {
            if (oResponse.error)
            {
               try
               {
                  var response = YAHOO.lang.JSON.parse(oResponse.responseText);
                  me.widgets.dataTable.set("MSG_ERROR", response.message);
               }
               catch(e)
               {
                  me._setDefaultDataTableErrors(me.widgets.dataTable);
               }
            }

            /* 		Tried to use paginators built in functions to set the value, it didn't work.
             * 		dataTablePaginator.setStartIndex(oResponse.meta.paginationRecordOffset,true);
             * 		Below may be a hacky solution but it works.
             **/
            dataTablePaginator._configs.recordOffset.value = oResponse.meta.paginationRecordOffset;

            // We don't get an renderEvent for an empty recordSet, but we'd like one anyway
            if (oResponse.results.length === 0)
            {
               this.fireEvent("renderEvent",
               {
                  type: "renderEvent"
               });
            }

            // Must return true to have the "Loading..." message replaced by the error message
            return true;
         };

         // File checked handler
         this.widgets.dataTable.subscribe("checkboxClickEvent", function(e)
         {
            var id = e.target.value;
            this.selectedItems[id] = e.target.checked;
            Bubbling.fire("selectedItemsChanged");
         }, this, true);

         // Rendering complete event handler
         this.widgets.dataTable.subscribe("renderEvent", function()
         {
            Alfresco.logger.debug("DataTable renderEvent");

            // IE6 fix for long filename rendering issue
            if (YAHOO.env.ua.ie < 7)
            {
               var ie6fix = this.widgets.dataTable.getTableEl().parentNode;
               ie6fix.className = ie6fix.className;
            }

            // Deferred functions specified?
            for (var i = 0, j = this.afterDataGridUpdate.length; i < j; i++)
            {
               this.afterDataGridUpdate[i].call(this);
            }
            this.afterDataGridUpdate = [];

            // Update the paginator if it's been created
            if (this.widgets.paginator)
            {
               Alfresco.logger.debug("Setting paginator state: page=" + this.currentPage + ", totalRecords=" + this.totalRecords);

               this.widgets.paginator.setState(
               {
                  page: this.currentPage,
                  totalRecords: this.totalRecords
               });

               if (this.totalRecordsUpper)
               {
                  this.widgets.paginator.set("pageReportTemplate", this.msg("pagination.template.page-report.more"));
               }
               else
               {
                  this.widgets.paginator.set("pageReportTemplate", this.msg("pagination.template.page-report"));
               }

               this.widgets.paginator.render();
            }
         }, this, true);

         // Enable row highlighting
         this.widgets.dataTable.subscribe("rowMouseoverEvent", this.onEventHighlightRow, this, true);
         this.widgets.dataTable.subscribe("rowMouseoutEvent", this.onEventUnhighlightRow, this, true);
      },


      /**
       * Retrieves the Data List from the Repository
       *
       * @method populateDataGrid
       */
      populateDataGrid: function ExtDataGrid_populateDataGrid()
      {
      	 this.populateFilterForm();
         if (!YAHOO.lang.isObject(this.datalistMeta))
         {
            return;
         }

         this.renderQueryListMeta();

         // Query the visible columns for this list's item type
         Alfresco.util.Ajax.jsonGet(
         {
            url: $combine(Alfresco.constants.URL_SERVICECONTEXT, "components/query-lists/config/columns?itemType=" + encodeURIComponent(this.datalistMeta.itemType) + "&site=" + Alfresco.constants.SITE),
            successCallback:
            {
               fn: this.onDatalistColumns,
               scope: this
            },
            failureCallback:
            {
               fn: this._onDataListFailure,
               obj:
               {
                  title: this.msg("message.error.columns.title"),
                  text: this.msg("message.error.columns.description")
               },
               scope: this
            }
         });
      },
      populateFilterForm: function ExtDataGrid_populateFilterForm()
      {
    	  var filterFormUrl = YAHOO.lang.substitute(Alfresco.constants.URL_SERVICECONTEXT + "components/form?itemKind={itemKind}&itemId={itemId}&mode={mode}&formId={formId}&site={site}&submitType=json&showCancelButton=false&showSubmitButton=false",
    	   {
              itemKind: "type",
              itemId: this.datalistMeta.itemType,
              formId:"querylist-filter",
              mode: "create",
              submitType: "json",
              site: Alfresco.constants.SITE
           });
    	  var data =
          {
             htmlid: this.id + "-extDgFilterForm"
          };

    	  Alfresco.util.Ajax.request(
		   {
	          url: filterFormUrl,
	          dataObj:data,
	          successCallback:
	          {
	             fn: this.onFilterFormTemplateLoaded,
	             scope: this
	          },
	          failureMessage: "Could not load dialog template from '" + this.options.templateUrl + "'.",
	          scope: this,
	          execScripts: true
	      });
      },

      /**
       * Event callback when filtr template has been loaded
       *
       * @method onFilterFormTemplateLoaded
       * @param response {object} Server response from load template XHR request
       */
      onFilterFormTemplateLoaded: function ExtDataGrid_onFilterFormTemplateLoaded(response)
      {
    	   Dom.get(this.id + "-filterform").innerHTML = response.serverResponse.responseText;
    	   //show filter form
    	   Dom.setStyle(this.id + "-filter", "visibility", "inherit");

      },

      onFilterFormSubmit: function ExtDataGrid_onFilterFormSubmit(){
    	  //alert(this.formsRuntime.getFormData());
    	  YAHOO.Bubbling.fire("changeFilter",
          {
             filterOwner: this.id,
             filterId: "filterform",
             filterData: Alfresco.util.toQueryString(this.formsRuntime.getFormData())
          });
      },

      onFilterFormClear: function ExtDataGrid_onFilterFormClear(){
          //alert(this.formsRuntime.getFormData());
    	  this.currentFilter="";
    	  this.formsRuntime.reset();
    	  // Some specialized controls - e.g. auto-complete  or object-finder don't implement reset
    	  YAHOO.Bubbling.fire("resetForm");
          // Seems nobody is is listening
    	  // YAHOO.Bubbling.fire("resetFinder"); // Seems nobody is is listening
          this.onFilterFormSubmit();
         // So this is the best way to get a "most of the time right" reset
         location.hash = '';
      },

      onChangeFilter: function DataGrid_onChangeFilter(layer, args)
      {
         var obj = args[1];
         if (obj !== null)
         {
        	this._addSortingToNewFilter(obj, this.sortAsc, this.sortColumn);
            var filter = Alfresco.util.cleanBubblingObject(obj);
            if(obj.filterId == "filterform"){
            	this.currentPage = 1;
            	this.widgets.paginator.setPage(1,true);
            }
            this._updateDataGrid.call(this,
            {
               filter: filter,
               page: this.currentPage
            });
       	 	//Alfresco.component.ExtDataGrid.superclass.onChangeFilter.call(this, layer, args);
         }
      },

      onFilterFormSave: function ExtDataGrid_onFilterFormSave(){
    	  Alfresco.util.PopupManager.getUserInput({
    		  okButtonText : "Save"
    	  });

      },

      /**
       * DataGrid View Filter changed event handler
       *
       * @method onFilterChanged
       * @param layer {object} Event fired (unused)
       * @param args {array} Event parameters (new filterId)
       */
      onFilterChanged: function DataGrid_onFilterChanged(layer, args)
      {
         var obj = args[1];
         if ((obj !== null) && (obj.filterId !== null))
         {
            obj.filterOwner = obj.filterOwner || Alfresco.util.FilterManager.getOwner(obj.filterId);

            // Should be a filterId in the arguments
            this.currentFilter = Alfresco.util.cleanBubblingObject(obj);
            Alfresco.logger.debug("DL_onFilterChanged: ", this.currentFilter);

            if (this.currentFilter.filterId === "filterform" && this.formsRuntime){
            	// refill filter form
            	var filterData =  Alfresco.util.getQueryStringParameters(this.currentFilter.filterData);

            	// get the form element
                var form = Dom.get(this.formsRuntime.formId);
                if (form !== null)
                {
                   var length = form.elements.length;
                   for (var i = 0; i < length; i++)
                   {
                      var element = form.elements[i];
                      var name = element.name;
                      if (name === "-" || element.disabled || element.type === "button")
                      {
                         continue;
                      }
                      if (name === undefined || name === "")
                      {
                         name = element.id;
                      }
                      var value = YAHOO.lang.trim(element.value);
                      if (name && filterData[name])
                      {
                    	//try to find form control component
                    	var control = Alfresco.util.ComponentManager.get(element.attributes.id.value+"-cntrl");
                    	if (control && control.name === "Alfresco.FilterDateRange"){
                    		var values = filterData[name].split("TO");
                    		if (values[0] !== ""){
                    			control.currentFromDate = values[0];
                    			var dateEntry = Alfresco.util.fromISO8601(control.currentFromDate).toString(control.msg("form.control.date-picker.entry.date.format"));
                            	Dom.get(control.id + "-date-from").value = dateEntry;
                    		}
                            if (values[1] !== ""){
                            	 control.currentToDate = values[1];
                             	var dateEntry = Alfresco.util.fromISO8601(control.currentToDate).toString(control.msg("form.control.date-picker.entry.date.format"));
                             	Dom.get(control.id + "-date-to").value = dateEntry;
                            }
                    	}else if (element.name === "prop_cm_categories"){
                    		element.value=filterData[name];

                    	}else if (element.type === "hidden" && filterData[name].split(",").length > 0 && !element.name.endsWith("_added"))
                        {
                    		element.value=filterData[name];
                    		var selectElement = Dom.get(element.id+"-entry");
                    		if (selectElement.type !== "checkbox"){
                    		   values = filterData[name].split(",");

                               for (var j = 0, jj = selectElement.options.length; j < jj; j++)
                               {
                                  if (Alfresco.util.arrayContains(values,selectElement.options[j].value))
                                  {
                                	  selectElement.options[j].selected= true;
                                  }else{
                                	  selectElement.options[j].selected= false;
                                  }
                               }
                    		}else {
                    			element.value=filterData[name];
                    		}

                        }
                    	else{
                    		element.value=filterData[name];
                    	}

                      }
                   }
                }
            }else if (this.currentFilter.filterId !== "filterform"){
            	//clear form
            	this.populateFilterForm();
            }
         }
      },


     /**
      * Event handler called when the "onAfterFormRuntimeInit" event is received.
      *
      * @method onAfterFormRuntimeInit
      * @param layer {String} Event type
      * @param args {Object} Event arguments
      * <pre>
      *    args.[1].component: Alfresco.FormUI component instance,
      *    args.[1].runtime: Alfresco.forms.Form instance
      * </pre>
      */
      onAfterFormRuntimeInit: function ExtDataGrid_onAfterFormRuntimeInit(layer, args)
      {
         var rt = args[1].runtime;
         if (rt.formId === this.id + "-extDgFilterForm-form") {
            this.formsRuntime = rt;
            var me = this,
                form = Dom.get(this.formsRuntime.formId);

            YAHOO.util.Event.removeListener(form, "keypress");
            YAHOO.util.Event.removeListener(form, KeyListener.KEYDOWN);
            /**
             * Prevent the Enter key from causing a double form submission
             */
            var fnStopEvent = function(id, keyEvent)
            {
               var event = keyEvent[1],
                   target = event.target ? event.target : event.srcElement;

               var targetId = target.id;
               if (!(targetId.contains(me.id) && targetId.contains("-extDgFilterForm")))
               {
                  //this is not from filterForm
                  return false;
               }

               if (target.tagName === "TEXTAREA")
               {
                  // Allow linefeeds in textareas
                  return false;
               }
               else if (target.tagName === "BUTTON" || Dom.hasClass(target, "yuimenuitemlabel"))
               {
                  // Eventlisteners for buttons and menus must be notified that the enter key was entered
               }
               else
               {
                  var targetName = target.name;
                  if (targetName && (targetName !== "-"))
                  {
                     me.onFilterFormSubmit(event);
                  }
                  Event.stopEvent(event);
                  return false;
               }
            };
            var enterListener = new KeyListener(form,
                {
                   keys: KeyListener.KEY.ENTER
                }, fnStopEvent, YAHOO.env.ua.ie > 0 ? KeyListener.KEYDOWN : "keypress");
            enterListener.enable();
            if (this.currentFilter.filterId === "filterform"){

               YAHOO.Bubbling.fire("filterChanged",
                   {
                      filterOwner: this.id,
                      filterId: this.currentFilter.filterId,
                      filterData: Alfresco.util.toQueryString(this.currentFilter.filterData)
                   });
            }
         }
      },
     /**
      * Build URI parameter string for doclist JSON data webscript
      *
      * @method _buildDataGridParams
      * @param p_obj.filter {string} [Optional] Current filter
      * @return {Object} Request parameters. Can be given directly to Alfresco.util.Ajax, but must be JSON.stringified elsewhere.
      */
     _buildDataGridParams: function DataGrid__buildDataGridParams(p_obj)
     {
        var request =
        {
           fields: this.dataRequestFields,
           page : this.currentPage,
           size : this.options.pageSize,
           total : this.totalRecords,
           noCache : new Date().getTime()
        };

        if (p_obj && p_obj.filter)
        {
        	if (p_obj.filter.sortField !=null && this.datalistColumns[p_obj.filter.sortField].dataType == "double"){
        		request.filter =
		        {
		        	// #36352 Alfresco canned queries does not support sorting on doubles, "short circuit" to use solr search.
        			filterId: "short circuit to solrSearch",
		            filterData:  Alfresco.util.getQueryStringParameters(p_obj.filter.filterData),
		            sortField: p_obj.filter.sortField,
		            sortAsc: p_obj.filter.sortAsc,
		            page : this.currentPage,
		            size : this.options.pageSize
		        };
		    } else if (!p_obj.filter.sortField){
       		    request.filter =
		        {
        	 	    filterId: p_obj.filter.filterId,
		            filterData:  Alfresco.util.getQueryStringParameters(p_obj.filter.filterData),
		            page : this.currentPage,
		            size : this.options.pageSize
		        };
        	} else {
        		request.filter =
		        {
        			filterId: p_obj.filter.filterId,
		            filterData:  Alfresco.util.getQueryStringParameters(p_obj.filter.filterData),
		            sortField: p_obj.filter.sortField,
		            sortAsc: p_obj.filter.sortAsc,
		            page : this.currentPage,
		            size : this.options.pageSize
		        };
        	}
        }
        this.widgets.dataSource.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);
        return request;

     },

     refreshItem: function ExtendedDataGrid_refreshItem(nodeRef, successMessageKey) {
            Alfresco.util.Ajax.jsonPost(
               {
                   url: Alfresco.constants.PROXY_URI + "slingshot/querylists/item/node/" + new Alfresco.util.NodeRef(nodeRef).uri,
                   dataObj: this._buildDataGridParams(),
                   successCallback:
                   {
                       fn: function refreshItem_refreshSuccess(response)
                       {
                           // Fire "itemUpdated" event
                           Bubbling.fire("dataItemUpdated",
                               {
                                   item: response.json.item
                               });
                           // Display success message
                           Alfresco.util.PopupManager.displayMessage(
                               {
                                   text: this.msg(successMessageKey)
                               });
                       },
                       scope: this
                   },
                   failureCallback:
                   {
                       fn: function refreshItem_refreshFailure(response)
                       {
                           Alfresco.util.PopupManager.displayPrompt(
                               {
                                   text: this.msg("message.details.failure")
                               });
                       },
                       scope: this
                   }
               });
       },

     /**
      * DataSource set-up and event registration
      *
      * @method _setupDataSource
      * @protected
      */
     _setupDataSource: function DataGrid__setupDataSource()
     {
        var listNodeRef = new Alfresco.util.NodeRef(this.datalistMeta.nodeRef);

        for (var i = 0, ii = this.datalistColumns.length; i < ii; i++)
        {
           var column = this.datalistColumns[i],
              columnName = column.name.replace(":", "_"),
              fieldLookup = (column.type === "property" ? "prop" : "assoc") + "_" + columnName;

           this.dataRequestFields.push(columnName);
           this.dataResponseFields.push(fieldLookup);
           this.datalistColumns[fieldLookup] = column;
        }

        // DataSource definition
        this.widgets.dataSource = new YAHOO.util.DataSource(Alfresco.constants.PROXY_URI + "lyse/querylists/query/" + Alfresco.constants.SITE + "/type/" + this.datalistMeta.itemType + "/" + this.datalistMeta.queryType + "/" + "querylist-datagrid",
        {
           connMethodPost: true,
           responseType: YAHOO.util.DataSource.TYPE_JSON,
           maxCacheEntries : 0,
           responseSchema:
           {
              resultsList: "items",
              metaFields:
              {
                 paginationRecordOffset: "startIndex",
                 totalRecords: "totalRecords",
                 totalRecordsUpper: "totalRecordsUpper"
              }
           }
        });
        this.widgets.dataSource.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);

        // Intercept data returned from data webscript to extract custom metadata
        this.widgets.dataSource.doBeforeCallback = function DataGrid_doBeforeCallback(oRequest, oFullResponse, oParsedResponse)
        {
        	this.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);
            this.connMgr.setDefaultXhrHeader(Alfresco.util.Ajax.JSON);

           // Container userAccess event
           var permissions = oFullResponse.metadata.parent.permissions;
           if (permissions && permissions.userAccess)
           {
              Bubbling.fire("userAccess",
              {
                 userAccess: permissions.userAccess
              });
           }

           return oParsedResponse;
        };
     },
     /**
      * Updates all Data Grid data by calling repository webscript with current list details
      *
      * @method _updateDataGrid
      * @private
      * @param p_obj.filter {object} Optional filter to navigate with
      */
     _updateDataGrid: function DataGrid__updateDataGrid(p_obj)
     {
        p_obj = p_obj || {};
        Alfresco.logger.debug("DataGrid__updateDataGrid: ", p_obj.filter);
        var successFilter = YAHOO.lang.merge({}, p_obj.filter !== undefined ? p_obj.filter : this.currentFilter),
           loadingMessage = null,
           timerShowLoadingMessage = null,
           me = this,
           params =
           {
              filter: successFilter
           };
        this._addSortingToNewFilter(params.filter, this.sortAsc, this.sortColumn);

        // Clear the current document list if the data webscript is taking too long
        var fnShowLoadingMessage = function DataGrid_fnShowLoadingMessage()
        {
           Alfresco.logger.debug("DataGrid__uDG_fnShowLoadingMessage: slow data webscript detected.");
           // Check the timer still exists. This is to prevent IE firing the event after we cancelled it. Which is "useful".
           if (timerShowLoadingMessage)
           {
              loadingMessage = Alfresco.util.PopupManager.displayMessage(
              {
                 displayTime: 0,
                 text: '<span class="wait">' + $html(this.msg("message.loading")) + '</span>',
                 noEscape: true
              });

              if (YAHOO.env.ua.ie > 0)
              {
                 this.loadingMessageShowing = true;
              }
              else
              {
                 loadingMessage.showEvent.subscribe(function()
                 {
                    this.loadingMessageShowing = true;
                 }, this, true);
              }
           }
        };

        // Reset the custom error messages
        this._setDefaultDataTableErrors(this.widgets.dataTable);

        // More Actions menu no longer relevant
        this.showingMoreActions = false;

        // Slow data webscript message
        this.loadingMessageShowing = false;
        timerShowLoadingMessage = YAHOO.lang.later(this.options.loadingMessageDelay, this, fnShowLoadingMessage);

        var destroyLoaderMessage = function DataGrid__uDG_destroyLoaderMessage()
        {
           if (timerShowLoadingMessage)
           {
              // Stop the "slow loading" timed function
              timerShowLoadingMessage.cancel();
              timerShowLoadingMessage = null;
           }

           if (loadingMessage)
           {
              if (this.loadingMessageShowing)
              {
                 // Safe to destroy
                 loadingMessage.destroy();
                 loadingMessage = null;
              }
              else
              {
                 // Wait and try again later. Scope doesn't get set correctly with "this"
                 YAHOO.lang.later(100, me, destroyLoaderMessage);
              }
           }
        };

        var successHandler = function DataGrid__uDG_successHandler(sRequest, oResponse, oPayload)
        {
           destroyLoaderMessage();
           // Updating the DotaGrid may change the item selection
           var fnAfterUpdate = function DataGrid__uDG_sH_fnAfterUpdate()
           {
              Bubbling.fire("selectedFilesChanged");
           };
           this.afterDataGridUpdate.push(fnAfterUpdate);

           Alfresco.logger.debug("currentFilter was:", this.currentFilter, "now:", successFilter);
           this.currentFilter = successFilter;
           this.currentPage = p_obj.page || 1;
           Bubbling.fire("filterChanged", successFilter);
           this.widgets.dataTable.onDataReturnReplaceRows.call(this.widgets.dataTable, sRequest, oResponse, oPayload);
//
           this.widgets.dataSource.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);
           this.widgets.dataSource.connMgr.setDefaultXhrHeader(Alfresco.util.Ajax.JSON);
        };

        var failureHandler = function DataGrid__uDG_failureHandler(sRequest, oResponse)
        {
           destroyLoaderMessage();
           // Clear out deferred functions
           this.afterDataGridUpdate = [];

           if (oResponse.status == 401)
           {
              // Our session has likely timed-out, so refresh to offer the login page
              window.location.reload(true);
           }
           else
           {
              try
              {
                 var response = YAHOO.lang.JSON.parse(oResponse.responseText);
                 this.widgets.dataTable.set("MSG_ERROR", response.message);
                 this.widgets.dataTable.showTableMessage(response.message + ". " + Alfresco.util.message("message.info.refresh"), YAHOO.widget.DataTable.CLASS_ERROR);
                 if (oResponse.status == 404)
                 {
                    // Site or container not found - deactivate controls
                    Bubbling.fire("deactivateAllControls");
                 }
              }
              catch(e)
              {
                 this._setDefaultDataTableErrors(this.widgets.dataTable);
              }
           }
        };

        // Update the DataSource
        var requestParams = this._buildDataGridParams(params);
        Alfresco.logger.debug("DataSource requestParams: ", requestParams);

        // TODO: No-cache? - add to URL retrieved from DataSource
        // "&noCache=" + new Date().getTime();

        if (Alfresco.util.CSRFPolicy.isFilterEnabled())
        {
           this.widgets.dataSource.connMgr.initHeader(Alfresco.util.CSRFPolicy.getHeader(), Alfresco.util.CSRFPolicy.getToken(), false);
        }
        this.widgets.dataSource.connMgr.setDefaultPostHeader(Alfresco.util.Ajax.JSON);
        this.widgets.dataSource.connMgr.setDefaultXhrHeader(Alfresco.util.Ajax.JSON);

        this.widgets.dataSource.sendRequest(YAHOO.lang.JSON.stringify(requestParams),
        {
           success: successHandler,
           failure: failureHandler,
           scope: this
        });
     },

     onDataGridPaging: function DataGrid_onDataGridPaging(paginator, args)
     {
    	this.currentPage = paginator.page;
        this._updateDataGrid.call(this,
        {
           page: this.currentPage
        });
     },

     onDataGridSort: function DataGrid_onDataGridSort(e, dataGrid)
     {
    	YAHOO.util.Event.stopEvent(e.event);
    	var column = dataGrid.widgets.dataTable.getColumn(e.target.cellIndex);
    	if (!column.sortable) {
    		this._removeSortingClasses();
    		return; // not sortable
    	}

    	var sortField = column.key;
    	if (dataGrid.sortColumn && dataGrid.sortColumn === sortField) {
    		dataGrid.sortAsc = (dataGrid.sortAsc) ? false : true;
    	} else {
    		dataGrid.sortColumn = sortField;
    		dataGrid.sortAsc = true;
    	}
   		dataGrid.currentFilter.sortField = sortField;
   		dataGrid.currentFilter.sortAsc = dataGrid.sortAsc;

   		dataGrid._updateDataGrid.call(dataGrid,
	    {
   			page: dataGrid.currentPage
	    });

   		this._addSortingClasses(e.target);

     },

     _removeSortingClasses : function ExtDataGrid_removeSortingClasses() {
		var addClass = YAHOO.widget.DataTable.CLASS_DESC;
		var removeClass = YAHOO.widget.DataTable.CLASS_ASC;

		var headers = this.widgets.dataTable.getColumnSet().flat;
		for (var i = 0; headers.length > i; i++) {
			var header = headers[i].getThEl();
			if (Dom.hasClass(header, addClass))
				Dom.removeClass(header, addClass);
			if (Dom.hasClass(header, removeClass))
				Dom.removeClass(header, removeClass);
		}
	},
     _addSortingClasses : function ExtDataGrid_addSortingClasses(elm) {
    	this._removeSortingClasses();
		if (this.sortAsc) {
			var addClass = YAHOO.widget.DataTable.CLASS_ASC;
			var removeClass = YAHOO.widget.DataTable.CLASS_DESC;
		} else {
			var addClass = YAHOO.widget.DataTable.CLASS_DESC;
			var removeClass = YAHOO.widget.DataTable.CLASS_ASC;
		}
		Dom.addClass(elm, addClass);
		if (Dom.hasClass(elm, removeClass))
			Dom.removeClass(elm, removeClass);
	},


     _addSortingToNewFilter : function ExtDataGrid_addSortingToNewFilter(newFilter, sortAsc, sortField)
     {
    	 if (null == newFilter.sortField) {
    		 newFilter.sortField = sortField;
    	 }
    	 if (null == newFilter.sortAsc) {
    		 newFilter.sortAsc = sortAsc;
    	 }

     },

	     /**
	      * Renders Query List metadata, i.e. title and description
	      *
	      * @method renderQueryListMeta
	      */
	     renderQueryListMeta: function DataGrid_renderQueryListMeta()
	     {
	        if (!YAHOO.lang.isObject(this.datalistMeta))
	        {
	           return;
	        }

	        Alfresco.util.populateHTML(
	           [ this.id + "-title", $html(this.datalistMeta.title) ],
	           [ this.id + "-description", $links($html(this.datalistMeta.description, true)) ]
	        );
	     },


       /**
        * Return data type-specific sorter
        *
        * @method getSortFunction
        * @return {function} Function to sort column by
        */
       getSortFunction: function DataGrid_getSortFunction()
       {
           /**
            * Data Type custom sorter
            *
            * @method sortFunction
            * @param a {object} Sort record a
            * @param b {object} Sort record b
            * @param desc {boolean} Ascending/descending flag
            * @param field {String} Field to sort by
            */
           return function DataGrid_sortFunction(a, b, desc, field)
           {
               var fieldA = a.getData().itemData[field],
                   fieldB = b.getData().itemData[field];

               if (YAHOO.lang.isArray(fieldA))
               {
                   fieldA = fieldA[0];
               }
               if (YAHOO.lang.isArray(fieldB))
               {
                   fieldB = fieldB[0];
               }

               // Deal with empty values
               if (!YAHOO.lang.isValue(fieldA))
               {
                   return (!YAHOO.lang.isValue(fieldB)) ? 0 : 1;
               }
               else if (!YAHOO.lang.isValue(fieldB))
               {
                   return -1;
               }

               var valA = fieldA.value,
                   valB = fieldB.value;

               // Lyse change: use the display value if it exists and is a string, the original code would only use displayValue if fieldA starts with "workspace://SpacesStore"
               if (typeof valA === "string" && YAHOO.lang.isValue(fieldA.displayValue))
               {
                   valA = fieldA.displayValue;
                   valB = fieldB.displayValue;
               }

               return YAHOO.util.Sort.compare(valA, valB, desc);
           };
       }

   }, true);

   function getTaskResponsibleFromRecord(oRecord){
        var responsiblesHtml = '';
        if (oRecord.getData("responsibles") != null) {

            for (i = 0; i < oRecord.getData("responsibles").length; i++) {
                var responsible = oRecord.getData("responsibles")[i];
                responsiblesHtml += responsible.name;
                if (i < oRecord.getData("responsibles").length-1){
                        responsiblesHtml += ", ";
                }
            }
        }
        return responsiblesHtml;
   }

   function isHTML(str) {
	    var a = document.createElement('div');
	    a.innerHTML = str;
	    for (var c = a.childNodes, i = c.length; i--; ) {
	        if (c[i].nodeType === 1) return true;
	    }
	    return false;
   }

   function strip(html)
   {
      var tmp = document.createElement("DIV");
      tmp.innerHTML = html;
      return tmp.textContent || tmp.innerText || "";
   }

})();
