// @overridden projects/slingshot/source/web/components/site-members/site-members.js

/**
 * _renderCellDescription override
 * Change from original: added more detailed data about members.
 * icluding groups, title and organization.
 */
(function(_renderCellDescription)
{
   var $html = Alfresco.util.encodeHTML;
      /**
       * Description/detail custom datacell formatter
       *
       * @method renderCellDescription
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */

     Alfresco.SiteMembers.prototype.renderCellDescription = function Lyse_SiteMembers_renderCellDescription(elCell, oRecord, oColumn, oData)
      {
         // Currently rendering all results the same way
         var userName = oRecord.getData("userName"),
            name = userName,
            firstName = oRecord.getData("firstName"),
            lastName = oRecord.getData("lastName"),
            userStatus = oRecord.getData("userStatus"),
            userStatusTime = oRecord.getData("userStatusTime");

         if ((firstName !== undefined) || (lastName !== undefined))
         {
            name = firstName ? firstName + " " : "";
            name += lastName ? lastName : "";
         }
         
         var url = Alfresco.constants.URL_PAGECONTEXT + "user/" + userName + "/profile",
            title = oRecord.getData("jobtitle") ? oRecord.getData("jobtitle") : "",
            organization = oRecord.getData("organization") ? oRecord.getData("organization") : "",
            desc = '<div id="people-wrapper"><h3><a href="' + url + '">' + $html(name) + '</a></h3>';

         if (title.length > 0)
         {
            desc += '<div><span class="attr-name">' + this.msg('label.title') + ': </span>&nbsp;<span class="attr-value">' + $html(title) + '</span></div>';
         }
         if (organization.length > 0)
         {
            desc += '<div><span class="attr-name">' + this.msg('label.company') + ':</span>&nbsp;<span class="attr-value">' + $html(organization) + '</span></div>';
         }
         if (typeof userStatus != "undefined" && userStatus.length > 0)
         {
            desc += '<div class="user-status">' + $html(userStatus) + ' <span>(' + Alfresco.util.relativeTime(Alfresco.util.fromISO8601(userStatusTime.iso8601)) + ')</span></div>';
         }

         elCell.innerHTML = desc + '</div>';
      };
         
}(Alfresco.SiteMembers.prototype.renderCellDescription));

/**
 * __setupDataTable override
 * Change from original: removed columns, custom field for members-groups
 */
(function(__setupDataTable) {
   Alfresco.SiteMembers.prototype._setupDataTable= function Lyse_SiteMembers_setupDataTable()
   {
   /**
    * DataTable Cell Renderers
    *
    * Each cell has a custom renderer defined as a custom function. See YUI documentation for details.
    * These MUST be inline in order to have access to the Alfresco.SiteMembers class (via the "me" variable).
    */
   var me = this;

   // DataTable column defintions
   var columnDefinitions =
       [
          { key: "userName", label: "User Name", sortable: false, formatter: this.bind(this.renderCellAvatar), width: 64 },
          { key: "bio", label: "Bio", sortable: false, formatter: this.bind(this.renderCellDescription) },
          { key: "role", label: "Groups", sortable: false, formatter: this.bind(this.renderCellRoleSelect), width: 500 }
       ];

   // DataTable definition
   this.widgets.dataTable = new YAHOO.widget.DataTable(this.id + "-members", columnDefinitions, this.widgets.dataSource,
       {
          renderLoopSize: 32,
          initialLoad: false,
          MSG_EMPTY: '<span style="white-space: nowrap;">' + this.msg("site-members.enter-search-term") + '</span>'
       });

   // Override abstract function within DataTable to set custom error message
   this.widgets.dataTable.doBeforeLoadData = function SiteMembers_doBeforeLoadData(sRequest, oResponse, oPayload)
   {
      if (oResponse.error)
      {
         try
         {
            var response = YAHOO.lang.JSON.parse(oResponse.responseText);
            me.widgets.dataTable.set("MSG_ERROR", response.message);
         }
         catch(e)
         {
            me._setDefaultDataTableErrors(me.widgets.dataTable);
         }
      }
      else if (oResponse.results)
      {
         if (oResponse.results.length === 0)
         {
            me.widgets.dataTable.set("MSG_EMPTY", '<span style="white-space: nowrap;">' + me.msg("message.empty") + '</span>');
         }
         me.renderLoopSize = Alfresco.util.RENDERLOOPSIZE;
      }

      // Must return true to have the "Searching..." message replaced by the error message
      return true;
   };
};
}(Alfresco.SiteMembers.prototype._setupDataTable));

/**
 * _renderCellRoleSelect override
 * Change from original: populate custom field with users group(s)
 */
(function(_renderCellRoleSelect) {
   Alfresco.SiteMembers.prototype.renderCellRoleSelect = function Lyse_SiteMembers_renderCellRoleSelect(elCell, oRecord, oColumn, oData) {
       Dom.setStyle(elCell.parentNode, "width", oColumn.width + "px");
       Dom.setStyle(elCell.parentNode, "text-align", "right");
       Dom.addClass(elCell, "overflow");

	   oRecord.getData("role").forEach(
	     function(item, index){
	       elCell.innerHTML +=  '<div class="lyse-group-item">' + item + '</div>';
         }
	   );
   
   }
}(Alfresco.SiteMembers.prototype.renderCellRoleSelect));


/**
 * onReady override
 * Change from original: Changed webscript to custom webscript.
 */
(function(onReady) {
   Alfresco.SiteMembers.prototype.onReady = function Lyse_SiteMembers_onReady() {
   var me = this;
   
   // DataSource definition
   var uriSearchResults = Alfresco.constants.PROXY_URI + "api/lyse/site/" + me.options.siteId + "/usersinsite?";
   this.widgets.dataSource = new YAHOO.util.DataSource(uriSearchResults,
   {
      responseType: YAHOO.util.DataSource.TYPE_JSON,
      connXhrMode: "queueRequests",
      responseSchema:
      {
          resultsList: "items"
      }
   });
   this.widgets.dataSource.doBeforeParseData = function SiteMembers_doBeforeParseData(oRequest , oFullResponse)
   {
      var updatedResponse = oFullResponse.data;
         
      if (oFullResponse.data)
      {
         var items = [],
            memberData, member;
         
         // create a data format that the DataTable can use
         for (var x = 0, xx = oFullResponse.data.length; x < xx; x++)
         {
            memberData = oFullResponse.data[x];
            
            // create object to represent member
            member = memberData.authority;
            member.role = memberData.role;
            member.isMemberOfGroup = memberData.isMemberOfGroup;
            
            // add member to list
            items.push(member);
         }
         
         // Sort the memeber list by name
         items.sort(function (membership1, membership2)
         {
            var name1 = membership1.firstName + membership1.lastName,
               name2 = membership2.firstName + membership2.lastName;
            
            return (name1 > name2) ? 1 : (name1 < name2) ? -1 : 0;
         });
         
         // we need to wrap the array inside a JSON object so the DataTable is happy
         updatedResponse =
         {
            "items": items
         };
      }
      
      return updatedResponse;
   };
   
   // determine if current user is a site administrator
   if (me.options.currentUserRole !== undefined &&
       me.options.currentUserRole === "SiteManager")
   {
      this.isCurrentUserSiteAdmin = true;
   }
   
   // setup of the datatable.
   this._setupDataTable();
   
   // setup the buttons
   this.widgets.searchButton = Alfresco.util.createYUIButton(this, "button", this.onSearch);
   if (Dom.get(this.id + "-invitePeople"))
   {
      this.widgets.invitePeople = Alfresco.util.createYUIButton(this, "invitePeople", null,
      {
         type: "link"
      });
   }

   // register the "enter" event on the search text field
   var searchInput = Dom.get(this.id + "-term"),
      enterListener = new YAHOO.util.KeyListener(searchInput,
   {
      keys:13
   },
   {
      fn: function() 
      {
         me.onSearch();
      },
      scope: this,
      correctScope: true
   }, "keydown");
   enterListener.enable();
   
   if (this.options.error)
   {
      enterListener.disable();
      this.widgets.dataTable.set("MSG_ERROR", this.options.error);
      this.widgets.dataTable.showTableMessage(this.options.error, YAHOO.widget.DataTable.CLASS_ERROR);
      // Deactivate controls
      YAHOO.Bubbling.fire("deactivateAllControls");
   }
   
   // Set initial focus?
   if (this.options.setFocus)
   {
      searchInput.focus();
   }
   
   // This is a quick solution for showing all members of the site. The default search string 
   // is empty and equates to a global search.
   if (window.location.hash === "#showall" || this.options.minSearchTermLength === 0)
   {
      this.onSearch();
   }
   
   // Finally show the component body here to prevent UI artifacts on YUI button decoration
   Dom.setStyle(this.id + "-body", "visibility", "visible");
   }
}(Alfresco.SiteMembers.prototype.onReady));



