// @overridden projects/slingshot/source/web/components/workflow/task-edit-header.js
/**
 * Copyright (C) 2005-2013 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * TaskEditHeader component.
 *
 * @namespace Alfresco.component
 * @class Alfresco.component.TaskEditHeader
 */
(function()
{
   /**
    * YUI Library aliases
    */
   var Dom = YAHOO.util.Dom,
      Event = YAHOO.util.Event,
      Selector = YAHOO.util.Selector;

  /**
    * Alfresco Slingshot aliases
    */
    var $html = Alfresco.util.encodeHTML,
       $hasEventInterest = Alfresco.util.hasEventInterest,
       $siteURL = Alfresco.util.siteURL;



   /**
    * TaskEditHeader constructor.
    *
    * @param {String} htmlId The HTML id of the parent element
    * @return {Alfresco.component.TaskEditHeader} The new TaskEditHeader instance
    * @constructor
    */
   Alfresco.component.LyseTaskEditHeader = function TaskEditHeader_constructor(htmlId)
   {
      Alfresco.component.LyseTaskEditHeader.superclass.constructor.call(this, htmlId, ["button"]);

      // Re-register with our own name
      this.name = "Alfresco.component.LyseTaskEditHeader";

      // Instance variables
      this.options = YAHOO.lang.merge(this.options, Alfresco.component.LyseTaskEditHeader.superclass.options);
      Alfresco.util.ComponentManager.reregister(this);
      this.isRunning = false;
      this.taskId = null;

      /* Decoupled event listeners */
      YAHOO.Bubbling.on("taskDetailedData", this.onTaskDetailedData, this);

      return this;
   };

   YAHOO.extend(Alfresco.component.LyseTaskEditHeader, Alfresco.component.ShareFormManager,
   {

      /**
       * Keeps track if this component is running an action or not
       *
       * @property isRunning
       * @type Boolean
       */
      isRunning: false,

      /**
       * The task instance id
       *
       * @property taskId
       * @type String
       */
      taskId: null,

      /**
       * The referrer value query string parameter
       *
       * @property referrerValue
       * @type String
       */
      referrerValue: null,


      /**
       * Fired by YUI when parent element is available for scripting.
       * Template initialisation, including instantiation of YUI widgets and event listener binding.
       *
       * @method onReady
       */
      onReady: function TEH_onReady()
      {
         YAHOO.Bubbling.on("formContentReady", this.hideSubmitButtonCheck, this);

          // Load in the People Finder component from the server
         Alfresco.util.Ajax.request(
         {
            url: Alfresco.constants.URL_SERVICECONTEXT + "components/people-finder/people-finder?site=" + Alfresco.constants.SITE,
            dataObj:
            {
               htmlid: this.id + "-peopleFinder"
            },
            successCallback:
            {
               fn: this.onPeopleFinderLoaded,
               scope: this
            },
            failureMessage: "Could not load People Finder component",
            execScripts: true
         });

         // Format dates (datatype could be 'date' or 'datetime')
         var dateEls = document.querySelectorAll('[data-datatype^="date"]');
         for (var i = 0; i < dateEls.length; i++) {
            var formatString = 'date-format.default'; // Default to date time format
            if (dateEls[i].getAttribute('data-datatype') === 'date') {
               formatString = 'date-format.mediumDate';
            }
            dateEls[i].innerHTML = Alfresco.util.formatDate(dateEls[i].innerHTML, Alfresco.util.message(formatString));
         };


      },

      /**
       * Called when the people finder template has been loaded.
       * Creates a dialog and inserts the people finder for choosing assignees.
       *
       * @method onPeopleFinderLoaded
       * @param response The server response
       */
      onPeopleFinderLoaded: function TEH_onPeopleFinderLoaded(response)
      {
           // Inject the component from the XHR request into it's placeholder DIV element
           var finderDiv = Dom.get(this.id + "-peopleFinder");
           finderDiv.innerHTML = response.serverResponse.responseText;

           // Create the Assignee dialog
           this.widgets.reassignPanel = Alfresco.util.createYUIPanel(this.id + "-reassignPanel");

           // Find the People Finder by container ID
           this.widgets.peopleFinder = Alfresco.util.ComponentManager.get(this.id + "-peopleFinder");
           Event.onContentReady("assoc_lysewf_relatedDatalistItem", function(ctx){

	          Alfresco.util.Ajax.jsonGet(
	 	      {
	 	        url: Alfresco.constants.PROXY_URI + "/api/lyse/" + Alfresco.util.NodeRef(this.value).uri + "/site-shortname",
	 	        responseContentType : "application/json",
	 	        successCallback:
	 	        {
	 	            fn: function (response)
	 	            {
	 	               siteShortname = response.json.shortName;

	 	               // Set the correct options for our use
                       ctx.widgets.peopleFinder.setOptions(
	 	               {
	 	            	  siteId: siteShortname,
	 	                  singleSelectMode: true,
	 	                  addButtonLabel: ctx.msg("button.select")
	 	               });

	 	              // Reinitialize dataSource definition
	 	              var peopleSearchUrl = Alfresco.constants.PROXY_URI + YAHOO.lang.substitute(ctx.widgets.peopleFinder.options.dataWebScript, ctx.widgets.peopleFinder.options);
	 	              peopleSearchUrl += (peopleSearchUrl.indexOf("?") < 0) ? "?" : "&";
                      ctx.widgets.peopleFinder.widgets.dataSource = new YAHOO.util.DataSource(peopleSearchUrl,
	 	              {
	 	                 responseType: YAHOO.util.DataSource.TYPE_JSON,
	 	                 connXhrMode: "queueRequests",
	 	                 responseSchema:
	 	                 {
	 	                     resultsList: "people"
	 	                 }
	 	              });


	 	               // Make sure we listen for events when the user selects a person
	 	               YAHOO.Bubbling.on("personSelected", ctx.onPersonSelected, ctx);
	 	            },
	 	            scope: ctx
	 	        }
	 	     });
         }, this);

      },

      /**
       * Called when the user has selected an assigne from the people finder.
       *
       * @method onPersonSelected
       * @param e DomEvent
       * @param args Event parameters (depends on event type)
       */
      onPersonSelected: function TEH_onPersonSelected(e, args)
      {
         // This is a "global" event so we ensure the event is for the current panel by checking panel visibility.
         if ($hasEventInterest(this.widgets.peopleFinder, args))
         {
            this.widgets.reassignPanel.hide();
            this._updateTaskProperties(
            {
               "cm_owner": args[1].userName
            }, "reassign");
         }
      },

      /**
       * Event handler called when the "taskDetailedData" event is received
       *
       * @method: onTaskDetailedData
       */
      onTaskDetailedData: function TEH_onTaskDetailedData(layer, args)
      {

         var task = args[1];

         // Save task id so we can use it when invoking actions later
         this.taskId = task.id;
         this.workflowId = task.workflowInstance.id;
         this.nodeRef = task.properties.lysewf_relatedDatalistItem;

         // Display actions and create yui buttons
         Selector.query("h1 span", this.id, true).innerHTML = $html(task.title);

         // ALF-13115 fix, inform user that this task has been completed
           if (!task.isEditable)
           {
              Alfresco.util.PopupManager.displayMessage(
              {
                 text: this.msg("message.task.completed"),
                 displayTime: 2
              });

            YAHOO.lang.later(2000, this, function()
            {
                if (window.opener) {
                  // Lyse change
                  Lyse.util.closeWindow();
                }
                else if(this.nodeRef)
                {
                  location.href = location.origin + "/share/page/lyse-datalist-details?nodeRef=" + this.nodeRef
                }
                else {
                    if (this.taskId != "") {
                        var datalistItem = Dom.get("assoc_lysewf_relatedDatalistItem");
                        var datalistItemNodeRef = new Alfresco.util.NodeRef(Alfresco.util.NodeRef(datalistItem.value).uri);
                        Alfresco.util.Ajax.request(
                            {
                                url: Alfresco.constants.PROXY_URI + "api/lyse/" + datalistItemNodeRef + "/datalist-item",
                                successCallback: {
                                    fn: function (response) {
                                        window.location = Alfresco.constants.URL_CONTEXT + "page/site/" + response.json.shortName + "/data-lists?list=" + response.json.datalistName;
                                    },
                                    scope: this
                                },
                                failureMessage: "Redirect failed"
                            });
                    }else{
                        Lyse.util.navigateForward();
                }
               }
            }, []);
         }

         // Lyse change: allow direct reassigning of a claimed pooled task as well
         if (task.isReassignable || task.isReleasable)
         {
            // Task is reassignable
            this.widgets.reassignButton = Alfresco.util.createYUIButton(this, "reassign", this.onReassignButtonClick);
            Dom.removeClass(Selector.query(".actions .reassign", this.id), "hidden");
         }

         if (task.isClaimable)
         {
            // Task is claimable
            this.widgets.claimButton = Alfresco.util.createYUIButton(this, "claim", this.onClaimButtonClick);
            Dom.removeClass(Selector.query(".actions .claim", this.id), "hidden");
            Dom.removeClass(Selector.query(".unassigned-message", this.id), "hidden");
         }

         if (task.isReleasable)
         {
            // Task is releasable
            this.widgets.releaseButton = Alfresco.util.createYUIButton(this, "release", this.onReleaseButtonClick);
            Dom.removeClass(Selector.query(".actions .release", this.id), "hidden");
         }

	    this.widgets.viewAllButton = Alfresco.util.createYUIButton(this, "view-all");
	    var nodeRef = task.properties.lysewf_relatedDatalistItem;
        var pageName, attachmentFolder, me = this;

        // Defined calls as functions so we can call them in the correct order IF noderef is missing!
        var activateLink = function(){
	        // Fetch nodeType & activate button
			Alfresco.util.Ajax.jsonGet( {
	           url: Alfresco.constants.PROXY_URI + "lyse/api/node-type?nodeRef=" + nodeRef,
	           successCallback: {
	            fn: function (response){
	               if (response.json !== undefined) {
	            	    var nodeType = response.json.nodeType;
	            	    if (nodeType == 'cm:content'){
	            	    	pageName = 'document-details';
	            	    }else if (nodeType == 'cm:folder'){
	            	    	pageName = 'folder-details';
	            	    }else {
	            	    	pageName = 'lyse-datalist-details';
			            }
	        			Dom.removeClass(Selector.query(".actions .view-all", me.id), "hidden");
	               }
	            }, scope: this }
	        });
        };
        var checkDoclib = function() {
	        // Fetch & check doclib
		    Alfresco.util.Ajax.jsonGet( {
		      url: Alfresco.constants.PROXY_URI + "api/lyse/attachment-folder?nodeRef=" + nodeRef,
		      successCallback: {
		        fn: function (response) {
		          if (response.json !== undefined && response.json.nodeRef !== "") {
		            //Success, this datalist item has an attachment folder
		            attachmentFolder = response.json.nodeRef;
		          }
		        }, scope: this }
		    });
	    };

        // Check nodeRef, if null fetch from workflowid
        if(!nodeRef){
      	  var workflowId = YAHOO.util.History.getQueryStringParameter('workflowId');
      	  Alfresco.util.Ajax.jsonGet( {
            url: Alfresco.constants.PROXY_URI + "/api/lyse/noderef-from-workflowid/" + workflowId,
            successCallback: {
              fn: function (response){
                if (response.json !== undefined && response.json.nodeRef !== "") {
    			  nodeRef = response.json.nodeRef;
		          checkDoclib();
		          activateLink();
                }
              },scope: this }
          });
        }else{
    	  checkDoclib();
    	  activateLink();
        }
        var workflowId;
        if(nodeRef)
        {
          Alfresco.util.Ajax.jsonGet({
            url: Alfresco.constants.PROXY_URI + "api/metadata?nodeRef=" + nodeRef,
            responseContentType: "application/json",
            successCallback: {
              fn: function (response) {
                workflowId = response.json.properties["{http://ns.lyse.no/model/datalist/1.0}workflowId"];
              },
              scope: this
            },
          });
        }

        this.widgets.viewAllButton.on("click",function() {
          var urlContext = Alfresco.constants.URL_PAGECONTEXT;
          if(Alfresco.constants.SITE) {
            urlContext += 'site/' + Alfresco.constants.SITE + '/';
          }
          if (attachmentFolder && workflowId) {
            window.open(YAHOO.lang.substitute(urlContext + pageName + "?workflowId=" + workflowId + "&docLib=" + attachmentFolder));
          } else if (workflowId) {
            window.open(YAHOO.lang.substitute(urlContext + pageName + "?workflowId=" + workflowId));
          }
          else if(nodeRef && attachmentFolder)
          {
            window.open(YAHOO.lang.substitute(urlContext + pageName + "?nodeRef=" + nodeRef + "&docLib=" + attachmentFolder));
          }
          else if(nodeRef)
          {
            window.open(YAHOO.lang.substitute(urlContext + pageName + "?nodeRef=" + nodeRef));
          }

        });
      },

      /**
       * Event handler called when the "release" button is clicked
       *
       * @method: onReleaseButtonClick
       */
      onReleaseButtonClick: function TEH_onReleaseButtonClick(layer, args)
      {
         this._updateTaskProperties(
         {
            "cm_owner": null
         }, "release");
      },

      /**
       * Event handler called when the "claim" button is clicked
       *
       * @method: onClaimButtonClick
       */
      onClaimButtonClick: function TEH_onClaimButtonClick(layer, args)
      {
         this._updateTaskProperties(
         {
            "cm_owner": Alfresco.constants.USERNAME
         }, "claim");
      },

      /**
       * Event handler called when the "reassign" button is clicked
       *
       * @method: onReassignButtonClick
       */
      onReassignButtonClick: function TEH_onReassignButtonClick(layer, args)
      {
         this.widgets.peopleFinder.clearResults();
         this.widgets.reassignPanel.show();
      },

      /**
       * Updates a task property
       *
       * @method: _updateTaskProperties
       * @private
       */
      _updateTaskProperties: function TEH__updateTaskProperties(properties, action)
      {
         this._disableActionButtons(true);
         YAHOO.lang.later(2000, this, function()
         {
            if (this.isRunning)
            {
               if (!this.widgets.feedbackMessage)
               {
                  this.widgets.feedbackMessage = Alfresco.util.PopupManager.displayMessage(
                  {
                     text: this.msg("message." + action),
                     spanClass: "wait",
                     displayTime: 0
                  });
               }
               else if (!this.widgets.feedbackMessage.cfg.getProperty("visible"))
               {
                  this.widgets.feedbackMessage.show();
               }
            }
         }, []);

         // Run rules for folder (and sub folders)
         if (!this.isRunning)
         {
            this.isRunning = true;

            // Start/stop inherit rules from parent folder
            Alfresco.util.Ajax.jsonPut(
            {
               url: Alfresco.constants.PROXY_URI_RELATIVE + "api/task-instances/" + this.taskId,
               dataObj: properties,
               successCallback:
               {
                  fn: function(response, action)
                  {
                     this.isRunning = false;
                     var data = response.json.data;
                     if (data)
                     {
                        var finish = (function (msg) {
                           Alfresco.util.PopupManager.displayMessage(
                             {
                                text: msg||this.msg("message." + action + ".success")
                             });
                           YAHOO.lang.later(3000, this, function (data) {
                              if (data.owner && data.owner.userName == Alfresco.constants.USERNAME) {
                                 // MNT-11418: Save previous URL
                                 if (YAHOO.env.ua.chrome > 0) {
                                    var prevUrl = (history.state && history.state.previous_url) ? history.state.previous_url : document.referrer;
                                    var currUrl = document.location.href;
                                    if (prevUrl && prevUrl != "") {
                                       history.pushState({
                                          'previous_url': prevUrl,
                                          'current_url': currUrl
                                       }, document.title, "");
                                    }
                                 }
                              }
                              document.location.reload();
                           }, data);
                        }).bind(this);

                        if(action=="reassign")
                        {
                           Alfresco.util.Ajax.jsonGet(
                             {
                                url: Alfresco.constants.PROXY_URI + "lyse/api/set-responsible/" + this.taskId,
                                dataObj: properties,
                                successCallback:
                                {
                                   fn:function()
                                   {
                                      finish();
                                   }
                                },
                                failureCallback:
                                {
                                   fn:function()
                                   {
                                      finish("Failed to set responsible on associated datalist item.");
                                   }
                                }
                             }
                           );
                        }
                        else
                        {
                           finish();
                        }
                     }
                  },
                  obj: action,
                  scope: this
               },
               failureCallback:
               {
                  fn: function(response)
                  {
                     this.isRunning = false;
                     this._disableActionButtons(false);
                     Alfresco.util.PopupManager.displayPrompt(
                     {
                        title: this.msg("message.failure"),
                        text: this.msg("message." + action + ".failure")
                     });
                  },
                  scope: this
               }
            });
         }
      },

      _disableActionButtons: function(disabled)
      {
         if (this.widgets.reassignButton)
         {
            this.widgets.reassignButton.set("disabled", disabled)
         }
         if (this.widgets.releaseButton)
         {
            this.widgets.releaseButton.set("disabled", disabled)
         }
         if (this.widgets.claimButton)
         {
            this.widgets.claimButton.set("disabled", disabled)
         }
      },

	  /**
	   *  We need to enable the button ONLY in MCC task "Review request for work permit"
	   *  Otherwise it should always be removed.
	   */
	  hideSubmitButtonCheck: function() {
	   	   var taskId = YAHOO.util.History.getQueryStringParameter('taskId');
	   	   if(taskId){
		       Alfresco.util.Ajax.jsonGet({
		           url: Alfresco.constants.PROXY_URI + "api/lyse/workflow-tasks/cansavedraft/" + taskId,
		           successCallback: {
		              fn:function(response) {
		              		var hide = true;
				            if(response.json) {

				            	// If task is current one and webscript says canSaveDraft === true.
							    if(response.json.canSaveDraft === true){
	          					    hide = false;
	          				    }
		               		}

		               		// Lets hide button by default if webscript didnt change hide variable.
		               		if(hide === false){
		               			this.showSubmitButton();
		               		}else{
		               			this.hideSubmitButton();
		               		}
		              },
		              scope: this
		           },
			      failureCallback: {
	        			// If some sort of failure, hide button.
						fn: this.hideSubmitButton(),
						scope: this
			      }
		        });
	        }else{
	        	// if missing task id hide button.
		    	this.hideSubmitButton();
	        }
	   },

       showSubmitButton: function()
       {
           var buttons = YAHOO.util.Selector.query('.form-buttons span.yui-submit-button');
           if (buttons.length > 0) {
              for (i = 0; i < buttons.length; i++) {
                 var button = buttons[i];
				 button.classList.remove("hidden");
               }
           }
       },
       hideSubmitButton: function()
       {
         var buttons = YAHOO.util.Selector.query('.form-buttons span.yui-submit-button');
         if (buttons.length > 0) {
           for (i = 0; i < buttons.length; i++) {
             var button = buttons[i];
             if(!button.classList.contains("hidden")){
               button.classList.add("hidden");
             }
           }
         }
       }

   });
})();
