var Lyse = Lyse || {};
Lyse.component = Lyse.component || {};

(function () {
    /**
     * YUI Library aliases
     */
    var Dom = YAHOO.util.Dom,
        Event = YAHOO.util.Event;

    /**
     * Alfresco Slingshot aliases
     */
    var $html = Alfresco.util.encodeHTML;

    /**
     * Constructor.
     *
     * @param {String} htmlId The HTML id of the parent element
     * @return {Lyse.component.WorkflowHistory} The new component instance
     * @constructor
     */
    Lyse.component.WorkflowHistory = function WorkflowHistory_constructor(htmlId) {
        Lyse.component.WorkflowHistory.superclass.constructor.call(this, "Lyse.component.WorkflowHistory", htmlId, ["container", "datasource", "datatable"]);
    };

    /**
     * Extend Alfresco.component.Base
     */
    YAHOO.extend(Lyse.component.WorkflowHistory, Alfresco.component.Base,
        {
            /**
             * Fired by YUI when parent element is available for scripting.
             * Component initialisation, including instantiation of YUI widgets and event listener binding.
             *
             * @method onReady
             */
            onReady: function WorkflowHistory_onReady() {
                this.createDataTable();
                if (Dom.get(this.id + "-close-button") !== null)
                {
                    Alfresco.util.createYUIButton(this, "close-button", null,
                        {
                            type: "submit"
                        });
                    Dom.get(this.id + "-close-button").name = "-";
                }

            },

            /**
             * Builds the url for the data table
             *
             * @method buildUrl
             * @retrun {string} The url for the data table
             */
            buildUrl: function WorkflowHistory_buildUrl() {
                var url = Alfresco.constants.PROXY_URI + "lyse/api/workflow-history?nodeRef={nodeRef}&property={property}&outcomeProperty={outcomeProperty}&state={state}";

                return YAHOO.lang.substitute(url,
                    {
                        nodeRef: encodeURIComponent(this.options.nodeRef),
                        property: encodeURIComponent(this.options.property),
                        outcomeProperty: encodeURIComponent(this.options.outcomeProperty),
                        state: encodeURIComponent(this.options.state)
                    });
            },

            /**
             * Creates the data table
             *
             * @method createDataTable
             */
            createDataTable: function WorkflowHistory_createDataTable() {

                this.widgets.alfrescoDataTable = new Alfresco.util.DataTable(
                    {
                        dataSource: {
                            url: this.buildUrl(),
                            config: {
                                responseSchema: {
                                    resultsList: 'workflows'
                                }
                            }
                        },
                        dataTable: {
                            container: this.id + "-workflow-history-table",
                            columnDefinitions: [
                                {key: "startDate", label: this.msg("column.startDate"), formatter: this.bind(this.renderCellDate)},
                                {key: "endDate", label: this.msg("column.endDate"), formatter: this.bind(this.renderCellDate)},
                                {key: "ownerLabel", label: this.msg("column.owner"), formatter: this.bind(this.renderCellOwner)},
                                {key: "outcomeLabel", label: this.msg("column.outcome"), formatter: this.bind(this.renderCellOutcome)}
                            ],
                            config: {
                                MSG_EMPTY: this.msg("no.result"),
                                className: "" // default = alfresco-datatable, which hides the column headings
                            }
                        }
                    });
                this.widgets.alfrescoDataTable.getDataTable().subscribe("cellClickEvent", function(e) {
                    var target = e.target,
                        record = this.getRecord(target),
                        url = YAHOO.lang.substitute("{context}lyse-datalist-details?workflowId={workflowId}&hideDL=true&showHistory=true",
                        {
                            context: Alfresco.constants.URL_PAGECONTEXT,
                            workflowId: record.getData("workflowId")
                        });
                    window.open(url);
                });
            },

            /**
             * Called by the DataTable to render the 'thumbnail' cell
             *
             * @method renderThumbnail
             * @param elCell {object}
             * @param oRecord {object}
             * @param oColumn {object}
             * @param oData {object|string}
             */
            renderCellDate: function WorkflowHistory_renderCellDate(elCell, oRecord, oColumn, oData) {
                if (oData) {
                    elCell.innerHTML = "<a href='#' style='text-decoration: underline;'>" + Alfresco.util.formatDate(Alfresco.util.fromISO8601(oData), this.msg("date-format.shortDate")) + " " + Alfresco.util.formatDate(Alfresco.util.fromISO8601(oData), this.msg("date-format.shortTime")) + "</a>" ;
                }
            },

            renderCellOutcome: function WorkflowHistory_renderCellOutcome(elCell, oRecord, oColumn, oData) {
                if (oRecord.getData("tasks")[0].outcomeLabel !== undefined) {
                    elCell.innerHTML = "<a href='#' style='text-decoration: underline;'>" + oRecord.getData("tasks")[0].outcomeLabel + "</a>";
                }
            },

            renderCellOwner: function WorkflowHistory_renderCellOwner(elCell, oRecord, oColumn, oData) {
                elCell.innerHTML = "<a href='#' style='text-decoration: underline;'>" + oRecord.getData("tasks")[0].ownerLabel + "</a>";
            }

        });
})();
