/**
 * Extension of forms-runtime with automatic redirect to the next workflowtask if one was
 * created and it is assigned to the current user. When no more tasks are available, the user will be redirected to its original start page.
 *
 * This is the rl-workflow-redirect module with some Lyse changes:
 * - redirect is NOT set to document.referrer, this way we can close the window via lyse-close-window-after-submit.js instead of going back to the referrer
 * - uses lyse-task-edit instead of task-edit page
 * - uses a different webscript to retrieve tasks for the current user for this workflow (also returns pooled tasks which the user can edit)
 */

(function(_submitInvoked) {
  var redirectCallback;
  /**
   * Help function to parse query parameters
   */
  _queryParam = function(key) {
    var vars = [], hash;
    var hashes = window.location.href.slice(window.location.href.indexOf('?') + 1).split('&');
    for ( var i = 0; i < hashes.length; i++) {
      hash = hashes[i].split('=');
      vars.push(hash[0]);
      vars[hash[0]] = hash[1];
    }
    return vars[key];
  };

  /**
   * Callback after workflow task details have been queried, when done, this will call the original callback.
   */
  var _workflowDetailsSuccessCallback = function(response, obj) {
    var queryParamRedir = _queryParam("redirect");
    var oldTaskId = _queryParam("taskId");
    var workflowId = _queryParam("workflowId");

    if (response.json.workflowTasks.length>0) {
      var task = response.json.workflowTasks[0];
      var taskId = task.id;
      var oldTaskTitle = YAHOO.util.Selector.query("h1 span", this.id, true).innerHTML;

      // Only redirect if the target task is not the same as the current one (in case of a save action) and the task title is not the same (same task with a new ID like VOR "Save as draf")
      if (oldTaskId != taskId && (oldTaskTitle == null || oldTaskTitle.length == 0 || oldTaskTitle !== Alfresco.util.encodeHTML(task.title))) {
        redirectCallback.scope.options.submitUrl = Alfresco.constants.URL_PAGECONTEXT + "lyse-task-edit?taskId=" + taskId +"&workflowId="+workflowId+ (queryParamRedir ? "&redirect=" + queryParamRedir : "");
      }
      else if (queryParamRedir != null) {
        redirectCallback.scope.options.submitUrl = decodeURIComponent(queryParamRedir);
      }      
    }
    else if (queryParamRedir != null) {
      redirectCallback.scope.options.submitUrl = decodeURIComponent(queryParamRedir);
    }
    redirectCallback.fn.call(redirectCallback.scope, obj.response);
  };
  
  /**
   * New callback function which is called after the submit button is pressed in a workflow form.
   * This function will query alfresco for the next workflow task.
   */
  var _newSuccessCallback = function(response) {
    var persistedObject = response.json.persistedObject;
    persistedObject = persistedObject.substr(persistedObject.indexOf("WorkflowInstance"));
    var startIndex = persistedObject.indexOf("id=activiti$")+3;
    var endIndex = persistedObject.indexOf(",", 17);
    var activitiId = persistedObject.substr(startIndex, endIndex-startIndex);
    Alfresco.util.Ajax.request({
      url : Alfresco.constants.PROXY_URI + "api/lyse/workflow-tasks/"+activitiId,
      successCallback : {
        fn : _workflowDetailsSuccessCallback,
        scope : this,
        obj : {response: response, activitiId : activitiId}              
      },
      failureMessage : Alfresco.util.message("message.failure"),
      scope : this,
      execScripts : false
    });
  };

  /**
   * Override of the _submitInvoked function for forms. Will redirect submits to a new callback function.
   */
  Alfresco.forms.Form.prototype._submitInvoked = function(event) {
    if (console) console.log("lyse-workflow-redirect submitInvoked");
    // Redirect the successcallback to a custom function
    if (redirectCallback===undefined || redirectCallback === null) {
      redirectCallback = this.ajaxSubmitHandlers.successCallback;
    }

    this.ajaxSubmitHandlers.successCallback = {
      fn : _newSuccessCallback,
      obj : null,
      scope : this
    };

    _submitInvoked.call(this, event);
  };
}(Alfresco.forms.Form.prototype._submitInvoked));
