/**
 * @module lyseproject/ProjectService
 * @extends module:alfresco/core/Core
 * @mixes module:alfresco/core/CoreXhr
 * @author Rasmus Melgaard
 */
// Define module requirements and callback...
define(["dojo/_base/declare",
        "alfresco/core/Core",
        "alfresco/core/CoreXhr",
        "alfresco/core/NotificationUtils",
        "dojo/request/xhr",
        "dojo/json",
        "dojo/_base/lang",
        "dojo/dom-construct",
        "alfresco/buttons/AlfButton"],
        function(declare, AlfCore, AlfXhr, NotificationUtils, xhr, JSON, lang, domConstruct, AlfButton) {
  YAHOO.namespace("lyse.extended.projectService");

   return declare([AlfCore, AlfXhr, NotificationUtils], {
      constructor: function(args) {
        this.alfSubscribe("LYSE_ADMIN_PROJECT", lang.hitch(this, "adminProject"));

        this.alfSubscribe("ALF_CREATE_PROJECT", lang.hitch(this, "createProject"));
        this.alfSubscribe("LYSE_EDIT_PROJECT", lang.hitch(this, "editProject"));

        this.alfSubscribe("ALF_CREATE_CONTRACTOR", lang.hitch(this, "createContractor"));
        this.alfSubscribe("LYSE_EDIT_CONTRACTOR", lang.hitch(this, "editContractor"));

        this.alfSubscribe("ALF_CREATE_INTERFACE", lang.hitch(this, "createInterface"));
        this.alfSubscribe("LYSE_EDIT_INTERFACE", lang.hitch(this, "editContractor"));

        this.alfSubscribe("ALF_CREATE_HSE", lang.hitch(this, "createHSE"));
        this.alfSubscribe("LYSE_EDIT_HSE", lang.hitch(this, "editHSE"));

        this.alfSubscribe("ALF_CREATE_MCC", lang.hitch(this, "createMCC"));
        this.alfSubscribe("LYSE_EDIT_MCC", lang.hitch(this, "editMCC"));

        this.alfSubscribe("ALF_CREATE_CIVIL", lang.hitch(this, "createCivil"));
        this.alfSubscribe("LYSE_EDIT_CIVIL", lang.hitch(this, "editCivil"));

        this.alfSubscribe("ALF_CREATE_EPC_ENGINEERING", lang.hitch(this, "createEPCEng"));
        this.alfSubscribe("LYSE_EDIT_EPC_ENGINEERING", lang.hitch(this, "createEPCEng"));

        this.alfSubscribe("ALF_CREATE_PS", lang.hitch(this, "createPS"));
        this.alfSubscribe("LYSE_EDIT_PS", lang.hitch(this, "editPS"));
          /*
          //Ensure the create-project.js javascript is included (Based on create-site.js)
          var _this = this;
           require([Alfresco.constants.URL_RESCONTEXT + "modules/lyse-create-project.js"], function() {
              _this.alfLog("log", "Create Project JavaScript resource loaded");
           });
           require([Alfresco.constants.URL_RESCONTEXT + "components/form/date.js"], function() {
              _this.alfLog("log", "Date resource loaded");
           });
           require([Alfresco.constants.URL_RESCONTEXT + "components/form/date-picker.js"], function() {
              _this.alfLog("log", "Date picker resource loaded");
           });
           require([Alfresco.constants.URL_RESCONTEXT + "components/object-finder/object-finder.js"], function() {
              _this.alfLog("log", "Object finder resource loaded");
           });
          */
      },

          /**
       * This method delegates site creation to the legacy YUI popup.
       *
       * @instance
       * @param {string} site
       */
      createProject: function lyse_ProjectService_createProject(config) {
         // TODO: When an edit site request is received we should display the edit site dialog.
         //       We need to wrap the existing YUI widget in a Dojo object.
         this.alfLog("log", "A request has been made to create a project: ", config);

         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreateProjectInstance === "function")
         {
            Alfresco.module.getCreateProjectInstance().show();
         }
         else
         {
            this.alfLog("error", "Could not find the 'Alfresco.module.getCreateProjectInstance' function - has 'lyse-create-project.js' been included in the page?");
         }
      },

     // Creates a dialog to lock project, change project phase and/or delete workflows.
     adminProject: function lyse_ProjectService_adminProject(config) {
       // Delete workflows function
       var confirmedCancelWorkflows = false;
       var cancelWorkflowCall = function(){
         if(confirmedCancelWorkflows === true){
           Alfresco.util.Ajax.jsonPut({
             url: Alfresco.constants.PROXY_URI + "api/lyse/delete/workflows/" + config.site,
             successCallback: {
               fn: function () {
                 this.destroy();
                 location.reload();
               },
               scope: this
             },
             failureCallback: {
               fn: function (){
                 this.destroy();
                 Alfresco.util.PopupManager.displayPrompt({
                   title: Alfresco.util.message("project.permission.workflow.title"),
                   text: Alfresco.util.message("project.permission.workflow.message")
                 });
               },
               scope: this
             }
           });
         } else {
           location.reload();
         }
       };

       /**
        * This method calls SetProjectPhasteProgress webscript on a 2s interval
        * an polls the current batch job progress (if it exists).
        */
       var getProgress = function(site) {
         Alfresco.util.Ajax.jsonGet({
           url: Alfresco.constants.PROXY_URI + "api/lyse/update/project/phase?site=" + site,
           responseContentType : "application/json",
           successCallback: {
             fn: function (response) {
               if (response.json !== undefined) {
                 var response = JSON.parse(response.serverResponse.responseText);
                 var text = response.text;
                 if(text === "In progress") {
                   setTimeout((function () {
                     getProgress(site);
                   }), 2000);
                 } else if (text === "Done"){
                   YAHOO.lyse.extended.projectService.wait.hide();
                   cancelWorkflowCall();
                 } else {
                    console.log("Help me");
                 }
               }
             },
             scope: this
           },
           failureCallback: {
             fn: function(response) {
               Alfresco.util.PopupManager.displayPrompt({
                 title: Alfresco.util.message("project.error.title"),
                 text: Alfresco.util.message("project.error.message")
               });
             },
             scope: this
           }
         });
       };

       var renderWaitingBar = function() {
         if (!YAHOO.lyse.extended.projectService.wait) {
           // Initialize the temporary Panel to display while waiting for external content to load
           YAHOO.lyse.extended.projectService.wait =
             new YAHOO.widget.Panel("wait", {
               width: "220px",
               fixedcenter: true,
               close: false,
               draggable: false,
               zindex:4,
               modal: true,
               visible: false
             });

           YAHOO.lyse.extended.projectService.wait.setHeader("Loading, please wait...");
           YAHOO.lyse.extended.projectService.wait.setBody("<img src=\"http://l.yimg.com/a/i/us/per/gr/gp/rel_interstitial_loading.gif\"/>");
           YAHOO.lyse.extended.projectService.wait.render(document.body);
         }
       };

       // Dialog popup show project settings, on success try delete workflows.
       var restrictionDialog = function(html){
         var prompt = Alfresco.util.PopupManager.getUserInput({
           title: Alfresco.util.message("project.phase.popup.label"),
           buttons: [{
             text: Alfresco.util.message("project.phase.popup.ok"),
             handler: function () {
               var e = document.getElementById("restrictbox");
               var selectedRestriction = e.options[e.selectedIndex].value;

               var e2 = document.getElementById("disable-project");
               var disable = (e2 && e2.checked) || false;

               var oReq = new XMLHttpRequest();
               var url = Alfresco.constants.PROXY_URI + "api/lyse/update/project/phase/" + config.site + "/" + selectedRestriction + "/" + disable;
               oReq.open("PUT", url, true);
               oReq.setRequestHeader(Alfresco.util.CSRFPolicy.getHeader(), Alfresco.util.CSRFPolicy.getToken());
               oReq.onload = function() {
                 if(oReq.status == 200) {
                   YAHOO.lyse.extended.projectService.wait.hide();
                   cancelWorkflowCall();
                 } else if(oReq.status == 400 || oReq.status == 500){
                   Alfresco.util.PopupManager.displayPrompt({
                     title: Alfresco.util.message("project.error.title"),
                     text: Alfresco.util.message("project.error.message")
                   });
                 }
               };
               oReq.send();
               this.destroy();
               renderWaitingBar();
               YAHOO.lyse.extended.projectService.wait.show()
               setTimeout((function () {
                 getProgress(config.site);
               }), 5000);
             },
             isDefault: true
           },
           {
             text: Alfresco.util.message("project.phase.popup.cancel"),
             handler: function () {
               this.destroy();
             }
           }],
           html: html
         });

         // Make OK button enabled.
         var okButton = prompt.getButtons()[0];
         okButton.set("disabled", false);
       };

       // Build dialog options and pop it.
       Alfresco.util.Ajax.jsonGet({
         url: Alfresco.constants.PROXY_URI + "api/lyse/get/project/phases/" + config.site,
         successCallback: {
           fn: function (response){
             if (response.json !== undefined) {
               var selections = "";
               var items = response.json.categories;

               for (var i = 0; i<items.length;  i++) {
                 selections = selections + '<option value="' + items[i] + '">' + items[i] +'</option>\n';
               }
               var html = '<div style="padding: 10px;"><select id="restrictbox">'+selections+'</select></div>';

               var checked = "";
               if(response.json.locked === true){
                 checked = "checked";
               }
/*
               html = html + '<div style="padding: 10px;"><div style="display: block; margin-bottom: 1em;"><label><input type="checkbox" id="disable-project"' + checked + '> '+Alfresco.util.message("project.disable.project.checkbox")+'</input></label></div>';
               html = html + '<div style="display: block;"><label><input type="checkbox" onclick="" id="cancel-workflows"> '+Alfresco.util.message("project.terminate.workflows.checkbox")+'</input></label></div></div>';
*/
               restrictionDialog(html);

               var cancelWorkflowsCheckbox = document.getElementById("cancel-workflows");
               if (cancelWorkflowsCheckbox) cancelWorkflowsCheckbox.addEventListener('click', function (event) {
                 if(event.toElement.checked === true){
                   Alfresco.util.PopupManager.displayPrompt( {
                     text: Alfresco.util.message("project.workflow.cancel.warning"),
                     title: Alfresco.util.message("project.workflow.cancel.title"),
                     noEscape: true,
                     buttons: [{
                       text: Alfresco.util.message("project.workflow.button.yes"),
                       handler: function(){
                         confirmedCancelWorkflows = true;
                         this.destroy();
                       }
                     },
                       {
                         text: Alfresco.util.message("project.workflow.cancel.button.no"),
                         handler: function(){
                           confirmedCancelWorkflows = false;
                           cancelWorkflowsCheckbox.checked = false;
                           this.destroy();
                         }
                       }
                     ]
                   });
                 } else {
                   confirmedCancelWorkflows = false;
                 }
               });
             }
           },
           scope: this
         }
       });
     },
      /**
       * This method delegates site creation to the legacy YUI popup.
       *
       * @instance
       * @param {string} site
       */
      createContractor: function lyse_ProjectService_createContractor(config) {
         // TODO: When an edit site request is received we should display the edit site dialog.
         //       We need to wrap the existing YUI widget in a Dojo object.
         this.alfLog("log", "A request has been made to create a contractor: ", config);

         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreateContractorInstance === "function")
         {
            Alfresco.module.getCreateContractorInstance(config).show();
         }
         else
         {
            this.alfLog("error", "Could not find the 'Alfresco.module.getCreateContractorInstance' function - has 'lyse-create-contractor.js' been included in the page?");
         }
      },
      /**
       * This method delegates site creation to the legacy YUI popup.
       *
       * @instance
       * @param {string} site
       */
      createEPCEng: function lyse_ProjectService_createEpcEng(config) {
         // TODO: When an edit site request is received we should display the edit site dialog.
         //       We need to wrap the existing YUI widget in a Dojo object.
         this.alfLog("log", "A request has been made to create a epc engineering site: ", config);

         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreateEPCEngInstance === "function")
         {
            Alfresco.module.getCreateEPCEngInstance(config).show();
         }
         else
         {
            this.alfLog("error", "Could not find the 'Alfresco.module.getCreateEPCEngInstance' function - has 'lyse-create-epc-eng.js' been included in the page?");
         }
      },


      createInterface: function lyse_ProjectService_createInterface(config) {
          // TODO: When an edit site request is received we should display the edit site dialog.
          //       We need to wrap the existing YUI widget in a Dojo object.
          this.alfLog("log", "A request has been made to create an interface site: ", config);

          // Just use the old YUI popup...
          if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreateInterfaceInstance === "function")
          {
             Alfresco.module.getCreateInterfaceInstance(config).show();
          }
          else
          {
             this.alfLog("error", "Could not find the 'Alfresco.module.getCreateInterfaceInstance' function - has 'lyse-create-interface.js' been included in the page?");
          }
       },

       createHSE: function lyse_ProjectService_createHSE(config) {
           // TODO: When an edit site request is received we should display the edit site dialog.
           //       We need to wrap the existing YUI widget in a Dojo object.
           this.alfLog("log", "A request has been made to create an hse site: ", config);

           // Just use the old YUI popup...
           if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreateHSEInstance === "function")
           {
               Alfresco.module.getCreateHSEInstance(config).show();
           }
           else
           {
               this.alfLog("error", "Could not find the 'Alfresco.module.getCreateHSEInstance' function - has 'lyse-create-hse.js' been included in the page?");
           }
       },

       createPS: function lyse_ProjectService_createPS(config) {
           // TODO: When an edit site request is received we should display the edit site dialog.
           //       We need to wrap the existing YUI widget in a Dojo object.
           this.alfLog("log", "A request has been made to create a ps site: ", config);

           // Just use the old YUI popup...
           if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreatePSInstance === "function")
           {
               Alfresco.module.getCreatePSInstance(config).show();
           }
           else
           {
               this.alfLog("error", "Could not find the 'Alfresco.module.getCreatePSInstance' function - has 'lyse-create-ps.js' been included in the page?");
           }
       },

       createMCC: function lyse_ProjectService_createMCC(config) {
         this.alfLog("log", "A request has been made to create a MCC site: ", config);

         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreateMCCInstance === "function")
         {
             Alfresco.module.getCreateMCCInstance(config).show();
         }
         else
         {
             this.alfLog("error", "Could not find the 'Alfresco.module.getCreateMCCInstance' function - has 'lyse-create-mcc.js' been included in the page?");
         }
      },
       createCivil: function lyse_ProjectService_createCivil(config) {
           // TODO: When an edit site request is received we should display the edit site dialog.
           //       We need to wrap the existing YUI widget in a Dojo object.
           this.alfLog("log", "A request has been made to create a Civil site: ", config);

           // Just use the old YUI popup...
           if (Alfresco && Alfresco.module && typeof Alfresco.module.getCreateCivilInstance === "function")
           {
               Alfresco.module.getCreateCivilInstance(config).show();
           }
           else
           {
               this.alfLog("error", "Could not find the 'Alfresco.module.getCreateCivilInstance' function - has 'lyse-create-civil.js' been included in the page?");
           }
       },
      /**
       * This method delegates site editing to the legacy YUI popup.
       *
       * @instance
       * @param {string} site
       */
      editProject: function lyse_ProjectService_editProject(config) {
         // TODO: When an edit site request is received we should display the edit site dialog.
         //       We need to wrap the existing YUI widget in a Dojo object.
         this.alfLog("log", "A request has been made to edit a site: ", config);

         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getEditProjectInstance === "function")
         {
           var test = Alfresco.module.getEditProjectInstance();

           Alfresco.module.getEditProjectInstance().show({
               shortName: config.site
            });
         }
         else
         {
            this.alfLog("error", "Could not find the 'Alfresco.module.getEditProjectInstance' function - has 'edit-project.js' been included in the page?");
         }
      },
      /**
       * This method delegates site editing to the legacy YUI popup.
       *
       * @instance
       * @param {string} site
       */
      editContractor: function lyse_ProjectService_editContractor(config) {
         // TODO: When an edit site request is received we should display the edit site dialog.
         //       We need to wrap the existing YUI widget in a Dojo object.
         this.alfLog("log", "A request has been made to edit a site: ", config);

         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getEditContractorInstance === "function")
         {
           var test = Alfresco.module.getEditContractorInstance();

           Alfresco.module.getEditContractorInstance().show({
               shortName: config.site
            });
         }
         else
         {
            this.alfLog("error", "Could not find the 'Alfresco.module.getEditContractorInstance' function - has 'edit-contractor.js' been included in the page?");
         }
      },
      editInterface: function lyse_ProjectService_editInterface(config) {
          // TODO: When an edit site request is received we should display the edit site dialog.
          //       We need to wrap the existing YUI widget in a Dojo object.
          this.alfLog("log", "A request has been made to edit a site: ", config);

          // Just use the old YUI popup...
          if (Alfresco && Alfresco.module && typeof Alfresco.module.getEditInterfaceInstance === "function")
          {
            var test = Alfresco.module.getEditInterfaceInstance();

            Alfresco.module.getEditInterfaceInstance().show({
                shortName: config.site
             });
          }
          else
          {
             this.alfLog("error", "Could not find the 'Alfresco.module.getEditInterfaceInstance' function - has 'edit-interface.js' been included in the page?");
          }
       },
       editHSE: function lyse_ProjectService_editHSE(config) {
           this.alfLog("log", "A request has been made to edit a site: ", config);

           // Just use the old YUI popup...
           if (Alfresco && Alfresco.module && typeof Alfresco.module.getEditHSEInstance === "function")
           {
               Alfresco.module.getEditHSEInstance().show({
                   shortName: config.site
               });
           }
           else
           {
               this.alfLog("error", "Could not find the 'Alfresco.module.getEditHSEInstance' function - has 'edit-hse.js' been included in the page?");
           }
       },
       editPS: function lyse_ProjectService_editPS(config) {
           this.alfLog("log", "A request has been made to edit a site: ", config);

           // Just use the old YUI popup...
           if (Alfresco && Alfresco.module && typeof Alfresco.module.getEditPSInstance === "function")
           {
               Alfresco.module.getEditPSInstance().show({
                   shortName: config.site
               });
           }
           else
           {
               this.alfLog("error", "Could not find the 'Alfresco.module.getEditPSInstance' function - has 'edit-ps.js' been included in the page?");
           }
       },
       editMCC: function lyse_ProjectService_editMCC(config) {
         this.alfLog("log", "A request has been made to edit a site: ", config);

         // Just use the old YUI popup...
         if (Alfresco && Alfresco.module && typeof Alfresco.module.getEditMCCInstance === "function")
         {
             Alfresco.module.getEditMCCInstance().show({
                 shortName: config.site
             });
         }
         else
         {
             this.alfLog("error", "Could not find the 'Alfresco.module.getEditMCCInstance' function - has 'edit-mcc.js' been included in the page?");
         }
       },
       editCivil: function lyse_ProjectService_editCivil(config) {
           // TODO: When an edit site request is received we should display the edit site dialog.
           //       We need to wrap the existing YUI widget in a Dojo object.
           this.alfLog("log", "A request has been made to edit a site: ", config);

           // Just use the old YUI popup...
           if (Alfresco && Alfresco.module && typeof Alfresco.module.getEditCivilInstance === "function")
           {
               var test = Alfresco.module.getEditCivilInstance();

               Alfresco.module.getEditCivilInstance().show({
                   shortName: config.site
               });
           }
           else
           {
               this.alfLog("error", "Could not find the 'Alfresco.module.getEditCivilInstance' function - has 'lyse-edit-civil.js' been included in the page?");
           }
       }
  });
});
