define(['dojo/_base/declare',
    'dijit/_WidgetBase',
    'dijit/_TemplatedMixin',
    'dojo/text!./datalist.html',
    'alfresco/core/Core',
    "alfresco/core/CoreXhr",
    "dojox/charting/Chart",
    "dojox/charting/plot2d/Pie",
    "dojox/charting/themes/Distinctive",
    "dojo/dom-construct",
    "dojox/charting/action2d/Highlight",
    "dojox/charting/action2d/MoveSlice",
    "dojox/charting/action2d/Tooltip",
    "dojox/charting/themes/MiamiNice",
    "dojox/charting/widget/Legend",
    "dojo/number"
  ],
  function (declare, _Widget, _Templated, template, Core, AlfCoreXhr, Chart, Pie, theme, domConstruct,
            Highlight, MoveSlice, Tooltip, MiamiNice, Legend, number) {
    return declare([_Widget, _Templated, Core, AlfCoreXhr], {

      cssRequirements: [{cssFile: './datalist.css', mediaType: 'all'}],

      i18nScope: 'Reports',

      i18nRequirements: [{i18nFile: 'reports.properties'}],

      templateString: template,

      dataList: '',

      dataListURLParam: '',

      columns: ['status', 'count'],

      showTotals: false,

      totalText: "Total",

      showChart: true,
      reportRows:[],

      buildRendering: function lyse_reports_datalist__buildRendering() {
        this.site = Alfresco.constants.SITE;
        if(this.uniqueCartId)
          this.chartId = this.site + "_" + this.uniqueCartId + "_" + "chartId";
        else
          this.chartId = this.site + "_" + this.dataListURLParam + "_" + "chartId";
        this.dataListURLBase = '/share/page/site/' + this.site + '/data-lists?list=' + encodeURIComponent(this.dataListURLParam.trim());

        this.showLinks = this.dataListURLParam != '';

		    if(this.dataListName == undefined){
        	this.dataListName = this.message(this.dataList);
        }

        if (this.dataListURLParam == '') {
          this.dataListHeader = this.dataListName;
        } else {
          this.dataListHeader = "";
        }

        this.inherited(arguments);
      },

      postCreate: function lyse_reports_datalist__postCreate() {
        if(this.orderList===undefined)
        {
          console.warn("Order was not defined for datalist %s",this.dataListName);
        }
        var isEmpty = function isEmpty(obj) {
            for(var prop in obj) {
                if(obj.hasOwnProperty(prop))
                    return false;
            }
            return JSON.stringify(obj) === JSON.stringify({});
        };

        var reportRows = this.reportRows;

        //These lines removed charts whith no data. Should we hide them or have them visible empty?
        if (!reportRows || reportRows.length === 0) {
          document.getElementById(this.id).className += " hidden";
          return console.warn("Datalist hidden - No results in response for datalist: " + this.dataListName);
        }
        else if(this.hideReports(reportRows) === true){
          document.getElementById(this.id).className += " hidden";
          return console.warn("Datalist hidden - No results in response for datalist: " + this.dataListName);
        }
        this.buildHeader();
        reportRows.forEach(function (item) {
          var row;
          if(item.status == undefined){
            row = domConstruct.create("tr", {className: "lyseoverview"}, this.bodyNode);
		      }
          else if(item.status==="Document overdue" || item.status=="Review overdue" || item.status.indexOf("Overdue") !== -1)
          {
            // Only show overdue if property is defined.
            if(!isEmpty(this.filterOverdueProperties) || item.status !== "Overdue"){
              row = domConstruct.create("tr", {className: "lyseoverview warning"}, this.bodyNode);
            }
          } else {
            row = domConstruct.create("tr", {className: "lyseoverview"}, this.bodyNode);
            if(this.excludeFromTotalCount != undefined){
              if(this.excludeFromTotalCount.indexOf(item.status) != -1){
                row.classList.add("cursive");
              }
	          }
          }
          this.populateRow(item, row);
        }, this);

        if (this.showTotals) {
          this.addTotalRow(reportRows)
        }
        if (this.showChart) {
          this.renderPieChart(this.pieSeries(reportRows));
        } else {
          //Don't show a chart; remove element from DOM
          domConstruct.destroy(this.chartId);
        }
      },

      hideReports: function(reportRows){
        var hideReportCounter = true;
        for (var report in reportRows) {
          if (!reportRows[report].count){continue;} // Ignore, go to next.}
          if (reportRows[report].count !== "0") {hideReportCounter = false; break;}
        }
        return hideReportCounter;
      },
      buildHeader: function () {
        var headerNode = this.headerNode;

        // If report title is a link.
        if(this.dataListURLParam){
          var validConstraints = headerNode.parentNode && headerNode.parentNode.parentElement &&
            headerNode.parentNode.parentElement.getElementsByTagName("h1")[0];
          if(validConstraints){
            var reportTitle = headerNode.parentNode.parentElement.getElementsByTagName("h1")[0];
            var createLink = document.createElement("a");
            createLink.setAttribute("href",this.dataListURLBase);
            createLink.setAttribute("target","_blank");
            createLink.className = "lyseoverview";
            createLink.innerHTML = this.dataListName;
            reportTitle.appendChild(createLink);
          }
        }
        var row = domConstruct.create("tr", {}, headerNode);
        this.columns.forEach(function (column) {
            domConstruct.create("th", {innerHTML: this.message(column)}, row);
        }, this);
      },

      populateRow: function (item, row) {
        this.columns.forEach(function (column) {
          var value = item[column];
          if (!value) value = "";
          var cell = domConstruct.create("td", {style:{padding:"8px"}}, row);
          var filterURL = "";
          if (this.dataListName === "Action Per Member") {
            if((column == 'user' || column === 'user_total') && item.userName != '') {
               filterURL = this.dataListURLBase + "#filter=memberfilterform%7C%253F" +  this.responsibleUser + "%253D" + encodeURIComponent(item.userName);
            } else {
              if(column === 'overdue_count') {
                if(this.filterBeforeOverdueProperties){
                  if(this.filterBeforeOverdueProperties[this.filterOverdueProperties[column]]){
                    filterURL = this.dataListURLBase + "#filter=memberfilterform%7C%253F" +  this.responsibleUser + "%253D" + encodeURIComponent(item.userName)
                               + "%26" + this.filterBeforeOverdueProperties[this.filterOverdueProperties[column]]
                               + "%26" + this.filterOverdueProperties[column] + "-date-range%253DTO" + new Date().toJSON();
                  }
                }
              } else if(item.userName != '') {
                filterURL = this.dataListURLBase + "#filter=memberfilterform%7C%253F" +  this.responsibleUser + "%253D" + encodeURIComponent(item.userName)
                            + "%26" + this.filterStatusProperty + "%253D" +  this.filterStatusConstraints[column];
              }
            }
          } else if (this.showLinks && column === "status") {
            if (this.filterOverdueProperties && value in this.filterOverdueProperties) {
              filterURL = this.dataListURLBase + "#filter=filterform%7C%253F" + this.filterOverdueProperties[value] +
                "-date-range%253DTO" + new Date().toJSON();

                // Other filters defined to be put before overdue properties.
                if(this.filterBeforeOverdueProperties){
                	if(this.filterBeforeOverdueProperties[this.filterOverdueProperties[value]]){
                		filterURL = this.dataListURLBase + "#filter=filterform%7C%253F" + this.filterBeforeOverdueProperties[this.filterOverdueProperties[value]]
                					+ "%26" + this.filterOverdueProperties[value] + "-date-range%253DTO" + new Date().toJSON();
                	}
                }

            } else if (this.filterStatusConstraints && value in this.filterStatusConstraints) {
              filterURL = this.dataListURLBase + "#filter=filterform%7C%253F" + this.filterStatusProperty + "%253D" + this.filterStatusConstraints[value];
            } else if(this.forceLinks != undefined) {
            	if(this.forceLinks == true) {
					      filterURL = this.dataListURLBase + "#filter=filterform%7C%253F" + this.filterStatusProperty + "%253D" + encodeURIComponent(value);
            	}
            }
          }
          if (filterURL != "") {
            domConstruct.create("a", {
              className: "lyseoverview",
              href: filterURL,
    		      target: "_blank",
              innerHTML: value
            }, cell);
          } else {
            if(!isNaN(value))
            {
              cell.innerHTML = this.formatNumber(value);
            }
            else
            {
              cell.innerHTML = value;
            }

          }
        }, this);
      },

      formatNumber:function(val)
      {
        return number.format(val,
          {
            locale:"fr-fr"
          })
      },

      addTotalRow: function (results) {
        var row = domConstruct.create("tr", {className: 'total lyseoverview'}, this.bodyNode);
        this.excludeTotals;
        this.columns.forEach(function (column) {
          var total = 0;
          results.forEach(function (item) {

            if(this.excludeFromTotalCount != undefined){
            	if(this.excludeFromTotalCount.indexOf(item.status) != -1){
            		return;
            	}
            }

            var value = parseFloat(item[column]);
            if (!isNaN(value)) {
              total += value;
            }
          },this);
          if (column == 'status' || column == 'user') {
            domConstruct.create("td", {innerHTML: this.totalText,style:{padding:"8px"}}, row);
          } else {
            domConstruct.create("td", {innerHTML: this.formatNumber(total),style:{padding:"8px"}}, row);
          }
        },this);
      },

      renderPieChart: function (pieSeries) {
        this.pieChart = new Chart(this.chartId);

        this.pieChart.setTheme(MiamiNice);
        this.pieChart.addPlot("default", {
          type: Pie,
          radius: 80,
          font: "normal normal 11pt Tahoma",
          fontColor: "black",
          // labelOffset: -30,
          labels: false,


        });
        this.pieChart.addSeries("Series 1",
          pieSeries
        );
        new MoveSlice(this.pieChart, "default");
        new Highlight(this.pieChart, "default");
        new Tooltip(this.pieChart, "default");
        this.pieChart.render();
        new Legend({chart:this.pieChart,horizontal:false},this.chartId + "-legend");
        this.pieChart.surface.rawNode.childNodes[1].setAttribute('fill-opacity','0');
        this.pieChart.surface.rawNode.childNodes[2].setAttribute('fill-opacity','0');

      },

      pieSeries: function (results) {

        var resultCount = results.reduce((function (acc, item) {
          if (this.filterOverdueProperties && item.status in this.filterOverdueProperties) {
            return acc;
          }
          else {
            return acc + parseFloat(item.count);
          }
        }).bind(this), 0);
        return results.reduce((function(acc,item)
        {
          if (this.filterOverdueProperties === undefined || !(item["status"] in this.filterOverdueProperties)) {
            var pieData = 'count';
            var pieText = 'status';
            var percent = Math.round(100 * item[pieData] / resultCount);

            acc.push({
              y: parseInt(item[pieData]),
              text: item[pieText] + ": " + percent + "%",
              tooltip: item[pieText] + ": " + percent + "%"
            });
          }
          return acc;
        }).bind(this),[]);
      },
    });
  });
