/**
 * AttachmentControl component. Extends the Alfresco.ObjectFinder so that in the onReady function we can do our modifications.
 *
 * @namespace Alfresco
 * @class Alfresco.AttachmentControl
 */
(function()
{
   /**
    * YUI Library aliases
    */
   var Dom = YAHOO.util.Dom,
      Event = YAHOO.util.Event;

   /**
    * Alfresco Slingshot aliases
    */
   var $html = Alfresco.util.encodeHTML,
      $hasEventInterest = Alfresco.util.hasEventInterest,
      $combine = Alfresco.util.combinePaths;

   /**
    * AttachmentControl constructor.
    *
    * @param {String} htmlId The HTML id of the control element
    * @param {String} currentValueHtmlId The HTML id of the current value of the control element
    * @return {Alfresco.AttachmentControl} The new AttachmentControl instance
    * @constructor
    */
   Alfresco.AttachmentControlMod = function(htmlId, currentValueHtmlId)
   {
      Alfresco.AttachmentControlMod.superclass.constructor.call(this, htmlId, currentValueHtmlId);
      this.fileUpload = null;
       this.name = "Alfresco.AttachmentControlMod";
      return this;
   };

   Alfresco.AttachmentControlMod.prototype.options = {

        /**
         * current nodeRef
         *
         * @property nodeRef
         * @type object
         */
        nodeRef: null,

        /**
         * Turn of selection of location in repo/site
         * @property allowNavigation
         * @type boolean
         */
        allowNavigation: true,

        /**
         * Specifies the location the object finder should start, the following
         * values are supported:
         *
         * - {companyhome}
         * - {userhome}
         * - {siteshome}
         * - {doclib}
         * - {self}
         * - {parent}
         * - A NodeRef
         * - An XPath
         *
         * @property startLocation
         * @type string
         */
        startLocation: null,

        /**
         * Specifies the parameters to pass to the node locator service
         * when determining the start location node.
         *
         * @property startLocationParams
         * @type string
         */
        startLocationParams: null,
        /**
         * The id of the item being edited
         *
         * @property currentItem
         * @type string
         */
        currentItem: null,
        parentNodeRef: null,

        /**
         * Lyse functionality, only show delete icon.
         */
        onlyDelete: false,

        /**
         * Lyse functionality, allow uploading a new version of an attachment.
         */
        allowUploadNewVersion: false,

        /**
         * Lyse functionality, when used in a workflow task form uploading a new
         * revision is mandatory.
         */
        newVersionMandatory: false,

        /**
         * Lyse functionality, when true it will use the "upload new revision" dialog where the user must enter a revision, revision
         * date and a comment. When false it will only upload a new version of the attachment.
         */
        uploadRevisions: false,

        /**
         * Lyse functionality, allow associating uploaded files to the current node from within this control, this was added to make
         * it possible to upload files from forms in "view" mode. Normally the associations are created when the form is persisted but
         * this won't work in view mode since the form is never posted to the form processor.
         */
        allowCreateAssociations: false
   };

   YAHOO.extend(Alfresco.AttachmentControlMod, Alfresco.ObjectFinder,
   {

	   loadNodeSuccess: function AttachmentControlMod_loadNodeSuccess(response)
   		{

   		  var punchDataExists = function(response){
   		    if(!response){
   		      return false;
          }
   		    if(!response.json){
   		      return false;
          }
   		    if(!response.json.item){
   		      return false;
          }
   		    if(!response.json.item.nodeType){
   		      return false;
          }
   		    return response.json.item.nodeType === "lysedl:punchList"
          || response.json.item.nodeType === "lysedl:issuePunchList"
          || response.json.item.nodeType === "lysedl:civilPunchList";
        };
   		  if(punchDataExists(response)){
          Alfresco.util.Ajax.jsonGet({
            url: Alfresco.constants.PROXY_URI + "api/sites/" + Alfresco.constants.SITE + "/memberships/" + Alfresco.constants.USERNAME,
            responseContentType :  "application/json",
            successCallback: {
              fn: function (response) {
                var role = response.json.role;
                if(role === "SiteConsumer") {
                  this.widgets.uploadAttachment.set("disabled", true);
                }
              },
              scope: this
            }
          });
        }
   		  else{
          // If we dont have create access, disable button.
          var canCreate = response.json.item.permissions.userAccess.create;
          if(!canCreate){
            if (this.widgets.uploadAttachment != null){
              this.widgets.uploadAttachment.set("disabled", true);
            }
          }
        }
   		},
     //https://redmine.redpill-linpro.com/issues/40901
     //Sort by date but in the reverse order, this probably could be solved by just array.reverse
     onRenderCurrentValue:function(layer, args)
     {
       var array = _.map(this.selectedItems,_.valuesIn);
       var sortedDocs = array.sort(function(a,b)
       {
         if(!a.modified)
         {
           a.modified = new Date();
         }
         if(!b.modified)
         {
           b.modified = new Date();
         }
         return new Date(b.modified) - new Date(a.modified);
       });
       this.selectedItems = {};
       sortedDocs.forEach(function(item)
       {
         this.selectedItems[item.nodeRef] = item;
       },this);

       Alfresco.AttachmentControlMod.superclass.onRenderCurrentValue.call(this,layer,args);
     },
       /**
       * Fired by YUI when parent element is available for scripting.
       * Component initialisation, including instantiation of YUI widgets and event listener binding.
       *
       * @method onReady
       */
      onReady: function AttachmentControlMod_onReady()
      {
          Alfresco.AttachmentControlMod.superclass.onReady.call(this);

          if (!this.fileUpload) {
              this.fileUpload = Alfresco.getFileUploadInstance();
          }

          var newVersionHiddenField = document.getElementById(this.id + '-newVersionUploaded');

          if (newVersionHiddenField != null){
        	  newVersionHiddenField.value = false;
          }

          var nodeRef = this.options.currentItem;

          var url;
          if (nodeRef !== undefined && nodeRef !== null) {
	          if (nodeRef.uri) {
	              url = Alfresco.constants.PROXY_URI + 'slingshot/doclib/node/' + nodeRef.uri;
	          }
	          else if (nodeRef.indexOf("activiti$") < 0) { // For workflow task forms currentItem is the task ID (activiti$12345), not a nodeRef!
	              var nodeRefObj = Alfresco.util.NodeRef(nodeRef);
	              url = Alfresco.constants.PROXY_URI + 'slingshot/doclib/node/' + nodeRefObj.storeType + "/" + nodeRefObj.storeId + "/" + nodeRefObj.id;
	          }
	          if (url) Alfresco.util.Ajax.jsonGet(
	              {
	                  url: url,
	                  successCallback: {
	                      fn: this.loadNodeSuccess,
	                      scope: this
	                  },
	                  failureMessage: "Failed to load data for permission details"
	              });
          }
           this.widgets.uploadAttachment = Alfresco.util.createYUIButton(this, "upload-button", this.onUploadAttachment, {
               disabled: !this.options.multipleSelectMode && this.selectedItems.lengh > 0
           });

           // Hide the "Add" button
           if (this.widgets.addButton) {
               this.widgets.addButton.set("disabled", true);
               YAHOO.util.Dom.setStyle(this.widgets.addButton, 'display', 'none');
           }

           YAHOO.Bubbling.on("formValueChanged", function (layer, args) {
               if ($hasEventInterest(this, args)) {
                   var formValue = args[1];
                   var allowUploading = this.options.multipleSelectMode || formValue.selectedItems.length == 0;
                   if (this.widgets.uploadAttachment) {
                       this.widgets.uploadAttachment.set("disabled", !allowUploading);

                       if (!allowUploading && !this.options.allowRemoveAction && !this.options.allowRemoveAllAction) {
                           YAHOO.util.Dom.setStyle(this.widgets.uploadAttachment, 'display', 'none');
                       }

                   }
               }
           }, this);

           if (this.options.onlyDelete) {
               YAHOO.util.Dom.setStyle(this.widgets.uploadAttachment, 'display', 'none');
           }

           // Add action to the list view to allow uploading a new revision of a list item
           if (this.options.onlyDelete != true && this.options.allowUploadNewVersion) {
               if (this.options.listItemActions) {
                   this.options.listItemActions.push(
                       {
                           name: "upload-new-list-item-revision",
                           event: "uploadNewListItemRevision",
                           label: "actions.document.upload-new-version"
                       });
               }
           }

           // Handle event for uploading new revisions of a list item
           YAHOO.Bubbling.on("uploadNewListItemRevision", function (layer, args) {
               if ($hasEventInterest(this, args)) {
                   var datalistDetailsActions = Alfresco.util.ComponentManager.findFirst("Lyse.DatalistDetailsActions");
                   if (this.options.uploadRevisions && datalistDetailsActions) {
                       datalistDetailsActions.onActionUploadNewRevision();
                   }
                   else {
                       var item = args[1].value;
                       if (item) {
                           var nodeRef = item.nodeRef;

                           if (!this.fileUpload) {
                               this.fileUpload = Alfresco.getFileUploadInstance();
                           }
                           var singleUpdateConfig =
                               {
                                   updateNodeRef: nodeRef,
                                   suppressRefreshEvent: true,
                                   overwrite: true,
                                   mode: this.fileUpload.MODE_SINGLE_UPDATE,
                                   onFileUploadComplete: {
                                       fn: this.onNewListItemRevisionUploaded,
                                       scope: this
                                   }

                               };
                           if (Alfresco.util.isValueSet(this.options.siteId)) {
                               singleUpdateConfig.siteId = this.options.siteId;
                               singleUpdateConfig.containerId = "documentLibrary";
                           }
                           this.fileUpload.show(singleUpdateConfig);
                       }
                   }
               }
           }, this);
       },

       onNewListItemRevisionUploaded: function(complete) {
           // Update the list view
           this._adjustCurrentValues();
           this.selectItems(this.getSelectedItems().toString());

           var newVersionHiddenField = document.getElementById(this.id + '-newVersionUploaded');
           console.log("newVersionFieldName:"+newVersionHiddenField);
           if (newVersionHiddenField != null){
          	 newVersionHiddenField.value = true;
           }
       },

       getDestinationDialog: function() {
           if (!this.widgets.destinationDialog) {
               this.widgets.destinationDialog = new Alfresco.module.DoclibGlobalFolder(this.id + "-destinationDialog");
               this.widgets.destinationDialog.setOptions(
                   {
                       title: this.msg("form.attachments.dialog.destination.title"),
                       siteId : Alfresco.constants.SITE,
                       containerId: "documentLibrary",
                       //or path
                       //pathNodeRef: {"uri": this.options.objectRenderer.options.parentNodeRef.replace("://","/")},
                       allowedViewModes:
                           [
                               Alfresco.module.DoclibGlobalFolder.VIEW_MODE_SITE
                           ]
                   });

               this._locateStartingNode();
               var me = this;
               YAHOO.Bubbling.on("refreshItemList", function (layer, args) {
                   if ($hasEventInterest(this, args)) {
                       me.getDestinationDialog().setOptions(
                           {
                               pathNodeRef: {"uri": this.options.objectRenderer.options.parentNodeRef.replace("://", "/")}
                           });
                       if (this.options.allowNavigation) { //} && !this.options.objectRenderer.startLocationResolved) {
                           me.getDestinationDialog().showDialog();
                       } else {
                           this.showFileUpload({
                               nodeRef: this.options.objectRenderer.options.parentNodeRef
                           });
                       }
                   }
               }, this);

               YAHOO.Bubbling.on("folderSelected", function (layer, args) {
                   if ($hasEventInterest(me.getDestinationDialog(), args)) {
                       var selectedFolder = args[1].selectedFolder;
                       if (selectedFolder !== null) {
                           this.showFileUpload(selectedFolder);

                       }
                   }
               }, this);
           }
           return this.widgets.destinationDialog;
       },

      /**
       * Upload Attachment button click handler
       *
       * @method onUploadAttachment
       * @param e {object} DomEvent
       * @param p_obj {object} Object passed back from addListener method
       */
       onUploadAttachment: function AttachmentControlMod_onUploadAttachment(e, p_obj) {
          this.getDestinationDialog();
          if (this.options.allowNavigation) {
              this.getDestinationDialog().showDialog();
          } else if (this.options.objectRenderer.startLocationResolved) {
              this.showFileUpload({
                  nodeRef: this.options.objectRenderer.options.parentNodeRef
              });
          }
      },

      showFileUpload: function AttachmentControlMod_showFileUpload(selectedFolder) {
    	  if (this.fileUpload === null)
          {
             this.fileUpload = Alfresco.getFileUploadInstance();
          }

          // Show uploader for multiple files
          var multiUploadConfig =
          {
             filter: [],
             mode: this.options.multipleSelectMode ? this.fileUpload.MODE_MULTI_UPLOAD : this.fileUpload.MODE_SINGLE_UPLOAD,
             thumbnails: "doclib",
             onFileUploadComplete:
             {
                fn: this.onFileUploadComplete,
                scope: this
             }
          };

          if (selectedFolder.nodeRef) {
        	  // from nodelocator resolve
        	  multiUploadConfig.destination = selectedFolder.nodeRef;
          } else {
        	  //from object-finder
              multiUploadConfig.siteId = selectedFolder.siteId;
              multiUploadConfig.containerId = selectedFolder.containerId;
              multiUploadConfig.uploadDirectory = selectedFolder.path;
          }
          this.fileUpload.show(multiUploadConfig);
      },
      /**
       * File Upload complete event handler
       *
       * @method onFileUploadComplete
       * @param complete {object} Object literal containing details of successful and failed uploads
       */
      onFileUploadComplete: function AttachmentControlMod_onFileUploadComplete(complete)
      {
         var success = complete.successful.length, file;
         if (success > 0)
         {
             // Associate the uploaded files to the current item, if needed (when in a view-mode form that allows uploading). In
             // edit forms the association is created in the form processor.
             if (this.options.allowCreateAssociations && this.options.field && this.options.field.indexOf("assoc_") == 0) {
                 var assocName = this.options.field.substr("assoc_".length).replace("_", ":");
                 var newAssociations = [];
                 for (i = 0; i < success; i++) {
                     file = complete.successful[i];
                     newAssociations.push({
                         type: assocName,
                         targetNodeRef: file.nodeRef
                     });
                 }
                 if (newAssociations.length > 0) {
                     Alfresco.util.Ajax.jsonPost({
                         url: Alfresco.constants.PROXY_URI + "api/lyse/forms/associate/" + this.options.currentItem.replace(":/", ""),
                         dataObj: {
                             associations: newAssociations
                         },
                         failureMessage: this.msg("Could not associate uploaded documents to the current item!")
                     });
                 }
             }

             for (i = 0; i < success; i++)
             {
                 file = complete.successful[i];
                 file.name = file.fileName;
                 YAHOO.Bubbling.fire("selectedItemAdded",
                     {
                         eventGroup: this,
                         item : file
                     });
             }
             YAHOO.Bubbling.fire("renderCurrentValue",
                 {
                     eventGroup: this
                 });

         }
      }
      // ,
      //_locateStartingNode: function AttachmentsMod__locateStartingNode()
      //{
    	//  this._locateStartingNode();
    	//  this.options.rootNode=this.options.objectRenderer.options.parentNodeRef;
      //}
   });
})();
