(function () {
    /*
     * Hide/show input fields based on user input, this javascript file is inspired by vor_validation.js
     */
    var initMCCSignalFormBehaviour = function (layer, args) {
        var formUI = args[1];
        // Check if it's a "new datalist item" or "task edit" form
        if (!formUI.runtime.formId.contains("createRow-form")
            && !formUI.runtime.formId.contains("editDetails-form")
            && !formUI.runtime.formId.contains("task-edit")
            && !formUI.runtime.formId.contains("start-workflow"))
            return;

        var htmlForm = YUIDom.get(formUI.runtime.formId);
        if (htmlForm) {
            // We have the correct type, now add an event handler
            var hasRemoteControl = document.getElementsByName('prop_lyse_mccSignalHasRemoteControl')[0];
            var remoteSignalId = document.getElementsByName('prop_lyse_mccSignalRemoteSignalId')[0];
            var remoteSignalDescription = document.getElementsByName('prop_lyse_mccSignalRemoteSignalDescription')[0];
            var testStatusRemote = document.getElementsByName('prop_lyse_mccSignalTestStatusRemote')[0];
            var contractorRole = false;

            Alfresco.util.Ajax.jsonGet({
                url: Alfresco.constants.PROXY_URI + "api/lyse/people/mccContractorRole/?userName="+Alfresco.constants.USERNAME,
                responseContentType : "application/json",
                successCallback: {
                    fn: function (response) {
                        if (response.json !== undefined) {
                          contractorRole = response.json.contractor;
                        }
                    },
                    scope: this
                }
            });

            if (hasRemoteControl && remoteSignalId && remoteSignalDescription && testStatusRemote) {

                var showHideField = function (element, enabled) {
                    if (element) {
                        // Enable/disable the actual input field so it will not be submitted if disabled
                        if (!enabled) {
                            element.disabled = 'disabled';
                        } else {
                            element.removeAttribute('disabled');
                        }
                        // Show/hide the parent container (so also the label and help is hidden)
                        var field = YAHOO.util.Dom.getAncestorByClassName(element, 'form-field');
                        if (field) {
                            YAHOO.util.Dom.setStyle(field, 'display', enabled ? 'inherit' : 'none');
                        }
                    }
                };

                var toggleTestStatusRemote = function(enabled) {
                    for (i = 0; i < testStatusRemote.options.length; i++) {
                        var option = testStatusRemote.options[i];
                        if ((enabled && option.value == "Not applicable") || (!enabled && option.value != "Not applicable")
                            || (contractorRole && !(option.value == "For Approval" || option.value == "Voided" || option.value == "New or Revised") )) {
                            option.disabled = true;
                        }
                        else {
                            option.disabled = false;
                        }
                    }
                };
                var updateFields = function () {
                    var enabled = "true" === hasRemoteControl.value;
                    showHideField(remoteSignalId, enabled);
                    showHideField(remoteSignalDescription, enabled);

                    if ("false" === hasRemoteControl.value && !testStatusRemote.disabled) {
                        testStatusRemote.value = "Not applicable";
                    }
                    else if ("true" === hasRemoteControl.value && !testStatusRemote.disabled) {
                        if(!formUI.runtime.formId.contains("editDetails-form"))
                            // Clear so that the user has to select a value
                            testStatusRemote.value = "";
                    }
                    // Enable/disable option values depending on hasRemoteControl
                    toggleTestStatusRemote(enabled);

                    YAHOO.Bubbling.fire("mandatoryControlValueUpdated", this);
                };

                // Set the initial state after loading the form
                updateFields();

                // Only initialize behaviour if the fields were not initially set to read only in the form
                if (!hasRemoteControl.disabled) {
                    YAHOO.util.Event.on(hasRemoteControl.id, 'change', function () {
                        // Update
                        updateFields();
                    });
                }

                // This makes sure it also runs updateFields after the testStatusRemote has been updated asynchronously
                var onCategorySelectLoaded = function(layer, args) {
                    if (args && args.length > 0) {
                        var obj = args[1];
                        if (obj.htmlId) {
                            var select = document.getElementById(obj.htmlId);
                            if (select === testStatusRemote) {
                                updateFields();
                            }
                        }
                    }
                };
                YAHOO.Bubbling.on("categorySelectLoaded", onCategorySelectLoaded);
            }
        }
    };

    YAHOO.Bubbling.on("afterFormRuntimeInit", initMCCSignalFormBehaviour, this);
})();
