(function () {
    /*
     * When the "Consequence" dropdown is changed show/hide other fields depending on the selected value.
     */
    var initVarationOrderFormBehaviour = function (layer, args) {
        var formUI = args[1];
        // Check if it's a "new datalist item" or "task edit" form
        if (!formUI.runtime.formId.contains("createRow-form")
            && !formUI.runtime.formId.contains("editDetails-form")
            && !formUI.runtime.formId.contains("task-edit")
            && !formUI.runtime.formId.contains("start-workflow"))
            return;

        var htmlForm = YUIDom.get(formUI.runtime.formId);
        if (htmlForm) {
            // Find out what content type the form creates by looking at the form's action URL
            // Looks like this: "http://localhost:8081/share/proxy/alfresco/api/type/lysedl%3avariationOrderRequestItem/formprocessor"
            // TODO: is there an easier way to do this (couldn't find itemKind/formId parameters anywhere?!)
            //var actionURL = decodeURIComponent(htmlForm.action);
            //if (actionURL.indexOf('/api/type/lysedl:variationOrderRequestItem/formprocessor') > -1) {

            // We have the correct type, now add an event handler
            var costConsequenceSelect = document.getElementsByName('prop_lysedl_vorCostConsequence')[0];
            var costConsequenceNOK = document.getElementsByName('prop_lysedl_vorConsequenceNOK')[0];
            var costConsequenceEUR = document.getElementsByName('prop_lysedl_vorConsequenceEUR')[0];


            if (costConsequenceSelect && costConsequenceNOK && costConsequenceEUR) {
                // Add mandatory indicator to label, this will be hidden and shown based on consequenceSelect value.
                var labelNOK = costConsequenceNOK.parentNode.children[0];
                var labelEUR = costConsequenceEUR.parentNode.children[0];
                var mandatoryIndicator = document.createElement("span");
                mandatoryIndicator.className = "mandatory-indicator";
                mandatoryIndicator.innerHTML= "*";
                labelEUR.appendChild(mandatoryIndicator);

                var mandatoryIndicator = document.createElement("span");
                mandatoryIndicator.className = "mandatory-indicator";
                mandatoryIndicator.innerHTML= "*";
                labelNOK.appendChild(mandatoryIndicator);

                var showHideField = function (element, isDisabled) {
                    if (element) {
                        // Enable/disable the actual input field so it will not be submitted if disabled
                        if (isDisabled) {
                            element.disabled = 'disabled'
                        }
                        else {
                            element.removeAttribute('disabled')
                        }
                        // Show/hide the parent container (so also the label and help is hidden)
                        var field = YAHOO.util.Dom.getAncestorByClassName(element, 'form-field');
                        if (field) {
                            YAHOO.util.Dom.setStyle(field, 'display', isDisabled ? 'none' : 'inherit')
                        }
                    }
                };
                var findIndex = function (arr, pred) {
                    for (var i = 0; i < arr.length; i++) {
                        if (pred(arr[i])) {
                            return i;
                        }
                    }
                    return -1;
                };

                var showHideCostConsequenceFields = function () {
                    var isDisabled = costConsequenceSelect.value == 'no_consequence';
                    if(costConsequenceSelect.value == 'consequence')
                    {
                        if(findIndex(formUI.runtime.validations,function(validation)
                          {
                              return validation.id=="EUR";
                          })==-1)
                        {
                            formUI.runtime.validations.push(
                              {
                                  event:"keyup",
                                  fieldId:costConsequenceNOK.id,
                                  handler:Alfresco.forms.validation.mandatory,
                                  message:'The value cannot be empty',
                                  originalTitle:"",
                                  id:"NOK"
                              }
                            );
                            formUI.runtime.validations.push(
                              {
                                  event:"keyup",
                                  fieldId:costConsequenceEUR.id,
                                  handler:Alfresco.forms.validation.mandatory,
                                  message:'The value cannot be empty',
                                  originalTitle:"",
                                  id:"EUR"
                              }
                            );
                        }
                        labelEUR.getElementsByTagName("span")[0].className ="mandatory-indicator";
                        labelNOK.getElementsByTagName("span")[0].className ="mandatory-indicator";
                        //use parent element and from there use the first element in children - label.
                        //add little span in label to indicate mandatory field..
                    }
                    else
                    {
                        var costConsEURIndex = findIndex(formUI.runtime.validations,function(validation)
                        {
                            return validation.id=="EUR";
                        });
                        if(costConsEURIndex!=-1)
                        {
                            formUI.runtime.validations.splice(costConsEURIndex,1);
                        }
                        var costConsNOKIndex = findIndex(formUI.runtime.validations,function(validation)
                        {
                            return validation.id=="NOK";
                        });
                        if(costConsNOKIndex!=-1)
                        {
                            formUI.runtime.validations.splice(costConsNOKIndex,1);
                        }

                        //remove little span in label to indicate no longer mandatory field..
                        labelEUR.getElementsByTagName("span")[0].className ="hidden";
                        labelNOK.getElementsByTagName("span")[0].className ="hidden";
                    }
                    showHideField(costConsequenceNOK, isDisabled);
                    showHideField(costConsequenceEUR, isDisabled);
                };

                // Only initialize behaviour if the fields were not initially set to read only in the form
                if (!costConsequenceSelect.disabled) {
                    showHideCostConsequenceFields();
                    YAHOO.util.Event.on(costConsequenceSelect.id, 'change', function () {
                        // Update
                        showHideCostConsequenceFields();
                    });
                }

            }

            var scheduleConsequenceSelect = document.getElementsByName('prop_lysedl_vorScheduleConsequence')[0];
            var scheduleConsequence = document.getElementsByName('prop_lysedl_vorConsequenceSchedule')[0];

            if (scheduleConsequenceSelect && scheduleConsequence) {
                var showHideScheduleConsequenceFields = function () {
                    var isDisabled = scheduleConsequenceSelect.value == 'no_consequence';
                    showHideField(scheduleConsequence, isDisabled);
                };

                // Only initialize behaviour if the fields were not initially set to read only in the form
                if (!scheduleConsequenceSelect.disabled) {
                    // Initialize
                    showHideScheduleConsequenceFields();
                    YAHOO.util.Event.on(scheduleConsequenceSelect.id, 'change', function () {
                        // Update
                        showHideScheduleConsequenceFields();
                    });
                }
            }
            //}
        }

    };

    YAHOO.Bubbling.on("afterFormRuntimeInit", initVarationOrderFormBehaviour, this);
})();
