//@overridden projects/slingshot/source/web/modules/create-site.js
/**
 * Copyright (C) 2005-2010 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * CreatePS module
 *
 * A dialog for creating sites
 *
 * @namespace Alfresco.module
 * @class Alfresco.module.CreatePS
 */
(function()
{

   var Dom = YAHOO.util.Dom,
      Event = YAHOO.util.Event,
      Element = YAHOO.util.Element,
      KeyListener = YAHOO.util.KeyListener;

   /**
    * CreatePS
    *
    * @param htmlId {string} A unique id for this component
    * @return {Alfresco.CreatePS} The new DocumentList instance
    * @constructor
    */
   Alfresco.module.CreatePS = function(containerId, config)
   {
      var instance = Alfresco.util.ComponentManager.get(this.id);
      if (instance !== null)
      {
         throw new Error("An instance of Alfresco.module.CreatePS already exists.");
      }

      Alfresco.module.CreatePS.superclass.constructor.call(this, "Alfresco.module.CreatePS", containerId, ["button", "container", "connection", "selector", "json"]);

      this.config = config;
      return this;
   };

   YAHOO.extend(Alfresco.module.CreatePS, Alfresco.component.Base,
   {

      /**
       * Shows the CreateSite dialog to the user.
       *
       * @method show
       */
      show: function CreatePS_show()
      {
         if (this.widgets.panel)
         {
            // Panel is already in the DOM, so just show it
            this._showPanel();
         }
         else
         {
            /**
             * Load the gui from the server and let the templateLoaded() method
             * handle the rest.
             */
            Alfresco.util.Ajax.request(
            {
               url: this.getTemplateUrl(),
               dataObj:
               {
                  htmlid: this.id
               },
               successCallback:
               {
                  fn: this.onTemplateLoaded,
                  scope: this
               },
               execScripts: true,
               failureMessage: "Could not load create site template"
            });
         }
      },
      getTemplateUrl: function CreatePS_getTemplateUrl() {
		   return Alfresco.constants.URL_SERVICECONTEXT + "lyse/modules/lyse-create-ps";
	  },
	  getFormUrl: function CreatePS_getFormUrl() {
		   return Alfresco.constants.URL_SERVICECONTEXT + "lyse/modules/lyse-create-project";
	  },
      /**
       * Called when the CreatePS html template has been returned from the server.
       * Creates the YUI gui objects such as buttons and a panel and shows it.
       *
       * @method onTemplateLoaded
       * @param response {object} a Alfresco.util.Ajax.request response object
       */
      onTemplateLoaded: function CreatePS_onTemplateLoaded(response)
      {
         // Inject the template from the XHR request into a new DIV element
         var containerDiv = document.createElement("div");
         containerDiv.innerHTML = response.serverResponse.responseText;

         // The panel is created from the HTML returned in the XHR request, not the container
         var panelDiv = Dom.getFirstChild(containerDiv);

         this.widgets.panel = Alfresco.util.createYUIPanel(panelDiv);

         // Create the cancel button
         this.widgets.cancelButton = Alfresco.util.createYUIButton(this, "cancel-button", this.onCancelButtonClick);

         // Create the ok button, the forms runtime will handle when its clicked
         this.widgets.okButton = Alfresco.util.createYUIButton(this, "ok-button", null,
         {
            type: "submit"
         });

         // Configure the forms runtime
         var createHSEForm = new Alfresco.forms.Form(this.id + "-form");
         this.widgets.form = createHSEForm;

         var elTitle = Dom.get(this.id + "-title"),
            elShortName = Dom.get(this.id + "-shortName");

        this.widgets.typeObj = Dom.get(this.id + "-type");

         /**
          * Title field
          */
         // Title is mandatory
         createHSEForm.addValidation(elTitle, Alfresco.forms.validation.mandatory, null, "keyup", this.msg("validation-hint.mandatory"));
         // ...and has a maximum length
         createHSEForm.addValidation(elTitle, Alfresco.forms.validation.length,
         {
            max: 256,
            crop: true
         }, "keyup");

         var createUrlName = function CreatePS_title_keyUp()
         {
           if (!this.shortNameEdited)
           {
              elShortName.value = this.safeURL(elTitle.value).substring(0, 72);
              createHSEForm.validate();
           }
         };

         // Auto-generate a short name as long as the user hasn't manually entered one first
         Event.addListener(elTitle, "keyup", createUrlName, this, true);

         // Auto-generate a short name as long as the user hasn't manually entered one first
         Event.addListener(elTitle, "change", createUrlName, this, true);


         /**
          * Short name field
          */
         this.shortNameEdited = false;

         // Shortname is mandatory
         createHSEForm.addValidation(elShortName, Alfresco.forms.validation.mandatory, null, "keyup", this.msg("validation-hint.mandatory"));
         // ...and is restricted to a limited set of characters
         createHSEForm.addValidation(elShortName, Alfresco.forms.validation.regexMatch,
         {
            pattern: /^[ ]*[0-9a-zA-Z\-]+[ ]*$/
         }, "keyup", this.msg("validation-hint.siteName"));
         // ...and has a maximum length
         createHSEForm.addValidation(elShortName, Alfresco.forms.validation.length,
         {
            max: 72,
            crop: true
         }, "keyup");

         // Flag that the user has edited the short name
         Event.addListener(elShortName, "keyup", function CreatePS_shortName_keyUp()
         {
            this.shortNameEdited = elShortName.value.length > 0;
         }, this, true);

         /**
          * Description field
          */
         //createHSEForm.addValidation(this.id + "-description", Alfresco.forms.validation.mandatory, null, "keyup", this.msg("validation-hint.mandatory"));

         // Description kept to a reasonable length
         createHSEForm.addValidation(this.id + "-description", Alfresco.forms.validation.length,
         {
            max: 512,
            crop: true
         }, "keyup");


         createHSEForm.addValidation(this.id + "-projectNumber", Alfresco.forms.validation.number, null, "keyup", this.msg("validation-hint.number"));

         createHSEForm.addValidation(this.id + "-projectEndDate-cntrl-date", Alfresco.forms.validation.regexMatch,
         {
            pattern: /^[0-9][0-9][0-9][0-9][-][0-9][0-9]-[0-9][0-9]$/
         }, "keyup", this.msg("validation-hint.date"));

         /**
          * Project Manager field
          */
         createHSEForm.addValidation(this.id + "-projectManager", Alfresco.forms.validation.mandatory, null, "change", this.msg("validation-hint.mandatory"), {errorField:this.id + "-projectManager-cntrl-currentValueDisplay"});

         createHSEForm.addValidation(this.id + "-projectStartDate-cntrl-date", Alfresco.forms.validation.mandatory, null, "keyup", this.msg("validation-hint.mandatory"));
         createHSEForm.addValidation(this.id + "-projectStartDate-cntrl-date", Alfresco.forms.validation.regexMatch,
         {
            pattern: /^[0-9][0-9][0-9][0-9][-][0-9][0-9]-[0-9][0-9]$/
         }, "keyup", this.msg("validation-hint.date"));
         createHSEForm.addValidation(this.id + "-projectEndDate-cntrl-date", Alfresco.forms.validation.regexMatch,
         {
            pattern: /^[0-9][0-9][0-9][0-9][-][0-9][0-9]-[0-9][0-9]$/
         }, "keyup", this.msg("validation-hint.date"));
         // Case number is mandatory
         createHSEForm.addValidation(this.id + "-projectCaseNumber", Alfresco.forms.validation.mandatory, null, "keyup", this.msg("validation-hint.mandatory"));
         // ...and is restricted to a limited set of characters
         createHSEForm.addValidation(this.id + "-projectCaseNumber", Alfresco.forms.validation.regexMatch,
         {
            pattern: /^((([^,]+)\/([^,]+))+)(,(([^,]+)\/([^,]+)))*$/
         }, "keyup", this.msg("validation-hint.caseNumber"));

         //createHSEForm.addValidation(this.id + "-projectId", Alfresco.forms.validation.mandatory, null, "keyup", this.msg("validation-hint.mandatory"));
         //createHSEForm.addValidation(this.id + "-projectId", Alfresco.forms.validation.number, null, "keyup", this.msg("validation-hint.number"));

         //Special handling of sub project field
         var fieldId = document.getElementById(this.id + "-subProject");
         fieldId.value=this.config.site;

//         var elSubProjectControlField = Dom.get(this.id + "-subProject-cntrl");
//
//         Event.addListener(elSubProjectControlField, "keyup", function clearSubProjectControlField()
//         {
//            var cntrlFieldId = document.getElementById(this.id + "-subProject-cntrl");
//            var displayFieldId = document.getElementById(this.id + "-subProject-display");
//            var fieldId = document.getElementById(this.id + "-subProject");
//
//            if (cntrlFieldId.value=="") {
//              fieldId.value="";
//              displayFieldId.innerHTML="";
//            }
//
//         }, this, true);
//

         // The ok button is the submit button, and it should be enabled when the form is ready
         createHSEForm.setSubmitElements(this.widgets.okButton);
         createHSEForm.doBeforeFormSubmit = {
            fn: this.doBeforeFormSubmit,
            obj: null,
            scope: this
         };

         // Submit as an ajax submit (not leave the page), in json format
         createHSEForm.setAJAXSubmit(true,
         {
            successCallback:
            {
               fn: this.onCreatePSSuccess,
               scope: this
            },
            failureCallback:
            {
               fn: this.onCreatePSFailure,
               scope: this
            }
         });
         createHSEForm.setSubmitAsJSON(true);
         // We're in a popup, so need the tabbing fix
         createHSEForm.applyTabFix();
         createHSEForm.doBeforeAjaxRequest = {
            fn: this.doBeforeAjaxRequest,
            scope: this
         };
         createHSEForm.init();

         // Show the panel
         this._showPanel();
      },

      /**
       * Called when a preset as been selected.
       * Implement to make it possible to dispay custom site property fields
       *
       * @method onSitePresetChange
       * @param sitePreset
       */
      onSitePresetChange: function(sitePreset) {},

      /**
       * Interceptor just before Ajax request is sent.
       *
       * @method doBeforeAjaxRequest
       * @param p_config {object} Object literal containing request config
       * @return {boolean} True to continue sending form, False to prevent it
       */
      doBeforeAjaxRequest: function CreatePS_doBeforeAjaxRequest(p_config)
      {
         return true;
      },

      /**
       * Called before the form is about to be submitted
       *
       * @method doBeforeFormSubmit
       * @param form {HTMLFormElement} The create site form
       * @param obj {Object} Callback object
       */
      doBeforeFormSubmit: function(form, obj)
      {
         var formEl = Dom.get(this.id + "-form");
         formEl.attributes.action.nodeValue = this.getFormUrl();

         this.widgets.cancelButton.set("disabled", true);


         //Force the type to be a project
         this.widgets.typeObj.value = "{http://ns.lyse.no/model/project/1.0}project";

         this.widgets.panel.hide();
         this.widgets.feedbackMessage = Alfresco.util.PopupManager.displayMessage(
         {
            text: Alfresco.util.message("message.creating", this.name),
            spanClass: "wait",
            displayTime: 0
         });
      },

      /**
       * Converts a user-entered string into a "readable" safe URL by stripping characters
       *
       * @method safeURL
       * @param text {string} The string to convert
       * @return {string} Safe and readable URL
       */
      safeURL: function CreatePS_safeURL(text)
      {
         // Strip unwanted characters and trim leading and trailing spaces
         text = YAHOO.lang.trim(text.replace(/[^0-9a-zA-Z\-\s]/g, ""));
         // Replace remaining spaces with dash & convert the whole string to lower case
         text = text.replace(/\s+/g, "-").toLowerCase();

         return text;
      },

      /**
       * Called when user clicks on the isPublic checkbox.
       *
       * @method onVisibilityChange
       * @param type
       * @param el
       */
      onVisibilityChange: function CreatePS_onVisibilityChange(type, el)
      {
         new Element(this.widgets.isModerated).set("disabled", el == this.widgets.isPrivate);
      },

      /**
       * Called when user clicks on the cancel button.
       * Closes the CreatePS panel.
       *
       * @method onCancelButtonClick
       * @param type
       * @param args
       */
      onCancelButtonClick: function CreatePS_onCancelButtonClick(type, args)
      {
         // Reset the form fields
         try
         {
            Dom.get(this.id + "-title").value = "";
            Dom.get(this.id + "-shortName").value = "";
            Dom.get(this.id + "-description").value = "";
            Dom.get(this.id + "-sitePreset").selectedIndex = 0;
            Dom.get(this.id + "-isPublic").checked = "checked";
            Dom.get(this.id + "-isModerated").checked = "";
            Dom.get(this.id + "-isModerated").disabled = false;
         }
         catch(e)
         {
         }

         this.widgets.panel.hide();
      },

      /**
       * Called when a site has been succesfully created on the server.
       * Redirects the user to the new site.
       *
       * @method onCreatePSSuccess
       * @param response
       */
      onCreatePSSuccess: function CreatePS_onCreatePSSuccess(response)
      {
         if (response.json !== undefined && response.json.success)
         {
            // The site has been successfully created - add it to the user's favourites and navigate to it
            var preferencesService = new Alfresco.service.Preferences(),
               shortName = response.config.dataObj.shortName;

            preferencesService.favouriteSite(shortName,
            {
               successCallback:
               {
                  fn: function CreatePS_onCreatePSSuccess_successCallback()
                  {
                     document.location.href = Alfresco.constants.URL_PAGECONTEXT + "site/" + shortName + "/dashboard";
                  }
               }
            });
         }
         else
         {
            this._adjustGUIAfterFailure(response);
         }
      },

      /**
       * Called when a site failed to be created.
       *
       * @method onCreatePSFailure
       * @param response
       */
      onCreatePSFailure: function CreatePS_onCreatePSFailure(response)
      {
         this._adjustGUIAfterFailure(response);
      },

      /**
       * Helper method that restores the gui and displays an error message.
       *
       * @method _adjustGUIAfterFailure
       * @param response
       */
      _adjustGUIAfterFailure: function CreatePS__adjustGUIAfterFailure(response)
      {
         this.widgets.feedbackMessage.destroy();
         this.widgets.cancelButton.set("disabled", false);
         this.widgets.panel.show();
         var text = Alfresco.util.message("message.failure", this.name);

         if (response.serverResponse.status === 403)
         {
            // User does not have permissions to create the site
            if (response.json.message)
            {
               text = Alfresco.util.message(response.json.message, this.name)
            }
            else
            {
               text = Alfresco.util.message("error.noPermissions", this.name);
            }
         }
         else if (response.json.message)
         {
            var tmp = Alfresco.util.message(response.json.message, this.name);
            text = tmp ? tmp : text;
         }
         Alfresco.util.PopupManager.displayPrompt(
         {
            title: Alfresco.util.message("message.failure", this.name),
            text: text
         });
      },

      /**
       * Prepares the gui and shows the panel.
       *
       * @method _showPanel
       * @private
       */
      _showPanel: function CreatePS__showPanel()
      {
         // Reset form before displaying it
         this.widgets.form.reset();

         // Show the upload panel
         this.widgets.panel.show();

         // Firefox insertion caret fix
         Alfresco.util.caretFix(this.id + "-form");

         if (!this.widgets.escapeListener)
         {
            // Register the ESC key to close the dialog
            this.widgets.escapeListener = new KeyListener(document,
            {
               keys: KeyListener.KEY.ESCAPE
            },
            {
               fn: function(id, keyEvent)
               {
                  this.onCancelButtonClick();
               },
               scope: this,
               correctScope: true
            });
            this.widgets.escapeListener.enable();
         }

         // Set the focus on the first field
         Dom.get(this.id + "-title").focus();
      }
   });
})();

Alfresco.module.getCreatePSInstance = function(config)
{
   var instanceId = "alfresco-createHSE-instance";
   //return Alfresco.util.ComponentManager.get(instanceId) || new Alfresco.module.CreatePS(instanceId, config);
   //We always need a new master parent project in the config
   return new Alfresco.module.CreatePS(instanceId, config);
};
